#!/bin/bash
# Note: before running this script, manually change the version number of the package you wish to create
# File to edit is found at: SignStream3.app/Contents/Info.plist
# Setting name is: CFBundleVersion

# Read dai.cs.rutgers.edu username and password
echo "Username for dai.cs.rutgers.edu:"
read USER
echo "Password for dai.cs.rutgers.edu:"
read -s PW

DATE=`date +%m-%d-%Y`

# The base directory of our project
SRC_DIR=..

# Directory to build
BUILD_DIR=build

# Directory containing this script
SCRIPTS_DIR=scripts

# Directory holding releases
RELEASE_DIR=release

# Directory holding readmes under $RELEASE_DIR
README_DIR=readme

# The application name
APP_NAME=SignStream3.app

# Version number 
VER=$(cat $SRC_DIR/SignStream3.app/Contents/Info.plist | grep -A 1 CFBundleVersion | grep string | sed -e "s/.*<string>\(.*\)<\/string>.*/\1/")

# Update signstream version number in coding scheme and SS3DBProfile.java
DB_PROFILE=$SRC_DIR/src/edu/bu/signstream/common/util/vo/ss3/db/SS3DBProfile.java
sed -E -e "s/ssVersion = \".*\"/ssVersion = \"$VER\"/g" -i "" $DB_PROFILE
sed -E -e "s/SIGNSTREAM-VERSION=\".*\" DATABASE-VERSION/SIGNSTREAM-VERSION=\"$VER\" DATABASE-VERSION/g" -i "" $SRC_DIR/newXMLfiles/prd-defCodingScheme.xml
sed -E -e "s/SIGNSTREAM-VERSION=\".*\" DATABASE-VERSION/SIGNSTREAM-VERSION=\"$VER\" DATABASE-VERSION/g" -i "" $SRC_DIR/newXMLfiles/defCodingScheme.xml
sed -E -e "s/UPDATE_COMMAND_MAX_VERSION=.*/UPDATE_COMMAND_MAX_VERSION=\"$VER\"/" -i "" $SRC_DIR/Update.command

# Directory to create for staging everything to zip up
OUTER_DIR=SignStream3-$VER
INNER_DIR=SignStream3
DEST_DIR=$OUTER_DIR/$INNER_DIR

# Name of the archive zip we want
ARCHIVE=$DEST_DIR-$VER.zip

# First, build the SignStream code using ant, and put the .jar into the .app
echo "Building a new SignStream3.jar and placing it into $APP_NAME ..."
cd $SRC_DIR/$BUILD_DIR
ant release -q

ERROR=$?
if [[ $ERROR -ne 0 ]] ; then
  echo 'Exiting signstream-build-and-release.sh due to ant failure.'; exit $ERROR
fi

echo 'cp SignStream3.$DATE.jar SignStream3.jar'
cp SignStream3.$DATE.jar SignStream3.jar
echo "Copying new SignStream3.jar into $SRC_DIR/$APP_NAME ..."
cp SignStream3.jar $SRC_DIR/$APP_NAME/Contents/Resources/Java/

# Copy all files and dir's into $DEST_DIR
echo "Creating destination directory $DEST_DIR ..."
cd $SRC_DIR
mkdir -p $DEST_DIR
echo "Copying all needed files and directories from $SRC_DIR into $DEST_DIR ..."
cp -r icons images dep lib RunCommand.scpt DisableQuarantine.command GetSignBank.command Update.command $APP_NAME README.txt NOTICES.txt LICENSE.txt LICENSES-3RD-PARTY.txt $DEST_DIR
# PRD-only files are pre-pended with "prd-"
mkdir $DEST_DIR/signbank
mkdir $DEST_DIR/newXMLfiles
for f in signbank/prd-*; do cp "$f" $DEST_DIR/$(echo "$f" | sed 's/prd-//g'); done
for f in newXMLfiles/prd-*; do cp "$f" $DEST_DIR/$(echo "$f" | sed 's/prd-//g'); done

# Set owner/group on everything recurisvely
# Set permissions on everything differently
echo "Setting appropriate permissions on everything in $DEST_DIR ..."
cd $DEST_DIR
sudo chown -R root:wheel .
sudo chmod -R go-w icons
sudo chmod -R go-w images
sudo chmod -R go-w newXMLfiles
sudo chmod -R go-w signbank
sudo chmod -R go-w dep
sudo chmod -R go-w lib
sudo chmod go-w RunCommand.scpt
sudo chmod go+x RunCommand.scpt
sudo chmod go-w DisableQuarantine.command
sudo chmod go+x DisableQuarantine.command
sudo chmod go-w GetSignBank.command
sudo chmod go+x GetSignBank.command
sudo chmod go-w Update.command
sudo chmod go+x Update.command
sudo chmod -R go-w SignStream3.app
sudo chmod go-xw README.txt
sudo chmod go-xw NOTICES.txt
sudo chmod go-xw LICENSE.txt
sudo chmod go-xw LICENSES-3RD-PARTY.txt
cd ../..

# Zip it up and remove created destination directory
echo "Zipping up $DEST_DIR into $RELEASE_DIR/$ARCHIVE ..."
mkdir -p $RELEASE_DIR/$README_DIR	# creates dir `$RELEASE_DIR` and also `$RELEASE_DIR/$README_DIR` if both not exist
mkdir -p $RELEASE_DIR/$OUTER_DIR
cd $OUTER_DIR
rm -rf ../$RELEASE_DIR/$ARCHIVE # just in case it already exists
zip -rq ../$RELEASE_DIR/$ARCHIVE $INNER_DIR
cd ..

README_NAME=SignStream3-README-$VER.txt
echo "Copying current version README to release/readme/$README_NAME ..."
cp $DEST_DIR/README.txt $RELEASE_DIR/$README_DIR/$README_NAME

echo "Cleaning up ..."
sudo rm -rf $OUTER_DIR

# Take the newly created zip archive and README file and scp them to the DAI server
cd $SCRIPTS_DIR
expect scp.exp $USER $PW ../$RELEASE_DIR/$ARCHIVE dai.cs.rutgers.edu /cbim/Prod/SignStream/download/$RELEASE_DIR
expect scp.exp $USER $PW ../$RELEASE_DIR/$README_DIR/$README_NAME dai.cs.rutgers.edu /cbim/Prod/SignStream/download/$RELEASE_DIR/$README_DIR

echo "Done!"
exit 1

# TODO - ssh to DAI server and set symlinks to our new files - not sure on this
#ssh to dai.cs and set symlinks on the two new files there
