package edu.bu.signstream.common.textgrid;

import edu.bu.signstream.common.util.Pair;
import edu.bu.signstream.time.Bound;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.time.Time;

import java.util.ArrayList;
import java.util.List;

public class TextGrid {
    public Time start;
    public Time end;

    public final List<IntervalTier> intervalTiers = new ArrayList<>();
    public final List<TextTier> textTiers = new ArrayList<>();

    public static class Tier {
        public String name;
        public Time start;
        public Time end;
    }

    public static class IntervalTier extends Tier {
        public final List<Pair<String, Interval>> intervals = new ArrayList<>();
    }

    public static class TextTier extends Tier {
        public final List<Pair<String, Time>> points = new ArrayList<>();
    }

    public void addTier(Tier tier) {
        if (tier instanceof IntervalTier)
            intervalTiers.add((IntervalTier) tier);
        else
            textTiers.add((TextTier) tier);
    }

    public void validate() throws Exception {
        // Ensure no intervals are overlapping
        for (var tier : intervalTiers)
            for (var i0 : tier.intervals)
                for (var i1 : tier.intervals)
                    if (i0 != i1 && i0.second.overlaps(i1.second, Bound.EXCLUSIVE))
                        throw new Exception("TextGrid file contains overlapping intervals.");

        // Ensure no points are equal
        for (var tier : textTiers)
            for (var p0 : tier.points)
                for (var p1 : tier.points)
                    if (p0 != p1 && p0.second.equals(p1.second))
                        throw new Exception("TextGrid file contains overlapping points.");
    }
}
