package edu.bu.signstream.common.util;

public class Version implements Comparable<Version> {

	private int majorVersion, minorVersion, bugVersion;
	
	public Version(String entireVersionString) {
		if (entireVersionString == null || entireVersionString.isBlank()) {
			this.majorVersion = 0;
			this.minorVersion = 0;
			this.bugVersion = 0;
			return;
		}
		
		String[] parts = entireVersionString.split("\\.");
		
		if (parts.length == 3) {
			this.majorVersion = Integer.valueOf(parts[0]);
			this.minorVersion = Integer.valueOf(parts[1]);
			this.bugVersion = Integer.valueOf(parts[2]);
		} else if (parts.length == 2) {
			this.majorVersion = Integer.valueOf(parts[0]);
			this.minorVersion = Integer.valueOf(parts[1]);
			this.bugVersion = 0;
		} else if (parts.length == 1) {
			this.majorVersion = Integer.valueOf(parts[0]);
			this.minorVersion = 0;
			this.bugVersion = 0;
		} else {
			this.majorVersion = 0;
			this.minorVersion = 0;
			this.bugVersion = 0;
		}
	}

	public int getMajorVersion() {
		return majorVersion;
	}

	public int getMinorVersion() {
		return minorVersion;
	}

	public int getBugVersion() {
		return bugVersion;
	}

	@Override
	public int compareTo(Version other) {
		if (this.getMajorVersion() == other.getMajorVersion()) {
			if (this.getMinorVersion() == other.getMinorVersion()) {
				if (this.getBugVersion() == other.getBugVersion()) {
					return 0;
				}
				else {
					return (this.getBugVersion() < other.getBugVersion()) ? -1 : 1;
				}
			} else {
				return (this.getMinorVersion() < other.getMinorVersion()) ? -1 : 1;
			}
		} else {
			return (this.getMajorVersion() < other.getMajorVersion()) ? -1 : 1;
		}
	}
	
	public boolean greaterThanStrict(Version other) {
		return this.compareTo(other) == 1;
	}
	
	public boolean lessThanStrict(Version other) {
		return this.compareTo(other) == -1;
	}
	
	public boolean equals(Version other) {
		return this.compareTo(other) == 0;
	}
	
	// ignore the bug version
	public int compareToCompatible(Version other) {
		if (this.getMajorVersion() == other.getMajorVersion()) {
			if (this.getMinorVersion() == other.getMinorVersion()) {
				return 0;
			} else {
				return (this.getMinorVersion() < other.getMinorVersion()) ? -1 : 1;
			}
		} else {
			return (this.getMajorVersion() < other.getMajorVersion()) ? -1 : 1;
		}
	}
	
	public boolean greaterThanCompatible(Version other) {
		return ( ! this.equals(other)) && (this.compareToCompatible(other) == 1);
	}
	
	public boolean lessThanCompatible(Version other) {
		return ( ! this.equals(other)) && (this.compareToCompatible(other) == -1);
	}
	
	@Override
	public String toString() {
		if (this.majorVersion == 0 && this.minorVersion == 0 && this.bugVersion == 0) {
			return null;
		}

		return this.getMajorVersion() + "." + this.getMinorVersion() + "." + this.getBugVersion();
	}
}
