package edu.bu.signstream.common.util.https.responses;

import org.apache.http.Header;

import java.util.Objects;

/**
 * This class is used to store the response from an HTTP request. It is the base class for all other response classes.
 */
public class BasicResponse {
    private final int httpStatusCode;
    private final long contentLength; // This name is misleading. It is the length of the content in bytes, not the length of the content itself.
    private final Header contentType; // JSON, HTML, YAML, etc.
    private final String contentJSON; // The content of the response as a JSON string (not a JSONObject)

    public BasicResponse(int statusCode, long contentLength, Header contentType, String contentJSON) {
        this.httpStatusCode = statusCode;
        this.contentLength = contentLength;
        this.contentType = Objects.requireNonNull(contentType);
        this.contentJSON = Objects.requireNonNull(contentJSON);
    }

    public int getHttpStatusCode() {
        return this.httpStatusCode;
    }

    public long getContentLength() {
        return this.contentLength;
    }

    public Header getContentType() {
        return this.contentType;
    }

    public String getContentJSON() {
        return this.contentJSON;
    }
}
