package edu.bu.signstream.common.util.vo.ss3.db;

import java.awt.Color;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.gestureField.locationField.GestureLocationEvent;
import edu.bu.signstream.grepresentation.fields.gestureField.gesturePropertyField.GesturePropertyEvent;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEvent;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignTypeEvent;

public class SS3Item {
	private String id = "";
	private String imageID = "";
	private String text = "";

	private String locationRegionID = "";
	private int locationTypeID;
	private Boolean locationIsContralateral;

	private Time startTimeInfo;
	private Time endTimeInfo;

	private boolean startTimeFinal = false;
	private boolean endTimeFinal = false;
	private boolean textOnly = false;

	private String startCompoundEntityID = "";
	private String endCompoundEntityID = "";

	private Color color;
	public int order;

	public SS3Item() {
	}

	public SS3Item(String text, Time startTimeInfo) {
		this.text = text;
		setStartTimeInfo(startTimeInfo);
		id = "" + Util.getLongUniqueNumber();
	}
	
	public SS3Item(String id, Time startTimeInfo, Time endTimeInfo) { 
		this.id = id;
		setStartTimeInfo(startTimeInfo);
		setEndTimeInfo(endTimeInfo);
	}

	public SS3Item(String id, String imageID, Time startTimeInfo, Time endTimeInfo) {
		this.id = id;
		this.imageID = imageID;
		setStartTimeInfo(startTimeInfo);
		setEndTimeInfo(endTimeInfo);
	}

	public SS3Item(LocationEvent event) {
		this.id = event.getID();
		this.locationRegionID = event.getLocation().getId();
		this.locationTypeID = event.getLocationType().getId();
		setStartTimeInfo(event.getStartTimeInfo());
		setEndTimeInfo(event.getEndTimeInfo());
	}
	
	public SS3Item(GestureLocationEvent event) {
		this.id = event.getID();
		this.locationRegionID = event.getGestureLocation().getId();
		this.locationTypeID = event.getLocationType().getId();
		this.locationIsContralateral = event.getIsContralateral();
		this.startTimeInfo = event.getStartTimeInfo();
		this.endTimeInfo = event.getEndTimeInfo();
	}
	
	public SS3Item(SignTypeEvent event) {
		this.id = event.getID();
		setStartTimeInfo(event.getStartTimeInfo());
		setEndTimeInfo(event.getEndTimeInfo());
	}
	
	public void setTextOnly(boolean textOnly) {
		this.textOnly = textOnly;
	}
	
	public boolean isTextOnly() {
		return textOnly;
	}

	public SS3Item(TrackValue value) {
		setId("" + value.getValueId());
		// the below shouldn't be needed
//		setStartTimeInfo(value.getStartTimeInfo());
//		setEndTimeInfo(value.getEndTimeInfo());
		setText(value.getTextValue());
		setStartTimeInfo(value.getStartTimeInfo());
		setEndTimeInfo(value.getEndTimeInfo());
	}

	public SS3Item(GesturePropertyEvent event) {
		this.id = event.getID();
		setStartTimeInfo(event.getStartTimeInfo());
		setEndTimeInfo(event.getEndTimeInfo());
	}

	public String getStartCompoundEntityID() {
		return startCompoundEntityID;
	}

	public void setStartCompoundEntityID(String startCompoundEntityID) {
		this.startCompoundEntityID = startCompoundEntityID;
	}

	public String getEndCompoundEntityID() {
		return endCompoundEntityID;
	}

	public void setEndCompoundEntityID(String endCompoundEntityID) {
		this.endCompoundEntityID = endCompoundEntityID;
	}

	public void setImageID(String imageID) {
		this.imageID = imageID;
	}

	public String getImageID() {
		return imageID;
	}

	public String getId() {
		if (id == null || id.length() == 0)		// GD: want to remove this entirely, but not sure if it would break anything
			id = "" + Util.getLongUniqueNumber();
		return id;
	}

	public String getText() {
		return text;
	}

	public void setId(String string) {
		id = string;
	}

	public void setText(String string) {
		text = string;
	}

	public void setLocationRegionID(String locationRegionID) {
		this.locationRegionID = locationRegionID;
	}

	public String getLocationRegionID() {
		return locationRegionID;
	}

	public int getLocationTypeID() {
		return locationTypeID;
	}

	public void setLocationTypeID(int locationTypeID) {
		this.locationTypeID = locationTypeID;
	}

	public Boolean getLocationIsContralateral() {
		return locationIsContralateral;
	}
	
	public void setLocationIsContralteral(Boolean isContralateral) {
		this.locationIsContralateral = isContralateral;
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder("SS3Item: \n");
		sb.append("id = ").append(id);
		sb.append(", text = ").append(text);
		sb.append(", startTime = ").append(startTimeInfo.doubleValue()).append(" ").append(startTimeInfo.unit());
		sb.append(", endTime = ").append(endTimeInfo.doubleValue()).append(" ").append(endTimeInfo.unit());
		sb.append(", locationRegionID = ").append(locationRegionID);
		sb.append(", locationTypeID = ").append(locationTypeID);
		sb.append(", locationIsContralateral = ").append(locationIsContralateral);
		return sb.toString();
	}

	public Color getColor() {
		return color;
	}

	public void setColor(Color color) {
		this.color = color;
	}

    public Time getStartTimeInfo() { return startTimeInfo; }
	public Time getEndTimeInfo() { return endTimeInfo; }
	public Interval interval() { return startTimeInfo.to(endTimeInfo); }

	public void setEndTimeInfo(Time info) {
		endTimeInfo = info;
	}

	public void setStartTimeInfo(Time info) {
		startTimeInfo = info;
	}

	public boolean isEndTimeFinal() {
		return endTimeFinal;
	}

	public boolean isStartTimeFinal() {
		return startTimeFinal;
	}

	public void setEndTimeFinal(boolean b) {
		endTimeFinal = b;
	}

	public void setStartTimeFinal(boolean b) {
		startTimeFinal = b;
	}
}
