package edu.bu.signstream.grepresentation.fields;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.HashSet;

import javax.swing.JPanel;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.time.*;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.RootLabelObject;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public abstract class EventsEntity extends JPanel implements Event {

	private static final long serialVersionUID = 1L;

	protected boolean selected = false;

	private boolean highlighted = false;

	private String id = "";

	private String refEntityId = "";

	//protected String fieldID = "";

	private Color color = null;
   
	private Field field;

	private ArrayList<PresentationEventsEntity> parentEntities = new ArrayList<PresentationEventsEntity>();

//	private String uniqueID = "";

	protected SignStreamSegmentPanel segmentPanel = null;
   
//   private int beforeCompoundID = -1;
//   private int afterCompoundID = -1;
   
//	public static final int ONSET = 0;
//	public static final int INITIAL_HOLD = 1;
//	public static final int FINAL_HOLD = 2;
//	public static final int OFFSET = 3;

	public EventsEntity(SignStreamSegmentPanel segmentPanel, Field field) {
		//this.fieldID = fieldID;
		this.field = field;
		this.segmentPanel = segmentPanel;
	}
	
//	@Override
//	public String getUniqueID() {
//		return uniqueID;
//	}
//	
//	@Override
//	public void setUniqueID(String uniqueID) {
//		this.uniqueID = uniqueID;
//	}

	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return segmentPanel;
	}

//	public void setFieldID(String fieldID) {
//		this.fieldID = fieldID;
//	}
//
//	public String getFieldID() {
//		return fieldID;
//	}

	// implement for every field type!!!
	public abstract SS3Entity getSS3Entity();  
//	{
//		SS3Entity entity = new SS3Entity();
//      //String entID = id.trim().length() > 0 ? id : Util.getLongUniqueNumber();       
//		entity.setID(id);
//		entity.setRefEntityId(refEntityId);
//		entity.setReference(ref);
//		
//      entity.setValueStartTimeLocked(isValueStartTimeLocked());
//      entity.setValueEndTimeLocked(isValueEndTimeLocked());
//      entity.setInitialHoldStartTimeLocked(isInitialHoldStartTimeLocked());
//      entity.setFinalHoldEndTimeLocked(isFinalHoldEndTimeLocked());
//
//		return entity;
//	}

	@Override
	public void setColor(Color color) {
		this.color = color;
	}

	@Override
	public Color getColor() {
		return color;
	}

	public Color color() {
	    SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
        var field = getField();
        var csField = dbScheme.getField(field.getFieldID());

        if (isSelected()) {
//            return Color.RED;
            return Color.BLACK;
        } else if (isHighlighted()) {
//            return Color.BLUE;
            return Color.BLACK;
        } else if (getColor() != null) {
            return getColor();
        } else if (field.getColor() != null) {
            return field.getColor();
        } else if (csField != null && csField.getColor() != null) {
            return csField.getColor();
        } else {
            return Color.BLACK;
        }
    }

//	public void setShowNote(boolean showNote) {
//		this.showNote = showNote;
//	}
//
//	public boolean isShowNote() {
//		return showNote;
//	}

	@Override
	public void setID(String id) {
		this.id = id;
	}

	@Override
	public String getID() {
		return id;
	}

	public void setRefEntityId(String refEntityId) {
		this.refEntityId = refEntityId;
	}

	public String getRefEntityId() {
		return refEntityId;
	}

	public abstract Event getFirstEvent();

	public abstract Event getLastEvent();

	@Override
	public boolean isSelected() {
		return selected;
	}

	public void highlight(boolean highlight) {
		this.highlighted = highlight;
	}

	@Override
	public boolean isHighlighted() {
		return highlighted;
	}

	@Override
	public double getStartTimeCoordinate() {
		return segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertTimeInfoToCoordinate(getStartTimeInfo());
	}

	@Override
	public double getEndTimeCoordinate() {
		return segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertTimeInfoToCoordinate(getEndTimeInfo());
	}

	@Override
	public void setStartTimeCoordinate(double coord) {
		setStartTimeInfo(TimeRange.ABSOLUTE.time(Math.round(coord), TimeUnit.COORDINATE)
            .in(SS3Singleton.getVideoTimeRange()));
	}

	@Override
	public void setEndTimeCoordinate(double coord) {
        setEndTimeInfo(TimeRange.ABSOLUTE.time(Math.round(coord), TimeUnit.TIMESTAMP)
            .in(SS3Singleton.getVideoTimeRange()));
	}

    @Override
    public Interval interval() {
	    return new Interval(getStartTimeInfo(), getEndTimeInfo());
    }

    public boolean contains(Time time) {
	    return interval().contains(time, Bound.INCLUSIVE);
    }

    public boolean contains(Time time, Bound b) {
	    return interval().contains(time, b);
    }

    public boolean overlaps(Interval o) {
	    return interval().overlaps(o, Bound.INCLUSIVE);
    }

    public boolean overlaps(Interval o, Bound b) {
	    return interval().overlaps(o, b);
    }

    public boolean overlaps(Time start, Time end) {
        var x0 = getStartTimeInfo();
        var x1 = getEndTimeInfo();
        return start.lessThanOrEq(x1) && end.greaterThanOrEq(x0);
    }

    @Deprecated
	public boolean contains(double x) {
		double startCoord = getStartTimeCoordinate();
		double endCoord = getEndTimeCoordinate();

		// JACK TODO: This seems like a bad idea, do we really need this with the inclusive comparisons below?
		if(getStartTimeInfo().doubleValue() == getEndTimeInfo().doubleValue()) {
			startCoord = startCoord - 5;
			endCoord = endCoord + 5;
		}
		
		if(startCoord > endCoord) {
		    // JACK TODO: Is this branch ever reached?

//			int startTime = getStartTimeInfo().getMovieTime();
//			int endTime = getEndTimeInfo().getMovieTime();
//			getStartTimeInfo().setMovieTime(endTime);
//			getEndTimeInfo().setMovieTime(startTime);
			// commenting for now - cannot imagine the reason this would be a good idea, but...
//			
			return (endCoord <= x && x <= startCoord);
		} else {		
			return (startCoord <= x && x <= endCoord);
		}
	}

	@Deprecated
	public boolean isEntityInsideElement(double x1, double x2) {
		return (x1 < getStartTimeCoordinate() && x2 > getEndTimeCoordinate());
	}

	@Deprecated
	public boolean isOverlapping(double coordx1, double coordx2) {
		double startCoord = getStartTimeCoordinate();
		double endCoord = getEndTimeCoordinate();

		return (coordx1 <= startCoord && coordx2 >= startCoord)
				|| (coordx1 <= endCoord && coordx2 >= endCoord);
	}

	public abstract boolean selectEntityIfEventSelected();

	public abstract boolean selectEventIfEntitySelected();

	@Override
	public abstract boolean select();

	@Override
	public abstract boolean unselect();

	public abstract boolean adjustEntity(int x1, int x2);

	public abstract Event getSelectedEvent();

	public abstract void reset();

	public abstract boolean delete();

	public abstract Event getEventAt(double x);

	public abstract ArrayList<String> getEventsToAdd();
	
	public abstract ArrayList<Event> getEventsList();

	public abstract String getToolTipText(int x);

	// public abstract boolean setSelectedEventStartCoordinate(int coord);
	// public abstract boolean setSelectedEventEndCoordinate(int coord);

	public abstract boolean selectEvent(double x);

	public abstract void paint(Graphics2D g2d, double Y, Font font);

	@Override
	public String toString() {
		StringBuffer buffer = new StringBuffer("EventEntity: startTime = ");
		
		Time startTimeInfo = getStartTimeInfo();
		Time endTimeInfo = getEndTimeInfo();
		
		if(startTimeInfo == null) {
			buffer.append(" null ");
		} else {
			buffer.append(startTimeInfo.doubleValue() + " " + startTimeInfo.unit());
		}
		
		buffer.append(", endTime = ");		
		if(endTimeInfo == null) {
			buffer.append(" null ");
		} else {
			buffer.append(endTimeInfo.doubleValue() + " " + endTimeInfo.unit());
		}
		return buffer.toString();
	}

//	public boolean isValueStartTimeLocked() {
//		return lockedStart;
//	}

//	public void lockValueStartTime() {
//		this.valueStartTimeLocked = true;
//	}
//
//	public void unlockValueStartTime() {
//		this.valueStartTimeLocked = false;
//	}

//	public boolean isValueEndTimeLocked() {
//		return lockedEnd;
//	}

//	public void lockValueEndTime() {
//		this.valueEndTimeLocked = true;
//	}
//
//	public void unlockValueEndTime() {
//		this.valueEndTimeLocked = false;
//	}
//
//	public boolean isInitialHoldStartTimeLocked() {
//		return lockedStart;
//	}
//
//	public void lockInitialHoldStartTime() {
//		this.initialHoldStartTimeLocked = true;
//	}
//
//	public void unlockInitialHoldStartTime() {
//		this.initialHoldStartTimeLocked = false;
//	}

//	 public boolean isInitialHoldEndTimeLocked() {
//		 return lockedEnd;
//	 }
//	
	// public void initialHoldLockEndTime() {
	// this.initialHoldEndTimeLocked = true;
	// }
	//
	// public void initialHoldUnlockEndTime() {
	// this.initialHoldEndTimeLocked = false;
	// }

	// public void unlockFinalHoldStartTime() {
	// this.finalHoldStartTimeLocked = false;
	// }

//	public boolean isFinalHoldEndTimeLocked() {
//		return lockedEnd;
//	}

//	public void lockFinalHoldEndTime() {
//		this.finalHoldEndTimeLocked = true;
//	}
//
//	public void unlockFinalHoldEndTime() {
//		this.finalHoldEndTimeLocked = false;
//	}

	@Override
	public void highlight() {
		this.highlighted = true;
	}

	@Override
	public void unhighlight() {
		this.highlighted = false;
	}

   public Field getField() {
      return field;
   }

   public void setField(Field field) {
      this.field = field;
   }
   
   public ArrayList<PresentationEventsEntity> getParentEntities() {
	   return parentEntities;
   }
   
   public void setParentEntities(ArrayList<PresentationEventsEntity> parentEntities) {
	   this.parentEntities = (ArrayList<PresentationEventsEntity>) parentEntities.clone();
   }

   private static EventsEntity findEntity(String id, ArrayList<String> potentialFieldIDs) {
	   if (potentialFieldIDs == null)
		   potentialFieldIDs = new ArrayList<String>();
	   
       var uttView = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getUtteranceView();
       if (uttView != null) {
    	   ArrayList<Segment> segments = uttView.getSegments();

			for (Segment segment : segments) {
				RootLabelObject rootLabelObject = segment.getRootLabelObject();
				LabelObject glossLabelObject = rootLabelObject.getGlossLabelObject();
				LabelObject nonManualLabelObject = rootLabelObject.getNonManualLabelObject();
				LabelObject gestureLabelObject = rootLabelObject.getGestureLabelObject();
				LabelObject customLabelObject = rootLabelObject.getCustomLabelObject();
				
				ArrayList<LabelObject> allObjects = nonManualLabelObject.getAllChildren();
				allObjects.addAll(glossLabelObject.getAllChildren());
				allObjects.addAll(gestureLabelObject.getAllChildren());
				allObjects.addAll(customLabelObject.getAllChildren());
				
				for (LabelObject obj : allObjects) {
					var wrapper = obj.getWrapper();
					var field = wrapper.getField();
					
					// limit this to just the fields where we would expect to find the item, if provided
					if ( !potentialFieldIDs.isEmpty() && !potentialFieldIDs.contains(field.getFieldID()))
						continue;
					
					var e = field.getEntity(id);
					if (e != null)
						return e;
				}
			}
       }

       return null;
   }

   // call this function when EventsEntity subclass is created dynamically (i.e. not from db parsing, etc)
   // it is also called in SignStreamSegmentPanel.updateLoadedDatabase() for every entity - to reset in case it was edited
   // the return is the list of id's of entities whose parents were also updated simultaneously to this entity
   public ArrayList<String> updateParentEntity() {
	   ArrayList<String> retEntityIDs = new ArrayList<String>();
	   
	   var utteranceField = field.getParentField();
	   if (utteranceField == null || this instanceof PresentationEventsEntity)
		   return retEntityIDs;

	   var thisSS3Entity = this.getSS3Entity();
	   if (thisSS3Entity.getItems().isEmpty())
		   return retEntityIDs;

	   String fieldID = this.getField().getFieldID();

	   ArrayList<EventsEntity> entitiesForInterval = new ArrayList<EventsEntity>();
	   entitiesForInterval.add(this);
	   
	   ArrayList<String> visited = new ArrayList<String>();
	   visited.add(thisSS3Entity.getID());

	   // assemble all the entities of this 'group' together to form one giant interval end to end
	   for (int i = 0; i < entitiesForInterval.size(); i++) {
		   EventsEntity currentEventsEntity = entitiesForInterval.get(i);
		   SS3Entity ss3Entity = currentEventsEntity.getSS3Entity();

		   if (ss3Entity.getRefEntityId() != null && !ss3Entity.getRefEntityId().isBlank() && !visited.contains(ss3Entity.getRefEntityId())) {
			   var potentialFieldsList = new ArrayList<String>();	// not exactly sure where ref_entity could be pointing, so leave list empty here
			   
			   var eventsEntity = findEntity(ss3Entity.getRefEntityId(), potentialFieldsList);
			   if (eventsEntity != null) {
				   entitiesForInterval.add(eventsEntity);
				   visited.add(ss3Entity.getRefEntityId());
			   }
		   }

		   if (ss3Entity.getDependentApexEntityID() != null && !ss3Entity.getDependentApexEntityID().isBlank() && !visited.contains(ss3Entity.getDependentApexEntityID())) {
			   var potentialFieldsList = new ArrayList<String>();
			   potentialFieldsList.add(Util.DOM_GESTURE_APEX_FIELD_ID); potentialFieldsList.add(Util.NDOM_GESTURE_APEX_FIELD_ID);
			   
			   var eventsEntity = findEntity(ss3Entity.getDependentApexEntityID(), potentialFieldsList);
			   if (eventsEntity != null) {
				   entitiesForInterval.add(eventsEntity);
				   visited.add(ss3Entity.getDependentApexEntityID());
			   }
		   }

		   if (ss3Entity.getDependentGestureEntityID() != null && !ss3Entity.getDependentGestureEntityID().isBlank() && !visited.contains(ss3Entity.getDependentGestureEntityID())) {
			   var potentialFieldsList = new ArrayList<String>();
			   potentialFieldsList.add(Util.DOM_GESTURE_FIELD_ID); potentialFieldsList.add(Util.NDOM_GESTURE_FIELD_ID);
			   
			   var eventsEntity = findEntity(ss3Entity.getDependentGestureEntityID(), potentialFieldsList);
			   if (eventsEntity != null) {
				   entitiesForInterval.add(eventsEntity);
				   visited.add(ss3Entity.getDependentGestureEntityID());
			   }
		   }

		   if (ss3Entity.getDependentGesturePropertyEntityID() != null && !ss3Entity.getDependentGesturePropertyEntityID().isBlank() && !visited.contains(ss3Entity.getDependentGesturePropertyEntityID())) {
			   var potentialFieldsList = new ArrayList<String>();
			   potentialFieldsList.add(Util.DOM_GESTURE_PROPERTY_FIELD_ID); potentialFieldsList.add(Util.NDOM_GESTURE_PROPERTY_FIELD_ID);
			   
			   var eventsEntity = findEntity(ss3Entity.getDependentGesturePropertyEntityID(), potentialFieldsList);
			   if (eventsEntity != null) {
				   entitiesForInterval.add(eventsEntity);
				   visited.add(ss3Entity.getDependentGesturePropertyEntityID());
			   }
		   }

		   if (ss3Entity.getDependentGlossEntityID() != null && !ss3Entity.getDependentGlossEntityID().isBlank() && !visited.contains(ss3Entity.getDependentGlossEntityID())) {
			   var potentialFieldsList = new ArrayList<String>();
			   potentialFieldsList.add(Util.DOM_GLOSS_FIELD_ID); potentialFieldsList.add(Util.NDOM_GLOSS_FIELD_ID);
			   
			   var eventsEntity = findEntity(ss3Entity.getDependentGlossEntityID(), potentialFieldsList);
			   if (eventsEntity != null) {
				   entitiesForInterval.add(eventsEntity);
				   visited.add(ss3Entity.getDependentGlossEntityID());
			   }
		   }

		   if (ss3Entity.getDependentHandEntityID() != null && !ss3Entity.getDependentHandEntityID().isBlank() && !visited.contains(ss3Entity.getDependentHandEntityID())) {
			   var potentialFieldsList = new ArrayList<String>();
			   potentialFieldsList.add(Util.DOM_HAND_FIELD_ID); potentialFieldsList.add(Util.NDOM_HAND_FIELD_ID);
			   
			   var eventsEntity = findEntity(ss3Entity.getDependentHandEntityID(), potentialFieldsList);
			   if (eventsEntity != null) {
				   entitiesForInterval.add(eventsEntity);
				   visited.add(ss3Entity.getDependentHandEntityID());
			   }
		   }

		   if (ss3Entity.getDependentHandShapeEntityID() != null && !ss3Entity.getDependentHandShapeEntityID().isBlank() && !visited.contains(ss3Entity.getDependentHandShapeEntityID())) {
			   var potentialFieldsList = new ArrayList<String>();
			   potentialFieldsList.add(Util.DOM_HANDSHAPE_FIELD_ID); potentialFieldsList.add(Util.NDOM_HANDSHAPE_FIELD_ID);
			   
			   var eventsEntity = findEntity(ss3Entity.getDependentHandShapeEntityID(), potentialFieldsList);
			   if (eventsEntity != null) {
				   entitiesForInterval.add(eventsEntity);
				   visited.add(ss3Entity.getDependentHandShapeEntityID());
			   }
		   }

		   if (ss3Entity.getDependentLocationEntityID() != null && !ss3Entity.getDependentLocationEntityID().isBlank() && !visited.contains(ss3Entity.getDependentLocationEntityID())) {
			   var potentialFieldsList = new ArrayList<String>();
			   potentialFieldsList.add(Util.DOM_LOCATION_FIELD_ID); potentialFieldsList.add(Util.NDOM_LOCATION_FIELD_ID);
			   
			   var eventsEntity = findEntity(ss3Entity.getDependentLocationEntityID(), potentialFieldsList);
			   if (eventsEntity != null) {
				   entitiesForInterval.add(eventsEntity);
				   visited.add(ss3Entity.getDependentLocationEntityID());
			   }
		   }

		   if (ss3Entity.getDependentOrientationFingerTipEntityID() != null && !ss3Entity.getDependentOrientationFingerTipEntityID().isBlank() && !visited.contains(ss3Entity.getDependentOrientationFingerTipEntityID())) {
			   var potentialFieldsList = new ArrayList<String>();
			   potentialFieldsList.add(Util.DOM_GESTURE_ORIENTATION_FINGER_TIP_FIELD_ID); potentialFieldsList.add(Util.NDOM_GESTURE_ORIENTATION_FINGER_TIP_FIELD_ID);
			   
			   var eventsEntity = findEntity(ss3Entity.getDependentOrientationFingerTipEntityID(), potentialFieldsList);
			   if (eventsEntity != null) {
				   entitiesForInterval.add(eventsEntity);
				   visited.add(ss3Entity.getDependentOrientationFingerTipEntityID());
			   }
		   }

		   if (ss3Entity.getDependentSignTypeEntityID() != null && !ss3Entity.getDependentSignTypeEntityID().isBlank() && !visited.contains(ss3Entity.getDependentSignTypeEntityID())) {
			   var potentialFieldsList = new ArrayList<String>();
			   potentialFieldsList.add(Util.DOM_SIGN_TYPE_FIELD_ID); potentialFieldsList.add(Util.NDOM_SIGN_TYPE_FIELD_ID);
			   
			   var eventsEntity = findEntity(ss3Entity.getDependentSignTypeEntityID(), potentialFieldsList);
			   if (eventsEntity != null) {
				   entitiesForInterval.add(eventsEntity);
				   visited.add(ss3Entity.getDependentSignTypeEntityID());
			   }
		   }
	   }

	   Time startTime = null, endTime = null;
	   for (int i = 1; i < entitiesForInterval.size(); i++) {
		   EventsEntity e1 = entitiesForInterval.get(i - 1);
		   EventsEntity e2 = entitiesForInterval.get(i);

		   if (e1.interval().t0.lessThan(e2.interval().t0))
			   startTime = e1.interval().t0;
		   else
			   startTime = e2.interval().t0;

		   if (e1.interval().t1.greaterThan(e2.interval().t1))
			   endTime = e1.interval().t1;
		   else
			   endTime = e2.interval().t1;
	   }

	   Interval combinedInterval = null;
	   if (startTime == null && endTime == null)
		   combinedInterval = this.interval();
	   else
		   combinedInterval = new Interval(startTime, endTime);

	   HashSet<PresentationEventsEntity> utterances = new HashSet<PresentationEventsEntity>();		// no duplicates

	   // inclusive check on utterance bounds for everything except translation entities (who must have exactly one parent utterance)
	   Bound bound = Bound.INCLUSIVE;
	   if (fieldID != null && ! fieldID.isEmpty() && fieldID.equals(Util.FREE_TEXT_FIELD_ID))
		   bound = Bound.EXCLUSIVE;

	   for (PresentationEventsEntity utteranceEntity: utteranceField.getEntities()) {
		   //        		if (utteranceEntity.overlaps(ss3Entity.interval(), bound)) {
		   if (utteranceEntity.overlaps(combinedInterval, bound)) {
			   utterances.add(utteranceEntity);
		   }
	   }

	   //        	boolean gapsExist = false;
	   //        	for (int i = 1; i < utterances.size(); i++) {
	   //        		PresentationEventsEntity first = utterances.get(i - 1);
	   //        		PresentationEventsEntity second = utterances.get(i);
	   //        		if (! first.getEndTimeInfo().equals(second.getStartTimeInfo())) {
	   //        			gapsExist = true;
	   //        		}
	   //        	}
	   //        	
	   //        	if (gapsExist)
	   //        		utterances.clear();

	   ArrayList<PresentationEventsEntity> utterancesList = new ArrayList<PresentationEventsEntity>(utterances);
	   for (EventsEntity e: entitiesForInterval) {
		   e.setParentEntities(utterancesList);

		   retEntityIDs.add(e.getID());
	   }

	   return retEntityIDs;
   }

//   public int getAfterCompoundID() {
//      return afterCompoundID;
//   }
//
//   public int getBeforeCompoundID() {
//      return beforeCompoundID;
//   }
//
//   public void setAfterCompoundID(int i) {
//      afterCompoundID = i;
//   }
//
//   public void setBeforeCompoundID(int i) {
//      beforeCompoundID = i;
//   }
}
