package edu.bu.signstream.grepresentation.fields;

import java.util.*;
import edu.bu.signstream.common.util.Sorter;

public class EventsIterator {
   private ArrayList sortedEvents;
   private double[] sortedStartCoordinates; //sortedStartCoordinates;

   public EventsIterator(ArrayList events) {
      sort(events);
   }
   
   public ArrayList getSortedEvents() {
	   return sortedEvents;
   }

   private ArrayList getSortedEvents(ArrayList events, double[] sorted) {
      ArrayList list = new ArrayList();
      for (int i = 0; i < sorted.length; i++) {
         Event event = getEvent(events, sorted[i]);
         list.add(event);
      }
      return list;
   }

   private void sort(ArrayList events) {
      sortedStartCoordinates = new double[events.size()];

      for (int i = 0; i < events.size(); i++) {
         Event event = (Event) events.get(i);
         sortedStartCoordinates[i] = event.getStartTimeInfo().doubleValue();
      }

      Sorter.sort(sortedStartCoordinates);

      this.sortedEvents = getSortedEvents(events, sortedStartCoordinates);
   }

   private Event getEvent(ArrayList events, double time) {
      for (int i = 0; i < events.size(); i++) {
         Event event = (Event) events.get(i);

         if (event.getStartTimeInfo().doubleValue() == time)
            return event;
      }
      return null;
   }

   public double nextEventStartCoordinate(int coord) {
      int element = elementAt(coord);

      if (element == -1 || element == sortedStartCoordinates.length - 1)
         return -1;

      return sortedStartCoordinates[element + 1];
   }

   public double previousEventStartCoordinate(int coord) {
      int element = elementAt(coord);

      if (element < 1)
         return -1;

      return sortedStartCoordinates[element - 1];
   }

   private int elementAt(int coord) {
      for (int i = 0; i < sortedStartCoordinates.length; i++) {
         if (sortedStartCoordinates[i] == coord)
            return i;
      }
      return -1;
   }

   private int lastIndexOf(Event event) {
      int index = 0;
      double startTimeCoord = event.getStartTimeCoordinate();
      for (int i = 0; i < sortedEvents.size(); i++) {
         Event e = (Event) sortedEvents.get(i);
         double coord = e.getStartTimeCoordinate();
         if (coord == startTimeCoord)
            index = i;
      }
      return index;
   }

   public Event next(Event event) {
         int index = //lastIndexOf(event);
   sortedEvents.lastIndexOf(event);

      if (index < sortedEvents.size() - 1) {
         //Event next = (Event) sortedEvents.get(index + 1);
         //		//
         //		System.out.println(
         //			"EVENT = "
         //				+ event.getStartTimeCoordinate()
         //				+ ", next = "
         //				+ next.getStartTimeCoordinate()
         //				+ ", sortedEvents.size = "
         //				+ sortedEvents.size()
         //				+ ", index = "
         //				+ index);
         //		//         		
         return (Event) sortedEvents.get(index + 1);
      }
      return null;
   }

   public double nextEventStartTime(int coord) {
	   
      for (int i = 0; i < sortedStartCoordinates.length; i++) {
         double ii = sortedStartCoordinates[i];
         if (ii == coord && i < sortedStartCoordinates.length - 1)
            return sortedStartCoordinates[i + 1];
      }
      return -1;
   }

   public Event previous(Event event) {
      int index = sortedEvents.indexOf(event);

      if (index > 0)
         return (Event) sortedEvents.get(index - 1);
      return null;
   }

   // returns Event that is in front of given coordinate
   public Event previous(int coord) {
      Event previous = null;

      for (int i = 0; i < sortedEvents.size(); i++) {
         Event event = (Event) sortedEvents.get(i);
         double startTime = event.getStartTimeCoordinate();

         if (startTime < coord) {
            if (previous == null) {
               previous = event;
            } else if (startTime > previous.getStartTimeCoordinate()) {
               previous = event;
            }
         }
      }
      return previous;
   }

   // returns Event that is following given coordinate
   public Event next(int coord) {
      Event next = null;

      for (int i = 0; i < sortedEvents.size(); i++) {
         Event event = (Event) sortedEvents.get(i);
         double startTime = event.getStartTimeCoordinate();

         if (startTime >= coord) {
            if (next == null) {
               next = event;
            } else if (startTime < next.getStartTimeCoordinate()) {
               next = event;
            }
         }
      }
      return next;
   }
}