package edu.bu.signstream.grepresentation.fields.apexField;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class ApexChainedApexEventsEntity extends ChainedApexEventsEntity implements CompoundableApex {

	private static final long serialVersionUID = 1L;
	
	private boolean isLockedDomNdomAlignments = false;
	
	private ApexChainedApexEventsEntity dependentApexEntity = null;

    private String dependentApexEntityID;

    private ApexChainedApexEventsEntity previousEntity = null;
    private ApexChainedApexEventsEntity nextEntity = null;

    private ChainedApexEventsCollection chainedEventsCollection = null;

    protected ChainedApexEventsCollectionManager manager;

    private String correspondingGlossEntityID = "";

    private CompoundableApex startCompound = null;
    private CompoundableApex endCompound = null;

    private ApexEntityProperties glossProperties = new ApexEntityProperties();

    public String getDependentApexEntityID() {
		return dependentApexEntityID;
	}

	public void setDependentApexEntityID(String dependentGlossEntityID) {
		this.dependentApexEntityID = dependentGlossEntityID;
	}

	public ApexChainedApexEventsEntity(ChainedApexEventsCollection collection, SignStreamSegmentPanel segmentPanel, Field field) {
		super(collection, segmentPanel, field);
		setID(Util.getLongUniqueNumber());
		this.manager = new ChainedApexEventsCollectionManager(collection,
				segmentPanel);

		this.chainedEventsCollection = collection;

//        this.getSS3Entity();
	}
	
	public boolean isLockedDomNdomAlignments() {
		return isLockedDomNdomAlignments;
	}
	
	public void setLockedDomNdomAlignments(boolean isLockedDomNdomAlignments) {
		this.isLockedDomNdomAlignments = isLockedDomNdomAlignments;
	}

	public ApexEntityProperties getApexEntityProperties() {
		if(glossProperties == null) {
			glossProperties = new ApexEntityProperties();
		}
		return glossProperties;
	}
	
	@Override
	public ApexChainedApexEventsEntity clone(String fieldID) {
		ChainedApexEventsCollection cll = manager.getChainedApexEventsCollection().cloneChainedApexEventsCollection();
		ApexChainedApexEventsEntity ret = new ApexChainedApexEventsEntity(cll, segmentPanel, getField());
		ret.updateParentEntity();
		return ret;
	}

	@Override
	public String addOffsetEvent() {
		if (nextEntity != null) {
			Time startMovieTimeInfo = nextEntity.getStartTimeInfo();

			ChainedApexEvent event = (ChainedApexEvent) manager.getFirstEvent();
			Time selectedEventEndTimeInfo = segmentPanel.getMultipleMovieController()
					.getFollowingFrameTimeInfo(event.getEndTimeInfo());

			if (selectedEventEndTimeInfo.greaterThan(startMovieTimeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		manager.addOffsetEvent();

		return null;
	}
	
	@Override
	public void deleteOnset() {
		super.deleteOnset();

		ApexChainedApexEventsEntity depGlsEntity = getDependentApexEntity();
		
		if (isLockedDomNdomAlignments()) {
			if (depGlsEntity != null && depGlsEntity.getOnset() != null) {
				depGlsEntity.deleteOnset();
				depGlsEntity.setStartTimeInfo(getStartTimeInfo());
			}
		}
	}
	
	@Override
	public void deleteOffset() {
		super.deleteOffset();

		ApexChainedApexEventsEntity depGlsEntity = getDependentApexEntity();
		
		if (isLockedDomNdomAlignments()) {
			if (depGlsEntity != null && depGlsEntity.getOffset() != null) {
				depGlsEntity.deleteOffset();
				depGlsEntity.setEndTimeInfo(getEndTimeInfo());
			}
		}
	}
	
	@Override
	public void deleteInitialHold() {
		super.deleteInitialHold();

		ApexChainedApexEventsEntity depGlsEntity = getDependentApexEntity();
		
		Time time = getStartTimeInfo();
		
		if (isLockedDomNdomAlignments()) {
			if (depGlsEntity != null && depGlsEntity.getInitialHold() != null) {
				depGlsEntity.deleteInitialHold();
				depGlsEntity.setStartTimeInfo(time);
			}
		}
	}
	
	@Override
	public void deleteFinalHold() {
		super.deleteFinalHold();

		ApexChainedApexEventsEntity depGlsEntity = getDependentApexEntity();
		
		Time time = getEndTimeInfo();
		
		if (isLockedDomNdomAlignments()) {
			if (depGlsEntity != null && depGlsEntity.getFinalHold() != null) {
				depGlsEntity.deleteFinalHold();
				depGlsEntity.setEndTimeInfo(time);
			}
		}
	}

	public ArrayList<EventsEntity> getDependentEntities() {
		ArrayList<EventsEntity> dEntities = new ArrayList<EventsEntity>();
		
		if(dependentApexEntity != null) {
			dEntities.add(dependentApexEntity);
		}
		return dEntities;
	}
	
	public void setApexEntityProperties(ApexEntityProperties glossProperties) {
		this.glossProperties = glossProperties;
	}
	
	public ApexChainedApexEventsEntity getDependentApexEntity() {
		return dependentApexEntity;
	}
	
	public void setDependentApexEntity(ApexChainedApexEventsEntity dependentApexEntity) {
		if (dependentApexEntity != null) {
			this.dependentApexEntityID = dependentApexEntity.getID();
		}
		this.dependentApexEntity = dependentApexEntity;
	}
	
	@Override
	public ChainedApexEventsCollection getChainedApexEventsCollection() {
		return chainedEventsCollection;
	}

	public void setCorrespondingGlossEntityID(String correspondingGlossEntityID) {
		this.correspondingGlossEntityID = correspondingGlossEntityID;
	}
	
	public String getCorrespondingGlossEntityID() {
		return correspondingGlossEntityID;
	}

	@Override
	public ApexChainedApexEventsEntity getPreviousEntity() {
		return previousEntity;
	}

	public void setPreviousEntity(ApexChainedApexEventsEntity previousEntity) {
		this.previousEntity = previousEntity;
	}

	@Override
	public ApexChainedApexEventsEntity getNextEntity() {
		return nextEntity;
	}

	public void setNextEntity(ApexChainedApexEventsEntity nextEntity) {
		this.nextEntity = nextEntity;
	}

	@Override
	public String getText() {
        if(getTextValue() == null) {
            return "";
        } else {
		    return getTextValue().getText();
        }
	}

	
	@Override
	public String getText(boolean start) {
		return getText();
	}

	@Override
	public boolean isEndCompound() {
		return endCompound != null;
	}

	@Override
	public boolean isStartCompound() {
		return startCompound != null;
	}

	@Override
	public CompoundableApex getEndCompound() {
		return endCompound;
	}

	@Override
	public CompoundableApex getStartCompound() {
		return startCompound;
	}

	@Override
	public void setEndCompound(CompoundableApex endCompound) {
		this.endCompound = endCompound;
	}

	@Override
	public void setStartCompound(CompoundableApex startCompound) {
		this.startCompound = startCompound;
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		g2d.setFont(font);
		FontMetrics fontMetrics = getFontMetrics(font);
		
		segmentPanel.getPainter().drawApexEntity(this, g2d, Y, fontMetrics, getField().getFieldID(), previousEntity, nextEntity);
	}
}
