package edu.bu.signstream.grepresentation.fields.apexField;

import java.util.ArrayList;

import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.grepresentation.fields.Event;

public class ChainedApexEventsCollection {
   private ChainedApexEvent onset = null;
   private ChainedApexEvent initialHold = null;
   private ChainedApexEvent value = null;
   private ApexChainedApexEvent textValue = null;
   private ChainedApexEvent finalHold = null;
   private ChainedApexEvent offset = null;

   public ChainedApexEvent getOnsetEvent() {
      return onset;
   }

   public ChainedApexEventsCollection cloneChainedApexEventsCollection() {
      ChainedApexEventsCollection collection = new ChainedApexEventsCollection();
      if (onset != null)
         collection.setOnsetEvent(onset.cloneChainedApexEvent());
      if (initialHold != null)
         collection.setInitialHoldEvent(initialHold.cloneChainedApexEvent());
      if (value != null)
         collection.setValueEvent(value.cloneChainedApexEvent());
      if (textValue != null)
         collection.setTextValueEvent(textValue.cloneApexChainedApexEvent());
      if (finalHold != null)
         collection.setFinalHoldEvent(finalHold.cloneChainedApexEvent());
      if (offset != null)
         collection.setOffsetEvent(offset.cloneChainedApexEvent());
      return collection;
   }
   
   public Event getNextEvent(Event event) {
      ArrayList<ChainedApexEvent> collection = getCollection();
      for(int i = 0; i < collection.size()-1; i++) {
         Event cEvent = collection.get(i);
         if(event.equals(cEvent)) {
            return collection.get(i+1);
         }
      }
      return null;
   }
   
   public Event getPreviousEvent(Event event) {
      ArrayList<ChainedApexEvent> collection = getCollection();
      for(int i = 1; i < collection.size(); i++) {
         Event cEvent = collection.get(i);
         if(event.equals(cEvent)) {
            return collection.get(i-1);
         }
      }
      return null;
   }

//   public ArrayList<NoteItem> getNoteItems(String entityID) {
//      ArrayList<NoteItem> noteItems = new ArrayList<NoteItem>();
//      ArrayList<ChainedApexEvent> col = getCollection();
//
//      for (ChainedApexEvent event : col) {
//         if (event != null) {
//            String noteID = event.getNoteID();
//            if (noteID != null && noteID.length() > 0) {
//               String valueID = event.getID();
//               String text = null;
//               if (event instanceof ApexChainedApexEvent)
//                  text = event.getText();
//               NoteItem item = new NoteItem();
//               item.setNoteID(noteID);
//               item.setEntityID(entityID);
//               item.setValueID(valueID);
//               item.setText(text);
//               noteItems.add(item);
//            }
//         }
//      }
//      return noteItems;
//   }

   public ArrayList<ChainedApexEvent> getCollection() {
      ArrayList<ChainedApexEvent> list = new ArrayList<ChainedApexEvent>();
      addEvent(list, onset);
      addEvent(list, initialHold);
      addEvent(list, value);
      addEvent(list, textValue);
      addEvent(list, finalHold);
      addEvent(list, offset);
      return list;
   }
   
   public ChainedApexEvent getEventAt(double x) {
      ArrayList<ChainedApexEvent> list = getCollection();
      for(ChainedApexEvent event : list) {
    	  double startTimeCoord = event.getStartTimeCoordinate();
    	  double endTimeCoord = event.getEndTimeCoordinate();
    	  
    	  if(event.getStartTimeInfo().value() == event.getEndTimeInfo().value()) {
    		  startTimeCoord = startTimeCoord - 5;
    		  endTimeCoord = endTimeCoord + 5; 
    	  }
    	  
    	  if(startTimeCoord <= x && x <= endTimeCoord) {
    		  return event;
    	  }
      }
      return null;
   }

   private void addEvent(ArrayList<ChainedApexEvent> list, ChainedApexEvent event) {
      if (event != null)
         list.add(event);
   }

   public void setOnsetEvent(ChainedApexEvent onset) {
      this.onset = onset;
      if (onset != null)
         onset.setText(Constants.ONSET);
   }

   public ChainedApexEvent getInitialHoldEvent() {
      return initialHold;
   }

   public void setInitialHoldEvent(ChainedApexEvent initialHold) {
      this.initialHold = initialHold;
      if (initialHold != null)
         initialHold.setText(Constants.INITIAL_HOLD);
   }

   public ChainedApexEvent getValueEvent() {
      return value;
   }

   public ChainedApexEvent getValueOrTextValueEvent() {
      return value == null ? textValue : value;
   }

   public void setValueEvent(ChainedApexEvent value) {
      this.value = value;
   }

   public ApexChainedApexEvent getTextValueEvent() {
      return textValue;
   }

   public void setTextValueEvent(ApexChainedApexEvent textValue) {
      this.textValue = textValue;
      if (textValue != null)
         textValue.setText(textValue.getText());
   }

   public ChainedApexEvent getFinalHoldEvent() {
      return finalHold;
   }

   public void setFinalHoldEvent(ChainedApexEvent finalHold) {
      this.finalHold = finalHold;
      if (finalHold != null)
         finalHold.setText(Constants.FINAL_HOLD);
   }

   public ChainedApexEvent getOffsetEvent() {
      return offset;
   }

   public void setOffsetEvent(ChainedApexEvent offset) {
      this.offset = offset;
      if (offset != null)
         offset.setText(Constants.OFFSET);
   }

   @Override
public String toString() {
      StringBuffer buffer = new StringBuffer("ChainedApexEventsCollection: ");
      buffer.append("\n ONSET: ");
      buffer.append(onset);
      buffer.append("\n INITIAL HOLD: ");
      buffer.append(initialHold);
      buffer.append("\n VALUE: ");
      buffer.append(value);
      buffer.append("\n TEXT VALUE: ");
      buffer.append(textValue);
      buffer.append("\n FINAL HOLD: ");
      buffer.append(finalHold);
      buffer.append("\n OFFSET: ");
      buffer.append(offset);

      return buffer.toString();
   }
}
