package edu.bu.signstream.grepresentation.fields.gestureField.dialog;

import java.util.ArrayList;

import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapeFormBean;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.domNdomField.GestureDomNdomHandFieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.handshapes.GestureHandShapeEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.handshapes.GestureHandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.DependentEntitiesManager;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntityTypes;

public class GestureHandShapePreprocessor {

	private FieldWrappersCollection fwc;
	private boolean rightHandDominant = true;

	public GestureHandShapePreprocessor(FieldWrappersCollection fwc) {
		this.fwc = fwc;
	}

	public void populateFormBean(HandShapeFormBean formBean) {
		char type = getSelectedHandShapeType();

		boolean rightHandFieldsUpdatable = isRightHandFieldUpdatable();
		boolean leftHandFieldsUpdatable = isLeftHandFieldUpdatable();
		boolean fingerspelledSelected = isFingerspelledSelected(type);
		boolean oneHandedSelected = isOneHandedSelected(type);

		boolean oneHandedRightSelected = isOneHandedRightSelected(leftHandFieldsUpdatable, rightHandFieldsUpdatable);

		SS3CodingScheme cs = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().getSS3CodingScheme();

		formBean.setSS3CodingScheme(cs);

		formBean.setType(type);

		formBean.setFingerspelledR(oneHandedRightSelected);
		formBean.setFingerspelledSelected(fingerspelledSelected);

		formBean.setOneHandedR(oneHandedRightSelected);
		formBean.setOneHandedSelected(oneHandedSelected);
	}

	private boolean isOneHandedRightSelected(boolean leftHandFieldsUpdatable,
			boolean rightHandFieldsUpdatable) {
		if (rightHandFieldsUpdatable) {
			if (rightHandDominant) {
				return true;
			}
			return !leftHandFieldsUpdatable;
		}
		return false;
	}

	private boolean isRightHandFieldUpdatable() {

		GestureEventsEntity glossEntity = (GestureEventsEntity) SS3Singleton
				.getMediaToolBar(null).selectedEntity();

		FieldWrapper rhWrapper = fwc
				.getFieldWrapper(Constants.TYPE_DOM_GESTURE_HAND);
		String rhID = rhWrapper.getField().getFieldID();

		GestureChainedEventsEntity depHandEntity = glossEntity
				.getDependentHandEntity();
		if (depHandEntity != null) {
			String id = depHandEntity.getField().getFieldID();
			if (rhID.equalsIgnoreCase(id)) {
				return true;
			}
		}

		GestureEventsEntity depGlossEntity = glossEntity
				.getDependentGestureEntity();
		if (depGlossEntity != null) {
			GestureChainedEventsEntity depHandEntity1 = depGlossEntity
					.getDependentHandEntity();
			if (depHandEntity1 != null) {
				String id1 = depHandEntity1.getField().getFieldID();
				if (rhID.equalsIgnoreCase(id1)) {
					return true;
				}
			}
		}

		GestureDomNdomHandFieldWrapper rlhWrp = (GestureDomNdomHandFieldWrapper) rhWrapper;

		return rlhWrp.getField().isIntervalEmpty(glossEntity.interval());
	}

	private boolean isLeftHandFieldUpdatable() {
		GestureEventsEntity glossEntity = (GestureEventsEntity) SS3Singleton
				.getMediaToolBar(null).selectedEntity();

		FieldWrapper lhWrapper = fwc
				.getFieldWrapper(Constants.TYPE_DOM_GESTURE_HAND);
		String lhID = lhWrapper.getField().getFieldID();

		GestureChainedEventsEntity depHandEntity = glossEntity
				.getDependentHandEntity();
		if (depHandEntity != null) {
			String id = depHandEntity.getField().getFieldID();
			if (lhID.equalsIgnoreCase(id)) {
				return true;
			}
		}

		GestureEventsEntity depGlossEntity = glossEntity
				.getDependentGestureEntity();
		if (depGlossEntity != null) {
			GestureChainedEventsEntity depHandEntity1 = depGlossEntity
					.getDependentHandEntity();
			if (depHandEntity1 != null) {
				String id1 = depHandEntity1.getField().getFieldID();
				if (lhID.equalsIgnoreCase(id1)) {
					return true;
				}
			}
		}

		GestureDomNdomHandFieldWrapper lhWrp = (GestureDomNdomHandFieldWrapper) lhWrapper;

		return lhWrp.getField().isIntervalEmpty(glossEntity.interval());
	}

	private boolean isFingerspelledSelected(char type) {
		return type == HandShapeEntityTypes.FINGERSPELLED;
	}

	private boolean isOneHandedSelected(char type) {
		return type == HandShapeEntityTypes.ONE_HANDED;
	}

	private char getSelectedHandShapeType() {

		GestureEventsEntity entity = (GestureEventsEntity) SS3Singleton
				.getMediaToolBar(null).selectedEntity();

		ArrayList<EventsEntity> dEntities = entity.getDependentEntities();

		DependentEntitiesManager depMngr = new DependentEntitiesManager(
				dEntities);

		GestureHandShapeFieldWrapper rHshWrp = (GestureHandShapeFieldWrapper) fwc.getFieldWrapper(
						Constants.TYPE_DOM_GESTURE_HANDSHAPE);

		if (rHshWrp != null) {
			EventsEntity rEnt = depMngr.getEventsEntityWithFieldID(rHshWrp
					.getField().getFieldID());

			if (rEnt != null) {
				GestureHandShapeEntity rHshEnt = (GestureHandShapeEntity) rEnt;
				return rHshEnt.getHandShapeType();
			}
		}

		GestureHandShapeFieldWrapper lHshWrp = (GestureHandShapeFieldWrapper) fwc.getFieldWrapper(
						Constants.TYPE_DOM_GESTURE_HANDSHAPE);
		if (lHshWrp != null) {
			EventsEntity lEnt = depMngr.getEventsEntityWithFieldID(lHshWrp
					.getField().getFieldID());

			if (lEnt != null) {
				GestureHandShapeEntity lHshEnt = (GestureHandShapeEntity) lEnt;
				return lHshEnt.getHandShapeType();
			}
		}
		return HandShapeEntityTypes.ONE_HANDED;
	}

	public void setRightHandDominant(boolean rightHandDominant) {
		this.rightHandDominant = rightHandDominant;
	}
}
