package edu.bu.signstream.grepresentation.fields.gestureField.dialog;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.*;
import java.util.Map.Entry;

import javax.swing.*;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.tree.*;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.gestureMenus.GestureCategory;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.gestureMenus.GestureMenu;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.gestureMenus.GestureMenuGroup;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.gestureMenus.GestureProperty;
import edu.bu.signstream.grepresentation.fields.gestureField.gesturePropertyField.*;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEventsEntity;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.GestureNRelatedView;
import edu.bu.signstream.grepresentation.view.LabelObject;

public class GesturePropertyPanel extends JPanel implements ItemListener, ActionListener {

	private GestureInfoPanel gestureInfoPanel = null;
	private GestureEventsEntity operativeGestureEntity = null;
	private GesturePropertyEntity gesturePropertyEntity = null;
	private final JButton expandAllButton;
	private final JButton collapseAllButton;
	private String currentGesturePropertyFieldID = null;

	private final JComboBox<String> gestureMenuSelector;
	private final JTree jTree;

	private String currentGestureMenuID;

	private final Set<TreePath> selectedNodes;

	public GesturePropertyPanel(GestureInfoPanel gestureInfoPanel) {
		this.gestureInfoPanel = gestureInfoPanel;
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.gestureMenuSelector = new JComboBox<>();
		this.gestureMenuSelector.setName("gestureMenuSelector");
		this.selectedNodes = new HashSet<>();

		DefaultMutableTreeNode top = new DefaultMutableTreeNode("Gesture Categories");

		this.currentGestureMenuID = this.initializeMenuSelector();

		this.expandAllButton = new JButton("Expand All");
		this.collapseAllButton = new JButton("Collapse All");
		this.expandAllButton.setName("expandAllButton");
		this.collapseAllButton.setName("collapseAllButton");
		this.expandAllButton.addActionListener(this);
		this.collapseAllButton.addActionListener(this);

		this.expandAllButton.setEnabled(true);
		this.collapseAllButton.setEnabled(true);

		JPanel panelControllerContainer = new JPanel();
		panelControllerContainer.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1.0;
		c.weighty = 0.0;

		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 2;
		panelControllerContainer.add(gestureMenuSelector, c);

		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 1;
		panelControllerContainer.add(expandAllButton, c);

		c.gridx = 1;
		c.gridy = 1;
		panelControllerContainer.add(collapseAllButton, c);

		this.add(panelControllerContainer);

		this.populateTree(top, this.currentGestureMenuID);

		this.jTree = new CheckBoxTree(top);

		this.setJTreeSelectionModel(this.jTree);

		CheckBoxNodeRenderer renderer = new CheckBoxNodeRenderer();
		this.jTree.setCellRenderer(renderer);

		this.jTree.addTreeWillExpandListener(new TreeWillExpandListener() {
			@Override
			public void treeWillExpand(TreeExpansionEvent event) throws ExpandVetoException {
				return;
			}

			@Override
			public void treeWillCollapse(TreeExpansionEvent event) throws ExpandVetoException {
				TreePath path = event.getPath();
				DefaultMutableTreeNode node = (DefaultMutableTreeNode) path.getLastPathComponent();
				GesturePropertyPanel.this.collapseChildren(node);
			}
		});

		this.jTree.addTreeExpansionListener(new TreeExpansionListener() {
			@Override
			public void treeExpanded(TreeExpansionEvent event) {
				TreePath path = event.getPath();
				// Determine if any of the newly visible nodes are leaf nodes.
				// If they are, figure out which ones are selected.

				Enumeration<TreeNode> childNodes = ((DefaultMutableTreeNode) path.getLastPathComponent()).children();

				while (childNodes.hasMoreElements()) {
					DefaultMutableTreeNode node = (DefaultMutableTreeNode) childNodes.nextElement();

					if (GesturePropertyPanel.this.selectedNodes.contains(new TreePath(node.getPath())) && !GesturePropertyPanel.this.jTree.isPathSelected(new TreePath(node.getPath()))) {
						// If the node is a leaf node and it is selected, deselect it.
						// Select the node
						GesturePropertyPanel.this.jTree.addSelectionPath(new TreePath(node.getPath()));
					}
				}

			}

			@Override
			public void treeCollapsed(TreeExpansionEvent event) {
				TreePath path = event.getPath();
			}
		});

		this.expandNodes(this.jTree, new TreePath(top));

		JScrollPane treeView = new JScrollPane(this.jTree);
		treeView.setPreferredSize(new Dimension(500, 450));
		this.add(treeView);

		this.update();
	}

	private void collapseChildren(DefaultMutableTreeNode node) {
		for (int i = 0; i < node.getChildCount(); i++) {
			DefaultMutableTreeNode child = (DefaultMutableTreeNode) node.getChildAt(i);
			this.collapseChildren(child);
			this.jTree.collapsePath(new TreePath(child.getPath()));
		}
	}


	private void setJTreeSelectionModel(JTree jTree) {
		jTree.setSelectionModel(new DefaultTreeSelectionModel() {
			@Override
			public void addSelectionPath(TreePath path) {
				DefaultMutableTreeNode node = (DefaultMutableTreeNode) path.getLastPathComponent();
				//prevent non-leaf (non checkbox) nodes from being selected
				if (node.isLeaf()) {
					super.addSelectionPath(path);
					//
				}
			}

			@Override
			public void removeSelectionPath(TreePath path) {
				super.removeSelectionPath(path);
			}

			@Override
			public void addSelectionPaths(TreePath[] paths) {
				if (paths != null) {
					for (TreePath path : paths) {
						TreePath[] toAdd = new TreePath[1];
						toAdd[0] = path;

						if (this.isPathSelected(path) && GesturePropertyPanel.this.selectedNodes.contains(path)) {
							this.removeSelectionPath(toAdd[0]);
							GesturePropertyPanel.this.selectedNodes.remove(path);
						} else {
							super.addSelectionPaths(toAdd);
							GesturePropertyPanel.this.selectedNodes.add(path);
						}
					}
				}
			}
		});
	}

	private String initializeMenuSelector() {
		//get categories and properties from coding schema
		SS3CodingScheme dbScheme = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel()
				.getSS3Database().getSS3CodingScheme();

		HashMap<String, GestureMenuGroup> gestureMenuGroups = dbScheme.getGestureMenuGroupHashMap();
		HashMap<String, GestureMenu> gestureMenus = dbScheme.getGestureMenuHashMap();

		GestureMenuGroup defaultGestureMenuGroup = gestureMenuGroups.get(Util.DEFAULT_MENU_GROUP);

		String defaultMenuID = defaultGestureMenuGroup.getDefaultGestureMenuReference();

		HashMap<String, String> gestureMenuReferences = defaultGestureMenuGroup.getGestureMenuReferences();
		ArrayList<String> menuOrderByIDs = new ArrayList<>();

		for (Entry<String, String> gestureMenuReference : gestureMenuReferences.entrySet()) {
			String gestureMenuID = gestureMenuReference.getKey();
			String gestureMenuOrder = gestureMenuReference.getValue();

			this.addInOrder(menuOrderByIDs, gestureMenuOrder, gestureMenuID);
		}

		for (String menuID : menuOrderByIDs) {
			GestureMenu gestureMenu = gestureMenus.get(menuID);
			this.gestureMenuSelector.addItem(gestureMenu.getName());
		}

		// Set the gestureMenuSelector to the default menu
		this.gestureMenuSelector.setSelectedItem(gestureMenus.get(defaultMenuID).getName());

		this.gestureMenuSelector.addItemListener(new ItemListener() {

			/**
			 * Invoked when an item has been selected or deselected by the user.
			 * The code written for this method performs the operations
			 * that need to occur when an item is selected (or deselected).
			 *
			 * @param e the event to be processed
			 */
			@Override
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					String selectedGestureMenu = (String) e.getItem();
					String gestureMenuID = null;

					for (Entry<String, GestureMenu> gestureMenuEntry : gestureMenus.entrySet()) {
						if (gestureMenuEntry.getValue().getName().equals(selectedGestureMenu)) {
							gestureMenuID = gestureMenuEntry.getKey();
							GesturePropertyPanel.this.currentGestureMenuID = gestureMenuID;
							GesturePropertyPanel.this.selectedNodes.clear();
							break;
						}
					}

					DefaultMutableTreeNode top = new DefaultMutableTreeNode("Gesture Categories");
					GesturePropertyPanel.this.populateTree(top, gestureMenuID);

					if (!top.isLeaf()) {
						// Update treeView with new tree
						GesturePropertyPanel.this.jTree.setModel(new DefaultTreeModel(top));
					} else {
						GesturePropertyPanel.this.jTree.setModel(null);
					}
				}
			}
		});

		return defaultMenuID;
	}

	private void addInOrder(ArrayList<String> list, String comparator, String value) {
		for (int i = 0; i < list.size(); i++) {
			if (Integer.parseInt(comparator) < Integer.parseInt(list.get(i))) {
				list.add(i, value);
				return;
			}
		}

		list.add(value);
	}

	//sets all nodes in the tree to an expanded state
	private void expandNodes(JTree tree, TreePath parent) {
		TreeNode node = (TreeNode) parent.getLastPathComponent();
		if (node.getChildCount() >= 0) {
			for (Enumeration e = node.children(); e.hasMoreElements();) {
				TreeNode n = (TreeNode)e.nextElement();
				TreePath path = parent.pathByAddingChild(n);
				expandNodes(tree, path);
			}
			tree.expandPath(parent);
		}
	}

	// populating the DefaultMutableTreeNode passed into the function, NOT the jtree
	private void populateTree(DefaultMutableTreeNode top, String menuID) {
		//get categories and properties from coding schema
		SS3CodingScheme dbScheme = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel()
				.getSS3Database().getSS3CodingScheme();

		HashMap<String, GestureCategory> gestureCategories = dbScheme.getGestureMenuHashMap().get(menuID).getGesturePropertyCategoryObject().getGestureCategories();

		//temporary lists to help construct the tree; one for categories, one for properties
		ArrayList<DefaultMutableTreeNode> categoryNodes = new ArrayList<>();

		//make nodes for all categories
		for (GestureCategory gc : gestureCategories.values()) {
			DefaultMutableTreeNode categoryNode = new DefaultMutableTreeNode(gc, true);
			categoryNodes.add(categoryNode);
		}

		categoryNodes.sort((o1, o2) -> {
			GestureCategory gc1 = (GestureCategory) o1.getUserObject();
			GestureCategory gc2 = (GestureCategory) o2.getUserObject();
			return Integer.compare(Integer.parseInt(gc1.getOrder()), Integer.parseInt(gc2.getOrder()));
		});

		//form tree links between categories
		for (DefaultMutableTreeNode categoryNode : categoryNodes) {
			GestureCategory gestureCategory = (GestureCategory) categoryNode.getUserObject();

			String parentID = gestureCategory.getParentCategoryId();

			if (parentID == null || parentID.isEmpty()) {
				top.add(categoryNode);
			} else {
				categoryNodes.stream()
						.filter(node -> ((GestureCategory) node.getUserObject()).getId().equals(parentID))
						.findFirst().ifPresent(parentNode -> parentNode.add(categoryNode));

			}

			//add properties to categories
			ArrayList<GestureProperty> gestureProperties = new ArrayList<>(gestureCategory.getGestureProperties().values());
			gestureProperties.sort(Comparator.comparingInt(o -> Integer.parseInt(o.getOrder())));

			for (GestureProperty gp : gestureProperties) {
				DefaultMutableTreeNode propertyNode = new DefaultMutableTreeNode(gp, false);
				categoryNode.add(propertyNode);
			}
		}
	}

	private void populateSelectedPaths(TreePath[] selectedPaths) {
		this.selectedNodes.addAll(Arrays.asList(selectedPaths));
	}

	public void clearSelectedPaths() {
		this.selectedNodes.clear();
	}

	public void setSelectedGestureProperties() {
		TreePath[] selectedPaths = new TreePath[this.selectedNodes.size()];
		selectedPaths = this.selectedNodes.toArray(selectedPaths);

		GestureNRelatedView view = gestureInfoPanel.getGestureNRelatedView();

		ArrayList<GestureProperty> selectedProperties = new ArrayList<>();
		HashMap<GestureCategory, ArrayList<GestureProperty>> categoryPropertyMap
				= new HashMap<GestureCategory, ArrayList<GestureProperty>>();

		for (TreePath selectedPath : selectedPaths) {
			//System.out.println(selectedPaths[i]);
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) selectedPath.getLastPathComponent();
			GestureProperty property = (GestureProperty) node.getUserObject();

			selectedProperties.add(property);

			// make another map: Category -> list of properties, then one list of properties per sub-gestureProperty-entity
			GestureCategory cat = property.getParentCategory();
			categoryPropertyMap.putIfAbsent(cat, new ArrayList<GestureProperty>());        // java8 and above
			categoryPropertyMap.get(cat).add(property);
		}

		GestureEventsEntity depEntity = operativeGestureEntity.getDependentGestureEntity();

		// delete old entities indiscriminately, before recreating below
		if (gesturePropertyEntity != null) {

			gesturePropertyEntity.setSelectedProperty(null);
			gesturePropertyEntity.getField().deleteEntity(gesturePropertyEntity);        // the field method overrides, to delete dependents also

			operativeGestureEntity.setDependentPropertyEntity(null);
			if (depEntity != null)
				depEntity.setDependentPropertyEntity(null);

			gesturePropertyEntity = null;
		}

		// resets operativeGestureEntity, gesturePropertyEntity, currentGesturePropertyFieldID
		this.updateGesturePropertyEntity();

		Time startTimeInfo = operativeGestureEntity.getValueOrTextValueEvent().getStartTimeInfo();
		Time endTimeInfo = operativeGestureEntity.getValueOrTextValueEvent().getEndTimeInfo();

		if (!selectedProperties.isEmpty()) {
			// (re)create gesture property entity on the current field/wrapper

			LabelObject gestObj = view.getSegment().getRootLabelObject().getGestureLabelObject();

			FieldWrapper topLevelWrapper = gestObj.getChildWithFieldID(currentGesturePropertyFieldID).getWrapper();

			GesturePropertyEvent topLevelEvent = new GesturePropertyEvent(topLevelWrapper.getSignStreamSegmentPanel(),startTimeInfo, endTimeInfo);

			gesturePropertyEntity = new GesturePropertyEntity(topLevelEvent, topLevelWrapper.getSignStreamSegmentPanel(), topLevelWrapper.getField());
			gesturePropertyEntity.setID(Util.getLongUniqueNumber());
			topLevelWrapper.addEventsEntity(gesturePropertyEntity);

			gesturePropertyEntity.updateParentEntity();
			gesturePropertyEntity.setRefGestureEntity(operativeGestureEntity);

			operativeGestureEntity.setDependentPropertyEntity(gesturePropertyEntity);

			SS3CodingScheme dbScheme = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel()
					.getSS3Database().getSS3CodingScheme();

			// reset the values to what is currently selected
			for (Entry<GestureCategory, ArrayList<GestureProperty>> entry : categoryPropertyMap.entrySet()) {
				ArrayList<GestureProperty> gestureProperties = entry.getValue();

				FieldWrapper wrapper = gestObj.getChildWithFieldID(currentGesturePropertyFieldID).getWrapper();

				ArrayList<LabelObject> labelObjectsWithWrappers = gestObj.getChildWithFieldID(currentGesturePropertyFieldID).getAllChildren();

				for (LabelObject obj : labelObjectsWithWrappers) {
					HashMap<String, SS3FieldValue> gesturePropertyFieldValues = dbScheme.getFieldValuesAsMap(obj.getFieldID());
					for (String valueID: gesturePropertyFieldValues.keySet()) {
						for (GestureProperty gestureProperty: gestureProperties) {
							String gesturePropertyID = gestureProperty.getId();

							if (valueID.equals(gesturePropertyID)) {
								wrapper = obj.getWrapper();
								break;
							}
						}
					}
				}

				GesturePropertyFieldWrapper gesturePropertyFieldWrapper = (GesturePropertyFieldWrapper) wrapper;

				GesturePropertyEvent event = new GesturePropertyEvent(
						gesturePropertyFieldWrapper.getSignStreamSegmentPanel(),
						startTimeInfo,
						endTimeInfo
				);
				GesturePropertyEntity ent = new GesturePropertyEntity(event, gesturePropertyFieldWrapper.getSignStreamSegmentPanel(), gesturePropertyFieldWrapper.getField());
				ent.updateParentEntity();
				ent.setSelectedProperty(gestureProperties);

				ent.setID(Util.getLongUniqueNumber());
				gesturePropertyFieldWrapper.addEventsEntity(ent);

				ent.setRefEntityId(gesturePropertyEntity.getID());
				gesturePropertyEntity.addDependentGesturePropertyEntity(ent);
			}
		}
	}

	public void updateGesturePropertyEntity() {
		GestureEventsEntity selectedEntity = (GestureEventsEntity) SS3Singleton.getMediaToolBar(null).selectedEntity();
		GestureEventsEntity depEntity = selectedEntity.getDependentGestureEntity();

		String selectedEntityGestureFieldID = selectedEntity.getField().getFieldID();
		boolean isNondom = selectedEntityGestureFieldID.equals(Util.NDOM_GESTURE_FIELD_ID);
		boolean isTwoHanded = gestureInfoPanel.isTwoHandedSelected();


		// Current parent field id of the selected property
		this.currentGesturePropertyFieldID = Util.DOM_GESTURE_PROPERTY_FIELD_ID;

		// GREG TODO: change the conditions here
		if (isNondom) {
			if (!isTwoHanded || depEntity == null) {
				this.operativeGestureEntity = selectedEntity;

				this.currentGesturePropertyFieldID = Util.NDOM_GESTURE_PROPERTY_FIELD_ID;
			} else {
				this.operativeGestureEntity = depEntity;
			}
		} else {
			this.operativeGestureEntity = selectedEntity;
		}

		this.gesturePropertyEntity = operativeGestureEntity.getDependentGesturePropertyEntity();
	}

	public void update() {
		updateGesturePropertyEntity();

		DefaultMutableTreeNode root = (DefaultMutableTreeNode) this.jTree.getModel().getRoot();
		deselectAll(root);

		if (this.gesturePropertyEntity != null) {
			setSelections(root);
		}
	}

	//traverses through the tree and sets all selections to null
	private void deselectAll(DefaultMutableTreeNode node) {
		for (int i = 0; i < node.getChildCount(); i++) {
			DefaultMutableTreeNode childNode = (DefaultMutableTreeNode) node.getChildAt(i);
			if (childNode.getChildCount() > 0) {
				deselectAll(childNode);
			} else {
				TreePath path = new TreePath(childNode.getPath());
				this.jTree.removeSelectionPath(path);
			}
		}
	}

	//traverses through the tree and sets selections
	private void setSelections(DefaultMutableTreeNode node) {
		for (int i = 0; i < node.getChildCount(); i++) {
			DefaultMutableTreeNode childNode = (DefaultMutableTreeNode) node.getChildAt(i);
			if (childNode.getChildCount() > 0) {
				setSelections(childNode);
			} else {
				if (childNode.getUserObject() instanceof GestureCategory) {
					continue;
				}
				GestureProperty currProperty = (GestureProperty)childNode.getUserObject();
				if (this.gesturePropertyEntity.containsGestureProperty(currProperty)) {
					TreePath path = new TreePath(childNode.getPath());
					this.selectedNodes.add(path);
					this.jTree.addSelectionPath(path);
				}
			}
		}
	}

	public String getCurrentGestureMenuID() {
		return this.currentGesturePropertyFieldID;
	}

	// Tracking the selected nodes on collapse event
	private void collapseTree() {
		for (int i = 1; i < this.jTree.getRowCount(); i++) {
			this.jTree.collapseRow(i);
		}
	}

	// Restoring the selected nodes on expand event
	private void expandTree() {
		for (int i = 0; i < this.jTree.getRowCount(); i++) {
			this.jTree.expandRow(i);
		}
	}

	// Modifying the actionPerformed method to call the new methods
	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == this.expandAllButton) {
			this.expandTree();
		} else if (e.getSource() == this.collapseAllButton) {
			this.collapseTree();
		}
	}


	@Override
	public void itemStateChanged(ItemEvent e) {

	}
}
