package edu.bu.signstream.grepresentation.fields.gestureField.dialog;


import edu.bu.signstream.grepresentation.fields.gestureField.dialog.associatedFields.locations.*;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.ui.ButtonPanel;

import javax.swing.*;

import java.awt.*;
import java.util.*;

public class LocationPanel extends JPanel {

    private ArrayList<LocationDisplayPanel> locDispPanels = new ArrayList<LocationDisplayPanel>();
    private HashMap<LocationDisplayPanel, LocationDisplayPanel> handSiblings = new HashMap<>();
    private JButton copyAllToOtherHand;
    private JButton clearAll;

    public LocationPanel(GestureInfoPanel gip) {
        // for each location type in the coding schema, a new LocationDisplayPanel is added to the list
        // in the order that is specified in the coding schema. To add a new location type, edit the coding schema itself.
        final ArrayList<GestureLocationType> locationTypes = SS3Singleton.getGestureLocationConfig().getLocationTypes();
        for (GestureLocationType locationType : locationTypes) {
        	locDispPanels.add(new LocationDisplayPanel(locationType, gip));
        }

        // all LocationDisplayPanels are siblings of each other. Go through the list of panels
        // and set each panel's list of siblings to the list of all panels that doesn't include itself.
        for (LocationDisplayPanel locDispPanel : locDispPanels) {
            ArrayList<LocationDisplayPanel> siblings = new ArrayList<LocationDisplayPanel>();
            for (LocationDisplayPanel locationDisplayPanel : locDispPanels) {
                if (!locationDisplayPanel.equals(locDispPanel)) {
                    siblings.add(locationDisplayPanel);
                }
            }
            locDispPanel.setSiblingPanels(siblings);
            locDispPanel.init();
        }

        initUI();
    }


    private void initUI() {
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        JPanel regionPanel = new JPanel();
        regionPanel.setLayout(new FlowLayout());
        for (LocationDisplayPanel locDispPanel : locDispPanels) {
            regionPanel.add(locDispPanel);
        }
        add(regionPanel);
    }

    protected ArrayList<LocationDisplayPanel> getLocDispPanels() {
        return locDispPanels;
    }

    public void createDepHandAssociations(ArrayList<LocationDisplayPanel> depLocDispPanels, String type) {
        createHandSiblings(depLocDispPanels);
        createCopyButtonsToOtherHand(type);
        generateButtonPanelForHand(type);
    }

    private void createHandSiblings(ArrayList<LocationDisplayPanel> depLocDispPanels) {
        for (int i = 0; i < locDispPanels.size(); i++) {
            handSiblings.put(locDispPanels.get(i), depLocDispPanels.get(i));
        }
    }

    private void createCopyButtonsToOtherHand(String type) {
        for (LocationDisplayPanel ldp : locDispPanels) {
            ldp.createCopyButtonToOtherHand(handSiblings.get(ldp), type);
        }
    }

    private void generateButtonPanelForHand(String type) {
        ButtonPanel buttonPanel = new ButtonPanel();
        copyAllToOtherHand = new JButton("Copy All To " + type + " Hand");
        copyAllToOtherHand.addActionListener(copy -> {
            for (LocationDisplayPanel ldp : handSiblings.keySet())
                handSiblings.get(ldp).copyFrom(ldp.getLocationObject());
        });
        buttonPanel.add(copyAllToOtherHand);
        clearAll = new JButton("Clear All");
        clearAll.addActionListener(clear -> resetHand());
        buttonPanel.add(clearAll);
        add(buttonPanel);
    }

    public void resetHand() {
        for (LocationDisplayPanel locDispPanel : locDispPanels) {
            locDispPanel.resetPanel();
        }
    }

    public void resetDialogs() {
        for (LocationDisplayPanel locDispPanel : locDispPanels) {
            locDispPanel.resetDialog();
        }
    }

    public void clearRegions() {
        for (LocationDisplayPanel locDispPanel : locDispPanels) {
            locDispPanel.clearCategorySelection();
            locDispPanel.setRegionToNull();
        }
    }

    public GestureLocationInfo getCurrentLocationInfo() {
        HashMap<GestureLocationType, GestureLocationObject> locations = new HashMap<GestureLocationType, GestureLocationObject>();
        for (LocationDisplayPanel locDispPanel : locDispPanels) {
            GestureLocationType locationType = locDispPanel.getLocationType();
            GestureRegion region = locDispPanel.getRegion();
            Boolean isConstralateral = locDispPanel.getIsContralateral();
            locations.put(locationType, new GestureLocationObject(region, isConstralateral));
        }
        return new GestureLocationInfo(locations);
    }

    public void setCurrentLocationInfo(GestureLocationInfo currentLocationInfo) {
        ArrayList<GestureLocationType> locationTypes = SS3Singleton.getGestureLocationConfig().getLocationTypes();
        for (int i = 0; i < locDispPanels.size(); i++) {
            GestureLocationType locationType = locationTypes.get(i);
            GestureLocationObject locObj = currentLocationInfo.getLocationObj(locationType);
            locDispPanels.get(i).setLocationObject(locObj);
            locDispPanels.get(i).setRegion(locDispPanels.get(i).getLocationObject().getRegion());
            if (locObj.getRegion().equals(GestureRegion.NONE)) {
                locDispPanels.get(i).clearCategorySelection();
            }
            else {
                locDispPanels.get(i).setIsContralateral(locObj.isContralateral());
            }
        }
    }

    public void disableButtons() {
        for (LocationDisplayPanel locDispPanel : locDispPanels) {
            locDispPanel.enableOrDisableButtons(false);
        }
        enableOrDisableButtonsOnButtonPanel(false);
    }

    public void enableButtons() {
        for (LocationDisplayPanel locDispPanel : locDispPanels) {
            locDispPanel.enableOrDisableButtons(true);
        }
        enableOrDisableButtonsOnButtonPanel(true);
    }

    public void enableOrDisableOtherLocationCopyButton(boolean flag) {
        enableOrDisableCopyToDepHand(flag);
        enableOrDisableButtonsOnButtonPanel(flag);
    }

    private void enableOrDisableButtonsOnButtonPanel(boolean flag) {
        copyAllToOtherHand.setEnabled(flag);
        clearAll.setEnabled(flag);
    }

    private void enableOrDisableCopyToDepHand(boolean flag) {
        for (LocationDisplayPanel locDispPanel : locDispPanels) {
            locDispPanel.enableOrDisableCopyToDepHand(flag);
        }
    }
}
