package edu.bu.signstream.grepresentation.fields.gestureField.dialog.associatedFields.locations;

import java.awt.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;

public class GestureRegion {

    public static final GestureRegion NONE = new GestureRegion("-1", "None Selected", "None");

    private String id;
    private String name;
    private String abbreviation;
    private boolean selectable = true;

    private String imageID;
    private GestureRegion parent;
    private HashMap<GestureRegion, ArrayList<Polygon>> polygonMap = new HashMap<GestureRegion, ArrayList<Polygon>>();
    private HashMap<GestureRegion, Integer> subregionOrders = new HashMap<GestureRegion, Integer>();
    private ArrayList<GestureRegion> subregions = new ArrayList<GestureRegion>();

    public GestureRegion(String id, String name, String abbreviation) {
        this.id = id;
        this.name = name;
        this.abbreviation = abbreviation;
    }

    public String getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public String getAbbreviation() {
        return abbreviation;
    }

    public boolean isSelectable() {
        return selectable;
    }

    public void setSelectable(boolean selectable) {
        this.selectable = selectable;
    }

    public boolean isParent() {
        return !subregions.isEmpty();
    }

    public String getImageID() {
        return imageID;
    }

    public void setImageID(String imageID) {
        this.imageID = imageID;
    }

    public ArrayList<Polygon> getPolygons(GestureRegion subregion) {
        return polygonMap.get(subregion);
    }

    public GestureRegion getParent() {
        return parent;
    }

    public void setParent(GestureRegion parent) {
        this.parent = parent;
    }

    public ArrayList<GestureRegion> getSubregions() {
        return new ArrayList<GestureRegion>(subregions);
    }

    public void addSubregion(final GestureRegion subregion, final int order, ArrayList<Polygon> polygons) {
        subregions.add(subregion);
        subregionOrders.put(subregion, order);
        Collections.sort(subregions, new Comparator<GestureRegion>() {
            @Override
            public int compare(GestureRegion o1, GestureRegion o2) {
                return subregionOrders.get(o1) - subregionOrders.get(o2);
            }
        });
        polygonMap.put(subregion, polygons);
    }

    @Override
    public String toString() {
        return abbreviation + " (" + name + ")";
    }

    @Override
    public boolean equals(Object obj) {
        return (obj instanceof GestureRegion) && ((GestureRegion)obj).id.equals(id);
    }

    @Override
    public int hashCode() {
        return Integer.parseInt(id);
    }
}
