package edu.bu.signstream.grepresentation.fields.gestureField.dialog.associatedFields.locations;

import edu.bu.signstream.grepresentation.fields.gestureField.dialog.GestureInfoPanel;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.LocationPanel;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.util.ArrayList;

/**
 * This class is responsible for showing a small thumbnail of a selected region
 * on the {@link LocationPanel} along with buttons to copy this panel's region info
 * to its siblings and a button that opens a dialog to select the region for this panel.
 */
public class LocationDisplayPanel extends JPanel {

    private RegionSelectDialog dialog;
    private RegionDisplayPanel regionDisplayPanel;
    private JButton selectRegionButton = new JButton("Select Region");
    private JButton clearButton = new JButton("Clear");

    ButtonGroup categoryGroup;

    private JCheckBox ipsilateralCheckBox = new JCheckBox("Ipsilateral");
    private JCheckBox contralateralCheckBox = new JCheckBox("Contralateral");

    private ArrayList<JButton> copyButtons = new ArrayList<JButton>();
    private JButton copyToOtherHand;
    private GridBagConstraints constraints = new GridBagConstraints();

    private ArrayList<LocationDisplayPanel> siblingPanels;
    private GestureLocationType locationType;
    public LocationDisplayPanel(GestureLocationType locationType, GestureInfoPanel afp) {
        this.locationType = locationType;
        regionDisplayPanel = new RegionDisplayPanel(afp);
        setRegion(null);
    }

    public GestureLocationObject getLocationObject() {
        Boolean categoryBtn = null;
        if (isRadioBtnSelected())
            categoryBtn = contralateralCheckBox.isSelected();
        return new GestureLocationObject(regionDisplayPanel.getRegion(), categoryBtn);
    }

    public void setLocationObject(GestureLocationObject locationObject) {
        if (locationObject.isEmpty()) {
            clearCategorySelection();
        }
        else {
            contralateralCheckBox.setSelected(contralateralCheckBox.isSelected());
            ipsilateralCheckBox.setSelected(ipsilateralCheckBox.isSelected());
        }
        setRegion(locationObject.getRegion());
    }

    public void init() {
        setLayout(new GridBagLayout());

        constraints.gridx = constraints.gridy = 0;

        setBorder(new TitledBorder(locationType.getName() + " Location"));
        selectRegionButton.addActionListener(selection -> {
            if (dialog == null) {
                dialog = new RegionSelectDialog(LocationDisplayPanel.this, locationType.getName());
            } else if (regionDisplayPanel.getRegion().equals(GestureRegion.NONE)) {
                dialog.reset();
            }
            dialog.setVisible(true);
        });

        Box sizeBox = Box.createVerticalBox();
        categoryGroup = new ButtonGroup() {
            @Override
            public void setSelected(ButtonModel model, boolean selected) {
                if (selected) {
                    super.setSelected(model, selected);
                } else {
                    clearSelection();
                }
            }
        };
        categoryGroup.add(ipsilateralCheckBox);
        categoryGroup.add(contralateralCheckBox);
        sizeBox.add(ipsilateralCheckBox);
        sizeBox.add(contralateralCheckBox);
        sizeBox.setBorder(BorderFactory.createTitledBorder("Category"));
        add(sizeBox, constraints);
        constraints.gridy++;

        add(selectRegionButton, constraints);
        constraints.gridy++;

        add(regionDisplayPanel, constraints);
        constraints.gridy++;
        for (final LocationDisplayPanel siblingPanel : siblingPanels) {
            JButton button = new JButton("Copy to " + siblingPanel.getLocationType().getName());
            button.addActionListener(copy -> siblingPanel.copyFrom(this.getLocationObject()));
            copyButtons.add(button);
            constraints.gridy++;
            add(button, constraints);
        }

        clearButton.addActionListener(clear -> {
            clearCategorySelection();
            setRegion(null);
        });

        constraints.gridy++;
        add(clearButton, constraints);
        constraints.gridy++;
    }

    public void setSiblingPanels(ArrayList<LocationDisplayPanel> siblings) {
        this.siblingPanels = new ArrayList<LocationDisplayPanel>(siblings);
    }

    public void createCopyButtonToOtherHand(LocationDisplayPanel depLocDisplayPanel, String type) {
        copyToOtherHand = new JButton("Copy To " + type);
        copyToOtherHand.addActionListener(copy -> depLocDisplayPanel.copyFrom(this.getLocationObject()));
        add(copyToOtherHand, constraints);
    }

    public GestureLocationType getLocationType() {
        return locationType;
    }

    public void copyFrom(GestureLocationObject locObj) {
        setRegion(locObj.getRegion());

        //Copies Radio Button Category Selection
//        clearCategorySelection();
//        if (!locObj.isEmpty()) {
//            if (locObj.isContralateral())
//                contralateralRadioBtn.setSelected(true);
//            else
//                ipsilateralRadioBtn.setSelected(true);
//        }
    }

    public GestureRegion getRegion() {
        return regionDisplayPanel.getRegion();
    }

    void selectedRegion(GestureRegion result) {
        GestureRegion r = result.getParent();
        if (r != null) {
            setRegion(result);
        }
    }

    public void setRegion(GestureRegion sr) {
        regionDisplayPanel.setRegion(sr);
    }
    
    public Boolean getIsContralateral() {
    	if (ipsilateralCheckBox.isSelected() && ! contralateralCheckBox.isSelected()) {
    		return false;
    	} else if ( ! ipsilateralCheckBox.isSelected() && contralateralCheckBox.isSelected()) {
    		return true;
    	} else {
    		return null;
    	}
    }

    public void setIsContralateral(Boolean isContralateral) {
        if (isContralateral == null) {
            clearCategorySelection();
        }
        else {
            if ((isContralateral)) {
                contralateralCheckBox.setSelected(true);
            } else {
                ipsilateralCheckBox.setSelected(true);
            }
        }
    }

    public Boolean isRadioBtnSelected() {
        return this.contralateralCheckBox.isSelected() || this.ipsilateralCheckBox.isSelected();
    }

    public void resetPanel() {
        clearCategorySelection();
        setRegionToNull();
    }

    public void resetDialog() {
        if (dialog != null) {
            dialog.reset();
        }
    }

    public void clearCategorySelection() {
        categoryGroup.clearSelection();
    }
    public void setRegionToNull(){
        setRegion(null);
    }

    public void enableOrDisableButtons(boolean flag) {
        ipsilateralCheckBox.setEnabled(flag);
        contralateralCheckBox.setEnabled(flag);
        selectRegionButton.setEnabled(flag);
        clearButton.setEnabled(flag);
        enableCopyFns(flag);
        enableOrDisableCopyToDepHand(flag);
        copyToOtherHand.setEnabled(flag);
    }

    public void enableOrDisableCopyToDepHand(boolean b) {
        copyToOtherHand.setEnabled(b);
    }

    private void enableCopyFns(boolean b) {
        for (JButton copyButton : copyButtons) {
            copyButton.setEnabled(b);
        }
    }
}