package edu.bu.signstream.grepresentation.fields.gestureField.gesturePropertyField;

import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.gestureMenus.GestureCategory;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.gestureMenus.GestureMenu;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.gestureMenus.GestureProperty;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEventsEntity;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

import java.awt.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;

public class GesturePropertyEntity extends EventsEntity {

    private GesturePropertyEvent event;
    private HashSet<GestureProperty> selectedProperties;
    private ArrayList<GesturePropertyEntity> dependentGesturePropertyEntities;

    private GestureEventsEntity refDomGestureEntity;

   // use constructor then call setSelectedProperties()
    public GesturePropertyEntity(GesturePropertyEvent event, SignStreamSegmentPanel segmentPanel, Field field) {
        super(segmentPanel, field);
        this.event = event;
        this.selectedProperties = new HashSet<>();
        this.dependentGesturePropertyEntities = new ArrayList<>();
    }

    public void setRefGestureEntity(GestureEventsEntity refDomGestureEntity) {
        this.refDomGestureEntity = refDomGestureEntity;
        this.setRefEntityId(refDomGestureEntity.getID());
    }

    public void setSelectedProperty(ArrayList<GestureProperty> selectedProperties) {
        if (selectedProperties == null)
            this.selectedProperties = new HashSet<>();
        else
            this.selectedProperties = new HashSet<>(selectedProperties);

        getSS3Entity();        // this will generate all the SS3Item's needed to represent each GestureProperty
    }
    
    public HashSet<GestureProperty> getSelectedProperties() {
        return this.selectedProperties;
    }
    public boolean containsGestureProperty(GestureProperty gestureProperty) {
        // GD 2023-04-05 the first for-loop was the original code here, when all gesture properties were stored in the top-level gesture entity
        // after not storing them in the top-level at all anymore, we must check in the second for-loop for the dependent values
        for (GestureProperty selectedGestureProperty : this.selectedProperties) {
            if (selectedGestureProperty.equals(gestureProperty)) {
                return true;
            }
        }
        for (GesturePropertyEntity depGesturePropertyEntity: dependentGesturePropertyEntities) {
            HashSet<GestureProperty> selectedProperties = depGesturePropertyEntity.getSelectedProperties();
            for (GestureProperty selectedGestureProperty: selectedProperties) {
                if (selectedGestureProperty.equals(gestureProperty)) {
                    return true;
                }
            }
        }
        return false;
    }
    public void addDependentGesturePropertyEntity(GesturePropertyEntity ent) {
        this.dependentGesturePropertyEntities.add(ent);
    }
    public void setDependentGesturePropertyEntities(ArrayList<GesturePropertyEntity> entities) {
        this.dependentGesturePropertyEntities = new ArrayList<GesturePropertyEntity>(entities);
    }
    public ArrayList<GesturePropertyEntity> getDependentGesturePropertyEntities() {
        return this.dependentGesturePropertyEntities;
    }
    @Override
    public SS3Entity getSS3Entity() {
        SS3Entity ss3entity = new SS3Entity();

        for (GestureProperty property : this.selectedProperties) {
            SS3Item propertyItem = new SS3Item(property.getId(), getStartTimeInfo(), getEndTimeInfo());
            ss3entity.addItem(propertyItem);
        }
        
        // GD 2023-04-05: the dummy is for preserving a single ITEM element under an ENTITY element in the collection xml
        if (selectedProperties.isEmpty()) {
        	SS3Item dummyItem = new SS3Item("", getStartTimeInfo());
        	dummyItem.setEndTimeInfo(getEndTimeInfo());
        	ss3entity.addItem(dummyItem);
        }
        
        ss3entity.setID(getID());
        if (getParentEntities() != null)
            ss3entity.setParentEntities(getParentEntities());
        ss3entity.setRefEntityId(getRefEntityId());
        return ss3entity;
    }
    
    public void setSS3Entity(SS3Entity ss3Entity) {
        for (SS3Item item: ss3Entity.getItemsAsList()) {
            String gesturePropertyID = item.getId();
            Collection<GestureMenu> gestureMenus = this.segmentPanel.getSS3Database().getSS3CodingScheme()
                                                            .getGestureMenuHashMap().values();

            // Potential solutions to this:
            // 1. Store a flag to not iterate through the next menu
            // 2. Modify the XML to store the menu
            // 3. Do not allow duplicates
            // 4. Break once we add (same as 1)
            // 5. Store the current menu in the object
            // 6. Something else...

            SS3CodingScheme scheme = this.segmentPanel.getSS3Database().getSS3CodingScheme();
            HashMap<String, GestureProperty> gesturePropertyHashMap = scheme.getGesturePropertyHashMap();

            GestureProperty propertyToAdd = gesturePropertyHashMap.get(gesturePropertyID);

            if (propertyToAdd == null) {
                return;
            }

            this.selectedProperties.add(propertyToAdd);
        }
    }

    public GesturePropertyEvent getEvent() {
        return event;
    }

    @Override
    public Event getFirstEvent() {
        return event;
    }

    @Override
    public Event getLastEvent() {
        return event;
    }

    @Override
    public boolean selectEntityIfEventSelected() {
        return false;
    }

    @Override
    public boolean selectEventIfEntitySelected() {
        return false;
    }

    @Override
    public Time getStartTimeInfo() {
        return event.getStartTimeInfo();
    }

    @Override
    public Time getEndTimeInfo() {
        return event.getEndTimeInfo();
    }

    @Override
    public void setStartTimeInfo(Time startTimeInfo) {
        event.setStartTimeInfo(startTimeInfo);
    }

    @Override
    public void setEndTimeInfo(Time endTimeInfo) {
        event.setEndTimeInfo(endTimeInfo);
    }

    public void updateTimes() {
        if (refDomGestureEntity != null) {
            Time startTimeInfo = refDomGestureEntity.getStartTimeInfo();
            Time endTimeInfo = refDomGestureEntity.getEndTimeInfo();

            setStartTimeInfo(startTimeInfo);
            setEndTimeInfo(endTimeInfo);
            
            if (dependentGesturePropertyEntities != null && !dependentGesturePropertyEntities.isEmpty()) {
                for (GesturePropertyEntity depEntity: dependentGesturePropertyEntities) {
                    depEntity.setStartTimeInfo(startTimeInfo);
                    depEntity.setEndTimeInfo(endTimeInfo);
                }
            }
        }
    }

    @Override
    public boolean select() {
        return false;
    }

    @Override
    public boolean unselect() {
        return false;
    }

    @Override
    public boolean adjustEntity(int x1, int x2) {
        return false;
    }

    @Override
    public Event getSelectedEvent() {
        return null;
    }

    @Override
    public void reset() {

    }

    @Override
    public boolean delete() {
        return false;
    }

    @Override
    public Event getEventAt(double x) {
        return null;
    }

    @Override
    public ArrayList<String> getEventsToAdd() {
        return null;
    }

    @Override
    public String getToolTipText(int x) {
        return null;
    }

    @Override
    public boolean selectEvent(double x) {
        return false;
    }

    @Override
    public void paint(Graphics2D g2d, double Y, Font font) {
        int y = (int) Y;
        g2d.setFont(font);
        g2d.setColor(Color.BLACK);

        // checking dependents are empty to just not paint the parent entity
        if (selectedProperties != null && !selectedProperties.isEmpty() && dependentGesturePropertyEntities.isEmpty()) {
            String gesturePropertyLabel = "";
            for (GestureProperty gt : selectedProperties) {
                gesturePropertyLabel = gesturePropertyLabel + " " + gt.getName();
            }
            double x1 = event.getStartTimeCoordinate();
            double x2 = event.getEndTimeCoordinate();
            int strWidth = g2d.getFontMetrics().stringWidth(gesturePropertyLabel);

            g2d.drawString(gesturePropertyLabel, (int) ((x1 + x2 - strWidth) / 2), y);
            g2d.drawLine((int)x1, y + 1, (int) x2, y + 1);
            g2d.drawLine((int)x1, y + 2, (int) x2, y + 2);
            g2d.drawLine((int)x1, y - 1, (int) x1, y + 2);
            g2d.drawLine((int)x2, y - 1, (int) x2, y + 2);
        }
    }

    @Override
    public ArrayList<Event> getEventsList() {
        ArrayList<Event> events = new ArrayList<Event>();
        events.add(getEvent());
        return events;
    }

    @Override
    public boolean equals(Object o) {
        GesturePropertyEntity other;
        if (o instanceof GesturePropertyEntity)
            other =  (GesturePropertyEntity) o;
        else
            return false;

        return (getSelectedProperties().equals(other.getSelectedProperties()))
                && (getDependentGesturePropertyEntities().equals(other.getDependentGesturePropertyEntities()));
    }
}
