package edu.bu.signstream.grepresentation.fields.gestureField.handshapes;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import edu.bu.signstream.grepresentation.fields.*;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapeFormBean;
import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.common.util.SSImage;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldTypeImg;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureChainedEvent;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.GestureHandShapeBean;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class GestureHandShapeEntity extends GestureHandShapeSubEntity implements ImageEventsEntity {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	private NewGestureHandShapeEntity newEntity = null;
	private char type;
	private boolean markedHand = false;

	private boolean isLockedGesturePhonAlignments = true;

	private GestureHandShapeSubEntitiesManager manager = new GestureHandShapeSubEntitiesManager();
	
	private GestureEventsEntity refGestureEntity;

	public GestureHandShapeEntity(SignStreamSegmentPanel segmentPanel, Field field) {
		super(segmentPanel, field);
	}

	public GestureHandShapeEntity(String fieldID, SS3Entity entity,
			SignStreamSegmentPanel segmentPanel, Field field) {
		super(segmentPanel, field);
		//this.updDelegate = new EntityUpdateDelegate(this, segmentPanel);

		GestureHandShapeSubEntity subEntity = new GestureHandShapeSubEntity(segmentPanel,
				field);
		this.addGestureHandShapeSubEntity(subEntity);

        ArrayList<SS3Item> items = entity.getItemsAsList();
		for (SS3Item item : items) {
			subEntity.addHandShape(item);
		}
		subEntity.updateParentEntity();
	}
	
	@Override
	public void setID(String id) {
		super.setID(id);
	}
	
	public void resetEntity(SS3Entity entity) {
		ArrayList<GestureHandShapeSubEntity> entities = this.getSubEntities();
		
		if(entities.size() < 1)
			return;
		
		GestureHandShapeSubEntity subEntity = entities.get(0);		
		GestureHandShapesManager manager = subEntity.getGestureHandShapesManager();
		manager.deleteEntity();
        ArrayList<SS3Item> items = entity.getItemsAsList();
		for (SS3Item item : items) {
			subEntity.addHandShape(item);
		}
	}

    public void cloneEvents(Time timeInfo) {
	    // JACK: Not needed anymore, since painting is unobtrusive...
//        Event firstEvent = getFirstGestureHandShapeEvent();
//        Event lastEvent = getLastGestureHandShapeEvent();
//
//        if(firstEvent.equals(lastEvent)) {
//            GestureHandShapeEvent hshEvent = (GestureHandShapeEvent)firstEvent;
//
//            SS3Entity entity = new SS3Entity();
//            entity.addItem(hshEvent.getSS3Item());
//            SS3Item item = hshEvent.getSS3Item();
//            SS3Item nextItem = new SS3Item(item.getId(), item.getImageID(), timeInfo, timeInfo);
//
//            nextItem.setImageID(item.getImageID());
//
//            entity.addItem(nextItem);
//            resetEntity(entity);
//        }
    }
    
    	public boolean mergeEvents(Time timeInfo) {
    		Event firstEvent = getFirstGestureHandShapeEvent();
    		Event lastEvent = getLastGestureHandShapeEvent();
    		
		GestureHandShapeEvent firstHshEvent = (GestureHandShapeEvent)firstEvent;
		GestureHandShapeEvent lastHshEvent = (GestureHandShapeEvent)lastEvent;
		
		String firstImageID = firstHshEvent.getImageID();
		String lastImageID = lastHshEvent.getImageID();
		
		if ( firstImageID.equals(lastImageID)) {
			SS3Entity entity = new SS3Entity();
			
            SS3Item firstItem = firstHshEvent.getSS3Item();
			SS3Item newItem = new SS3Item(firstItem.getId(), firstItem.getImageID(), timeInfo, timeInfo);
			
			entity.addItem(newItem);
			resetEntity(entity);
			
			return true;
		}
    		
    		return false;
    	}
    	
    	public boolean isPossibleToMergeEvents() {
		Event firstEvent = getFirstGestureHandShapeEvent();
		Event lastEvent = getLastGestureHandShapeEvent();
		
		GestureHandShapeEvent firstHshEvent = (GestureHandShapeEvent)firstEvent;
		GestureHandShapeEvent lastHshEvent = (GestureHandShapeEvent)lastEvent;
		
		String firstImageID = firstHshEvent.getImageID();
		String lastImageID = lastHshEvent.getImageID();
		
		if ( firstImageID.equals(lastImageID)) {
			return true;
		}
    		
    		return false;
    	}
		
	public boolean isLockedGesturePhonAlignments() {
		return isLockedGesturePhonAlignments;
	}

	public void setLockedGesturePhonAlignments(boolean isLockedGesturePhonAlignments) {
		this.isLockedGesturePhonAlignments = isLockedGesturePhonAlignments;
	}

	public boolean minifyAllIcons() {
		boolean minified = false;
        ArrayList<GestureHandShapeEvent> events = manager.getEvents();
		for (GestureHandShapeEvent event : events) {
			if (event.isMagnifiedImage()) {
				event.setMagnifiedImage(false);
				minified = true;
			}
		}
		return minified;
	}

	public void setHandShapeType(char type) {
		this.type = type;
	}

	public char getHandShapeType() {
		return type;
	}

	public NewGestureHandShapeEntity getNewGestureHandShapeEntity() {
		return newEntity;
	}

	public void setNewGestureHandShapeEntity(NewGestureHandShapeEntity newEntity) {
		this.newEntity = newEntity;
	}

	public EventsIterator getEventsIterator() {
		return new EventsIterator(getEvents());
	}

	public boolean hasSubEntities() {
		return !manager.getSubEntities().isEmpty();
	}

	public void addGestureHandShapeSubEntity(GestureHandShapeSubEntity subEntity) {
		manager.addSubEntity(subEntity);
	}

	@Override
	public boolean contains(double x) {
		if (getFirstEvent() == null || getLastEvent() == null) {
			return false;
		}
		double startTimeCoord = getFirstEvent().getStartTimeCoordinate();
		double endTimeCoord = getLastEvent().getStartTimeCoordinate();

		return (x >= startTimeCoord && x <= endTimeCoord);
	}

	@Override
	public SS3Entity getSS3Entity() {
		SS3Entity entity = super.getSS3Entity();

        ArrayList<GestureHandShapeEvent> col = manager.getEvents(); // .getHandShapesCollection();
		for (GestureHandShapeEvent event : col) {
			if (event != null) {
				SS3Item item = new SS3Item();
				item.setId(event.getID());
				item.setImageID(event.getImageID());
				item.setStartTimeInfo(event.getStartTimeInfo());
				item.setEndTimeInfo(event.getEndTimeInfo());
				item.setText(event.getText());
				entity.addItem(item);
			}
		}
		
		entity.setID(getID());
		if (getParentEntities() != null)
			entity.setParentEntities(getParentEntities());
		entity.setRefEntityId(getRefEntityId());
		entity.setLockedGesturePhoneAlignments(isLockedGesturePhonAlignments);

		return entity;
	}

	@Override
	public boolean isSelected() {
		return false;
	}

	@Override
	public boolean isOverlapping(double coordx1, double coordx2) {
		double startCoord = getStartTimeCoordinate();
		double endCoord = getEndTimeCoordinate();

		return (coordx1 <= startCoord && coordx2 >= startCoord)
				|| (coordx1 <= endCoord && coordx2 >= endCoord);
	}

	@Override
	public double getStartTimeCoordinate() {
		// if (initialHold != null)
		// return initialHold.getStartTimeCoordinate();
		// return manager.getStartTimeCoordinate();
		return getFirstEvent().getStartTimeCoordinate();
	}

	@Override
	public Event getFirstEvent() {
		// if (initialHold != null)
		// return initialHold;
		return manager.getFirstEvent();
	}

	@Override
	public Event getLastEvent() {
		// if (finalHold != null)
		// return finalHold;
		return manager.getLastEvent();
	}

	public Event getFirstGestureHandShapeEvent() {
		return manager.getFirstEvent();
	}

	public Event getLastGestureHandShapeEvent() {
		return manager.getLastEvent();
	}

	@Override
	public double getEndTimeCoordinate() {
		// if (finalHold != null)
		// return finalHold.getEndTimeCoordinate();
		// return manager.getEndTimeCoordinate();
		return getLastGestureHandShapeEvent().getEndTimeCoordinate();
	}

	@Override
	public void addHandShape(SS3Item item) {
		// GestureHandShapeSubEntity ent = manager.getSuperEntity();
		// ent.addHandShape(item);
		// manager.addHandShape(item);
	}

	public void addGestureHandShapeEvent(GestureHandShapeEvent event) {
		GestureHandShapeSubEntity entity = manager.getSuperEntity();
		entity.addHandShape(event);
	}

	@Override
	@Deprecated
	public void addHandShape(TrackValue value) {
		// hsEventsManager.addHandShape(value);
	}

	@Override
	@Deprecated
	public void setStartTime(int startTime) {
		Event event = manager.getFirstEvent();
		event.setStartTimeInfo(SS3Singleton.getVideoTimeRange().time(startTime, TimeUnit.TIMESTAMP));
//		event.getEndTimeInfo().setMovieTime(startTime);	// moved into if-block below
		if (!manager.getFirstEvent().equals(manager.getLastEvent())) {
			// in the case where it's not only one hand shape event, also set the start time info
			// don't want to do this if only one because it will mess with the painting logic for one event
            event.setEndTimeInfo(SS3Singleton.getVideoTimeRange().time(startTime, TimeUnit.TIMESTAMP));
		}
	}

	@Override
	@Deprecated
	public void setEndTime(int endTime) {
		Event event = manager.getLastEvent();
		event.setEndTimeInfo(SS3Singleton.getVideoTimeRange().time(endTime, TimeUnit.TIMESTAMP));
//		event.getStartTimeInfo().setMovieTime(endTime);	// moved into if-block below
		if (!manager.getFirstEvent().equals(manager.getLastEvent())) {
			// in the case where it's not only one hand shape event, also set the start time info
			// don't want to do this if only one because it will mess with the painting logic for one event
            event.setStartTimeInfo(SS3Singleton.getVideoTimeRange().time(endTime, TimeUnit.TIMESTAMP));
		}
	}

	@Override
	public Time getStartTimeInfo() {
		Event event = getFirstEvent();
		if (event == null)
		    return SS3Singleton.getVideoTimeRange().zero();

		return event.getStartTimeInfo();
	}

	@Override
	public Time getEndTimeInfo() {
		Event event = getLastEvent();
		if (event == null)
            return SS3Singleton.getVideoTimeRange().zero();

		return event.getEndTimeInfo();		// GREG TODO: is this correct? shouldn't it be getEndTimeInfo ?
		// GREG TODO: changed to end - test it, see what's different
		// .getEndTimeInfo();
	}

	   @Override
	public void setStartTimeInfo(Time startTimeInfo) {
	      manager.getFirstEvent().setStartTimeInfo(startTimeInfo);
	   }

	   @Override
	public void setEndTimeInfo(Time endTimeInfo) {
	      manager.getLastEvent().setEndTimeInfo(endTimeInfo);
	   }


    public void setStartTime() {
        // check if entity has only one handshape, create another handshape
        if(getFirstEvent().equals(getLastEvent())) {
            getLastEvent();
        }
    }
    
	@Override
	public boolean selectEntityIfEventSelected() {
		if (manager.selectEntityIfEventSelected())
			select();
		return selected;
	}

	@Override
	public boolean selectEventIfEntitySelected() {
        return selected && manager.selectEventIfEntitySelected();
    }

	@Override
	public boolean select() {
		// hsEventsManager.highlightEntity(Palette.HIGHLIGHTED_ENTITY_COLOR);

		// QTMultiMovieController qtControl =
		// SSSingleton.getMultiMovieController();
		// qtControl.setStartTime((int)
		// hsEventsManager.getFirstEvent().getStartTime());
		// qtControl.setEndTime((int)
		// hsEventsManager.getLastEvent().getEndTime());
		selected = true;
		// mToolBar.setEnable(true);
		return selected;
	}

	@Override
	public boolean unselect() {
		selected = false;
		// mToolBar.setEnable(false);
		manager.unselectEntity();
		// if (initialHold != null)
		// initialHold.unselect();
		//
		// if (finalHold != null)
		// finalHold.unselect();

		return selected;
	}

	// public boolean adjustEntity(int x1, int x2) {
	// return manager.adjustEntity(x1, x2);
	// }

	@Override
	public Event getSelectedEvent() {
		// if (initialHold != null && initialHold.isSelected())
		// return initialHold;
		//
		// if (finalHold != null && finalHold.isSelected())
		// return finalHold;

		return manager.getSelectedEvent();
	}

	@Override
	public void reset() {
		manager.reset();
	}

	@Override
	public boolean delete() {
		// this.initialHold = null;
		// this.finalHold = null;
		manager.clear();
		// Field field =
		// SS3Singleton.getManager().getLoadedSegment().getField(fieldID);
		// field.deleteEmptyEntities();
		return true;
	}

    @Override
	public Event getEventAt(double x) {
		// if (initialHold != null && initialHold.contains(x))
		// return initialHold;
		//
		// if (finalHold != null && finalHold.contains(x))
		// return finalHold;

		return manager.getEventAt(x);
	}

	@Override
	public ArrayList getEventsToAdd() {
		ArrayList list = new ArrayList();
		list.add("Add HandShape");
		return list;
	}

	@Override
	public String getToolTipText(int x) {
		GestureHandShapeEvent event = (GestureHandShapeEvent) manager.getEventAt(x);

		if (event == null)
			return "";

		StringBuilder toolTipText = new StringBuilder();
		toolTipText.append(event.getText());

		if (event.equals(getFirstEvent())) {
			toolTipText.append(", starts at ");
		} else if (event.equals(getLastEvent())) {
			toolTipText.append(", ends at ");
		} else {
			toolTipText.append(", at ");
		}

		toolTipText.append(event.getStartTimeInfo().doubleValue());

		return toolTipText.toString();
	}

	@Override
	public boolean selectEvent(double x) {
		Event event = manager.getEventAt(x);
        return event != null && event.select();
    }

	public ArrayList<GestureHandShapeSubEntity> getSubEntities() {
		return manager.getSubEntities();
	}

	@Override
	public ArrayList getEvents() {
		ArrayList events = new ArrayList();
		// if (getInitialHold() != null)
		// events.add(getInitialHold());
		events.addAll(manager.getEvents());
		// if (getFinalHold() != null)
		// events.add(getFinalHold());
		return events;
	}

	public ArrayList<GestureHandShapeEvent> getGestureHandShapeEvents() {
		return manager.getEvents();
	}
	
	public GestureEventsEntity getRefGestureEntity() {
		return refGestureEntity;
	}
	
	public void setRefGestureEntity(GestureEventsEntity refGestureEntity) {
		this.refGestureEntity = refGestureEntity;
	}
	
	public void updateTimes() {
        if (refGestureEntity != null) {
        	getFirstGestureHandShapeEvent().setStartTimeInfo(refGestureEntity.getValueOrTextValueEvent().getStartTimeInfo());
        	getFirstGestureHandShapeEvent().setEndTimeInfo(refGestureEntity.getValueOrTextValueEvent().getStartTimeInfo());
        	
        	getLastGestureHandShapeEvent().setStartTimeInfo(refGestureEntity.getValueOrTextValueEvent().getEndTimeInfo());
        	getLastGestureHandShapeEvent().setEndTimeInfo(refGestureEntity.getValueOrTextValueEvent().getEndTimeInfo());

            double startTime = getStartTimeInfo().value();
            double endTime = getEndTimeInfo().value();
            ArrayList<GestureHandShapeEvent> events = getGestureHandShapeEvents();
            for (int i = 0; i < events.size(); i++) {
                GestureHandShapeEvent event = events.get(i);
                double time;
                if (i == 0) {
                    time = startTime;
                } else if (i == events.size() - 1) {
                    time = endTime;
                } else {
                    time = startTime + (i * (endTime - startTime) / (events.size() - 1));
                }
                var startTimeInfo = getStartTimeInfo().range().time(Math.round(time), TimeUnit.TIMESTAMP);
                event.setStartTimeInfo(startTimeInfo);
            }
        }
    }

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
        ImageEventsEntityPainter.paint(g2d, (int)Math.round(Y) - 22, 28, this);
	}

	public boolean isMarkedHand() {
		return markedHand;
	}

	public static SS3Entity createSS3Entity(ArrayList<HandShapeImage> handShapeImgs,
                                      Time startTimeInfo, Time endTimeInfo) {
		SS3Entity ent = new SS3Entity();

		if (handShapeImgs.size() == 1) {
			HandShapeImage img = handShapeImgs.get(0);
			SS3FieldValue value = img.getSS3FieldValue();

			SS3Item item1 = new SS3Item(value.getId(), value.getIid(),
					startTimeInfo, endTimeInfo);
			ent.addItem(item1);
		} else {
			Duration dur = endTimeInfo.distance(startTimeInfo.absolute()).divide(handShapeImgs.size() - 1);

			HandShapeImage img1 = handShapeImgs.get(0);
			SS3FieldValue value1 = img1.getSS3FieldValue();

			SS3Item item1 = new SS3Item(value1.getId(), value1.getIid(),
					startTimeInfo, startTimeInfo);
			ent.addItem(item1);

			Time nextDurTime = startTimeInfo;

			for (int i = 1; i < handShapeImgs.size(); i++) {
				HandShapeImage img = handShapeImgs.get(i);
				SS3FieldValue value = img.getSS3FieldValue();
				nextDurTime = nextDurTime.add(dur);
				SS3Item item3 = new SS3Item(value.getId(), value.getIid(),
											nextDurTime, nextDurTime);
				ent.addItem(item3);
			}

		}
		return ent;
	}

	public static GestureHandShapeEntity createGestureHandShapeEntity(HandShapeFormBean bean,
																	  ArrayList<HandShapeImage> handShapeImages,
																	  GestureChainedEventsEntity superEntity, Field field) {

		SignStreamSegmentPanel segmentPanel = superEntity
				.getSignStreamSegmentPanel();

		GestureHandShapeEntity hsEntity = new GestureHandShapeEntity(segmentPanel, field);
		hsEntity.setID(Util.getLongUniqueNumber());

		hsEntity.setField(field);
		hsEntity.setHandShapeType(bean.getType());
		hsEntity.setRefEntityId(superEntity.getID());
		GestureHandShapeSubEntity subEntity = new GestureHandShapeSubEntity(segmentPanel, field);
		subEntity.setID(hsEntity.getID());
		hsEntity.addGestureHandShapeSubEntity(subEntity);

		GestureChainedEvent event = superEntity.getTextValue();
		Time start = event.getStartTimeInfo();
		Time end = event.getEndTimeInfo();

		if (handShapeImages.size() == 1) {
			HandShapeImage img = handShapeImages.get(0);
			SS3FieldValue value = img.getSS3FieldValue();
			String vid = value.getId();

			SS3Item item1 = new SS3Item();
			item1.setId(vid);
			item1.setImageID(value.getIid());

			item1.setStartTimeInfo(start);
			item1.setEndTimeInfo(end);
			subEntity.addHandShape(item1);
		} else if (handShapeImages.size() > 1) {
			HandShapeImage img1 = handShapeImages.get(0);
			SS3FieldValue value1 = img1.getSS3FieldValue();
			String vid1 = value1.getId();

			SS3Item item1 = new SS3Item();
			item1.setId(vid1);
			item1.setImageID(value1.getIid());

			item1.setStartTimeInfo(start);
			item1.setEndTimeInfo(start);
			subEntity.addHandShape(item1);

			Duration dur = end.distance(start.absolute()).divide(handShapeImages.size() - 1);
			Time nextDurTime = start;

			for (int i = 1; i < handShapeImages.size(); i++) {
				HandShapeImage img = handShapeImages.get(i);
				SS3FieldValue value = img.getSS3FieldValue();
				String vid = value.getId();
				SS3Item item = new SS3Item();
				item.setId(vid);
				item.setImageID(value.getIid());
				
				nextDurTime = nextDurTime.add(dur);

				item.setStartTimeInfo(nextDurTime);
				item.setEndTimeInfo(nextDurTime);

				subEntity.addHandShape(item);
			}
		}

		hsEntity.setLockedGesturePhonAlignments(true);
		hsEntity.highlight();

		subEntity.updateParentEntity();
		hsEntity.updateParentEntity();

		return hsEntity;
	}

	public void resetTimes(Time startTimeInfo, Time endTimeInfo) {
		
		Time start = startTimeInfo;
		Time end = endTimeInfo;
		
		ArrayList<GestureHandShapeEvent> events = getGestureHandShapeEvents();

		Duration dur = end.distance(start.absolute()).divide(events.size() - 1);
		Time nextDurTime = start;

		double frameRate = SS3Singleton.getVideoControlManager().getZoomer().getCoordinateTimeConvertor().getFramerate();
		if (events.size() == 1) {
			GestureHandShapeEvent event = events.get(0);
			SS3Item item = event.getSS3Item();
			
			event.setStartTimeInfo(start);
			event.setEndTimeInfo(end);
			
			item.setStartTimeInfo(start);
			item.setEndTimeInfo(end);
			
			return;
		}

		TimeUnit unit = TimeUnit.FRAME;
		unit.setBase(frameRate);
		
		for (int i = 0; i < events.size(); i++) {
			GestureHandShapeEvent event = events.get(i);
			SS3Item item = event.getSS3Item();

			event.setStartTimeInfo(nextDurTime.convert(unit));
			event.setEndTimeInfo(nextDurTime.convert(unit));
			
			item.setStartTimeInfo(nextDurTime.convert(unit));
			item.setEndTimeInfo(nextDurTime.convert(unit));
			
			nextDurTime = nextDurTime.absolute().add(dur);
			// Convert nextDurTime back to frame time
		}
	}

    @Override
    public List<SSImage> images() {
        var handshapes = SS3Singleton.getGestureHandShapesImageManager();

        var dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
        var field = dbScheme.getField(getField().getFieldID());

        return getGestureHandShapeEvents().stream()
            .map(event -> {
                SS3FieldValue fieldValue = field.getFieldValue(event.getID());
                String id = fieldValue.getId();
                String iid = fieldValue.getIid();
                SS3FieldTypeImg image = dbScheme.getImage(iid);
                
                if (image == null || iid.equals("HSH_999")) // "No Image"
                    return null;
                else
                    return new SSImage(handshapes.getScaledImageInstance(image.getPath(), this), id, field.getId());
            })
            .collect(Collectors.toList());
    }
}
