package edu.bu.signstream.grepresentation.fields.gestureField.handshapes;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.geom.Line2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldTypeImg;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsIterator;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;

public class GestureHandshapeEntityPainter {

    private static final Color entitySelectedColor = Color.GREEN;
    private static final Color eventSelectedColor = Color.RED;
    private static final Color eventHighlightedColor = Color.BLUE;

    private static final Color COLOR_WHITE = Color.WHITE;

    private static final int DEFAULT_ICON_WIDTH = 23;

    public static void drawGestureHandShapeEntity(GestureHandShapeEntity entityOrig, Graphics2D g2d,
                                           double Y, String fieldID) {

        if (entityOrig == null || entityOrig.getFirstEvent() == null || !entityOrig.hasSubEntities())
            return;
        
		GestureHandShapeEntity entity = new GestureHandShapeEntity(entityOrig.getSignStreamSegmentPanel(), entityOrig.getField());
		

        entity.setID(Util.getLongUniqueNumber());
        entity.setField(entityOrig.getField());
        entity.setHandShapeType(entityOrig.getHandShapeType());
        entity.setRefEntityId(entityOrig.getRefEntityId());		// referring back to same hand entity object based on ID
		GestureHandShapeSubEntity subEntityCopy = new GestureHandShapeSubEntity(entity.getSignStreamSegmentPanel(), entity.getField());
		subEntityCopy.setID(entity.getID());
		subEntityCopy.updateParentEntity();
		entity.addGestureHandShapeSubEntity(subEntityCopy);
                
        for (GestureHandShapeEvent e: entityOrig.getGestureHandShapeEvents()) {
        		entity.addGestureHandShapeEvent(e);
        }
        
		entity.updateParentEntity();
        
        // we copy the image item to display twice in this case (only one handshape image in data)
        // and we use a copy of the GestureHandShapeEntity (entity is a copy of entityOrig) to modify and use below
        if (entity.getEvents().size() == 1) {            		
	        	GestureHandShapeEvent firstGestureHandShapeEvent = (GestureHandShapeEvent)entity.getFirstEvent();
	    		
	    		SS3Item firstItem = firstGestureHandShapeEvent.getSS3Item();
	    		
	    		Time firstItemStartTimeInfo = firstItem.getStartTimeInfo();
	    		Time firstItemEndTimeInfo = firstItem.getEndTimeInfo();
	    		
	    		Time newStartTimeInfo = firstItemStartTimeInfo.range().time(
	    				firstItemStartTimeInfo.value(), firstItemStartTimeInfo.unit());
	    		Time newEndTimeInfo = firstItemEndTimeInfo.range().time(
	    				firstItemEndTimeInfo.value(), firstItemEndTimeInfo.unit());
	    		
                firstItem.setEndTimeInfo(newStartTimeInfo);
                
                SS3Item secondItem = new SS3Item(firstItem.getId(), firstItem.getImageID(),
                		newEndTimeInfo, newEndTimeInfo);

                SS3Entity ent = new SS3Entity();
	    		ent.addItem(firstItem);
	    		ent.addItem(secondItem);
	    		
	    		entity.resetEntity(ent);	    		
        	}

        // LOAD IMAGES
        loadHandShapeIcons(entity, fieldID);

        if (!drawTwoIdenticalHandshapes(entity, fieldID, g2d, (int)Y)) {
            EventsIterator eventsIterator = entity.getEventsIterator();

            GestureHandShapeEvent magnifiedEvent = null;

            Color entityColor = getValueColor(entity, entity.isSelected(),
                    entity.isHighlighted(), fieldID, entity.getField());

            Color color = entityColor;
            g2d.setColor(color);

            ArrayList<GestureHandShapeSubEntity> subEntities = entity.getSubEntities();

            // iterator returns sorted events
            GestureHandShapeSubEntity superEntity = subEntities.get(0);

            if (superEntity.getEvents().isEmpty())
                return;

            if (superEntity.isSelected())
                color = Color.RED;

            g2d.draw(new Line2D.Double(superEntity.getStartTimeCoordinate(),
                    Y - 5, superEntity.getEndTimeCoordinate(), Y - 5));

            // draw entity base line
            g2d.draw(new Line2D.Double(
                    superEntity.getStartTimeCoordinate() + 2, Y - 5,
                    superEntity.getEndTimeCoordinate() - 2, Y - 5));

            // draw super entity icons
            ArrayList<GestureHandShapeEvent> superEvents = superEntity.getEvents();
            
            boolean lineByStart = superEvents.size() == 1 ? true : false;

            GestureHandShapeEvent superEntityLastEvent = (GestureHandShapeEvent) superEntity
                    .getLastEvent();

            g2d.setColor(color);

            for (int i = 0; i < superEvents.size() - 1; i++) {
                GestureHandShapeEvent event = superEvents.get(i);
                // boolean lineByStartCoord = true;
                // if (event == superEntityLastEvent)
                // lineByStartCoord = false;
                // g2d.setColor(color);
                boolean drawImageOriginalSize = drawGestureHandShapeEvent(event,
                        entity, eventsIterator, true, g2d, Y);

                if (event.isMagnifiedImage() && !drawImageOriginalSize) {
                    magnifiedEvent = event;
                }
            }

            // g2d.setColor(color);
            
            boolean drawImageOriginalSize1 = drawGestureHandShapeEvent(
                    superEntityLastEvent, entity, eventsIterator, lineByStart, g2d, Y);

            if (superEntityLastEvent.isMagnifiedImage()
                    && !drawImageOriginalSize1) {
                magnifiedEvent = superEntityLastEvent;
            }

            // draw new sub entity
            NewGestureHandShapeEntity newEntity = entity.getNewGestureHandShapeEntity();

            if (newEntity != null) {
                g2d.setColor(color);

                drawSubEntityBaseLine(newEntity.getStartTimeCoord(), newEntity
                        .getEndTimeCoord(), (int) Y, g2d);
            }

            for (int i = 1; i < subEntities.size(); i++) {
                GestureHandShapeSubEntity subEntity = subEntities.get(i);

                color = subEntity.isSelected() ? Color.RED : entityColor;

                g2d.setColor(color);

                // draw sub entity base line
                drawSubEntityBaseLine(subEntity.getStartTimeCoordinate(),
                        subEntity.getEndTimeCoordinate(), (int) Y, g2d);

                Event subEntityLastEvent = subEntity.getLastEvent();
                ArrayList subEvents = subEntity.getEvents();

                for (Object subEvent : subEvents) {
                    GestureHandShapeEvent sEvent = (GestureHandShapeEvent) subEvent;
                    boolean lineByStartCoord = true;
                    if (sEvent == subEntityLastEvent)
                        lineByStartCoord = false;

                    g2d.setColor(color);

                    boolean drawImageOriginalSize = drawGestureHandShapeEvent(sEvent,
                            entity, eventsIterator, lineByStartCoord, g2d, Y);

                    if (sEvent.isMagnifiedImage() && !drawImageOriginalSize) {
                        magnifiedEvent = sEvent;
                    }
                }
            }

            if (magnifiedEvent != null) {
                Image img = magnifiedEvent.getLargeImage();

                Image image = img.getScaledInstance(img.getWidth(entity), 23,
                        Image.SCALE_AREA_AVERAGING);

                int width1 = image.getWidth(entity) + 1;
                int height1 = image.getHeight(entity) + 1;

                double w = magnifiedEvent.getEndTimeCoordinate()
                        - magnifiedEvent.getStartTimeCoordinate();
                double w1 = magnifiedEvent.getStartTimeCoordinate() + (w / 2);
                double startCoord = w1 - width1 / 2;

                g2d.drawRect((int) startCoord - 1, (int) Y - 18 - 1, width1, height1);
                g2d.drawImage(image, (int) startCoord, (int) Y - 18, COLOR_WHITE,
                        entity);
            }
        }
    }

    private static boolean drawTwoIdenticalHandshapes(GestureHandShapeEntity entity,
                                                      String fieldID, Graphics2D g2d, int Y) {
        ArrayList events = entity.getEvents();

        if (events.size() != 2) {
            return false;
        }
        // start here!
        GestureHandShapeEvent firstEvent = (GestureHandShapeEvent) events.get(0);
        GestureHandShapeEvent lastEvent = (GestureHandShapeEvent) events.get(1);

        if (!firstEvent.getImageID().equalsIgnoreCase(lastEvent.getImageID())) {
            return false;
        }

        // LOAD IMAGES
        loadHandShapeIcons(entity, fieldID);

        double width = entity.getEndTimeCoordinate()
                - entity.getStartTimeCoordinate();

        Image imgL = firstEvent.getLargeImage();
        Image imgS = firstEvent.getSmallImage();
        int lL = imgL.getWidth(entity);
        int lS = imgS.getWidth(entity);
        // int lLR1 = (int)(lL*0.8);
        int lLR2 = (int) (lL * 1.3);
        int lSR1 = (int) (lS * 0.8);
        int lSR2 = (int) (lS * 1.2);

        if (width == lS) {
            // draw one small handshape
            // DRAW

            drawIdenticalIcons(imgS, firstEvent, lastEvent,
                    entity, entity.getStartTimeCoordinate(), width, Y, g2d);

            return true;
        } else if (width < lS) {
            if (width >= lSR1) {
                // draw resized small image

                drawIdenticalIcons(imgS, firstEvent, lastEvent,
                        entity, entity.getStartTimeCoordinate(), width, Y, g2d);

                return true;
            } else {

                g2d.drawRect((int) entity.getStartTimeCoordinate(), Y - 18 - 1, (int) width, DEFAULT_ICON_WIDTH);

                // draw rectangles
                //DRAW that rectangle here and return true!!!
                //int l = lenght > 3 ? lenght : 3;
                //event1.setRectangle(new Rectangle(entity.getStartTimeCoordinate(), 1, l, 23));
                //g2d.drawRect(entity.getStartTimeCoordinate(), (int) Y - 18 - 1, l, 18);
                //return true;
                return false;
            }
        } else if (width < lL) {
            if (width <= lSR2) {

                // DRAW
                drawIdenticalIcons(imgS, firstEvent, lastEvent,
                        entity, entity.getStartTimeCoordinate(), width, Y, g2d);

                return true;
            } else {

                // DRAW
                drawIdenticalIcons(imgL, firstEvent, lastEvent,
                        entity, entity.getStartTimeCoordinate(), width, Y, g2d);

                return true;
            }
        } else if (width == lL) {
            // draw large handshape
            // DRAW
            drawIdenticalIcons(imgL, firstEvent,
                    null, entity, entity.getStartTimeCoordinate(), width, Y, g2d);

            return true;
        } else if (width < 2 * lS) {
            if (width <= lLR2) {

                // DRAW
                drawIdenticalIcons(imgL, firstEvent, lastEvent,
                        entity, entity.getStartTimeCoordinate(), width, Y, g2d);

                return true;
            } else {
                // draw a rectangle (black)
                int height = imgL.getHeight(null);
                g2d.setColor(Color.black);
                g2d.fillRect((int) entity.getStartTimeCoordinate(), Y - 18,
                        (int) width, height); // 23);

                double coordinate = entity.getStartTimeCoordinate() + (width - lL) / 2;

                // DRAW
                drawIdenticalIcons(imgL, firstEvent, lastEvent, entity, coordinate, width, Y, g2d);

                return true;
            }
        }

        return false;
		/*
		 * else if(lenght > 2*lL) { // draw two large handshapes } else
		 * if(lenght > 2*lS) { // draw two small handshapes }
		 *
		 * // if (lenght < img.getWidth(entity)) { if (lenght == 0) lenght = 1;
		 *
		 * Image image = event1.getLargeImage(); //.getSmallImage(); //Image
		 * image = img2.getScaledInstance(lenght, 23, //
		 * Image.SCALE_AREA_AVERAGING
		 *
		 * g2d.drawImage(image, entity.getStartTimeCoordinate(), (int) Y - 18,
		 * Color.WHITE, entity);
		 *
		 * event1.setRectangle(new Rectangle(entity.getStartTimeCoordinate() -
		 * 1, 1, lWidth, 23)); event2.setRectangle(new
		 * Rectangle(entity.getStartTimeCoordinate() - 1 + lWidth, 1, lWidth,
		 * 23)); // g2d.setColor(Color.RED); //
		 * g2d.drawRect(entity.getStartTimeCoordinate() - 1, (int) Y - 18, //
		 * lWidth, 23); // g2d.drawRect(entity.getStartTimeCoordinate() - 1 +
		 * lWidth, (int) Y - // 18, lWidth, 23); return true;
		 */
    }

    private static void drawIdenticalIcons(Image image, GestureHandShapeEvent firstEvent,
                                           GestureHandShapeEvent lastEvent, GestureHandShapeEntity entity,
                                           double startCoordinate, double width, int Y, Graphics2D g2d) {

        g2d.drawImage(image, (int) startCoordinate, Y - 18, COLOR_WHITE, entity);

        if(firstEvent != null)
            firstEvent.setRectangle(new Rectangle((int)entity
                    .getStartTimeCoordinate() - 1, 1, (int) width, DEFAULT_ICON_WIDTH));

        if(lastEvent != null)
            lastEvent.setRectangle(new Rectangle((int)entity
                    .getStartTimeCoordinate() - 1, 1, (int) width, DEFAULT_ICON_WIDTH));
    }

    private static void drawSubEntityBaseLine(double x1, double x2, int Y, Graphics2D g2d) {
        g2d.draw(new Line2D.Double(x1 + 2, Y - 2, x2 - 2, Y - 2));
        g2d.draw(new Line2D.Double(x1 + 2, Y - 8, x2 - 2, Y - 8));
    }

    private static boolean drawGestureHandShapeEvent(GestureHandShapeEvent event,
                                              GestureHandShapeEntity entity, EventsIterator eventsIterator,
                                              boolean lineByStart, Graphics2D g2d, double Y) {
    	if (event.getImageID().equals(Util.DNI_HANDSHAPE_IMAGE_ID)) {
        	return true;
        }
        
    	if (event.isSelected())
            g2d.setColor(Color.RED);

        double startTimeCoordinate = event.getStartTimeCoordinate();

        Image image;

        if (lineByStart) {
            Event nextEvent = eventsIterator.next(event);

            image = getImageToDrawLinedByStart(event, nextEvent, entity);

            if(image == null) {
                g2d.drawRect((int) startTimeCoordinate - 1, (int) Y - 18 - 1, 2, 24);
                event.setRectangle(new Rectangle((int) startTimeCoordinate - 1, 1, 2, 24));
                return false;
            }
        } else {
            Event previousEvent = eventsIterator.previous(event);

            image = getImageToDrawLinedByEnd(event, previousEvent, entity);

            if (image == null) {
                g2d.drawRect((int) startTimeCoordinate - 1, (int) Y - 18 - 1, 2, 24);
                event.setRectangle(new Rectangle((int) startTimeCoordinate - 1, 1, 2, 24));
                return false;
            } else {
                startTimeCoordinate = event.getStartTimeCoordinate() - image.getWidth(entity);
            }
        }

        int width = image.getWidth(entity) + 1;
        int height = image.getHeight(entity) + 1;

        g2d.drawRect((int) startTimeCoordinate - 1, (int) Y - 18 - 1, width, height);
        event.setRectangle(new Rectangle((int) startTimeCoordinate, 1, width, height));
        g2d.drawImage(image, (int) startTimeCoordinate, (int) Y - 18, Color.WHITE, entity);

        return true;
    }

    private static Image getImageToDrawLinedByEnd(GestureHandShapeEvent event, Event previousEvent, GestureHandShapeEntity entity) {

        loadHandshapeIcon(event, entity, entity.getField().getFieldID());

        Image image = event.getLargeImage();

        double w = image.getWidth(entity);

        if(previousEvent != null)
        {
            w = event.getStartTimeCoordinate() - previousEvent.getStartTimeCoordinate() - 25;
        }

        if (w < event.getSmallImage().getWidth(entity)) {
            return null;
        } else if (w < image.getWidth(entity)) {
            return event.getSmallImage();
        }
        return image;
    }

    private static Image getImageToDrawLinedByStart(GestureHandShapeEvent event, Event nextEvent, GestureHandShapeEntity entity) {

        Image image = event.getLargeImage();

       double w = nextEvent != null ? nextEvent.getStartTimeCoordinate()
                - event.getStartTimeCoordinate() - 5 : 0;
             // changed the else case here from 10000 to 0
             // unclear why it was 10000 to begin with - should be 0 width if there is no next event
        if (w < event.getSmallImage().getWidth(entity)) {
            return null;
        } else if (w < image.getWidth(entity)) {
            return event.getSmallImage();
        }
        return image;
    }

    private static void loadHandShapeIcons(GestureHandShapeEntity entity, String fieldID) {
        ArrayList hShapes = entity.getGestureHandShapeEvents();
        for (Object hShape : hShapes) {
            GestureHandShapeEvent event = (GestureHandShapeEvent) hShape;
            loadHandshapeIcon(event, entity, fieldID);
        }
    }

    private static void loadHandshapeIcon(GestureHandShapeEvent event, GestureHandShapeEntity entity, String fieldID) {

        SS3CodingScheme dbScheme = SS3Singleton
                .getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().getSS3CodingScheme();

        GestureHandShapesImageManager hsManager = SS3Singleton.getGestureHandShapesImageManager();

        SS3Field csField = dbScheme.getField(fieldID);

        if (event.getLargeImage() == null || event.getSmallImage() == null) {

            SS3FieldValue value = csField.getFieldValue(event.getID());

            if(value != null) {
                SS3FieldTypeImg ftImg = dbScheme.getImage(value.getIid());

                if(ftImg != null) {
                    if (event.getLargeImage() == null) {
                        Image imgL = hsManager.getScaledImageInstance(ftImg.getPath(), entity);
                        event.setLargeImage(imgL);
                    }

                    if (event.getSmallImage() == null) {
                        Image imgS = hsManager.getScaledImageInstance(ftImg.getPath(), entity);
                        event.setSmallImage(imgS);
                    }
                }
            }
        }
    }

    private static Color getValueColor(Event event, boolean selected,
                                       boolean entityHighlighted, String fieldID, Field field) {

        SS3CodingScheme dbScheme = SS3Singleton
                .getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().getSS3CodingScheme();

        if (selected) {
            return entitySelectedColor;
        } else if (event != null && event.isSelected()) {
            return eventSelectedColor;
        } else if (entityHighlighted || (event != null && event.isHighlighted())) {
            return eventHighlightedColor;
        } else if (event != null && event.getColor() != null) {
            return event.getColor();
        } else if (field != null && field.getColor() != null) {
            return field.getColor();
        } else {
            SS3Field csField = dbScheme.getField(fieldID);
            if (csField != null)
                return csField.getColor();
            return dbScheme.getColor();
        }
    }
}


