package edu.bu.signstream.grepresentation.fields.gestureField.orientationField;

import java.awt.Image;
import java.awt.Rectangle;
import java.io.File;
import java.util.ArrayList;
import java.util.logging.Level;

import javax.imageio.ImageIO;

import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldTypeImg;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class GestureOrientationEvent extends ChainedEvent {
   private Image lImg = null;
   private Image sImg = null;
   private Rectangle rect;
   
   private boolean magnifiedImage = false;
   
   private String imageID;
   
   private SS3FieldTypeImg fieldTypeImg = null;
   
   private ArrayList dependentEvents = new ArrayList(); 
   
   private String imgPath = null; 

   public GestureOrientationEvent(TrackValue value, SignStreamSegmentPanel segmentPanel) {
      super(value, segmentPanel);
   }

   public GestureOrientationEvent(SS3Item item, SignStreamSegmentPanel segmentPanel) {
      super(item, segmentPanel);
      setText(item.getText());
   }
   
   @Override
public SS3Item getSS3Item() {
	   SS3Item item = super.getSS3Item();
	   item.setImageID(this.imageID);
	   return item;
   }
   
   public void setImagePath(String imgPath) {
      this.imgPath = imgPath;
   }
   
   public String getImagePath() {
      return imgPath;
   }
   
   public void setMagnifiedImage(boolean magnifiedImage) {
      this.magnifiedImage = magnifiedImage;
   }
   
   public boolean isMagnifiedImage() {
      return magnifiedImage;
   }
   
   public void setImageID(String imageID) {
      this.imageID = imageID;
   }
   
   public String getImageID() {
      return imageID;
   }
   
   public void setSS3FieldTypeImg(SS3FieldTypeImg fieldTypeImg) {
      this.fieldTypeImg = fieldTypeImg;
   }
   
   public SS3FieldTypeImg getSS3FieldTypeImg() {
      return fieldTypeImg;
   }
   
   public void addDependentEvent(ChainedEvent event) {
      dependentEvents.add(event);
   }

   public void removeDependentEvent(ChainedEvent event) {
      dependentEvents.remove(event);
   }

   public void setLargeImage(Image image) {
      this.lImg = image;
   }

   public void setSmallImage(Image image) {
      this.sImg = image;
   }

   @Override
public void setStartTimeCoordinate(double coord) {
      super.setStartTimeCoordinate(coord);
      super.setEndTimeCoordinate(coord);

      double movieTime = segmentPanel.getZoomer().getCoordinateTimeConvertor().convertCoordinateToTime(coord);

      startTimeInfo = SS3Singleton.getVideoTimeRange().time(Math.round(movieTime), TimeUnit.TIMESTAMP);
      endTimeInfo = SS3Singleton.getVideoTimeRange().time(Math.round(movieTime), TimeUnit.TIMESTAMP);

      for(int i = 0; i < dependentEvents.size(); i++) {
         ChainedEvent event = (ChainedEvent)dependentEvents.get(i);
         event.setStartTimeCoordinate(coord);
      }
   }

   @Override
public void setEndTimeCoordinate(double coord) {
      //super.setStartTimeCoordinate(coord);
      super.setEndTimeCoordinate(coord);
      
      double movieTime = segmentPanel.getZoomer().getCoordinateTimeConvertor().convertCoordinateToTime(coord);

      startTimeInfo = SS3Singleton.getVideoTimeRange().time(Math.round(movieTime), TimeUnit.TIMESTAMP);
      endTimeInfo = SS3Singleton.getVideoTimeRange().time(Math.round(movieTime), TimeUnit.TIMESTAMP);

      for(int i = 0; i < dependentEvents.size(); i++) {
         ChainedEvent event = (ChainedEvent)dependentEvents.get(i);
         event.setEndTimeCoordinate(coord);
      }
   }

   @Override
public boolean contains(double x) {
      
      if (rect == null)
         return false;
         
      return rect.contains(x, 10);
   }

   public void setRectangle(Rectangle rect) {
      this.rect = rect;
   }

   public void loadLargeImage(String imgPath) {
      if (imgPath != null) {
         try {
            File file = new File(imgPath);
            lImg = ImageIO.read(file);
         } catch (Exception ex) {
            ex.printStackTrace();
            SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
         }
      }
   }

   public Image getLargeImage() {
      return lImg;
   }

   public void loadSmallImage(String imgPath) {
      if (imgPath != null) {
         try {
            File file = new File(imgPath);
            sImg = ImageIO.read(file);
         } catch (Exception ex) {
            ex.printStackTrace();
            SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
         }
      }
   }

   public Image getSmallImage() {
      return sImg;
   }

    // in case of Orientation use textValue for toolTipText
    @Override
    public String getToolTipText() {
        return String.format("%s, starts at %,.2fs", getText(), startTimeInfo.absolute().as(TimeUnit.SECOND));
    }

    @Override
public String toString() {
      return "GestureOrientationEvent: startTimeCoord = "
         + getStartTimeCoordinate()
         + ", endTimeCoord = "
         + getEndTimeCoordinate();
   }
}
