
package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.vo.ss3.db.*;
import edu.bu.signstream.grepresentation.fields.*;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.dai.DAISearch;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.domNdomField.DomNdomHandFieldWrapper;
import edu.bu.signstream.grepresentation.fields.glossField.*;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.handshapeTypes.DomNdomHandshapesType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.handshapeTypes.StartEndHandshapesType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationInfo;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationType;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEvent;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEntity;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEvent;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEventsCollection;
import edu.bu.signstream.grepresentation.fields.locationField.LocationFieldWrapper;
import edu.bu.signstream.grepresentation.fields.nonManualField.*;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignType;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignTypeEntity;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignTypeEvent;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignTypeFieldWrapper;
import edu.bu.signstream.grepresentation.listener.WeakActionListener;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.GlossNRelatedView;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.UtteranceView;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolbarCommons;
import edu.bu.signstream.grepresentation.view.signbank.SignBankCollection;
import edu.bu.signstream.grepresentation.view.signbank.SignBankMainGloss;
import edu.bu.signstream.grepresentation.view.signbank.SignBankOccurrence;
import edu.bu.signstream.grepresentation.view.signbank.SignBankVariant;
import edu.bu.signstream.time.*;
import edu.bu.signstream.ui.ButtonPanel;
import edu.bu.signstream.ui.panels.signbank.SignBankDialog;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class AssociatedFieldsPanel extends JPanel implements ItemListener, ActionListener {

	private static final long serialVersionUID = 1L;

	private final JDialog parent;

	private SignBankDialog signBankDialog = null;

	private JTabbedPane tabbedPane;

	private GlossFieldWrapper wrapper = null;
	private HandShapePanel hshPanel = null;

	private LocationPanel locationPanel;

	private final JLabel primaryEntryLb = new JLabel("Primary Entry");
	private final JTextField primaryEntryField = new JTextField();

	private final JLabel variantLb = new JLabel("Entry/Variant");
	private final JTextField variantField = new JTextField();

	private final JButton searchSignBank = new JButton("Search Sign Bank");
	private final JButton addNewSign = new JButton("Add To Sign Bank");

	private final JButton daiVideoSearch = new JButton("DAI Video Search");

	private final JLabel domHandLbl = new JLabel("Dominant Hand");
	private final JRadioButton domHandR = new JRadioButton("Right");
	private final JRadioButton domHandL = new JRadioButton("Left");

	private final JButton enter = new JButton("Enter");
	private final JButton cancel = new JButton("Cancel");
	private final JButton bypass = new JButton("Bypass Sign Bank Check");

	private SignType currentSignType = SignType.NONE;

	private final int ENTER_BYPASS = -1;
	private final int ENTER = 0;
	private final int CANCEL = 1;

	private final int SIGN_BANK = 8;
	private final int ADD_NEW = 9;
	private final int DAI_VIDEO_SEARCH = 10;

	public static final int TAB_HANDSHAPES = 0;
	public static final int TAB_LOCATION = 1;

	private GlossNRelatedView view = null;

	private HandShapePreprocessor processor = null;
	private HandShapeFormBean hshFormBean = null;

	private final LoadedProperties loadedProperties = new LoadedProperties();

	private GlossChainedEventsEntity currentGlossEntity;
	private boolean currentPartipantRightHanded;

	private String dominantHand;

	public AssociatedFieldsPanel(GlossFieldWrapper wrapper, GlossNRelatedView view, JDialog parent) {
		this.wrapper = wrapper;
		this.view = view;
		this.parent = parent;
		this.processor = new HandShapePreprocessor(view.getFieldWrappersCollection());

		enter.setActionCommand(String.valueOf(ENTER));
		cancel.setActionCommand(String.valueOf(CANCEL));
		bypass.setActionCommand(String.valueOf(ENTER_BYPASS));
		addGlossFieldDocumentListener();

		enter.addActionListener(new WeakActionListener(this, enter));
		cancel.addActionListener(new WeakActionListener(this, cancel));
		bypass.addActionListener(new WeakActionListener(this, bypass));

		searchSignBank.addActionListener(this);
		searchSignBank.setActionCommand(String.valueOf(SIGN_BANK));
		SignBankCollection collection = SS3Singleton.getSignStreamApplication().getSignBankCollection();

		addNewSign.addActionListener(this);
		addNewSign.setActionCommand(String.valueOf(ADD_NEW));
		if ( !collection.canAddToSignBank()) {
			addNewSign.setEnabled(false);
		}

		daiVideoSearch.addActionListener(this);
		daiVideoSearch.setActionCommand(String.valueOf(DAI_VIDEO_SEARCH));
		daiVideoSearch.setEnabled(DAISearch.isDAIEnabled());


		ButtonGroup domHandGrp = new ButtonGroup();
		domHandGrp.add(domHandR);
		domHandGrp.add(domHandL);

		domHandR.setEnabled(false);
		domHandL.setEnabled(false);
		
		update(wrapper, view);

		initUI();
	}

	private void addGlossFieldDocumentListener() {
		variantField.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void insertUpdate(DocumentEvent e) {
				changed();
			}

			@Override
			public void removeUpdate(DocumentEvent e) {
				changed();
			}

			@Override
			public void changedUpdate(DocumentEvent e) {
			}

			public void changed() {
				String variantStr = variantField.getText().trim();

				if (variantStr == null || variantStr.isBlank()) {
					enter.setEnabled(false);
				}
				else if (hshPanel != null && (hshPanel.isFingerspelled() || hshPanel.isLoan() || hshPanel.isGesture() || hshPanel.isClassifier()))
					enter.setEnabled(true);
				else {
					SignBankCollection coll = SS3Singleton.getSignStreamApplication().getSignBankCollection();
					enter.setEnabled(coll.contains(variantStr) != null);
				}
			}
		});
	}

	public void showTab(int tabNum) {
		if (tabbedPane != null) {
			tabbedPane.setSelectedIndex(tabNum);
		}
	}

	public JButton getDaiVideoSearch() {
		return this.daiVideoSearch;
	}

	private void populateHandshapeFormBean(GlossChainedEventsEntity entity) {
		// retrieve selected handshapes for right and left hands
		HandShapeEntity hshEntity1 = entity.getDependentHandShapeEntity();
		// 1000, 2000
		SS3CodingScheme dbScheme = entity.getSignStreamSegmentPanel().getSS3Database().getSS3CodingScheme();

		ArrayList<HandShapeImage> selectedHandImagesRight = new ArrayList<HandShapeImage>();
		ArrayList<HandShapeImage> selectedHandImagesLeft = new ArrayList<HandShapeImage>();

		if (hshEntity1 != null) {
			String fID = hshEntity1.getField().getFieldID();
			SS3Field field = dbScheme.getField(fID);
			ArrayList<HandShapeEvent> hShapes = hshEntity1.getHandShapeEvents();

			for (HandShapeEvent event : hShapes) {
				SS3FieldValue value = field.getFieldValue(event.getID());
				HandShapeImage image = new HandShapeImage(value, dbScheme);

				if (image.getSS3FieldValue().getLabel().matches("^[A-Z]$")) { // TODO: Hasnain, ask Greg about this.
					image.getSS3FieldValue().setLetter(image.getSS3FieldValue().getLabel().strip());
				} else if (image.getSS3FieldValue().getLabel().strip().matches("^[A-Z]/[A-Z]$")) {
					String letter = String.valueOf(image.getSS3FieldValue().getLabel().charAt(0));
					image.getSS3FieldValue().setLetter(letter);
				}

				if (currentPartipantRightHanded) {
					selectedHandImagesRight.add(image);
				} else {
					selectedHandImagesLeft.add(image);
				}
			}
		}

		GlossChainedEventsEntity ndEntity = entity.getDependentGlossEntity();

		if (ndEntity != null && ndEntity.getDependentHandShapeEntity() != null) {
			HandShapeEntity hshEntity2 = ndEntity.getDependentHandShapeEntity();
			String ndfID = hshEntity2.getField().getFieldID();

			SS3Field ndfield = dbScheme.getField(ndfID);
			ArrayList<HandShapeEvent> ndHShapes = hshEntity2.getHandShapeEvents();

			for (HandShapeEvent event : ndHShapes) {
				SS3FieldValue value = ndfield.getFieldValue(event.getID());
				HandShapeImage image = new HandShapeImage(value, dbScheme);
				if (currentPartipantRightHanded) {
					selectedHandImagesLeft.add(image);
				} else {
					selectedHandImagesRight.add(image);
				}
			}
		}

		if (wrapper.getGlossField().getFieldID().equals(Util.DOM_GLOSS_FIELD_ID)) {
			if (isDominantHandRight()) {
				hshFormBean.setSelectedDomHandShapeImages(selectedHandImagesRight);
				hshFormBean.setSelectedNdomHandShapeImages(selectedHandImagesLeft);
			} else {
				hshFormBean.setSelectedDomHandShapeImages(selectedHandImagesLeft);
				hshFormBean.setSelectedNdomHandShapeImages(selectedHandImagesRight);
			}
		} else {
			if (isDominantHandRight()) {
				hshFormBean.setSelectedDomHandShapeImages(selectedHandImagesLeft);
				hshFormBean.setSelectedNdomHandShapeImages(selectedHandImagesRight);
			} else {
				hshFormBean.setSelectedDomHandShapeImages(selectedHandImagesRight);
				hshFormBean.setSelectedNdomHandShapeImages(selectedHandImagesLeft);
			}
		}
	}

	protected void update(GlossFieldWrapper wrapper, GlossNRelatedView glsView) {

		this.wrapper = wrapper;
		this.view = glsView;

		loadedProperties.reset();

		EventsEntity ent = SS3Singleton.getMediaToolBar(null).selectedEntity();

		if (ent == null)
			return;

		currentGlossEntity = (GlossChainedEventsEntity) ent;
		PresentationField parentField = currentGlossEntity.getField().getParentField();
		SS3GlossWindowSegment segment = parentField.getSS3GlossWindowSegment();
		currentPartipantRightHanded = segment.getParticipant() == null || segment.getParticipant().isDominantHandRight();

		dominantHand = currentPartipantRightHanded ? "R" : "L";
		if (currentPartipantRightHanded) {
			domHandR.setSelected(true);
		} else {
			domHandL.setSelected(true);
		}

		hshFormBean = new HandShapeFormBean();
		processor.setRightHandDominant(currentPartipantRightHanded);
		processor.populateFormBean(hshFormBean);
		hshFormBean.setGlsFieldWrapper(wrapper);

		populateHandshapeFormBean(currentGlossEntity);

		GlossChainedEvent event = currentGlossEntity.getGlossChainedEvent();

		variantField.setText(event.getText().trim());

		primaryEntryField.setText(currentGlossEntity.getPrimaryEntry());

		String fieldID = currentGlossEntity.getField().getFieldID();

		GlossEntityProperties properties = currentGlossEntity.getGlossEntityProperties();

		ArrayList<HandShapeImage> rhList = new ArrayList<>(hshFormBean.getSelectedDomHandShapeImages());
		ArrayList<HandShapeImage> lhList = new ArrayList<>(hshFormBean.getSelectedNdomHandShapeImages());

		if((rhList.size() > 0 && lhList.size() == 0) || (rhList.size() == 0 && lhList.size() > 0)) {
			properties.setTwoHanded(false);
		}

		if (this.hshPanel == null)
			this.hshPanel = new HandShapePanel(this);

		if (fieldID.equalsIgnoreCase(Util.DOM_GLOSS_FIELD_ID)) {
			// if dominant hand right (right handed signer), check right hand;
			// otherwise check left
			if (currentPartipantRightHanded) {
				properties.setOnehandedSignHand("R");
			} else {
				properties.setOnehandedSignHand("L");
			}
		} else {
			if (currentPartipantRightHanded) {
				properties.setOnehandedSignHand("L");
			} else {
				properties.setOnehandedSignHand("R");
			}
		}

		properties.setOneFrameGloss(currentGlossEntity.getStartTimeInfo().equals(currentGlossEntity.getEndTimeInfo()));

		FieldWrappersCollection collection = view.getFieldWrappersCollection();
		GlossFieldWrapper ndWrp;


		hshPanel.clearSignTypeSelection();
		currentSignType = SignType.NONE;

		if (properties.getSignType() == null) {

			boolean rhEmpty = rhList.isEmpty();
			boolean lhEmpty = lhList.isEmpty();

			if (rhEmpty && !lhEmpty) {
				properties.setTwoHanded(false);
				boolean rIdentical = isIdenticalStartEndHandshapes(lhList);

				if (rIdentical) {
					properties
							.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
				} else {
					properties
							.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
				}

			} else {
				if (!rhEmpty && lhEmpty) {
					properties.setTwoHanded(false);
					boolean lIdentical = isIdenticalStartEndHandshapes(rhList);

					if (lIdentical) {
						properties
								.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
					} else {
						properties
								.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
					}
				} else if (!rhEmpty && !lhEmpty) {
					properties.setTwoHanded(true);

					boolean isIdenticalDomNdom = isIdenticalHandshapes(rhList, lhList);

					if (isIdenticalDomNdom) {
						properties.setTwoHandedHandshapesClassification(GlossEntityProperties.SAME_HANDSHAPES_DOM_NDOM);
						boolean lIdentical = isIdenticalStartEndHandshapes(rhList);

						if (lIdentical) {
							properties.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
						} else {
							properties.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
						}
					} else {
						properties.setTwoHandedHandshapesClassification(GlossEntityProperties.DIFFERENT_HANDSHAPES_DOM_NDOM);

						if (isIdenticalStartEndHandshapes(lhList)) {
							properties
									.setStartEndHandshapeNdomHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
						} else {
							properties
									.setStartEndHandshapeNdomHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
						}

						if (isIdenticalStartEndHandshapes(rhList)) {
							properties
									.setStartEndHandshapeDomHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
						} else {
							properties
									.setStartEndHandshapeDomHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
						}
					}

					properties.setTwoHandedEnabled(true);
				}
			}

			String gls = currentGlossEntity.getText();

			String handednessPattern = "(!)?(neu:|([a-z_1-3][p:]))?((.)([12][h])(.))(alt.)?(.*)?";
			Pattern.compile(handednessPattern);
			Pattern handednessR = Pattern.compile(handednessPattern);
			Matcher handednessMatcher = handednessR.matcher(gls);

			// handedness
			if (handednessMatcher.find()) {

				String oneHandedPattern = "(!)?(neu:)?((1h)){1}(.*)?";
				Pattern oneHandedR = Pattern.compile(oneHandedPattern);
				Matcher oneHandedMatcher = oneHandedR.matcher(gls);

				if (oneHandedMatcher.find()) {
					properties.setTwoHanded(false);

				} else {
					properties.setTwoHanded(true);


					String twoHandedAltPattern = "(!)?(neu:)?((2h).alt)(.*)?";
					Pattern twoHandedAltR = Pattern
							.compile(twoHandedAltPattern);
					Matcher twoHandedAltMatcher = twoHandedAltR
							.matcher(gls);

					if (twoHandedAltMatcher.find()) {
						properties
								.setTwoHandedHandshapesClassification(GlossEntityProperties.ALTERNATING_SAME_HANDSHAPES_DOM_NDOM);
					}
				}
			}

			// name sign
			String nameSignPattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][(2h)][(2h)alt.])?(ns-){1}(.*)?";
			Pattern nameSignR = Pattern.compile(nameSignPattern);
			Matcher nameSignMatcher = nameSignR.matcher(gls);

			hshPanel.setNameSign(nameSignMatcher.find());

			String fingerspelledPattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][(2h)][(2h)alt.])?(fs-){1}(.*)?";
			Pattern fingerspelledR = Pattern.compile(fingerspelledPattern);
			Matcher fingerspelledMatcher = fingerspelledR.matcher(gls);

			if (fingerspelledMatcher.find()) {
				hshPanel.setFingerspelled(true);
				currentSignType = SignType.FINGERSPELLED;
			} else {
				// loan sign
				String loanSignPattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][(2h)][(2h)alt.])?(#){1}(.*)?";
				Pattern loanSignR = Pattern.compile(loanSignPattern);
				Matcher loanSignMatcher = loanSignR.matcher(gls);

				if (loanSignMatcher.find()) {
					hshPanel.setLoan(true);
					currentSignType = SignType.LOAN;
				} else {
					String classifierPattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][`][(2h)alt.])?(ns-)?((CL)|(DCL)|(LCL)|(SCL)|(BCL)|(BPCL)|(PCL)|(ICL)){1}([-_:].*)?$";
					Pattern classifierR = Pattern.compile(classifierPattern);
					Matcher classifierMatcher = classifierR.matcher(gls);

					if (classifierMatcher.find()) {
						hshPanel.setClassifier(true);
						currentSignType = SignType.CLASSIFIER;
					} else {

						String gesturePattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][(2h)][(2h)alt.])?(ns-)?((\"){1}(.){1,}(\"){1})(.*)?";
						Pattern gestureR = Pattern.compile(gesturePattern);
						Matcher gestureMatcher = gestureR.matcher(gls);

						if (gestureMatcher.find()) {
							hshPanel.setGesture(true);
							properties.setGesture();
							currentSignType = SignType.GESTURE;
						} else {
							hshPanel.setLexical(true);
							properties.setLexical();
							currentSignType = SignType.LEXICAL;

							if (fieldID.equalsIgnoreCase(Util.DOM_GLOSS_FIELD_ID)) {
								ndWrp = (GlossFieldWrapper) collection
										.getFieldWrapperWithFieldID(Util.NDOM_GLOSS_FIELD_ID);
							} else {
								ndWrp = (GlossFieldWrapper) collection
										.getFieldWrapperWithFieldID(Util.DOM_GLOSS_FIELD_ID);
							}
						}
					}
				}
			}
		} else {
			hshPanel.setFingerspelled(properties.isFingerspelled());
			hshPanel.setLoan(properties.isLoan());
			hshPanel.setNumber(properties.isNumber());
			hshPanel.setLexical(properties.isLexical());
			hshPanel.setClassifier(properties.isClassifier());
			hshPanel.setGesture(properties.isGesture());
			hshPanel.setNameSign(properties.isNameSign());

			currentSignType = SignType.NONE;
			if (properties.isFingerspelled()) {
				currentSignType = SignType.FINGERSPELLED;
				loadedProperties.setFingerspelled(true);
			} else if (properties.isLoan()) {
				currentSignType = SignType.LOAN;
				loadedProperties.setLoan(true);
			} else if (properties.isLexical()) {
				currentSignType = SignType.LEXICAL;
				loadedProperties.setLexical(true);
			} else if (properties.isClassifier()) {
				currentSignType = SignType.CLASSIFIER;
				loadedProperties.setClassifier(true);
			} else if (properties.isGesture()) {
				currentSignType = SignType.GESTURE;
				loadedProperties.setGesture(true);
			} else if (properties.isNumber()) {
				currentSignType = SignType.NUMBER;
				loadedProperties.setNumber(true);
			}

			if (fieldID.equalsIgnoreCase(Util.DOM_GLOSS_FIELD_ID)) {
				ndWrp = (GlossFieldWrapper) collection
						.getFieldWrapperWithFieldID(Util.NDOM_GLOSS_FIELD_ID);
			} else {
				ndWrp = (GlossFieldWrapper) collection
						.getFieldWrapperWithFieldID(Util.DOM_GLOSS_FIELD_ID);
			}

			NonManualField ndField = (NonManualField) ndWrp.getField();
            boolean isPossibleToAddEvent = ndField.isIntervalEmpty(currentGlossEntity.interval());

			if (!isPossibleToAddEvent) {
				String correspondingID = currentGlossEntity.getCorrespondingGlossEntityID();
				EventsEntity correspondingEntity = ndField
						.getEntity(correspondingID);
				if (correspondingEntity != null) {
					isPossibleToAddEvent = true;
				}
			}

			properties.setTwoHandedEnabled(isPossibleToAddEvent);

		}

		hshFormBean.setGlossEntityProperties(properties);

		if (this.hshPanel != null) {
			this.hshPanel.update();
		}

		checkSignTypeAndUpdateSignBankFunctionality();

		// populate LoadedProperties object
		loadedProperties.setGloss(variantField.getText().trim());

		loadedProperties.setTwoHanded(properties.isTwoHanded());
		loadedProperties.setMarkedNumberOfHands(properties.isMarkedNumberOfHands());
		loadedProperties.setPassiveBaseArm(properties.isPassiveBaseArm());
		loadedProperties.setOneFrameGloss(properties.isOneFrameGloss());

		loadedProperties.setFingerspelled(properties.isFingerspelled());
		loadedProperties.setLoan(properties.isLoan());
		loadedProperties.setNumber(properties.isNumber());
		loadedProperties.setLexical(properties.isLexical());
		loadedProperties.setClassifier(properties.isClassifier());
		loadedProperties.setGesture(properties.isGesture());
		loadedProperties.setNameSign(properties.isNameSign());

		if (hshPanel != null) {
			loadedProperties.setSameDifStartEndHandshapes(hshPanel.getSelectedSameDifStartEndHandshapes());
			loadedProperties.setSameDifferentDomNdomHandshapes(hshPanel.getSelectedSameDifDomNdomHandshapes());
			loadedProperties.setSameDifStartEndHandshapesL(hshPanel.getSelectedSameDifStartEndHandshapesNdom());
			loadedProperties.setSameDifStartEndHandshapesR(hshPanel.getSelectedSameDifStartEndHandshapesDom());
		}

		// hshPanel is for the current gloss, so we need to get the handshapes from the form bean

		loadedProperties.setRightHandSelectedHandshapes(hshFormBean.getSelectedDomHandShapeImages());
		loadedProperties.setLeftHandSelectedHandshapes(hshFormBean.getSelectedNdomHandShapeImages());

		if (locationPanel == null) {
			locationPanel = new LocationPanel(this);
		}

		locationPanel.resetDialogs();
		GlossChainedEventsEntity depGlsEntity = currentGlossEntity.getDependentGlossEntity();
		if (isCurrentGlossDominant() || (!isCurrentGlossDominant() && depGlsEntity == null)) {
			locationPanel.setCurrentLocationInfo(properties.getLocationInfo().cloneInfo());
		} else {
			GlossEntityProperties depProperties = depGlsEntity.getGlossEntityProperties();
			locationPanel.setCurrentLocationInfo(depProperties.getLocationInfo().cloneInfo());
		}

		if (loadedProperties.isLexical() || loadedProperties.isNumber()) {
			SignBankCollection coll = SS3Singleton.getSignStreamApplication().getSignBankCollection();
			enter.setEnabled(coll.contains(loadedProperties.getGloss()) != null);
		}

		if (tabbedPane != null) {
			showTab(TAB_HANDSHAPES);
		}

		// We need to update the handshape panel to reflect the changes in the gloss entity properties
		// This is because the handshape panel is not updated when the gloss entity properties are changed
		// (e.g. when the user changes the sign type)
		if (hshPanel != null) {
			hshPanel.update();
		}


		revalidate();
		parent.pack();
	}

	public HandShapeFormBean getHandShapeFormBean() {
		return hshFormBean;
	}

	private boolean isIdenticalHandshapes(ArrayList<HandShapeImage> rhList,
										  ArrayList<HandShapeImage> lhList) {
		if (rhList.size() == lhList.size()) {
			for (int i = 0; i < rhList.size(); i++) {
				HandShapeImage rImg = rhList.get(i);
				HandShapeImage lImg = lhList.get(i);
				String rId = rImg.getSS3FieldValue().getId();
				String lId = lImg.getSS3FieldValue().getId();

				if (!rId.equalsIgnoreCase(lId)) {
					return false;
				}
			}
			return true;
		} else {
			return false;
		}
	}

	private boolean isIdenticalStartEndHandshapes(ArrayList<HandShapeImage> list) {
		if (list.isEmpty()) {
			return true;
		}

		HandShapeImage fImg = list.get(0);
		HandShapeImage lImg = list.get(list.size() - 1);

		String rId = fImg.getSS3FieldValue().getId();
		String lId = lImg.getSS3FieldValue().getId();

		return rId.equalsIgnoreCase(lId);

	}

	private JPanel createGlossPanel() {
		JPanel panel = new JPanel(new GridBagLayout());

		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		panel.add(primaryEntryLb, c);

		c.gridx++;
		c.weightx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = 2;
		panel.add(primaryEntryField, c);

		c.gridx += 2;
		c.weightx = 0;
		c.fill = GridBagConstraints.NONE;
		c.gridwidth = 1;

		c.gridx = 0;
		c.gridy++;
		c.weightx = 0;
		c.fill = GridBagConstraints.NONE;
		c.gridwidth = 1;
		panel.add(variantLb, c);

		c.gridx++;
		c.weightx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = 2;
		panel.add(variantField, c);

		c.gridx += 2;
		c.weightx = 0;
		c.fill = GridBagConstraints.NONE;
		c.gridwidth = 1;
		panel.add(searchSignBank, c);


		c.gridx = 0;
		c.gridy++;
		panel.add(domHandLbl, c);

		c.gridx++;
		c.insets = new Insets(0, 15, 0, 0);
		panel.add(domHandR, c);

		c.gridx++;
		c.insets = new Insets(0, 0, 0, 30);
		panel.add(domHandL, c);

		c.gridx++;
		c.insets = new Insets(0, 0, 0, 0);
		panel.add(addNewSign, c);

		c.gridy++;
		c.insets = new Insets(0, 0, 0, 0);
		panel.add(daiVideoSearch, c);

		panel.setBorder(new EmptyBorder(5, 5, 5, 5));
		return panel;
	}

	private JPanel createPosPanel() {
		JPanel panel = new JPanel();

		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();

		panel.setLayout(gbl);

		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;

		gbc.anchor = GridBagConstraints.NORTH;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0, 0, 10, 0);

		ButtonPanel buttonPanel = new ButtonPanel();
		buttonPanel.add(cancel);
		buttonPanel.add(enter);
		buttonPanel.add(bypass);

		gbc.anchor = GridBagConstraints.SOUTH;
		gbc.insets = new Insets(15, 0, 0, 0);
		gbc.fill = GridBagConstraints.HORIZONTAL;
		panel.add(buttonPanel, gbc);

		return panel;
	}

	private JPanel createCenterPanel() {
		JPanel panel = new JPanel();

		OrientationPanel oPanel = new OrientationPanel(new OrientationBean());
		MovementPanel mPanel = new MovementPanel(new MovementFormBean());

		tabbedPane = new JTabbedPane();
		tabbedPane.addTab("Sign Type & Handshapes", hshPanel);
		tabbedPane.addTab("Location", locationPanel);
		tabbedPane.addTab("<html><p style='color: gray;'>Orientation</p></html>", oPanel);
		tabbedPane.addTab("<html><p style='color: gray;'>Movement</p></html>", mPanel);

		tabbedPane.setEnabledAt(2, false);
		tabbedPane.setEnabledAt(3, false);

		tabbedPane.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
//				int newTabIndex = ((JTabbedPane)e.getSource()).getSelectedIndex();
//				boolean enable = newTabIndex == 0;
//				addNewSign.setEnabled(enable);
//				enter.setEnabled(enable);

				locationPanel.resetDialogs();
			}
		});

		panel.setLayout(new BorderLayout());

		panel.add(tabbedPane, BorderLayout.CENTER);

		return panel;
	}

	private void initUI() {
		JPanel glsPanel = createGlossPanel();
		JPanel posPanel = createPosPanel();
		JPanel cPanel = createCenterPanel();

		setLayout(new BorderLayout(0, 10));
		add(glsPanel, BorderLayout.NORTH);

		Dimension size = new Dimension(600, 500);
		cPanel.setMinimumSize(size);
		cPanel.setPreferredSize(size);
		add(cPanel, BorderLayout.CENTER);

		add(posPanel, BorderLayout.SOUTH);

//		GridBagConstraints gbc = new GridBagConstraints();
//		setLayout(new GridBagLayout());
//
//		gbc.anchor = GridBagConstraints.NORTHWEST;
//		gbc.gridwidth = GridBagConstraints.REMAINDER;
//		gbc.fill = GridBagConstraints.HORIZONTAL;
//		gbc.insets = new Insets(0, 0, 10, 0);
//		add(glsPanel, gbc);
//
//		gbc.anchor = GridBagConstraints.NORTH;
//		gbc.fill = GridBagConstraints.HORIZONTAL;
//		gbc.insets = new Insets(0, 0, 10, 0);
//		add(new JSeparator(), gbc);
//
//		gbc.anchor = GridBagConstraints.NORTHWEST;
//		gbc.gridwidth = GridBagConstraints.REMAINDER;
//		add(cPanel, gbc);
//
//		gbc.anchor = GridBagConstraints.NORTH;
//		gbc.fill = GridBagConstraints.HORIZONTAL;
//		gbc.insets = new Insets(0, 0, 10, 0);
//		add(posPanel, gbc);
	}

	@Override
	public void itemStateChanged(ItemEvent event) {
		removeAll();
		initUI();
		revalidate();
		parent.pack();
	}

	public boolean isChanged() {

		String glsText = variantField.getText().strip();
		if (!glsText.equalsIgnoreCase(loadedProperties.getGloss())) {
			return true;
		}

		if(hshPanel.isFingerspelled() != loadedProperties.isFingerspelled()) {
			return true;
		}

		if(hshPanel.isLoan() != loadedProperties.isLoan()) {
			return true;
		}

		if(hshPanel.isNumber() != loadedProperties.isNumber()) {
			return true;
		}

		if(hshPanel.isLexical() != loadedProperties.isLexical()) {
			return true;
		}

		if(hshPanel.isClassifier() != loadedProperties.isClassifier()) {
			return true;
		}

		if(hshPanel.isGesture() != loadedProperties.isGesture()) {
			return true;
		}

		if(hshPanel.isNameSign() != loadedProperties.isNameSign()) {
			return true;
		}

		if(hshPanel.isPassiveBaseArm() != loadedProperties.isPassiveBaseArm()) {
			return true;
		}

		if(hshPanel.isMarkedNumberOfHands() != loadedProperties.isMarkedNumberOfHands()) {
			return true;
		}

		if(hshPanel.isTwoHandedSelected() != loadedProperties.isTwoHanded()) {
			return true;
		}

		DomNdomHandshapesType sameDifHandshapes = hshPanel.getSelectedSameDifDomNdomHandshapes();
		StartEndHandshapesType sameDifStartEndHandshapes = hshPanel.getSelectedSameDifStartEndHandshapes();
		StartEndHandshapesType sameDifStartEndHandshapesL = hshPanel.getSelectedSameDifStartEndHandshapesNdom();
		StartEndHandshapesType sameDifStartEndHandshapesR = hshPanel.getSelectedSameDifStartEndHandshapesDom();

		DomNdomHandshapesType loadedSameDifHandshapes = loadedProperties.getSameDifferentDomNdomHandshapes();
		StartEndHandshapesType loadedSameDifStartEndHandshapes = loadedProperties.getSameDifStartEndHandshapes();
		StartEndHandshapesType loadedSameDifStartEndHandshapesR = loadedProperties.getSameDifStartEndHandshapesR();
		StartEndHandshapesType loadedSameDifStartEndHandshapesL = loadedProperties.getSameDifStartEndHandshapesL();


		if(sameDifHandshapes != loadedSameDifHandshapes) {
			return true;
		}

		if(sameDifStartEndHandshapes != loadedSameDifStartEndHandshapes) {
			return true;
		}

		if(sameDifStartEndHandshapesL != loadedSameDifStartEndHandshapesL) {
			return true;
		}

		if(sameDifStartEndHandshapesR != loadedSameDifStartEndHandshapesR) {
			return true;
		}

		ArrayList<HandShapeImage> selLeftHandShapes = hshPanel.getSelectedNdomHandshapesPane().getHandshapes();
		ArrayList<HandShapeImage> selRightHandShapes = hshPanel.getSelectedDomHandshapesPane().getHandshapes();

		ArrayList<HandShapeImage> selLeftHandShapes1 = loadedProperties.getLeftHandSelectedHandshapes();
		ArrayList<HandShapeImage> selRightHandShapes1 = loadedProperties.getRightHandSelectedHandshapes();

		if(selLeftHandShapes.isEmpty() != selLeftHandShapes1.isEmpty()) {
			return true;
		}

		if(selRightHandShapes.isEmpty() != selRightHandShapes1.isEmpty()) {
			return true;
		}

		int rSize1 = selRightHandShapes1.size();
		int rSize = selRightHandShapes.size();

		if(rSize1 != rSize) {
			if(rSize1 == 2 && rSize == 1) {
				HandShapeImage rHandShape11 = selRightHandShapes1.get(0);
				HandShapeImage rHandShape12 = selRightHandShapes1.get(1);
				HandShapeImage rHandShape = selRightHandShapes.get(0);
				String rHandShape11ID = rHandShape11.getSS3FieldValue().getId();
				String rHandShape12ID = rHandShape12.getSS3FieldValue().getId();
				if(!rHandShape11ID.equalsIgnoreCase(rHandShape12ID)) {
					return true;
				}
				String rHandShapeID = rHandShape.getSS3FieldValue().getId();
				if(!rHandShape11ID.equalsIgnoreCase(rHandShapeID)) {
					return true;
				}
			} else {
				return true;
			}
		}

		int i = 0 ;
		for(HandShapeImage rHandShape : selRightHandShapes) {
			HandShapeImage rHandShape1 = selRightHandShapes1.get(i);
			String rhShapeID = rHandShape.getSS3FieldValue().getId();
			String rhShapeID1 = rHandShape1.getSS3FieldValue().getId();
			if(!rhShapeID.equalsIgnoreCase(rhShapeID1)) {
				return true;
			}
			i++;
		}

		int lSize1 = selLeftHandShapes1.size();
		int lSize = selLeftHandShapes.size();

		if(lSize1 != lSize) {
			if(lSize1 == 2 && lSize == 1) {
				HandShapeImage lHandShape11 = selLeftHandShapes1.get(0);
				HandShapeImage lHandShape12 = selLeftHandShapes1.get(1);
				HandShapeImage lHandShape = selLeftHandShapes.get(0);
				String lHandShape11ID = lHandShape11.getSS3FieldValue().getId();
				String lHandShape12ID = lHandShape12.getSS3FieldValue().getId();
				if(!lHandShape11ID.equalsIgnoreCase(lHandShape12ID)) {
					return true;
				}
				String lHandShapeID = lHandShape.getSS3FieldValue().getId();
				if(!lHandShape11ID.equalsIgnoreCase(lHandShapeID)) {
					return true;
				}
			} else {
				return true;
			}
		}

		int j = 0 ;
		for(HandShapeImage lHandShape : selLeftHandShapes) {
			HandShapeImage lHandShape1 = selLeftHandShapes1.get(j);
			String lhShapeID = lHandShape.getSS3FieldValue().getId();
			String lhShapeID1 = lHandShape1.getSS3FieldValue().getId();
			if(!lhShapeID.equalsIgnoreCase(lhShapeID1)) {
				return true;
			}
			j++;
		}

		LocationInfo oldInfo = currentGlossEntity.getGlossEntityProperties().getLocationInfo();
		LocationInfo newInfo = locationPanel.getCurrentLocationInfo();

		if (oldInfo == null && newInfo != null)
			return true;

		return oldInfo != null && newInfo != null && !oldInfo.equals(newInfo);
	}

	public boolean saveChanges() {
		boolean success = updateChanges();
		return success;
	}

	public boolean saveChanges(SignBankVariant variant) {
		MediaToolBar mediaToolBar = SS3Singleton.getMediaToolBar(null);
		GlossChainedEventsEntity sEntity = (GlossChainedEventsEntity) mediaToolBar.selectedEntity();

		if (sEntity == null) {
			sEntity = wrapper.getGlossField()
					.getGlossChainedEventsEntityCollection()
					.getSelectedEntity();
		}

		GlossEntityProperties properties = sEntity.getGlossEntityProperties();

		properties.setVerifiedFromSignBank(true);

		// compare against the defaultOccurrence - from which we get the source resource
		SignBankOccurrence defaultOccurrence = variant.getDefaultOccurrence();
		String sourceResource = defaultOccurrence.getSignBankResourceFileName();
		properties.setSourceResource(sourceResource);

		String mainGlossID = variant.getParentMainGloss().getMainGlossID();
		String variantID = variant.getVariantID();
		String occurrenceID = defaultOccurrence.getOccurrenceID();
		properties.setMainGlossID(mainGlossID);
		properties.setVariantID(variantID);
		properties.setOccurrenceID(occurrenceID);

		ArrayList<HandShapeImage> domImages, nondomImages;
		int domImagesSize, nondomImagesSize;

		// TODO: FIX
		if (isDominantHandRight()) {
			domImages = hshPanel.getSelectedDomHandshapesPane().getHandshapes();
			nondomImages = hshPanel.getSelectedNdomHandshapesPane().getHandshapes();
		} else {
			domImages = hshPanel.getSelectedNdomHandshapesPane().getHandshapes();
			nondomImages = hshPanel.getSelectedDomHandshapesPane().getHandshapes();
		}
		domImagesSize = domImages.size();
		nondomImagesSize = nondomImages.size();

		String domStartHS = null;
		String nondomStartHS = null;
		String domEndHS = null;
		String nondomEndHS = null;

		if (domImagesSize > 0) {
			domStartHS = domImages.get(0).getSS3FieldValue().getLabel();
			domEndHS = domImages.get(domImagesSize - 1).getSS3FieldValue().getLabel();
		}

		if (nondomImagesSize > 0) {
			nondomStartHS = nondomImages.get(0).getSS3FieldValue().getLabel();
			nondomEndHS = nondomImages.get(nondomImagesSize - 1).getSS3FieldValue().getLabel();
		}

		String occDomStartHS = defaultOccurrence.getDomStartHS();
		String occNondomStartHS = defaultOccurrence.getNondomStartHS();
		String occDomEndHS = defaultOccurrence.getDomEndHS();
		String occNondomEndHS = defaultOccurrence.getNondomEndHS();

		String occLeftRightHS = defaultOccurrence.getLeftRightHS();
		String occStartEndHS = defaultOccurrence.getStartEndHS();
		String occStartEndLeftHS = defaultOccurrence.getStartEndLeftHS();
		String occStartEndRightHS = defaultOccurrence.getStartEndRightHS();

		boolean occTwoHanded = defaultOccurrence.isTwoHanded();
		boolean occMarkedHands = defaultOccurrence.isMarkedHands();
		boolean occPassiveBaseArm = defaultOccurrence.isPassiveBaseArm();

		String occSignType = defaultOccurrence.getSignType();

		boolean differingFromOccurrence = occDomStartHS != null && domStartHS != null && !occDomStartHS.equals(domStartHS);

		if (occNondomStartHS != null && nondomStartHS != null && !occNondomStartHS.equals(nondomStartHS))
			differingFromOccurrence = true;
		if (occDomEndHS != null && domEndHS != null && !occDomEndHS.equals(domEndHS))
			differingFromOccurrence = true;
		if (occNondomEndHS != null && nondomEndHS != null && !occNondomEndHS.equals(nondomEndHS))
			differingFromOccurrence = true;

		DomNdomHandshapesType domNdomHS = hshPanel.getSelectedSameDifDomNdomHandshapes();
		StartEndHandshapesType startEndHS = hshPanel.getSelectedSameDifStartEndHandshapes();
		StartEndHandshapesType startEndLeftHS = hshPanel.getSelectedSameDifStartEndHandshapesNdom();
		StartEndHandshapesType startEndRightHS = hshPanel.getSelectedSameDifStartEndHandshapesDom();

		boolean twoHanded = hshPanel.isTwoHandedSelected();
		boolean markedHands = hshPanel.isMarkedNumberOfHands();
		boolean passiveBaseArm = hshPanel.isPassiveBaseArm();

		String signType = null;
		if (hshPanel.isLexical())
			signType = "LEXICAL";
		if (hshPanel.isFingerspelled())
			signType = "FINGERSPELLED";
		if (hshPanel.isLoan())
			signType = "LOAN";
		if (hshPanel.isNumber())
			signType = "NUMBER";
		if (hshPanel.isClassifier())
			signType = "CLASSIFER";
		if (hshPanel.isGesture())
			signType = "GESTURE";

		if (occLeftRightHS != null) {
			if (occLeftRightHS.equals("SAME") && domNdomHS != DomNdomHandshapesType.SAME) {
				differingFromOccurrence = true;

				if (occStartEndHS != null) {
					if (occStartEndHS.equals("SAME") && startEndHS != StartEndHandshapesType.SAME)
						differingFromOccurrence = true;
					else if (occStartEndHS.equals("DIFFERENT") && startEndHS != StartEndHandshapesType.DIFFERENT)
						differingFromOccurrence = true;
				}
			} else if (occLeftRightHS.equals("DIFFERENT") && domNdomHS != DomNdomHandshapesType.DIFFERENT) {
				differingFromOccurrence = true;

				if (occStartEndLeftHS != null) {
					if (occStartEndLeftHS.equals("SAME") && startEndLeftHS != StartEndHandshapesType.SAME)
						differingFromOccurrence = true;
					else if (occStartEndLeftHS.equals("DIFFERENT") && startEndLeftHS != StartEndHandshapesType.DIFFERENT)
						differingFromOccurrence = true;
				}

				if (occStartEndRightHS != null) {
					if (occStartEndRightHS.equals("SAME") && startEndRightHS != StartEndHandshapesType.SAME)
						differingFromOccurrence = true;
					else if (occStartEndRightHS.equals("DIFFERENT") && startEndRightHS != StartEndHandshapesType.DIFFERENT)
						differingFromOccurrence = true;
				}
			}
		}

		if (occTwoHanded != twoHanded)
			differingFromOccurrence = true;

		if (occMarkedHands != markedHands)
			differingFromOccurrence = true;

		if (occPassiveBaseArm != passiveBaseArm)
			differingFromOccurrence = true;

		if ( !occSignType.equals(signType))
			differingFromOccurrence = true;

		properties.setDifferingFromOccurrence(differingFromOccurrence);

		// we need to set the same sign bank properties on the dependent (other hand) gloss entity
		// because on "file open" both hands (both SS3Track objects) will be need to be parsed separately
		// (there could be one handed signs on the nondominant hand - an unlikely but possible use case)
		GlossChainedEventsEntity dependentEntity = sEntity.getDependentGlossEntity();
		if (dependentEntity != null) {
			GlossEntityProperties dependentProperties = dependentEntity.getGlossEntityProperties();
			dependentProperties.setVerifiedFromSignBank(true);
			dependentProperties.setSourceResource(sourceResource);
			dependentProperties.setDifferingFromOccurrence(differingFromOccurrence);
			dependentProperties.setMainGlossID(mainGlossID);
			dependentProperties.setVariantID(variantID);
			dependentProperties.setOccurrenceID(occurrenceID);
		}

		updateChanges();

		return true;
	}

	protected boolean updateChanges() {

		MediaToolBar mediaToolBar = SS3Singleton.getMediaToolBar(null);

		if (currentGlossEntity == null) {
			currentGlossEntity = wrapper.getGlossField()
					.getGlossChainedEventsEntityCollection()
					.getSelectedEntity();
			boolean rightHanded = true;
			var parentEntities = currentGlossEntity.getParentEntities();
			for (PresentationEventsEntity parentEntity: parentEntities) {
				SS3GlossWindowItem windowItem = parentEntity.getSS3GlossWindowItem();
				SS3TemporalPartition partition = currentGlossEntity.getSignStreamSegmentPanel().getSS3Database()
						.getSS3GlossWindow().getTemporalPartitionByLabel(windowItem.getPartitionLabel());
				SS3GlossWindowSegment segment = partition.getTemporalPartitionSegmentByLabel(windowItem.getSegmentLabel());
				if (segment.getParticipant() != null && ! segment.getParticipant().isDominantHandRight()) {
					rightHanded = false;
				}
			}
			currentPartipantRightHanded = rightHanded;
		}

		GlossChainedEventsEntity depGlsEntity = currentGlossEntity.getDependentGlossEntity();
		GlossChainedEvent selectedTextEvent = (GlossChainedEvent) currentGlossEntity.getTextValue();
		GlossClassifications glsClassification = selectedTextEvent.getGlossClassifications();

		GlossEntityProperties properties = currentGlossEntity.getGlossEntityProperties();

		if(hshPanel.isFingerspelled()) {
			glsClassification.setSignClassifications(GlossClassifications.SIGN_FINGERSPELLED);
		} else if(hshPanel.isLoan()) {
			glsClassification.setSignClassifications(GlossClassifications.SIGN_LOAN);
		} else if(hshPanel.isNumber()) {
			glsClassification.setSignClassifications(GlossClassifications.SIGN_NUMBERS);
		} else if(hshPanel.isLexical()) {
			glsClassification.setSignClassifications(GlossClassifications.SIGN_LEXICAL);
		} else if(hshPanel.isClassifier()) {
			glsClassification.setSignClassifications(GlossClassifications.SIGN_CLASSIFIER);
		} else if(hshPanel.isGesture()) {
			glsClassification.setSignClassifications(GlossClassifications.SIGN_GESTURE);
		}

		glsClassification.setNameSign(hshPanel.isNameSign());
		glsClassification.setOneFrameGloss(hshPanel.isOneFrameEntity());
		glsClassification.setPassiveBaseArm(hshPanel.isPassiveBaseArm());
		glsClassification.setMarkedNamberOfHands(hshPanel.isMarkedNumberOfHands());

		properties.setOneFrameGloss(hshPanel.isOneFrameEntity());

		if(hshPanel.isTwoHandedSelected()) {
			glsClassification.setNumberOfHands(2);
			properties.setTwoHanded(true);
		} else {
			glsClassification.setNumberOfHands(1);
			properties.setTwoHanded(false);
		}

		DomNdomHandshapesType handshapeClassification = hshPanel.getSelectedSameDifDomNdomHandshapes();
		boolean sameHandshapes = handshapeClassification == DomNdomHandshapesType.SAME;

		glsClassification.setSameHandshapes(sameHandshapes);

		if(sameHandshapes) {
			StartEndHandshapesType sameDiffStartEndHandshapes = hshPanel.getSelectedSameDifStartEndHandshapes();
			glsClassification.setDomHandShapesSameDIfferent(sameDiffStartEndHandshapes);
			glsClassification.setNondomHandShapesSameDIfferent(sameDiffStartEndHandshapes);
		} else {
			StartEndHandshapesType sameDifStartEndHandshapesL = hshPanel.getSelectedSameDifStartEndHandshapesNdom();
			StartEndHandshapesType sameDifStartEndHandshapesR = hshPanel.getSelectedSameDifStartEndHandshapesDom();

			if(isDominantHandRight()) {
				glsClassification.setDomHandShapesSameDIfferent(sameDifStartEndHandshapesR);
				glsClassification.setNondomHandShapesSameDIfferent(sameDifStartEndHandshapesL);
			} else {
				glsClassification.setDomHandShapesSameDIfferent(sameDifStartEndHandshapesL);
				glsClassification.setNondomHandShapesSameDIfferent(sameDifStartEndHandshapesR);
			}
		}

		String fieldID = null;

		Field field = SS3Singleton.getMediaToolBar(null).selectedField();
		if(field != null) {
			fieldID = field.getFieldID();
		}

		if(fieldID == null) {
			EventsEntity ent = SS3Singleton.getMediaToolBar(null).selectedEntity();

			if (ent == null)
				return false;

			GlossChainedEventsEntity entity = (GlossChainedEventsEntity) ent;
			fieldID = entity.getField().getFieldID();
		}

		LabelObject glossLabelObj = view.getSegment().getRootLabelObject().getGlossLabelObject();

		String newID = fieldID.equalsIgnoreCase(Util.DOM_GLOSS_FIELD_ID) ? Util.NDOM_GLOSS_FIELD_ID : Util.DOM_GLOSS_FIELD_ID;

		GlossFieldWrapper depGlsWrapper = (GlossFieldWrapper) glossLabelObj
				.getChildWithFieldID(newID).getWrapper();

		if (hshPanel.isTwoHandedSelected()) {
			if(depGlsEntity != null) {
				String text = currentGlossEntity.getText();
				depGlsEntity.getTextValue().setText(text);
				depGlsEntity.getGlossEntityProperties().setTwoHanded(true);
			} else {
				GlossField glsField = (GlossField) depGlsWrapper.getField();

				ArrayList<GlossChainedEventsEntity> entities = glsField
						.getEntitiesInInterval(currentGlossEntity.interval(), Bound.EXCLUSIVE);

				if (entities.isEmpty()) {
					GlossChainedEventsCollection chainedEventsCollection = new GlossChainedEventsCollection();

					GlossChainedEvent ndTextEvent = new GlossChainedEvent(
							currentGlossEntity.getSignStreamSegmentPanel());
					ndTextEvent.setStartTimeInfo(selectedTextEvent.getStartTimeInfo());
					ndTextEvent.setEndTimeInfo(selectedTextEvent.getEndTimeInfo());
					ndTextEvent.setText(selectedTextEvent.getText());

					chainedEventsCollection.setTextValueEvent(ndTextEvent);
					depGlsEntity = new GlossChainedEventsEntity(
							chainedEventsCollection,
							currentGlossEntity.getSignStreamSegmentPanel(), glsField);
					depGlsEntity.updateParentEntity();
					depGlsEntity.setCorrespondingGlossEntityID(currentGlossEntity.getID());
					currentGlossEntity.setCorrespondingGlossEntityID(depGlsEntity.getID());
					currentGlossEntity.setDependentGlossEntity(depGlsEntity);
					depGlsEntity.setDependentGlossEntity(currentGlossEntity);
					depGlsEntity.getGlossEntityProperties().setTwoHanded(true);

					currentGlossEntity.setLockedDomNdomAlignments(true);
					depGlsEntity.setLockedDomNdomAlignments(true);
					depGlsWrapper.getGlossField().addGlossChainedEventsEntity(depGlsEntity);

					LocationEntity depLocEntity = currentGlossEntity.getDependentLocationEntity();

					if (depLocEntity != null) {
						LocationFieldWrapper domLocWrapper = (LocationFieldWrapper) glossLabelObj.getChildWithFieldID(Util.DOM_LOCATION_FIELD_ID).getWrapper();
						LocationFieldWrapper ndomLocWrapper = (LocationFieldWrapper) glossLabelObj.getChildWithFieldID(Util.NDOM_LOCATION_FIELD_ID).getWrapper();
						if (!isCurrentGlossDominant()) {
							ndomLocWrapper.getField().deleteEntity(depLocEntity);
							domLocWrapper.getField().addEventsEntity(depLocEntity);

							currentGlossEntity.setDependentLocationEntity(null);
							depGlsEntity.setDependentLocationEntity(depLocEntity);
						}
					}

					SignTypeEntity depSignTypeEnt = currentGlossEntity.getDependentSignTypeEntity();

					if (depSignTypeEnt != null) {
						SignTypeFieldWrapper domSignTypeWrapper = (SignTypeFieldWrapper) glossLabelObj.getChildWithFieldID(Util.DOM_SIGN_TYPE_FIELD_ID).getWrapper();
						SignTypeFieldWrapper ndomSignTypeWrapper = (SignTypeFieldWrapper) glossLabelObj.getChildWithFieldID(Util.NDOM_SIGN_TYPE_FIELD_ID).getWrapper();
						if (!isCurrentGlossDominant()) {
							ndomSignTypeWrapper.getField().deleteEntity(depSignTypeEnt);
							domSignTypeWrapper.getField().addEventsEntity(depSignTypeEnt);

							currentGlossEntity.setDependentSignTypeEntity(null);
							depGlsEntity.setDependentSignTypeEntity(depSignTypeEnt);
						}
					}
				} else {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.CHAINS_CANNOT_OVERLAP);
					return false;
				}
			}
		} else {
			currentGlossEntity.setLockedDomNdomAlignments(false);

			// delete second gloss and related
			if (depGlsEntity != null) {
				depGlsWrapper.getGlossField().deleteEntity(depGlsEntity);

				ChainedEventsEntity depHandEntity = depGlsEntity.getDependentHandEntity();
				HandShapeEntity depHandShapeEntity = depGlsEntity.getDependentHandShapeEntity();

				DomNdomHandFieldWrapper depHandWrapper;

				if(depGlsWrapper.getGlossField().getFieldID().equalsIgnoreCase(Util.DOM_GLOSS_FIELD_ID)) {
					depHandWrapper = (DomNdomHandFieldWrapper) glossLabelObj.getChildWithFieldID(Util.DOM_HAND_FIELD_ID).getWrapper();
				} else {
					depHandWrapper = (DomNdomHandFieldWrapper) glossLabelObj.getChildWithFieldID(Util.NDOM_HAND_FIELD_ID).getWrapper();
				}

				if(depHandEntity != null) {
					NonManualField rl = (NonManualField)depHandWrapper.getField();
					rl.deleteEntity(depHandEntity);
				}

				if(depHandShapeEntity != null) {
					depHandShapeEntity.getField().deleteEntity(depHandShapeEntity);
				}

				LocationEntity depLocEntity = depGlsEntity.getDependentLocationEntity();
				if (depLocEntity != null) {
					LocationFieldWrapper domLocWrapper = (LocationFieldWrapper) glossLabelObj
							.getChildWithFieldID(Util.DOM_LOCATION_FIELD_ID).getWrapper();
					LocationFieldWrapper ndomLocWrapper = (LocationFieldWrapper) glossLabelObj
							.getChildWithFieldID(Util.NDOM_LOCATION_FIELD_ID).getWrapper();
					if (wrapper.getGlossField().getFieldID().equals(Util.NDOM_GLOSS_FIELD_ID)) {

						// GD: copying the above sign type block, but for locations - in case it is ever necessary
						if (domLocWrapper == null || ndomLocWrapper == null) {								
							ArrayList<LabelObject> arr = view.getLabelView().getLabels();
							LabelObject domLocObj = null, ndomLocObj = null;
							for (LabelObject obj: arr) {
								if (obj.getFieldID().equals(Util.DOM_LOCATION_FIELD_ID))
									domLocObj = obj;
								else if (obj.getFieldID().equals(Util.NDOM_LOCATION_FIELD_ID))
									ndomLocObj = obj;
							}

							if (domLocObj != null)
								domLocWrapper = (LocationFieldWrapper) domLocObj.getWrapper();
							if (ndomLocObj != null)
								ndomLocWrapper = (LocationFieldWrapper) ndomLocObj.getWrapper();
						}

						domLocWrapper.getField().deleteEntity(depLocEntity);
						ndomLocWrapper.getField().addEventsEntity(depLocEntity);

						depGlsEntity.setDependentLocationEntity(null);
						currentGlossEntity.setDependentLocationEntity(depLocEntity);
					}
					depLocEntity.setRefGlossEntity(currentGlossEntity);
				}

				SignTypeEntity depSignTypeEntity = depGlsEntity.getDependentSignTypeEntity();
				if (depSignTypeEntity != null) {
					SignTypeFieldWrapper domSignTypeWrapper = (SignTypeFieldWrapper) glossLabelObj
							.getChildWithFieldID(Util.DOM_SIGN_TYPE_FIELD_ID).getWrapper();
					SignTypeFieldWrapper ndomSignTypeWrapper = (SignTypeFieldWrapper) glossLabelObj
							.getChildWithFieldID(Util.NDOM_SIGN_TYPE_FIELD_ID).getWrapper();
					if (wrapper.getGlossField().getFieldID().equals(Util.NDOM_GLOSS_FIELD_ID)) {

						// GD: copying the above sign type block
						if (domSignTypeWrapper == null || ndomSignTypeWrapper == null) {						
							ArrayList<LabelObject> arr = view.getLabelView().getLabels();
							LabelObject domSignTypeObj = null, ndomSignTypeObj = null;
							for (LabelObject obj: arr) {
								if (obj.getFieldID().equals(Util.DOM_SIGN_TYPE_FIELD_ID))
									domSignTypeObj = obj;
								else if (obj.getFieldID().equals(Util.NDOM_SIGN_TYPE_FIELD_ID))
									ndomSignTypeObj = obj;
							}

							if (domSignTypeObj != null)
								domSignTypeWrapper = (SignTypeFieldWrapper) domSignTypeObj.getWrapper();
							if (ndomSignTypeObj != null)
								ndomSignTypeWrapper = (SignTypeFieldWrapper) ndomSignTypeObj.getWrapper();
						}

						domSignTypeWrapper.getField().deleteEntity(depSignTypeEntity);
						ndomSignTypeWrapper.getField().addEventsEntity(depSignTypeEntity);

						depGlsEntity.setDependentSignTypeEntity(null);
						currentGlossEntity.setDependentSignTypeEntity(depSignTypeEntity);
					}
					depSignTypeEntity.setRefGlossEntity(currentGlossEntity);
				}

				depGlsEntity = null;
			}
		}

		GlossEntityProperties properties2 = null;

		if (depGlsEntity != null)
			properties2 = depGlsEntity.getGlossEntityProperties();

		if (hshPanel.isFingerspelled()) {
			properties.setFingerspelled();
			if (properties2 != null)
				properties2.setFingerspelled();
		} else if (hshPanel.isLoan()) {
			properties.setLoan();
			if (properties2 != null)
				properties2.setLoan();
		} else if (hshPanel.isNumber()) {
			properties.setNumber();
			if (properties2 != null)
				properties2.setNumber();
		} else if (hshPanel.isLexical()) {
			properties.setLexical();
			if (properties2 != null)
				properties2.setLexical();
		} else if (hshPanel.isClassifier()) {
			properties.setClassifier();
			if (properties2 != null)
				properties2.setClassifier();
		} else if (hshPanel.isGesture()) {
			properties.setGesture();
			if (properties2 != null)
				properties2.setGesture();
		}

		properties.setNameSign(hshPanel.isNameSign());
		properties.setMarkedNumberOfHands(hshPanel.isMarkedNumberOfHands());
		properties.setPassiveBaseArm(hshPanel.isPassiveBaseArm());

		if (properties2 != null) {
			properties2.setNameSign(hshPanel.isNameSign());
			properties2
					.setMarkedNumberOfHands(hshPanel.isMarkedNumberOfHands());
			properties2.setPassiveBaseArm(hshPanel.isPassiveBaseArm());
		}

		String text = variantField.getText().trim();

		currentGlossEntity.getGlossChainedEvent().setText(text);

		if (depGlsEntity != null) {
			depGlsEntity.getGlossChainedEvent().setText(text);
		}

		String primaryEntry = primaryEntryField.getText().trim();
		currentGlossEntity.setPrimaryEntry(primaryEntry);

		HandShapeFormBean hspBean = hshPanel.getHandShapeBean();

		addOrUpdateHandShapes(hspBean, currentGlossEntity, glossLabelObj);

		if (depGlsEntity != null) {
			depGlsEntity.setPrimaryEntry(primaryEntry);

			Event event = currentGlossEntity.getValueOrTextValueEvent();
			ChainedEvent onset = currentGlossEntity.getOnset();
			ChainedEvent initialHold = currentGlossEntity.getInitialHold();
			ChainedEvent offset = currentGlossEntity.getOffset();
			ChainedEvent finalHold = currentGlossEntity.getFinalHold();

			Event ndEvent = depGlsEntity.getValueOrTextValueEvent();
			ChainedEvent ndOnset = depGlsEntity.getOnset();
			ChainedEvent ndInitialHold = depGlsEntity.getInitialHold();
			ChainedEvent ndOffset = depGlsEntity.getOffset();
			ChainedEvent ndFinalHold = depGlsEntity.getFinalHold();

			if (currentGlossEntity.isLockedDomNdomAlignments() && depGlsEntity.isLockedDomNdomAlignments()) {
				ndEvent.setStartTimeInfo(event.getStartTimeInfo());
				ndEvent.setEndTimeInfo(event.getEndTimeInfo());

				SignStreamSegmentPanel panel = wrapper.getField()
						.getSignStreamSegmentPanel();

				if (initialHold != null) {
					if (ndInitialHold != null) {
						ndInitialHold.setStartTimeInfo(initialHold.getStartTimeInfo());
						ndInitialHold.setEndTimeInfo(initialHold.getEndTimeInfo());
					} else {
						ndInitialHold = new ChainedEvent(panel);
                        ndInitialHold.setStartTimeInfo(initialHold.getStartTimeInfo());
                        ndInitialHold.setEndTimeInfo(initialHold.getEndTimeInfo());
						depGlsEntity.setInitialHoldEvent(ndInitialHold);
					}
				} else {
					depGlsEntity.setInitialHoldEvent(null);
				}

				if (onset != null) {
					if (ndOnset != null) {
						ndOnset.setStartTimeInfo(onset.getStartTimeInfo());
						ndOnset.setEndTimeInfo(onset.getEndTimeInfo());
					} else {
						ndOnset = new ChainedEvent(panel);
                        ndOnset.setStartTimeInfo(onset.getStartTimeInfo());
                        ndOnset.setEndTimeInfo(onset.getEndTimeInfo());
						depGlsEntity.setOnsetEvent(ndOnset);
					}
				} else {
					depGlsEntity.setOnsetEvent(null);
				}

				if (finalHold != null) {
					if (ndFinalHold != null) {
						ndFinalHold.setStartTimeInfo(finalHold.getStartTimeInfo());
						ndFinalHold.setEndTimeInfo(finalHold.getEndTimeInfo());
					} else {
						ndFinalHold = new ChainedEvent(panel);
                        ndFinalHold.setStartTimeInfo(finalHold.getStartTimeInfo());
                        ndFinalHold.setEndTimeInfo(finalHold.getEndTimeInfo());
						depGlsEntity.setFinalHoldEvent(ndFinalHold);
					}
				} else {
					depGlsEntity.setFinalHoldEvent(null);
				}

				if (offset != null) {
					if (ndOffset != null) {
						ndOffset.setStartTimeInfo(offset.getStartTimeInfo());
						ndOffset.setEndTimeInfo(offset.getEndTimeInfo());
					} else {
						ndOffset = new ChainedEvent(panel);
                        ndOffset.setStartTimeInfo(offset.getStartTimeInfo());
                        ndOffset.setEndTimeInfo(offset.getEndTimeInfo());
						depGlsEntity.setOffsetEvent(ndOffset);
					}
				} else {
					depGlsEntity.setOffsetEvent(null);
				}

			}
		}

		properties.clearHandshapeInformation();

		if(hshPanel.isTwoHandedSelected()) {
			properties2.clearHandshapeInformation();
			if(handshapeClassification == DomNdomHandshapesType.SAME) {
				properties.setTwoHandedHandshapesClassification(GlossEntityProperties.SAME_HANDSHAPES_DOM_NDOM);
				properties2.setTwoHandedHandshapesClassification(GlossEntityProperties.SAME_HANDSHAPES_DOM_NDOM);
				StartEndHandshapesType sameDiffHandshapes = hshPanel.getSelectedSameDifStartEndHandshapes();
				if(sameDiffHandshapes == StartEndHandshapesType.SAME) {
					properties.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
					properties2.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
				} else {
					properties.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
					properties2.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
				}
			} else if(handshapeClassification == DomNdomHandshapesType.ALTERNATING) {
				properties.setTwoHandedHandshapesClassification(GlossEntityProperties.ALTERNATING_SAME_HANDSHAPES_DOM_NDOM);
				properties2.setTwoHandedHandshapesClassification(GlossEntityProperties.ALTERNATING_SAME_HANDSHAPES_DOM_NDOM);
				StartEndHandshapesType sameDiffHandshapes = hshPanel.getSelectedSameDifStartEndHandshapes();
				if(sameDiffHandshapes == StartEndHandshapesType.SAME) {
					properties.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
					properties2.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
				} else {
					properties.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
					properties2.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
				}
			} else {
				properties.setTwoHandedHandshapesClassification(GlossEntityProperties.DIFFERENT_HANDSHAPES_DOM_NDOM);
				properties2.setTwoHandedHandshapesClassification(GlossEntityProperties.DIFFERENT_HANDSHAPES_DOM_NDOM);

				StartEndHandshapesType selectedStartEndHandshapeR = hshPanel.getSelectedSameDifStartEndHandshapesDom();
				StartEndHandshapesType selectedStartEndHandshapeL = hshPanel.getSelectedSameDifStartEndHandshapesNdom();

				if(selectedStartEndHandshapeR == StartEndHandshapesType.SAME) {
					properties.setStartEndHandshapeDomHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
					properties2.setStartEndHandshapeDomHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
				} else {
					properties.setStartEndHandshapeDomHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
					properties2.setStartEndHandshapeDomHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
				}

				if(selectedStartEndHandshapeL == StartEndHandshapesType.SAME) {
					properties.setStartEndHandshapeNdomHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
					properties2.setStartEndHandshapeNdomHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
				} else {
					properties.setStartEndHandshapeNdomHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
					properties2.setStartEndHandshapeNdomHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
				}
			}
		} else {
			StartEndHandshapesType sameDiffHandshapes = hshPanel.getSelectedSameDifStartEndHandshapes();
			if(sameDiffHandshapes == StartEndHandshapesType.SAME) {
				properties.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
			} else {
				properties.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
			}
		}

		if(!properties.isTwoHanded()) {
			currentGlossEntity.setDependentGlossEntity(null);
			currentGlossEntity.setCorrespondingGlossEntityID(null);
		}

		if (depGlsEntity != null && depGlsEntity.isLockedGlossPhonAlignments()) { //check if locked to hand shape
			Event valueOrTextEvent = depGlsEntity.getValueOrTextValueEvent();

			Time startTimeInfo = valueOrTextEvent.getStartTimeInfo();
			Time endTimeInfo = valueOrTextEvent.getEndTimeInfo();

			Event initialHoldEvent = depGlsEntity.getInitialHold();
			startTimeInfo = initialHoldEvent != null
					? initialHoldEvent.getStartTimeInfo()
					: valueOrTextEvent.getStartTimeInfo();

			Event finalHoldEvent = depGlsEntity.getFinalHold();
			endTimeInfo = finalHoldEvent != null
					? finalHoldEvent.getEndTimeInfo()
					: valueOrTextEvent.getEndTimeInfo();

			ChainedEventsEntity depHandEntity = depGlsEntity.getDependentHandEntity();
			if (depHandEntity != null) {
				depHandEntity.setStartTimeInfo(startTimeInfo);
				depHandEntity.setEndTimeInfo(endTimeInfo);
			}

			HandShapeEntity depHandShapeEntity = depGlsEntity.getDependentHandShapeEntity();
			if (depHandShapeEntity != null) {
				MediaToolbarCommons.setHandshapeStartTime(depHandShapeEntity, startTimeInfo);
				MediaToolbarCommons.setHandshapeEndTime(depHandShapeEntity, endTimeInfo);
			}
		}

		hshPanel.getSelectedDomHandshapesPane().removeAll();
		hshPanel.getSelectedNdomHandshapesPane().removeAll();

		GlossChainedEventsEntity glossWithLocationAndSignType = null;
		if (isCurrentGlossDominant() || (!isCurrentGlossDominant() && depGlsEntity == null)) {
			properties.setLocationInfo(locationPanel.getCurrentLocationInfo().cloneInfo());
			glossWithLocationAndSignType = currentGlossEntity;
		} else {
			properties2.setLocationInfo(locationPanel.getCurrentLocationInfo().cloneInfo());
			glossWithLocationAndSignType = depGlsEntity;
		}

		addOrUpdateLocations(glossWithLocationAndSignType, glossLabelObj);
		addOrUpdateSignTypes(glossWithLocationAndSignType, glossLabelObj);

		wrapper.getSignStreamSegmentPanel().repaint();
		wrapper.getSignStreamSegmentPanel().getUtteranceView().updateUtteranceLists();
		wrapper.getSignStreamSegmentPanel().getUtteranceView().updateUtteranceIndex();

		// set start/end times to themselves to make sure handshapes won't get misaligned for badly saved glosses
		MediaToolbarCommons.setSelectedEventStartTime(mediaToolBar.selectedEvent().getStartTimeInfo());
		MediaToolbarCommons.setSelectedEventEndTime(mediaToolBar.selectedEvent().getEndTimeInfo());

		return true;
	}

	private void addOrUpdateSignTypes(GlossChainedEventsEntity glossWithLocationAndSignType, LabelObject glossLabelObj) {
		SignTypeEntity dependentSignTypeEntity = glossWithLocationAndSignType.getDependentSignTypeEntity();
		Time startTimeInfo = glossWithLocationAndSignType.getStartTimeInfo();
		Time endTimeInfo = glossWithLocationAndSignType.getEndTimeInfo();

		GlossChainedEventsEntity depGlsEntity = glossWithLocationAndSignType.getDependentGlossEntity();

		SignTypeFieldWrapper signTypeFieldWrapper;
		String signTypeFieldID;
		if (!isCurrentGlossDominant() && depGlsEntity == null) { // one-handed non-dominant gloss
			signTypeFieldID = Util.NDOM_SIGN_TYPE_FIELD_ID;
		} else {
			signTypeFieldID = Util.DOM_SIGN_TYPE_FIELD_ID;
		}
        signTypeFieldWrapper = (SignTypeFieldWrapper) glossLabelObj
				.getChildWithFieldID(signTypeFieldID).getWrapper();

		if (dependentSignTypeEntity != null) {
			dependentSignTypeEntity.getEvent().setSignType(currentSignType);
			dependentSignTypeEntity.getEvent().setNameSign(glossWithLocationAndSignType.getGlossEntityProperties().isNameSign());
			dependentSignTypeEntity.setStartTimeInfo(startTimeInfo);
			dependentSignTypeEntity.setEndTimeInfo(endTimeInfo);
		} else {
			SignTypeEvent event = new SignTypeEvent(
					signTypeFieldWrapper.getSignStreamSegmentPanel(),
					startTimeInfo,
					endTimeInfo
			);
			event.setSignType(currentSignType);
			event.setNameSign(glossWithLocationAndSignType.getGlossEntityProperties().isNameSign());
			dependentSignTypeEntity = new SignTypeEntity(event, signTypeFieldWrapper.getSignStreamSegmentPanel(), signTypeFieldWrapper.getField());
			dependentSignTypeEntity.setID(Util.getLongUniqueNumber());
			signTypeFieldWrapper.addEventsEntity(dependentSignTypeEntity);

			dependentSignTypeEntity.updateParentEntity();
		}

		glossWithLocationAndSignType.setDependentSignTypeEntity(dependentSignTypeEntity);

		dependentSignTypeEntity.setRefGlossEntity(glossWithLocationAndSignType);
		dependentSignTypeEntity.setRefEntityId(glossWithLocationAndSignType.getID());
	}

	private void addOrUpdateLocations(GlossChainedEventsEntity glossWithLocationAndSignType, LabelObject glossLabelObj) {
		SignStreamSegmentPanel segmentPanel = glossWithLocationAndSignType.getSignStreamSegmentPanel();
		LocationEntity depLocEntity = glossWithLocationAndSignType.getDependentLocationEntity();
		LocationInfo newInfo = glossWithLocationAndSignType.getGlossEntityProperties().getLocationInfo();

		GlossChainedEventsEntity depGlsEntity = glossWithLocationAndSignType.getDependentGlossEntity();

		LocationFieldWrapper lfWrapper;
		String locFieldID;
		if (!isCurrentGlossDominant() && depGlsEntity == null) { // one-handed non-dominant gloss
			locFieldID = Util.NDOM_LOCATION_FIELD_ID;
		} else {
			locFieldID = Util.DOM_LOCATION_FIELD_ID;
		}
		lfWrapper = (LocationFieldWrapper) glossLabelObj.getChildWithFieldID(locFieldID).getWrapper();


		if (newInfo.isEmpty()) {
			if (depLocEntity != null) {
                lfWrapper.getField().deleteEntity(depLocEntity);
                glossWithLocationAndSignType.setDependentLocationEntityID(null);
                glossWithLocationAndSignType.setDependentLocationEntity(null);
			}
			return;
		}

		var startUnit = glossWithLocationAndSignType.getStartTimeInfo().unit();
		var endUnit = glossWithLocationAndSignType.getEndTimeInfo().unit();
        Time start = glossWithLocationAndSignType.getStartTimeInfo();
        Time end = glossWithLocationAndSignType.getEndTimeInfo();
        
		Time startTime = glossWithLocationAndSignType.getStartTimeInfo();
        Time endTime = glossWithLocationAndSignType.getEndTimeInfo();

        ArrayList<LocationEvent> eventsList = new ArrayList<LocationEvent>();

        // populates the list of LocationEvents that will get passed to the LocationEntity.
        // sets the start/end times of each LocationEvent relative to what order they appear in on-screen
        ArrayList<LocationType> locationTypes = SS3Singleton.getLocationConfig().getLocationTypes();
        	int size = locationTypes.size();
		Duration dur = end.distance(start.absolute()).divide(size - 1);
		
        for (int i = 0; i < size; i++) {
            LocationType type = locationTypes.get(i);
            Time time;
            if (i == 0) {
                time = startTime;
            } else if (i == locationTypes.size() - 1) {
                time = endTime;
            } else {
				time = start.add(dur.multiply(i));
            }

			var s = time.convert(startUnit);
			var e = time.convert(endUnit);

			LocationEvent event = new LocationEvent(segmentPanel, type, newInfo.getLocation(type));
            event.setStartTimeInfo(s);
            event.setEndTimeInfo(e);

            eventsList.add(event);
        }

        if (depLocEntity != null) {
            depLocEntity.reset();
            depLocEntity.setEvents(eventsList);
        } else {
            LocationEventsCollection newLocEventsColl = new LocationEventsCollection();
            for (LocationEvent locationEvent : eventsList) {
                newLocEventsColl.addEvent(locationEvent);
            }

            depLocEntity = new LocationEntity(Util.getLongUniqueNumber(), newLocEventsColl, segmentPanel, lfWrapper.getField());
            depLocEntity.setStartTimeInfo(newLocEventsColl.getFirstEvent().getStartTimeInfo());
            depLocEntity.setEndTimeInfo(newLocEventsColl.getLastEvent().getEndTimeInfo());
            lfWrapper.addEventsEntity(depLocEntity);

			depLocEntity.updateParentEntity();
        }

        glossWithLocationAndSignType.setDependentLocationEntity(depLocEntity);

        depLocEntity.setRefGlossEntity(glossWithLocationAndSignType);
        depLocEntity.setRefEntityId(glossWithLocationAndSignType.getID());
	}

	private void addOrUpdateHandShapes(HandShapeFormBean bean, GlossChainedEventsEntity entity, LabelObject glossLabelObj) {

		String fieldID = entity.getField().getFieldID();

		// 10000 - right handshapes, 10001 - left handshapes

		ArrayList<HandShapeImage> domHndShapeImgs = new ArrayList<HandShapeImage>(bean.getSelectedDomHandShapeImages());
		ArrayList<HandShapeImage> ndomHndShapeImgs = new ArrayList<HandShapeImage>(bean.getSelectedNdomHandShapeImages());

		ChainedEventsEntity dhEntity, ndhEntity;
		HandShapeEntity dHshEntity, ndHshEntity;

		SignStreamSegmentPanel segmentPanel = entity.getSignStreamSegmentPanel();

		ChainedEventsEntity handEntity = entity.getDependentHandEntity();
		HandShapeEntity handShapeEntity = entity.getDependentHandShapeEntity();
		GlossChainedEventsEntity depGlsEntity = entity.getDependentGlossEntity();
		ChainedEventsEntity depHandEntity = null;
		HandShapeEntity depHandShapeEntity = null;

		if(depGlsEntity != null) {
			depHandEntity = depGlsEntity.getDependentHandEntity();
			depHandShapeEntity = depGlsEntity.getDependentHandShapeEntity();
		}

		DomNdomHandFieldWrapper domHandWrapper = (DomNdomHandFieldWrapper) glossLabelObj
				.getChildWithFieldID(Util.DOM_HAND_FIELD_ID).getWrapper();
		HandShapeFieldWrapper domHandShapeWrapper = (HandShapeFieldWrapper) glossLabelObj
				.getChildWithFieldID(Util.DOM_HANDSHAPE_FIELD_ID).getWrapper();

		DomNdomHandFieldWrapper ndomHandWrapper = (DomNdomHandFieldWrapper) glossLabelObj
				.getChildWithFieldID(Util.NDOM_HAND_FIELD_ID).getWrapper();
		HandShapeFieldWrapper ndomHandShapeWrapper = (HandShapeFieldWrapper) glossLabelObj
				.getChildWithFieldID(Util.NDOM_HANDSHAPE_FIELD_ID).getWrapper();

		if(!domHndShapeImgs.isEmpty()) {
			// add/update handshapes on right hand
			if(fieldID.equals(Util.DOM_GLOSS_FIELD_ID)) {
				if(handEntity != null) {
					handEntity.getTextValue().setText(entity.getText());
				}
				if(handShapeEntity != null) {
					Time startTimeInfo = handShapeEntity.getStartTimeInfo();
					Time endTimeInfo = handShapeEntity.getEndTimeInfo();

					if (entity.isLockedGlossPhonAlignments()) {
						ChainedEvent value = entity.getTextValue();
						ChainedEvent initHold = entity.getInitialHold();
						ChainedEvent finalHold = entity.getFinalHold();

						if (initHold != null) {
							startTimeInfo = initHold.getStartTimeInfo();
						} else {
							startTimeInfo = value.getStartTimeInfo();
						}

						if (finalHold != null) {
							endTimeInfo = finalHold.getEndTimeInfo();
						} else {
							endTimeInfo = value.getEndTimeInfo();
						}
					}

					SS3Entity ent = HandShapeEntity.createSS3Entity(domHndShapeImgs, startTimeInfo, endTimeInfo);
					handShapeEntity.resetEntity(ent);
				} else {
					// add entity to Right Hand field
					dhEntity = addHandEntity(domHandWrapper, entity, segmentPanel);

					dHshEntity = HandShapeEntity.createHandShapeEntity(bean, domHndShapeImgs, dhEntity,
							domHandShapeWrapper.getHandShapeField());

					domHandShapeWrapper.getHandShapeField().addHandShapeEntity(dHshEntity);
					entity.setDependentHandEntity(dhEntity);
					entity.setDependentHandShapeEntity(dHshEntity);

					entity.setLockedGlossPhonAlignments(true);
					dhEntity.setLockedGlossPhonAlignments(true);
					dHshEntity.setLockedGlossPhonAlignments(true);
				}
			} else {
				if(depHandEntity != null) {
					depGlsEntity.getTextValue().setText(entity.getText());
				}
				if(depHandShapeEntity != null) {
					Time startTimeInfo = depHandShapeEntity.getStartTimeInfo();
					Time endTimeInfo = depHandShapeEntity.getEndTimeInfo();

					if (depGlsEntity.isLockedGlossPhonAlignments()) {
						ChainedEvent value = depGlsEntity.getTextValue();
						ChainedEvent initHold = depGlsEntity.getInitialHold();
						ChainedEvent finalHold = depGlsEntity.getFinalHold();

						if (initHold != null) {
							startTimeInfo = initHold.getStartTimeInfo();
						} else {
							startTimeInfo = value.getStartTimeInfo();
						}

						if (finalHold != null) {
							endTimeInfo = finalHold.getEndTimeInfo();
						} else {
							endTimeInfo = value.getEndTimeInfo();
						}
					}

					SS3Entity ent = HandShapeEntity.createSS3Entity(domHndShapeImgs, startTimeInfo, endTimeInfo);
					depHandShapeEntity.resetEntity(ent);
				} else {
					// add entity to Right Hand field
					dhEntity = addHandEntity(domHandWrapper, depGlsEntity, segmentPanel);

					dHshEntity = HandShapeEntity.createHandShapeEntity(bean, domHndShapeImgs, dhEntity,
							domHandShapeWrapper.getHandShapeField());

					domHandShapeWrapper.getHandShapeField().addHandShapeEntity(dHshEntity);
					depGlsEntity.setDependentHandEntity(dhEntity);
					depGlsEntity.setDependentHandShapeEntity(dHshEntity);

					depGlsEntity.setLockedGlossPhonAlignments(true);
					dhEntity.setLockedGlossPhonAlignments(true);
					dHshEntity.setLockedGlossPhonAlignments(true);
				}
			}
		} else {
			// delete handshapes
			if(fieldID.equals(Util.DOM_GLOSS_FIELD_ID)) {
				if(handEntity != null) {
					domHandWrapper.getField().deleteEntity(handEntity);
				}

				if(handShapeEntity != null) {
					domHandShapeWrapper.getHandShapeField().deleteEntity(handShapeEntity);
				}

				entity.setDependentHandEntity(null);
				entity.setDependentHandShapeEntity(null);
			} else {
				if(depHandEntity != null) {
					domHandWrapper.getField().deleteEntity(depHandEntity);
				}
				if(depHandShapeEntity != null) {
					domHandShapeWrapper.getHandShapeField().deleteEntity(depHandShapeEntity);
				}

				if(depGlsEntity != null) {
					depGlsEntity.setDependentHandEntity(null);
					depGlsEntity.setDependentHandShapeEntity(null);
				}
			}
		}

		if(!ndomHndShapeImgs.isEmpty()) {
			// add/update handshapes on right hand
			if(fieldID.equals(Util.NDOM_GLOSS_FIELD_ID)) {
				if(handEntity != null) {
					handEntity.getTextValue().setText(entity.getText());
				}
				if(handShapeEntity != null) {
					Time startTimeInfo = handShapeEntity.getStartTimeInfo();
					Time endTimeInfo = handShapeEntity.getEndTimeInfo();

					if (entity.isLockedGlossPhonAlignments()) {
						ChainedEvent value = entity.getTextValue();
						ChainedEvent initHold = entity.getInitialHold();
						ChainedEvent finalHold = entity.getFinalHold();

						if (initHold != null) {
							startTimeInfo = initHold.getStartTimeInfo();
						} else {
							startTimeInfo = value.getStartTimeInfo();
						}

						if (finalHold != null) {
							endTimeInfo = finalHold.getEndTimeInfo();
						} else {
							endTimeInfo = value.getEndTimeInfo();
						}
					}

					SS3Entity ent = HandShapeEntity.createSS3Entity(ndomHndShapeImgs, startTimeInfo, endTimeInfo);
					handShapeEntity.resetEntity(ent);
				} else {
					// add entity to Right Hand field
					ndhEntity = addHandEntity(ndomHandWrapper, entity, segmentPanel);

					ndHshEntity = HandShapeEntity.createHandShapeEntity(bean, ndomHndShapeImgs, ndhEntity,
							ndomHandShapeWrapper.getHandShapeField());

					ndomHandShapeWrapper.getHandShapeField().addHandShapeEntity(ndHshEntity);
					entity.setDependentHandEntity(ndhEntity);
					entity.setDependentHandShapeEntity(ndHshEntity);

					entity.setLockedGlossPhonAlignments(true);
					ndhEntity.setLockedGlossPhonAlignments(true);
					ndHshEntity.setLockedGlossPhonAlignments(true);
				}
			} else {
				// assert ndEntity != null;
				if(depHandEntity != null) {
					depHandEntity.getTextValue().setText(entity.getText());
				}
				if(depHandShapeEntity != null) {
					Time startTimeInfo = depHandShapeEntity.getStartTimeInfo();
					Time endTimeInfo = depHandShapeEntity.getEndTimeInfo();

					if (depGlsEntity.isLockedGlossPhonAlignments()) {
						ChainedEvent value = depGlsEntity.getTextValue();
						ChainedEvent initHold = depGlsEntity.getInitialHold();
						ChainedEvent finalHold = depGlsEntity.getFinalHold();

						if (initHold != null) {
							startTimeInfo = initHold.getStartTimeInfo();
						} else {
							startTimeInfo = value.getStartTimeInfo();
						}

						if (finalHold != null) {
							endTimeInfo = finalHold.getEndTimeInfo();
						} else {
							endTimeInfo = value.getEndTimeInfo();
						}
					}

					SS3Entity ent = HandShapeEntity.createSS3Entity(ndomHndShapeImgs, startTimeInfo, endTimeInfo);
					depHandShapeEntity.resetEntity(ent);
				} else {
					// add entity to Right Hand field
					ndhEntity = addHandEntity(ndomHandWrapper, depGlsEntity, segmentPanel);

					ndHshEntity = HandShapeEntity.createHandShapeEntity(bean, ndomHndShapeImgs, ndhEntity,
							ndomHandShapeWrapper.getHandShapeField());

					ndomHandShapeWrapper.getHandShapeField().addHandShapeEntity(ndHshEntity);

					if(depGlsEntity != null) {
						depGlsEntity.setDependentHandEntity(ndhEntity);
						depGlsEntity.setDependentHandShapeEntity(ndHshEntity);
						depGlsEntity.setLockedGlossPhonAlignments(true);
					}

					ndhEntity.setLockedGlossPhonAlignments(true);
					ndHshEntity.setLockedGlossPhonAlignments(true);
				}
			}
		} else {
			// delete handshapes
			if(fieldID.equals(Util.NDOM_GLOSS_FIELD_ID)) {
				if(handEntity != null) {
					ndomHandWrapper.getField().deleteEntity(handEntity);
				}

				if(handShapeEntity != null) {
					ndomHandShapeWrapper.getHandShapeField().deleteEntity(handShapeEntity);
				}

				entity.setDependentHandEntity(null);
				entity.setDependentHandShapeEntity(null);
			} else {
				if(depHandEntity != null) {
					ndomHandWrapper.getField().deleteEntity(depHandEntity);
				}
				if(depHandShapeEntity != null) {
					ndomHandShapeWrapper.getHandShapeField().deleteEntity(depHandShapeEntity);
				}

				if(depGlsEntity != null) {
					depGlsEntity.setDependentHandEntity(null);
					depGlsEntity.setDependentHandShapeEntity(null);
				}
			}
		}
	}

	private ChainedEventsEntity addHandEntity(DomNdomHandFieldWrapper wrp,
											  GlossChainedEventsEntity glsEntity, SignStreamSegmentPanel panel) {

		ChainedEvent value = glsEntity.getTextValue();
		ChainedEvent initHold = glsEntity.getInitialHold();
		ChainedEvent finalHold = glsEntity.getFinalHold();

		Time startTimeInfo; // = null;
		Time endTimeInfo; // = null;

		glsEntity.setLockedGlossPhonAlignments(true);

		if (initHold != null) {
			startTimeInfo = initHold.getStartTimeInfo();
		} else {
			startTimeInfo = value.getStartTimeInfo();
		}

		if (finalHold != null) {
			endTimeInfo = finalHold.getEndTimeInfo();
		} else {
			endTimeInfo = value.getEndTimeInfo();
		}

		ChainedEventsCollection collection = new ChainedEventsCollection();

		GlossChainedEvent gEvent = new GlossChainedEvent(panel);
		gEvent.setStartTimeInfo(startTimeInfo);
		gEvent.setEndTimeInfo(endTimeInfo);
		gEvent.setText(value.getText());
		collection.setTextValueEvent(gEvent);

		ChainedEventsEntity handEntity = new ChainedEventsEntity(collection,
				panel, wrp.getField());

		handEntity.setID(Util.getLongUniqueNumber());
		handEntity.setField(wrp.getField());
		handEntity.updateParentEntity();

		wrp.addEventsEntity(handEntity);
		handEntity.setRefEntityId(glsEntity.getID());

		handEntity.setLockedGlossPhonAlignments(true);
		return handEntity;
	}

	public void addToSignBank(String mainGlossStr, String variantStr) {
		SignBankCollection collection = SS3Singleton.getSignStreamApplication().getSignBankCollection();


		// check first if this sign exists in collection already - we will disallow if so
		SignBankVariant matchingVariant = collection.contains(mainGlossStr, variantStr);
		if (matchingVariant != null) {
			String warningDialogText =
					"Sign \""+ variantStr + "\" already found in bank.\nPlease enter a different set of values.";
			String warningTitleText = "Duplicate Sign";
			int optionType = JOptionPane.YES_NO_OPTION;
			int messageType = JOptionPane.ERROR_MESSAGE;
			String[] options = {"OK"};

			try {
				JOptionPane.showOptionDialog(this,
						warningDialogText,
						warningTitleText,
						optionType,
						messageType,
						null,
						options,
						options[0]);
			} catch (HeadlessException he) {
				System.out.println();
			}

			return;
		}

		// next check that there is at least one handshape on the dominant (right) hand before allowing the addition
		ArrayList<HandShapeImage> domHandShapeImages;
		if (isDominantHandRight()) {
			domHandShapeImages = hshPanel.getSelectedDomHandshapesPane().getHandshapes();
		} else {
			domHandShapeImages = hshPanel.getSelectedNdomHandshapesPane().getHandshapes();
		}
		if (domHandShapeImages.size() < 1) {
			String warningDialogText =
					"You must enter at least one handshape for the dominant hand before adding to the Sign Bank.";
			String warningTitleText = "Select Handshapes";
			int optionType = JOptionPane.YES_NO_OPTION;
			int messageType = JOptionPane.ERROR_MESSAGE;
			String[] options = {"OK"};

			try {
				JOptionPane.showOptionDialog(this,
						warningDialogText,
						warningTitleText,
						optionType,
						messageType,
						null,
						options,
						options[0]);
			} catch (HeadlessException he) {
				System.out.println();
			}

			return;
		}

		// and when two-handed is selected, it means you must be intending to add both hands, so check nondominant (left) handshapes as well
		if (hshPanel.isTwoHandedSelected()) {
			ArrayList<HandShapeImage> nondomHandShapeImages;
			if (isDominantHandRight()) {
				nondomHandShapeImages = hshPanel.getSelectedNdomHandshapesPane().getHandshapes();
			} else {
				nondomHandShapeImages = hshPanel.getSelectedDomHandshapesPane().getHandshapes();
			}
			if (nondomHandShapeImages.size() < 1) {
				String warningDialogText =
						"You must enter at least one handshape for the nondominant hand before adding to the Sign Bank.";
				String warningTitleText = "Select Handshapes";
				int optionType = JOptionPane.YES_NO_OPTION;
				int messageType = JOptionPane.ERROR_MESSAGE;
				String[] options = {"OK"};

				try {
					JOptionPane.showOptionDialog(this,
							warningDialogText,
							warningTitleText,
							optionType,
							messageType,
							null,
							options,
							options[0]);
				} catch (HeadlessException he) {
					System.out.println();
				}

				return;
			}
		}

		String resourceFilename = Util.SIGN_BANK_DEFAULT_LOCAL_FILE;

//		SignBankResource resource;
//		String resourceFilename = (String) resourceSelector.getSelectedItem();
//		resource = resourceCollection.getResourceByFileName(resourceFilename);

		String mgID = Util.getLongUniqueNumber();
		String varID = Util.getLongUniqueNumber();
		String occID = Util.getLongUniqueNumber();

		// check whether ID already taken in SB collection - and if so get a new one
		while (collection.getOccurrenceWithID(mgID, varID, occID) != null) {
			occID = Util.getLongUniqueNumber();
		}

		// if there is not an existing main gloss with the label given, then create a new main gloss object to save
		SignBankMainGloss mainGloss = collection.getMainGlossesList().get(mainGlossStr);
		if (mainGloss == null) {
			mainGloss = new SignBankMainGloss(mgID, mainGlossStr.toUpperCase(), varID);
		}
		SignBankVariant newVariant = new SignBankVariant(mainGloss, varID, variantStr, occID, "1");
		SignBankOccurrence newOccurrence = new SignBankOccurrence(newVariant, occID);

		// get data points to set in the occurrence object

		ArrayList<HandShapeImage> domImages, nondomImages;
		int domImagesSize, nondomImagesSize;
		if (isDominantHandRight()) {
			domImages = hshPanel.getSelectedDomHandshapesPane().getHandshapes();
			nondomImages = hshPanel.getSelectedNdomHandshapesPane().getHandshapes();
		} else {
			domImages = hshPanel.getSelectedNdomHandshapesPane().getHandshapes();
			nondomImages = hshPanel.getSelectedDomHandshapesPane().getHandshapes();
		}
		domImagesSize = domImages.size();
		nondomImagesSize = nondomImages.size();

		String domStartHS = null;
		String nondomStartHS = null;
		String domEndHS = null;
		String nondomEndHS = null;

		if (domImagesSize > 0) {
			domStartHS = domImages.get(0).getSS3FieldValue().getLabel();
			domEndHS = domImages.get(domImagesSize - 1).getSS3FieldValue().getLabel();
		}

		if (nondomImagesSize > 0) {
			nondomStartHS = nondomImages.get(0).getSS3FieldValue().getLabel();
			nondomEndHS = nondomImages.get(nondomImagesSize - 1).getSS3FieldValue().getLabel();
		}

		String participant = null;

		String signType = null;
		if (hshPanel.isLexical())
			signType = "LEXICAL";
		if (hshPanel.isFingerspelled())
			signType = "FINGERSPELLED";
		if (hshPanel.isLoan())
			signType = "LOAN";
		if (hshPanel.isNumber())
			signType = "NUMBER";
		if (hshPanel.isClassifier())
			signType = "CLASSIFER";
		if (hshPanel.isGesture())
			signType = "GESTURE";

		String twoHanded = hshPanel.isTwoHandedSelected() ? "T" : "F";

		String passiveBaseArm = hshPanel.isPassiveBaseArm() ? "T" : "F";

		String markedHands = hshPanel.isMarkedNumberOfHands() ? "T" : "F";

		DomNdomHandshapesType domNdomHSIndex = hshPanel.getSelectedSameDifDomNdomHandshapes();
		StartEndHandshapesType startEndHSIndex = hshPanel.getSelectedSameDifStartEndHandshapes();
		StartEndHandshapesType startEndLeftHSIndex = hshPanel.getSelectedSameDifStartEndHandshapesNdom();
		StartEndHandshapesType startEndRightHSIndex = hshPanel.getSelectedSameDifStartEndHandshapesDom();

		String domNdomHS = domNdomHSIndex.shortName();

		String startEndHS = "", startEndLeftHS = "", startEndRightHS = "";

		if (domNdomHSIndex == DomNdomHandshapesType.SAME) {    // if left right is the same
			startEndHS = startEndHSIndex == StartEndHandshapesType.SAME ? "SAME" : "DIFFERENT";
		} else {    // if left right is different
			startEndLeftHS = startEndLeftHSIndex == StartEndHandshapesType.SAME ? "SAME" : "DIFFERENT";
			startEndRightHS = startEndRightHSIndex == StartEndHandshapesType.SAME ? "SAME" : "DIFFERENT";
		}


		newOccurrence.setSignBankResourceFileName(resourceFilename);
		newOccurrence.setDomStartHS(domStartHS);
		newOccurrence.setNondomStartHS(nondomStartHS);
		newOccurrence.setDomEndHS(domEndHS);
		newOccurrence.setNondomEndHS(nondomEndHS);
		newOccurrence.setParticipant(participant);
		newOccurrence.setSignType(signType);
		newOccurrence.setTwoHanded(twoHanded);
		newOccurrence.setPassiveBaseArm(passiveBaseArm);
		newOccurrence.setMarkedHands(markedHands);
		newOccurrence.setLeftRightHS(domNdomHS);
		newOccurrence.setStartEndHS(startEndHS);
		newOccurrence.setStartEndLeftHS(startEndLeftHS);
		newOccurrence.setStartEndRightHS(startEndRightHS);

		newVariant.getOccurrencesList().put(newOccurrence.getOccurrenceID(), newOccurrence);
		mainGloss.getVariantsList().put(newVariant.getVariantLabel(), newVariant);

		// no harm in overwriting the same main gloss with an additional variant
		collection.getMainGlossesList().put(mainGloss.getMainGlossLabel(), mainGloss);

		// write file now
		SS3Singleton.getSignBankWriter().writeResourceToSignBank(collection, resourceFilename);

		// re-parse the entire sign bank collection fresh, application class reference will be set
		SS3Singleton.getSignStreamApplication().parseSignBank();

		// notify the user of a successful addition to the sign bank
		String warningDialogText =
				"Successfully added to sign bank.";
		String warningTitleText = "Added Sign";
		int optionType = JOptionPane.YES_NO_OPTION;
		int messageType = JOptionPane.ERROR_MESSAGE;
		String[] options = {"OK"};

		try {
			JOptionPane.showOptionDialog(this,
					warningDialogText,
					warningTitleText,
					optionType,
					messageType,
					null,
					options,
					options[0]);
		} catch (HeadlessException he) {
			System.out.println();
		}
	}

	/**
	 * To be called by the sign bank panel exclusively.
	 * Take the occurrence object passed in and set the values on this panel accordingly.
	 * @param signBankOccurrence The sign bank occurrence object from the sign bank results list
	 */
	public void populateFromSignBankOccurrence(SignBankOccurrence signBankOccurrence) {
		SignBankVariant parentVariant = signBankOccurrence.getParentVariant();
		SignBankMainGloss parentMainGloss = parentVariant.getParentMainGloss();

		String variantLabel = parentVariant.getVariantLabel().trim();
		String mainGlossLabel = parentMainGloss.getMainGlossLabel().trim();

		String domStartHS = signBankOccurrence.getDomStartHS();
		String nondomStartHS = signBankOccurrence.getNondomStartHS();
		String domEndHS = signBankOccurrence.getDomEndHS();
		String nondomEndHS = signBankOccurrence.getNondomEndHS();
		String signType = signBankOccurrence.getSignType();
		boolean twoHanded = signBankOccurrence.isTwoHanded();
		boolean passiveBaseArm = signBankOccurrence.isPassiveBaseArm();
		boolean markedHands = signBankOccurrence.isMarkedHands();
		String leftRightHS = signBankOccurrence.getLeftRightHS();
		String startEndHS = signBankOccurrence.getStartEndHS();
		String startEndLeftHS = signBankOccurrence.getStartEndLeftHS();
		String startEndRightHS = signBankOccurrence.getStartEndRightHS();


		//Save existing handshapes in case switching is not necessary
		ArrayList<HandShapeImage> existingSelectedHandImagesDom
				= (ArrayList<HandShapeImage>) hshPanel.getHandShapeBean().getSelectedDomHandShapeImages().clone();
		ArrayList<HandShapeImage> existingSelectedHandImagesNdom
				= (ArrayList<HandShapeImage>) hshPanel.getHandShapeBean().getSelectedNdomHandShapeImages().clone();


		//Clear Handshapes to remove any popups about handedness.
		ArrayList<HandShapeImage> selectedHandImagesDom = new ArrayList<HandShapeImage>();
		ArrayList<HandShapeImage> selectedHandImagesNdom = new ArrayList<HandShapeImage>();
		hshPanel.getHandShapeBean().setSelectedDomHandShapeImages(selectedHandImagesDom);
		hshPanel.getHandShapeBean().setSelectedNdomHandShapeImages(selectedHandImagesNdom);
		hshPanel.updateParentInfoSelectedHandshapes(selectedHandImagesDom, selectedHandImagesNdom);
		hshPanel.setNameSign(false);

		variantField.setText(variantLabel);

		primaryEntryField.setText(mainGlossLabel);

		hshPanel.setNameSign(variantLabel.startsWith("ns-"));

		if (signType != null) {
			if (signType.equals("LEXICAL")) {
				hshPanel.setLexical(true);
				hshPanel.onLexicalClassifierSelected();
				setCurrentSignType(SignType.LEXICAL);
			}
			if (signType.equals("FINGERSPELLED")) {
				hshPanel.setFingerspelled(true);
				hshPanel.onFingerspelledLoanNumberSelected();
				setCurrentSignType(SignType.NUMBER);
			}
			if (signType.equals("LOAN")) {
				hshPanel.setLoan(true);
				hshPanel.onFingerspelledLoanNumberSelected();
				setCurrentSignType(SignType.LOAN);
			}
			if (signType.equals("NUMBER")) {
				hshPanel.setNumber(true);
				hshPanel.onFingerspelledLoanNumberSelected();
				setCurrentSignType(SignType.NUMBER);
			}
			if (signType.equals("CLASSIFIER")) {
				hshPanel.setClassifier(true);
				hshPanel.onLexicalClassifierSelected();
				setCurrentSignType(SignType.CLASSIFIER);
			}
			if (signType.equals("GESTURE")) {
				hshPanel.setGesture(true);
				hshPanel.onGestureSelected();
				setCurrentSignType(SignType.GESTURE);
			}
		}

		if (twoHanded) {
			hshPanel.selectTwoHanded();
			DomNdomHandshapesType domNdomType = DomNdomHandshapesType.valueOf(leftRightHS);
			hshPanel.setSelectedSameDifDomNdomHandshapes(domNdomType);
			if (domNdomType == DomNdomHandshapesType.SAME) {
				if (startEndHS != null && !startEndHS.isEmpty()) {
					hshPanel.setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.valueOf(startEndHS));
				} else {
					if (startEndRightHS != null && !startEndRightHS.isEmpty()) {
						hshPanel.setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.valueOf(startEndRightHS));
					} else if (startEndLeftHS != null && !startEndLeftHS.isEmpty()) {
						hshPanel.setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.valueOf(startEndLeftHS));
					}
				}
			} else {
				if (startEndRightHS != null && !startEndRightHS.isEmpty()) {
					hshPanel.setSelectedSameDifStartEndHandshapesDom(StartEndHandshapesType.valueOf(startEndRightHS));
				}
				if (startEndLeftHS != null && !startEndLeftHS.isEmpty()) {
					hshPanel.setSelectedSameDifStartEndHandshapesNdom(StartEndHandshapesType.valueOf(startEndLeftHS));
				}
			}
		} else {
			hshPanel.selectOneHanded();
			if (startEndHS != null && !startEndHS.isEmpty()) {
				hshPanel.setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.valueOf(startEndHS));
			} else {
				if (startEndRightHS != null && !startEndRightHS.isEmpty()) {
					hshPanel.setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.valueOf(startEndRightHS));
				} else if (startEndLeftHS != null && !startEndLeftHS.isEmpty()) {
					hshPanel.setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.valueOf(startEndLeftHS));
				}
			}
		}

		hshPanel.setPassiveBaseArmSelected(passiveBaseArm);

		hshPanel.setMarkedNumberOfHands(markedHands);

		SS3CodingScheme dbScheme = currentGlossEntity.getSignStreamSegmentPanel().getSS3Database().getSS3CodingScheme();

		ArrayList<SS3FieldValue> domFieldValues = dbScheme.getFieldImg(Util.DOM_HANDSHAPE_FIELD_ID);
		ArrayList<SS3FieldValue> ndomFieldValues = dbScheme.getFieldImg(Util.NDOM_HANDSHAPE_FIELD_ID);

		SS3FieldValue domHandStartValue = null, domHandEndValue = null, ndomHandStartValue = null, ndomHandEndValue = null;

		// dom start hs
		domHandStartValue = Util.matchSignBank(domFieldValues, domStartHS);

		if (domHandStartValue != null) {
			HandShapeImage image = new HandShapeImage(domHandStartValue, dbScheme);
			selectedHandImagesDom.add(image);
		}

		// right end hs
		domHandEndValue = Util.matchSignBank(domFieldValues, domEndHS);


		if (domHandEndValue != null) {
			HandShapeImage image = new HandShapeImage(domHandEndValue, dbScheme);
			selectedHandImagesDom.add(image);
		}

		// left start hs
		ndomHandStartValue = Util.matchSignBank(ndomFieldValues, nondomStartHS) ;

		if (ndomHandStartValue != null) {
			HandShapeImage image = new HandShapeImage(ndomHandStartValue, dbScheme);
			selectedHandImagesNdom.add(image);
		}

		// left end hs
		ndomHandEndValue = Util.matchSignBank(ndomFieldValues, nondomEndHS);

		if (ndomHandEndValue != null) {
			HandShapeImage image = new HandShapeImage(ndomHandEndValue, dbScheme);
			selectedHandImagesNdom.add(image);
		}

		// set the arraylists of handshapes
//		if (selectedHandImagesDom.size() > 0 && selectedHandImagesNdom.size() > 0) {
		// whether ndom handshapes are incoming are irrelevant, set the list anyway
		// i.e. if its a one-handed sign in the sign bank, all the handshapes are on the dom
		if (selectedHandImagesDom.size() > 0) {
			hshPanel.getHandShapeBean().setUpdated(true);
			if(hshPanel.isNondomHandSelected()){
				hshPanel.getHandShapeBean().setSelectedDomHandShapeImages(selectedHandImagesNdom);
				hshPanel.getHandShapeBean().setSelectedNdomHandShapeImages(selectedHandImagesDom);
				hshPanel.updateParentInfoSelectedHandshapes(selectedHandImagesNdom, selectedHandImagesDom);
			}else{
				hshPanel.getHandShapeBean().setSelectedDomHandShapeImages(selectedHandImagesDom);
				hshPanel.getHandShapeBean().setSelectedNdomHandShapeImages(selectedHandImagesNdom);
				hshPanel.updateParentInfoSelectedHandshapes(selectedHandImagesDom, selectedHandImagesNdom);
			}
		} else {
			hshPanel.getHandShapeBean().setSelectedDomHandShapeImages(existingSelectedHandImagesDom);
			hshPanel.getHandShapeBean().setSelectedNdomHandShapeImages(existingSelectedHandImagesNdom);
			hshPanel.updateParentInfoSelectedHandshapes(existingSelectedHandImagesDom, existingSelectedHandImagesNdom);
		}
		hshPanel.repaint();

		// Disable/Enable SignBank search buttons if necessary when inserting from SignBank
		checkSignTypeAndUpdateSignBankFunctionality();

		revalidate();
		repaint();
	}

	/**
	 * To be called by the sign bank panel exclusively.
	 * Set the main gloss text and variant text to the labels passed in.
	 * @param mainGlossLabel The main gloss label from the sign bank panel results list
	 * @param variantLabel The variant label from the sign bank panel results list
	 */
	public void populateMainGlossAndVariantLabels(String mainGlossLabel, String variantLabel) {
		if (mainGlossLabel == null || variantLabel == null)
			return;

		primaryEntryField.setText(mainGlossLabel.trim());
		variantField.setText(variantLabel.trim());
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {
			case ENTER_BYPASS:

				if (hshPanel.isTwoHanded()) {
					ArrayList<HandShapeImage> rhImages = hshPanel.getSelectedDomHandshapesPane().getHandshapes();
					ArrayList<HandShapeImage> lhImages = hshPanel.getSelectedNdomHandshapesPane().getHandshapes();

					boolean handSizeNotMatch = rhImages.size() != lhImages.size();

					if (handSizeNotMatch && (rhImages.size() == 0 || lhImages.size() == 0)) {
						// user must select any number of handshape images for two-handed, unless empty
						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GLOSS_EMPTY_HANDSHAPES_ORIENTATIONS);
						break;
					}
				}

				String text = variantField.getText().trim();
				
				// string that either consists entirely of non-space and non-quote characters, 
				// or consists of quoted strings ("[^"]*") and non-space/non-quote characters interleaved
				String quoteSpaceRegex = "^(\"[^\"]*\"|'[^']*'|[^\\s\"'])*$";
				boolean isValid = text.matches(quoteSpaceRegex);

				// check if valid by regex, and then if not blank and not a single quote character (which the regex does not account for)
				if ( (! isValid && ! text.isBlank()) || (text.equals("\"") || text.equals("'"))) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GLOSS_TEXT_NO_SPACES);
					break;
				}


				// GD 2024-07-17: taking this out here and below to allow anything to pass for glosses. Intending to put it back in later, 
				// after we do validation to remove old "legacy" characters as mentioned in https://github.com/DCS-LCSR/SignStream3/issues/677
//				boolean matches = text.matches(Util.LABEL_REGEX_PATTERN);
//				if (!matches) {
//					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.VALUE_MUST_MATCH_CHARACTERS);
//					break;
//				}


				boolean success = saveChanges();
				if (!success)
					return;

				view.update();
				view.getSignStreamSegmentPanel().getUtteranceView().getNonmanualFieldsView().getCSFieldDialog()
                        .getCSFieldPanel().showAllFieldsCurrentUtterance();
				SS3Singleton.getSignStreamApplication().repaint();
				parent.setVisible(false);
				System.gc();
				break;
			case ENTER:

				if (hshPanel.isTwoHanded()) {
					ArrayList<HandShapeImage> rhImages = hshPanel.getSelectedDomHandshapesPane().getHandshapes();
					ArrayList<HandShapeImage> lhImages = hshPanel.getSelectedNdomHandshapesPane().getHandshapes();
					if (rhImages.isEmpty() || lhImages.isEmpty()) {
						// user must select any number of handshape images for two-handed
						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GLOSS_EMPTY_HANDSHAPES_ORIENTATIONS);
						break;
					}
				}

				text = variantField.getText().trim();
				
				// string that either consists entirely of non-space and non-quote characters, 
				// or consists of quoted strings ("[^"]*") and non-space/non-quote characters interleaved
				quoteSpaceRegex = "^(\"[^\"]*\"|[^\\s\"])*$";
				isValid = text.matches(quoteSpaceRegex);

				// check if valid by regex, and then if not blank and not a single quote character (which the regex does not account for)
				if ( ! isValid && ! text.isBlank() && !text.equals("\"")) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GLOSS_TEXT_NO_SPACES);
					break;
				}

//				matches = text.matches(Util.LABEL_REGEX_PATTERN);
//				if (!matches) {
//					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.VALUE_MUST_MATCH_CHARACTERS);
//					break;
//				}


				SignBankCollection collection = null;
				String mainGlossStr = "", trimmedVariantStr = "", originalVariantStr = "";

				// first, before closing out, check that the user didn't edit the labels of main gloss & variant - to ensure that those labels are in the SB
				if (hshPanel.isLexical() || hshPanel.isNumber()) {
					collection = SS3Singleton.getSignStreamApplication().getSignBankCollection();

					mainGlossStr = primaryEntryField.getText().trim();
					originalVariantStr = variantField.getText().trim();
					trimmedVariantStr = Util.trimAsterisksAndPlusSigns(originalVariantStr);
					// check first if this sign DOES NOT exist in collection already - we will disallow if so
					SignBankVariant matchingVariant = collection.contains(trimmedVariantStr);
					SignBankVariant matchingOriginalVariant = collection.contains(originalVariantStr);
					if (matchingVariant == null && matchingOriginalVariant == null) {
						String warningDialogText =
								"Sign was not found in the sign bank.\nPlease either search for an existing sign, or add a new one.";
						String warningTitleText = "Sign Not Found";
						int optionType = JOptionPane.YES_NO_OPTION;
						int messageType = JOptionPane.ERROR_MESSAGE;
						String[] options = {"OK"};

						try {
							JOptionPane.showOptionDialog(this,
									warningDialogText,
									warningTitleText,
									optionType,
									messageType,
									null,
									options,
									options[0]);
						} catch (HeadlessException he) {
							System.out.println();
						}

						return;
					} else if (matchingOriginalVariant == null) {
						//addToSignBank(mainGlossStr, untrimmedVariantStr);

						// reset this variant in the case we just added to sign bank
						// (inside addToSignBank() the sign bank itself got re-parsed into `collection`)
						//matchingUntrimmedVariant = collection.contains(mainGlossStr, untrimmedVariantStr);

						success = saveChanges();
					} else {
						success = saveChanges(matchingOriginalVariant);
					}

				} else {
					success = saveChanges();
				}

				if (!success)
					return;

				view.update();
				view.getSignStreamSegmentPanel().getUtteranceView().getNonmanualFieldsView().getCSFieldDialog()
                        .getCSFieldPanel().showAllFieldsCurrentUtterance();
				SS3Singleton.getSignStreamApplication().repaint();
				parent.setVisible(false);
				System.gc();
				break;
			case CANCEL:
				parent.setVisible(false);
				break;
			case SIGN_BANK:
				collection = SS3Singleton.getSignStreamApplication().getSignBankCollection();
				if ( !collection.canSearchSignBank()) {
					// an error occurred - please fix the error
					String warningDialogText =
							"An error occurred in accessing the sign bank.\n" +
									"Either a sign bank file is missing, or a local file was not found.\n" +
									"Make sure files are named exactly globalSignBank.xml and localSignBank.xml";
					String warningTitleText = "Cannot Access Sign Bank";
					int optionType = JOptionPane.YES_NO_OPTION;
					int messageType = JOptionPane.ERROR_MESSAGE;
					String[] options = {"OK"};

					try {
						JOptionPane.showOptionDialog(this,
								warningDialogText,
								warningTitleText,
								optionType,
								messageType,
								null,
								options,
								options[0]);
					} catch (HeadlessException he) {
						System.out.println();
					}

					return;
				}

				String variantLabel = variantField.getText().trim();

				signBankDialog = new SignBankDialog(this, variantLabel);
				signBankDialog.setLocation(new Point(this.getWidth(), 0));
				signBankDialog.setVisible(true);

				break;
			case ADD_NEW:

				// disallow non-sign-bank signs from being added to the sign bank
				if (hshPanel.isFingerspelled() || hshPanel.isGesture() || hshPanel.isClassifier()) {
					return;
				}

				collection = SS3Singleton.getSignStreamApplication().getSignBankCollection();
				if ( !collection.canSearchSignBank()) {
					// an error occurred - please fix the error
					String warningDialogText =
							"An error occurred in accessing the sign bank.\n" +
									"Either a sign bank file is missing, or a local file was not found.\n" +
									"Make sure files are named exactly globalSignBank.xml and localSignBank.xml";
					String warningTitleText = "Cannot Access Sign Bank";
					int optionType = JOptionPane.YES_NO_OPTION;
					int messageType = JOptionPane.ERROR_MESSAGE;
					String[] options = {"OK"};

					try {
						JOptionPane.showOptionDialog(this,
								warningDialogText,
								warningTitleText,
								optionType,
								messageType,
								null,
								options,
								options[0]);
					} catch (HeadlessException he) {
						System.out.println();
					}

					return;
				}

				mainGlossStr = primaryEntryField.getText().toUpperCase().trim();
				trimmedVariantStr = variantField.getText().toUpperCase().trim();

				// first check empty labels
				if (mainGlossStr == null || mainGlossStr.isEmpty() || trimmedVariantStr == null || trimmedVariantStr.isEmpty()) {

					String warningDialogText =
							"Enter values for both variant and main gloss";
					String warningTitleText = "Enter Values";
					int optionType = JOptionPane.YES_NO_OPTION;
					int messageType = JOptionPane.ERROR_MESSAGE;
					String[] options = {"OK"};

					try {
						JOptionPane.showOptionDialog(this,
								warningDialogText,
								warningTitleText,
								optionType,
								messageType,
								null,
								options,
								options[0]);
					} catch (HeadlessException he) {
						System.out.println();
					}

					primaryEntryField.requestFocus();
					return;
				}
				
				trimmedVariantStr = trimmedVariantStr.replace("+", "");

				// then check to sanitize input text
				// disallow these characters:  "  '  ,  !  <  >
				String regex = ".*[\"',!<>].*";

				if (mainGlossStr.matches(regex) || trimmedVariantStr.matches(regex)) {
					String warningDialogText =
							"Cannot enter these characters for main gloss or variant label:\n" +
									" \"  '  ,  !  <  >";
					String warningTitleText = "Invalid Label";
					int optionType = JOptionPane.YES_NO_OPTION;
					int messageType = JOptionPane.ERROR_MESSAGE;
					String[] options = {"OK"};

					try {
						JOptionPane.showOptionDialog(this,
								warningDialogText,
								warningTitleText,
								optionType,
								messageType,
								null,
								options,
								options[0]);
					} catch (HeadlessException he) {
						System.out.println();
					}

					primaryEntryField.requestFocus();
					return;
				}
				
				addToSignBank(mainGlossStr, trimmedVariantStr);
				
				break;

			case DAI_VIDEO_SEARCH:
				Time startTime = this.hshPanel.getSelectedEntity().getStartTimeInfo().convert(TimeUnit.MILLISECOND, Rounding.NEAREST);
				Time endTime = this.hshPanel.getSelectedEntity().getEndTimeInfo().convert(TimeUnit.MILLISECOND, Rounding.NEAREST);
				// Initialize the DAI search
				// Get all the current segments

				// Get current segment
				Segment segment = this.view.getSegment();
				// segment.getParentField().getSS3GlossWindowSegment()

				// this.view.getSignStreamSegmentPanel().getSS3Database().getSS3GlossWindow().getTemporalPartitions()

				HashMap<String, String> defaultDAIVideoNames = UtteranceView.getDefaultDAIVideoNames();


				DAISearch daiSearch = new DAISearch(startTime, endTime, this, this.parent, segment, defaultDAIVideoNames);
				daiSearch.prepareAndStartDAISearch();
		}

		this.checkSignTypeAndUpdateSignBankFunctionality();
	}

	public void checkSignTypeAndUpdateSignBankFunctionality() {
		if (hshPanel.isLexical() || hshPanel.isNumber() || hshPanel.isLoan())
			enableSignBankFunctionality(true);
		else if (hshPanel.isFingerspelled() || hshPanel.isGesture() || hshPanel.isClassifier())
			enableSignBankFunctionality(false);
	}

	public void enableSignBankFunctionality(boolean enabled) {
		SignBankCollection collection = SS3Singleton.getSignStreamApplication().getSignBankCollection();
		if (collection.canAddToSignBank()) {
			primaryEntryField.setEnabled(enabled);
			addNewSign.setEnabled(enabled);
		}
	}

	public boolean isOneHandedSelected() {
		return hshPanel.isOneHandedSelected();
	}

	public boolean isTwoHandedSelected() {
		return hshPanel.isTwoHandedSelected();
	}

	public boolean isDominantHandRight() {
		return Util.RIGHT_HAND.equals(dominantHand);
	}

	public boolean isCurrentGlossDominant() {
		return wrapper.getGlossField().getFieldID().equals(Util.DOM_GLOSS_FIELD_ID);
	}

	public String getDominantHand() {
		return dominantHand;
	}

	public JDialog getParentDialog() {
		return parent;
	}

	public JButton getEnterButton() {
		return enter;
	}

	public void touchVariantField() {
		variantField.setText(variantField.getText());
	}

	public GlossChainedEventsEntity getCurrentGlossEntity() {
		return currentGlossEntity;
	}

	public SignType getCurrentSignType() {
		return currentSignType;
	}

	public void setCurrentSignType(SignType currentSignType) {
		this.currentSignType = currentSignType;
	}
}

class LoadedProperties {

	private String gloss = "";

	private boolean fingerspelled = false;
	private boolean loan = false;
	private boolean number = false;
	private boolean lexical = false;
	private boolean classifier = false;
	private boolean gesture = false;
	private boolean nameSign = false;
	private boolean twoHanded = false;
	private boolean oneFrameGloss = false;
	private boolean markedNumberOfHands = false;
	private boolean passiveBaseArm = false;

//	private int sameDifHandshapes = 0;
private DomNdomHandshapesType sameDifferentDomNdomHandshapes = DomNdomHandshapesType.SAME;
	private StartEndHandshapesType sameDifStartEndHandshapesL = StartEndHandshapesType.SAME;
	private StartEndHandshapesType sameDifStartEndHandshapesR = StartEndHandshapesType.SAME;
	private StartEndHandshapesType sameDifStartEndHandshapes = StartEndHandshapesType.SAME;

	private final ArrayList<HandShapeImage> rightHandSelectedHandshapes = new ArrayList<HandShapeImage>();
	private final ArrayList<HandShapeImage> leftHandSelectedHandshapes = new ArrayList<HandShapeImage>();

	//	public int getSameDifHandshapes() {
//		return sameDifHandshapes;
//	}
//	public void setSameDifHandshapes(int sameDifHandshapes) {
//		this.sameDifHandshapes = sameDifHandshapes;
//	}
	public DomNdomHandshapesType getSameDifferentDomNdomHandshapes() {
		return sameDifferentDomNdomHandshapes;
	}

	public void setSameDifferentDomNdomHandshapes(
			DomNdomHandshapesType sameDifferentDomNdomHandshapes) {
		this.sameDifferentDomNdomHandshapes = sameDifferentDomNdomHandshapes;
	}

	public void setSameDifStartEndHandshapes(
			StartEndHandshapesType sameDifStartEndHandshapes) {
		this.sameDifStartEndHandshapes = sameDifStartEndHandshapes;
	}

	public StartEndHandshapesType getSameDifStartEndHandshapes() {
		return sameDifStartEndHandshapes;
	}

	public StartEndHandshapesType getSameDifStartEndHandshapesL() {
		return sameDifStartEndHandshapesL;
	}

	public void setSameDifStartEndHandshapesL(StartEndHandshapesType sameDifStartEndHandshapesL) {
		this.sameDifStartEndHandshapesL = sameDifStartEndHandshapesL;
	}

	public StartEndHandshapesType getSameDifStartEndHandshapesR() {
		return sameDifStartEndHandshapesR;
	}

	public void setSameDifStartEndHandshapesR(StartEndHandshapesType sameDifStartEndHandshapesR) {
		this.sameDifStartEndHandshapesR = sameDifStartEndHandshapesR;
	}

	public boolean isFingerspelled() {
		return fingerspelled;
	}

	public void setFingerspelled(boolean fingerspelled) {
		this.fingerspelled = fingerspelled;
	}

	public boolean isLoan() {
		return loan;
	}

	public void setLoan(boolean loan) {
		this.loan = loan;
	}

	public boolean isNumber() {
		return number;
	}

	public void setNumber(boolean number) {
		this.number = number;
	}

	public boolean isLexical() {
		return lexical;
	}

	public void setLexical(boolean lexical) {
		this.lexical = lexical;
	}

	public boolean isClassifier() {
		return classifier;
	}

	public void setClassifier(boolean classifier) {
		this.classifier = classifier;
	}

	public boolean isGesture() {
		return gesture;
	}

	public void setGesture(boolean gesture) {
		this.gesture = gesture;
	}

	public boolean isNameSign() {
		return nameSign;
	}

	public void setNameSign(boolean nameSign) {
		this.nameSign = nameSign;
	}

	public boolean isTwoHanded() {
		return twoHanded;
	}

	public void setTwoHanded(boolean twoHanded) {
		this.twoHanded = twoHanded;
	}

	public boolean isOneFrameGloss() {
		return oneFrameGloss;
	}

	public void setOneFrameGloss(boolean oneFrameGloss) {
		this.oneFrameGloss = oneFrameGloss;
	}

	public boolean isMarkedNumberOfHands() {
		return markedNumberOfHands;
	}

	public void setMarkedNumberOfHands(boolean markedNumberOfHands) {
		this.markedNumberOfHands = markedNumberOfHands;
	}

	public boolean isPassiveBaseArm() {
		return passiveBaseArm;
	}

	public void setPassiveBaseArm(boolean passiveBaseArm) {
		this.passiveBaseArm = passiveBaseArm;
	}

	public ArrayList<HandShapeImage> getRightHandSelectedHandshapes() {
		return rightHandSelectedHandshapes;
	}

	public void setRightHandSelectedHandshapes(
			ArrayList<HandShapeImage> rightHandSelectedHandshapes) {
		this.rightHandSelectedHandshapes.clear();
		this.rightHandSelectedHandshapes.addAll(rightHandSelectedHandshapes);
	}

	public ArrayList<HandShapeImage> getLeftHandSelectedHandshapes() {
		return leftHandSelectedHandshapes;
	}

	public void setLeftHandSelectedHandshapes(
			ArrayList<HandShapeImage> leftHandSelectedHandshapes) {
		this.leftHandSelectedHandshapes.clear();
		this.leftHandSelectedHandshapes.addAll(leftHandSelectedHandshapes);
	}

	public String getGloss() {
		return this.gloss;
	}

	public void setGloss(String gloss) {
		this.gloss = gloss;
	}

	public void reset() {
		this.gloss = "";
		this.fingerspelled = false;
		this.loan = false;
		this.number = false;
		this.lexical = false;
		this.classifier = false;
		this.gesture = false;
		this.nameSign = false;
		this.twoHanded = false;
		this.oneFrameGloss = false;
		this.markedNumberOfHands = false;
		this.passiveBaseArm = false;
//		this.sameDifHandshapes = 0;
		this.sameDifferentDomNdomHandshapes = DomNdomHandshapesType.SAME;
		this.sameDifStartEndHandshapesL = StartEndHandshapesType.SAME;
		this.sameDifStartEndHandshapesR = StartEndHandshapesType.SAME;

		this.rightHandSelectedHandshapes.clear();
		this.leftHandSelectedHandshapes.clear();
	}
}
