package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Locale;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.text.AbstractDocument;
import javax.swing.text.BadLocationException;
import javax.swing.text.DocumentFilter;

import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;

public class LettersHandshapesPalette extends PalettePanel {

	private static final long serialVersionUID = 1L;

	private HashMap<String, String> idNLetter = new HashMap<String, String>();

	private PaletteUtil paletteUtil = null;

	public LettersHandshapesPalette(ArrayList<SS3FieldValue> handshapes, PaletteUtil paletteUtil, ArrayList<HandShapeImage> selectedHshImages) {
		this.paletteUtil = paletteUtil;

		paletteUtil.setLetter(true);

		for (int i = 0; i < handshapes.size(); i++) {
			Object obj = handshapes.get(i);
			SS3FieldValue value = (SS3FieldValue) obj;
			paletteUtil.addLetter(value.getId(), value);
		}

		populate();
		initUI();

		synchronizeHandshapeWithTextfield(selectedHshImages);
	}

	public JTextField getLettersTextField() {
		return this.paletteUtil.getLettersTxtfld();
	}

	public static boolean validConfigurationCheck(char[] textField, int charAt) {
		if (textField == null || textField.length == 0 || charAt < 0 || charAt >= textField.length) {
			throw new IllegalArgumentException("Invalid arguments");
		} else if (charAt == 0) {
			return false;
		}
		boolean check;
		check = (textField[charAt - 1] == '<' && textField[charAt + 1] == '/' && textField[charAt + 3] == '>') || (textField[charAt - 1] == '/' && textField[charAt - 3] == '<' && textField[charAt + 1] == '>');

		return check;
	}


	public void synchronizeHandshapeWithTextfield(ArrayList<HandShapeImage> handshapes) {
		StringBuffer sb = new StringBuffer();
		for(HandShapeImage handshape : handshapes) {
			SS3FieldValue value = handshape.getSS3FieldValue();
			String letter = value.getLetter();
			if (paletteUtil.containsLetter(letter)) {
				if (value.getLabel().contains("/")) {
					sb.append(String.format("<%s>", value.getLabel()));
				} else {
					sb.append(letter);
				}
			}
		}
		paletteUtil.getLettersTxtfld().setText(sb.toString());
	}

	private JPanel handshapesPanel = new JPanel();
	GridBagConstraints c = new GridBagConstraints();
//	JScrollPane scrollPane = new JScrollPane(handshapesPanel);

	private void initUI() {
		this.setLayout(new GridBagLayout());

		this.handshapesPanel.setLayout(new GridBagLayout());

//		JPanel controlsPanel = new JPanel();
//		GridBagLayout gridbag1 = new GridBagLayout();
//		GridBagConstraints c1 = new GridBagConstraints();
//		controlsPanel.setLayout(gridbag1);
//
//		c1.insets = new Insets(1, 1, 1, 1);

		this.populateHandshapesPanel();

		int h = this.paletteUtil.getLettersTxtfld().getPreferredSize().height;
		this.paletteUtil.getLettersTxtfld().setPreferredSize(new Dimension(450, h));


//		scrollPane.setPreferredSize(new Dimension(450, 385 - h));
//		scrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

		this.c.gridwidth = 1;
		this.c.gridy = 1;
		this.c.gridx = 1;
//		add(controlsPanel, c);
//
//		c.gridwidth = 1;
//		c.gridy = 3;
//		c.gridx = 1;
		add(handshapesPanel, c);

		c.gridy = 4;
		c.gridx = 1;
	}

	private void populateHandshapesPanel() {
		handshapesPanel.removeAll();
		handshapesPanel.setBackground(UIPalette.getBackground());

		c.insets = new Insets(0, 0, 0, 0);

		int longestHandshapeeList = sortedHandshapesCollectionLeft.size();
		if (longestHandshapeeList < sortedHandshapesCollectionCenter.size()) {
			longestHandshapeeList = sortedHandshapesCollectionRight.size();
		}

		if (longestHandshapeeList < sortedHandshapesCollectionRight.size()) {
			longestHandshapeeList = sortedHandshapesCollectionRight.size();
		}

		int y = 0;
		int height = 10;
		for (int i = 0; i < longestHandshapeeList; i++) {

			c.gridwidth = 1;
			c.gridy = y;
			c.gridx = 1;
			c.anchor = GridBagConstraints.NORTHEAST;

			if (sortedHandshapesCollectionLeft.size() > i) {
				FieldValueWrapper wrapperLeft = sortedHandshapesCollectionLeft.get(i);
				SS3FieldValue valueLeft = wrapperLeft.getSS3FieldValue();

				JLabel l1 = paletteUtil.getHandshapeVisualRepresentation(valueLeft, wrapperLeft.getBackground());
				handshapesPanel.add(new JLabel(" "), c);

				c.gridx = 2;
				c.weightx = 1;
				c.anchor = GridBagConstraints.NORTHWEST;
				handshapesPanel.add(l1, c);
			}

			c.gridx = 3;

			if (sortedHandshapesCollectionCenter.size() > i) {
				final FieldValueWrapper wrapperCenter = sortedHandshapesCollectionCenter
						.get(i);
				final SS3FieldValue valueCenter = wrapperCenter
						.getSS3FieldValue();

				c.anchor = GridBagConstraints.NORTHEAST;

				JLabel l2 = paletteUtil.getHandshapeVisualRepresentation(valueCenter, wrapperCenter.getBackground());
				handshapesPanel.add(new JLabel(" "), c);

				c.gridx = 4;
				c.weightx = 1;
				c.anchor = GridBagConstraints.NORTHWEST;
				handshapesPanel.add(l2, c);
			}

			c.gridx = 5;

			if (sortedHandshapesCollectionRight.size() > i) {
				final FieldValueWrapper wrapperRight = sortedHandshapesCollectionRight
						.get(i);
				final SS3FieldValue valueRight = wrapperRight
						.getSS3FieldValue();

				c.anchor = GridBagConstraints.NORTHEAST;

				JLabel l3 = paletteUtil.getHandshapeVisualRepresentation(valueRight, wrapperRight.getBackground());
				handshapesPanel.add(new JLabel(" "), c);

				c.gridx = 6;
				c.weightx = 1;
				c.anchor = GridBagConstraints.NORTHWEST;
				handshapesPanel.add(l3, c);
			}
			y++;
		}

		int j = 20 - y;
		for (int k = 0; k < j; k++) {
			c.gridwidth = 1;
			c.gridy = GridBagConstraints.RELATIVE;
			c.gridy = y;
			c.gridx = 2;
			JLabel label = createPlainLabel(height);
			handshapesPanel.add(label, c);
			y++;
		}
	}

	ArrayList<FieldValueWrapper> sortedHandshapesCollectionLeft = new ArrayList<FieldValueWrapper>();
	ArrayList<FieldValueWrapper> sortedHandshapesCollectionCenter = new ArrayList<FieldValueWrapper>();
	ArrayList<FieldValueWrapper> sortedHandshapesCollectionRight = new ArrayList<FieldValueWrapper>();

	private void populate() {
		Color color2 = new Color(255, 255, 255);
		Color color1 = new Color(123, 166, 172);

		FieldValueWrapper aw = createFieldValueWrapper("14", "HSH_014", "A",
				false, false, color1);
		paletteUtil.addLetter("A", aw.getSS3FieldValue());
		idNLetter.put("HSH_014", "A");
		sortedHandshapesCollectionLeft.add(aw);

		FieldValueWrapper bw = createFieldValueWrapper("20", "HSH_020", "B",
				false, false, color2);
		paletteUtil.addLetter("B", bw.getSS3FieldValue());
		idNLetter.put("HSH_020", "B");
		sortedHandshapesCollectionLeft.add(bw);

		FieldValueWrapper cw = createFieldValueWrapper("5", "HSH_005", "C",
				false, false, color1);
		paletteUtil.addLetter("C", cw.getSS3FieldValue());
		idNLetter.put("HSH_005", "C");
		sortedHandshapesCollectionLeft.add(cw);

		FieldValueWrapper dw = createFieldValueWrapper("42", "HSH_042", "D",
				false, false, color2);
		paletteUtil.addLetter("D", dw.getSS3FieldValue());
		idNLetter.put("HSH_042", "D");
		sortedHandshapesCollectionLeft.add(dw);

		FieldValueWrapper ew = createFieldValueWrapper("43", "HSH_043", "E",
				false, false, color1);
		paletteUtil.addLetter("E", ew.getSS3FieldValue());
		idNLetter.put("HSH_043", "E");
		sortedHandshapesCollectionLeft.add(ew);

		FieldValueWrapper fw = createFieldValueWrapper("44", "HSH_044", "F/9", "F",
				false, false, color2);
		paletteUtil.addLetter("F", fw.getSS3FieldValue());
		idNLetter.put("HSH_044", "F");
		sortedHandshapesCollectionLeft.add(fw);

		FieldValueWrapper gw = createFieldValueWrapper("53", "HSH_053", "G/Q", "G",
				false, false, color1);
		paletteUtil.addLetter("G", gw.getSS3FieldValue());
		idNLetter.put("HSH_053", "G");
		sortedHandshapesCollectionLeft.add(gw);

		FieldValueWrapper hw = createFieldValueWrapper("76", "HSH_076", "U/H", "H",
				false, false, color2);
		paletteUtil.addLetter("H", hw.getSS3FieldValue());
		idNLetter.put("HSH_076", "H");
		sortedHandshapesCollectionLeft.add(hw);

		FieldValueWrapper iw = createFieldValueWrapper("57", "HSH_057", "I",
				false, false, color1);
		paletteUtil.addLetter("I", iw.getSS3FieldValue());
		idNLetter.put("HSH_057", "I");
		sortedHandshapesCollectionLeft.add(iw);

		FieldValueWrapper jw = createFieldValueWrapper("97", "HSH_097", "J",
				false, false, color1);
		paletteUtil.addLetter("J", jw.getSS3FieldValue());
		idNLetter.put("HSH_097", "J");
		sortedHandshapesCollectionCenter.add(jw);

		FieldValueWrapper kw = createFieldValueWrapper("58", "HSH_058", "P/K", "K",
				false, false, color2);
		paletteUtil.addLetter("K", kw.getSS3FieldValue());
		idNLetter.put("HSH_058", "K");
		sortedHandshapesCollectionCenter.add(kw);

		FieldValueWrapper lw = createFieldValueWrapper("60", "HSH_060", "L",
				false, false, color1);
		paletteUtil.addLetter("L", lw.getSS3FieldValue());
		idNLetter.put("HSH_060", "L");
		sortedHandshapesCollectionCenter.add(lw);

		FieldValueWrapper mw = createFieldValueWrapper("62", "HSH_062", "M",
				false, false, color2);
		paletteUtil.addLetter("M", mw.getSS3FieldValue());
		idNLetter.put("HSH_062", "M");
		sortedHandshapesCollectionCenter.add(mw);

		FieldValueWrapper nw = createFieldValueWrapper("63", "HSH_063", "N",
				false, false, color2);
		paletteUtil.addLetter("N", nw.getSS3FieldValue());
		idNLetter.put("HSH_063", "N");
		sortedHandshapesCollectionCenter.add(nw);

		FieldValueWrapper ow = createFieldValueWrapper("65", "HSH_065", "O",
				false, false, color2);
		paletteUtil.addLetter("O", ow.getSS3FieldValue());
		idNLetter.put("HSH_065", "O");
		sortedHandshapesCollectionCenter.add(ow);

		FieldValueWrapper pw = createFieldValueWrapper("58", "HSH_058", "P/K", "P",
				false, false, color1);
		paletteUtil.addLetter("P", pw.getSS3FieldValue());
		idNLetter.put("HSH_058", "P");
		sortedHandshapesCollectionCenter.add(pw);

		FieldValueWrapper qw = createFieldValueWrapper("53", "HSH_053", "G/Q", "Q",
				false, false, color2);
		paletteUtil.addLetter("Q", qw.getSS3FieldValue());
		idNLetter.put("HSH_053", "Q");
		sortedHandshapesCollectionCenter.add(qw);

		FieldValueWrapper rw = createFieldValueWrapper("70", "HSH_070", "R",
				false, false, color1);
		paletteUtil.addLetter("R", rw.getSS3FieldValue());
		idNLetter.put("HSH_070", "R");
		sortedHandshapesCollectionCenter.add(rw);

		FieldValueWrapper sw = createFieldValueWrapper("82", "HSH_082", "S",
				false, false, color1);
		paletteUtil.addLetter("S", sw.getSS3FieldValue());
		idNLetter.put("HSH_082", "S");
		sortedHandshapesCollectionRight.add(sw);

		FieldValueWrapper tw = createFieldValueWrapper("72", "HSH_072", "T",
				false, false, color2);
		paletteUtil.addLetter("T", tw.getSS3FieldValue());
		idNLetter.put("HSH_072", "T");
		sortedHandshapesCollectionRight.add(tw);

		FieldValueWrapper uw = createFieldValueWrapper("76", "HSH_076", "U/H", "U",
				false, false, color1);
		paletteUtil.addLetter("U", uw.getSS3FieldValue());
		idNLetter.put("HSH_076", "U");
		sortedHandshapesCollectionRight.add(uw);

		FieldValueWrapper vw = createFieldValueWrapper("77", "HSH_077", "V/2", "V",
				false, false, color2);
		paletteUtil.addLetter("V", vw.getSS3FieldValue());
		idNLetter.put("HSH_077", "V");
		sortedHandshapesCollectionRight.add(vw);

		FieldValueWrapper ww = createFieldValueWrapper("78", "HSH_078", "W",
				false, false, color1);
		paletteUtil.addLetter("W", ww.getSS3FieldValue());
		idNLetter.put("HSH_078", "W");
		sortedHandshapesCollectionRight.add(ww);

		FieldValueWrapper xw = createFieldValueWrapper("79", "HSH_079", "X",
				false, false, color2);
		paletteUtil.addLetter("X", xw.getSS3FieldValue());
		idNLetter.put("HSH_079", "X");
		sortedHandshapesCollectionRight.add(xw);

		FieldValueWrapper yw = createFieldValueWrapper("81", "HSH_081", "Y",
				false, false, color1);
		paletteUtil.addLetter("Y", yw.getSS3FieldValue());
		idNLetter.put("HSH_081", "Y");
		sortedHandshapesCollectionRight.add(yw);

		FieldValueWrapper zw = createFieldValueWrapper("96", "HSH_096", "Z",
				false, false, color2);
		paletteUtil.addLetter("Z", zw.getSS3FieldValue());
		idNLetter.put("HSH_096", "Z");
		sortedHandshapesCollectionRight.add(zw);
	}

	public void clearEnteredIcons() {
		paletteUtil.getLettersTxtfld().setText("");
	}
}

class LetterTypedListener extends KeyAdapter {

	private PaletteUtil paletteUtil = null;
	private boolean deleteExtraChar = false;

	public LetterTypedListener(PaletteUtil paletteUtil) {
		this.paletteUtil = paletteUtil;
	}

/*	@Override
	public void keyPressed(KeyEvent e) {
		JTextField tf = (JTextField) e.getSource();

		if (e.getKeyCode() == KeyEvent.VK_BACK_SPACE) {
			String input = tf.getText().strip().toUpperCase(Locale.ROOT);
			int caretPosition = tf.getCaretPosition();

			// Ensure we only check within string bounds
			try {
				if (caretPosition > 0) {
					String t = input.substring(0, caretPosition - 1) + input.substring(caretPosition + 3);
					if (input.startsWith("<", caretPosition - 1) && caretPosition + 3 <= input.length() &&
							input.substring(caretPosition - 1, caretPosition + 3).matches("<./.>")) {
						tf.setText(t);
						tf.setCaretPosition(caretPosition - 1);
						return;
					} else if (caretPosition >= 2 && input.startsWith("/", caretPosition - 2) &&
							caretPosition + 2 <= input.length() && input.substring(caretPosition - 2, caretPosition + 2).matches("./.>")) {
						tf.setText(input.substring(0, caretPosition - 2) + input.substring(caretPosition + 2));
						tf.setCaretPosition(caretPosition - 2);
						return;
					} else if (input.startsWith("/", caretPosition - 1) && caretPosition + 3 <= input.length() &&
							input.substring(caretPosition - 1, caretPosition + 3).matches("/./.>")) {
						tf.setText(t);
						tf.setCaretPosition(caretPosition - 1);
						return;
					} else if (caretPosition + 4 <= input.length() && input.startsWith("<", caretPosition) && input.substring(caretPosition - 1, caretPosition + 4).matches(".<./>")) {
						tf.setText(input.substring(0, caretPosition - 1) + input.substring(caretPosition + 4));
						tf.setCaretPosition(caretPosition - 1);
					} else if (caretPosition >= 5 && input.startsWith(">", caretPosition - 1) &&
							input.substring(caretPosition - 5, caretPosition).matches("<./.>")) {
						tf.setText(input.substring(0, caretPosition - 5) + input.substring(caretPosition));
						tf.setCaretPosition(caretPosition - 5);
						return;
					} else {
						// Remove the character at caretPosition - 1 as usual
						tf.setText(input.substring(0, caretPosition - 1) + input.substring(caretPosition));
						tf.setCaretPosition(caretPosition);
						return;
					}
				}
			} catch (ArrayIndexOutOfBoundsException | StringIndexOutOfBoundsException ex) {
				tf.setText(input.substring(0, caretPosition - 1) + input.substring(caretPosition));
				tf.setCaretPosition(caretPosition);
				return;
			}

			tf.setText(input.substring(0, caretPosition - 1) + input.substring(caretPosition));
			tf.setCaretPosition(caretPosition);
			return;
		}
	}*/ // This code is almost working and it can work, just was not allowed to spend more time debugging it
	// If you have the time, it would be nice if you could fix it

	@Override
	public void keyPressed(KeyEvent e) {
		JTextField textField = (JTextField) e.getSource();

		if (Character.isLetter(e.getKeyChar())) {
			// Convert the input to upper case
			char letter = Character.toUpperCase(e.getKeyChar());
			SS3FieldValue fieldValue = this.paletteUtil.getFieldValueForLetter(String.valueOf(letter));

			if (fieldValue == null) {
				return;
			}

			int caretPosition = textField.getCaretPosition();
			StringBuilder currentText = new StringBuilder(textField.getText().substring(0, caretPosition).toUpperCase(Locale.ROOT));

			if (fieldValue.getLabel().contains("/")) {
				// Remove the filter from the text field
				((AbstractDocument) textField.getDocument()).setDocumentFilter(null);

				currentText.append(String.format("<%s>", fieldValue.getLabel()));
				textField.setText(currentText.toString());
				// Add the filter back to the text field
				((AbstractDocument) textField.getDocument()).setDocumentFilter(new NonNumericFilter());
				this.deleteExtraChar = true;
			} else {
				this.deleteExtraChar = false;
			}
			e.consume();
		} else if (e.getKeyCode() == KeyEvent.VK_LEFT || e.getKeyCode() == KeyEvent.VK_RIGHT) {
			e.consume();  // ignore the event
		} else if (e.getKeyCode() == KeyEvent.VK_BACK_SPACE) {
			try {
				int caretPosition = textField.getCaretPosition();
				String textFieldText = textField.getText().toUpperCase(Locale.ROOT);
				char characterToBeRemoved = textFieldText.charAt(caretPosition - 1);

				if (characterToBeRemoved == '>') {
					// Remove the '>' character and the four characters before it too
					String newText = textFieldText.substring(0, caretPosition - 5) + textFieldText.substring(caretPosition);
					// Remove document filter
					((AbstractDocument) textField.getDocument()).setDocumentFilter(null);
					textField.setText(newText);
					((AbstractDocument) textField.getDocument()).setDocumentFilter(new NonNumericFilter());
					e.consume();
				}
			} catch (StringIndexOutOfBoundsException ex) {
				// Do nothing
			}
			// Else if the key pressed is a letter, then we do nothing
		}
	}

	@Override
	public void keyTyped(KeyEvent e) {
		if (this.deleteExtraChar) {
			e.setKeyChar('\0');
		}
	}


	@Override
	public void keyReleased(KeyEvent e) {
		JTextField tf = (JTextField) e.getSource();

		this.paletteUtil.getHandShapePalletClient().removeAll();
		char[] c = tf.getText().toUpperCase().toCharArray();
		for (int i = 0; i < c.length; i++) {
			char cc = c[i];
			if (Character.isLetter(cc)) {
				String s = String.valueOf(cc);
				SS3FieldValue obj = this.paletteUtil.getFieldValueForLetter(s);

				this.paletteUtil.getHandShapePalletClient().setItem(obj);
			} else if (c[i] == '<') {
				SS3FieldValue obj;
				try {
					obj = this.paletteUtil.getFieldValueForLetter(String.valueOf(c[i + 1]));
				} catch (ArrayIndexOutOfBoundsException e1) {
					continue;
				}

				if (obj != null) {
					this.paletteUtil.getHandShapePalletClient().setItem(obj);
				}

				i += 4;
			}
		}
	}
}

class NonNumericFilter extends DocumentFilter {

	@Override
	public void insertString(DocumentFilter.FilterBypass fb, int offset,
			String text, javax.swing.text.AttributeSet attr)
			throws BadLocationException {

		fb.insertString(offset, text.toUpperCase(Locale.ROOT).replaceAll("\\d", ""), attr);
	}

	@Override
	public void replace(DocumentFilter.FilterBypass fb, int offset, int length,
			String text, javax.swing.text.AttributeSet attr)
			throws BadLocationException {

		if (text.matches("^[a-zA-Z]*$")) {
			fb.replace(offset, length, text.toUpperCase(Locale.ROOT).replaceAll("\\d", ""), attr);
		}
	}
}
