package edu.bu.signstream.grepresentation.fields.movementField;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.CoordinateTimeConverter;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class MovementEntity extends EventsEntity {
	private MovementEventsCollection events = new MovementEventsCollection(segmentPanel);

	public MovementEntity(MovementEventsCollection events, SignStreamSegmentPanel segmentPanel, Field field) {
		super(segmentPanel, field);
		this.events = events;
	}



	@Override
	public SS3Entity getSS3Entity() {
		SS3Entity entity = new SS3Entity();

		ArrayList col = events.getEvents();
		for (int i = 0; i < col.size(); i++) {
			ChainedEvent event = (ChainedEvent) col.get(i);
			if (event != null) {
				SS3Item item = new SS3Item();
				item.setId(event.getID());
				item.setStartTimeInfo(event.getStartTimeInfo());
				item.setEndTimeInfo(event.getEndTimeInfo());
				item.setText(event.getText());
				entity.addItem(item);
			}
		}
		return entity;
	}

	public void addMovementEvent(ChainedEvent event) {
		events.addEvent(event);
	}

	public void setStartTime(int startTime) {
		ChainedEvent event = events.getFirstEvent();
		event.setStartTimeInfo(SS3Singleton.getVideoTimeRange().time(startTime, TimeUnit.TIMESTAMP));
	}

	public void setEndTime(int endTime) {
		ChainedEvent event = events.getLastEvent();

		if (endTime <= event.getStartTimeInfo().doubleValue()) {
			events.remove(event);
		}
        event.setEndTimeInfo(SS3Singleton.getVideoTimeRange().time(endTime, TimeUnit.TIMESTAMP));
	}

	@Override
	public Event getFirstEvent() {
		return events.getFirstEvent();
	}

	@Override
	public Event getLastEvent() {
		return events.getLastEvent();
	}

	public double getStartTime() {
		ChainedEvent event = events.getFirstEvent();
		if (event == null)
			return 0;
		return event.getStartTimeInfo().doubleValue();
	}

	public double getEndTime() {
		ChainedEvent event = events.getLastEvent();
		if (event == null)
			return 0;
		return event.getEndTimeInfo().doubleValue();
	}

	@Override
	public boolean selectEntityIfEventSelected() {
		if (events.getSelectedEvent() != null) {
			select();
			return true;
		}
		return false;
	}

	public ArrayList getEvents() {
		return events.getEvents();
	}

	// i don't know how to do it - b. l.
	@Override
	public boolean selectEventIfEntitySelected() {
		// if (selected) {
		// manager.unselectEntity();
		// //ChainedEvent valueEvent =
		// // manager.getChainedEventsCollection().getValueEvent();
		//				
		// manager.selectChainedEvent(valueEvent);
		// selected = false;
		// return true;
		// }
		return false;
	}

	@Override
	public boolean select() {
		// manager.highlightEntity(Palette.HIGHLIGHTED_ENTITY_COLOR);

		// QTMultiMovieController qtControl =
		// SSSingleton.getMultiMovieController();
		// qtControl.setStartTime((int) manager.getFirstEvent().getStartTime());
		// qtControl.setEndTime((int) manager.getLastEvent().getEndTime());
		selected = true;
		// mToolBar.setEnable(true);
		return selected;
	}

	@Override
	public boolean unselect() {
		selected = false;
		// mToolBar.setEnable(false);
		events.unselectEntity();
		return selected;
	}

	@Override
	public boolean adjustEntity(int x1, int x2) {
		CoordinateTimeConverter convertor = segmentPanel.getZoomer()
		.getCoordinateTimeConvertor();

		Time startTimeInfo = convertor.convertCoordinateToTimeInfo(x1);
		Time endTimeInfo = convertor.convertCoordinateToTimeInfo(x2);

		return events.adjustEntity(startTimeInfo, endTimeInfo);
	}

	@Override
	public Event getSelectedEvent() {
		if (selected)
			return this;
		return events.getSelectedEvent();
	}

	@Override
	public void reset() {
		selected = false;
		// mToolBar.setEnable(false);
		events.resetAll();
	}

	@Override
	public boolean delete() {
		return events.deleteAll();
	}

	@Override
	public Event getEventAt(double x) {
		return events.getEventAt(x);
	}

	@Override
	public String getToolTipText(int x) {
		ChainedEvent event = events.getEventAt(x);

		if (event != null)
			return event.getToolTipText();
		return null;
	}

	/*
	 * 
	 * public boolean setSelectedEventStartCoordinate(int coord) { double time =
	 * coord / SegmentGraphReprUtil.pixelInTimeFrame;
	 *  // if (selected) // return manager.setEntityStartTime((int) time); // //
	 * return manager.setSelectedEventStartTime((int) time); return true; }
	 * 
	 * public boolean setSelectedEventEndCoordinate(int coord) { double time =
	 * coord / SegmentGraphReprUtil.pixelInTimeFrame;
	 * 
	 * if (selected) return manager.setEntityEndTime((int) time);
	 * 
	 * return manager.setSelectedEventEndTime((int) time); }
	 */

	public boolean setSelectedEventStartCoordinate(double coord) {
		if (selected) {
			setStartTimeCoordinate(coord);
			return true;
		} else {
			return events.setSelectedEventStartCoordinate(coord);
		}
	}

	public boolean setSelectedEventEndCoordinate(double coord) {
		if (selected) {
			setEndTimeCoordinate(coord);
			return true;
		} else {
			return events.setSelectedEventEndCoordinate(coord);
		}
	}

	@Override
	public boolean selectEvent(double x) {
		Event event = events.getEventAt(x);
		if (event == null)
			return false;
		return event.select();
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		// double X1, X2;
		g2d.setFont(font);
		FontMetrics fontMetrics = getFontMetrics(font);

		segmentPanel.getPainter().drawMovementEntity(this, g2d, fontMetrics, Y,
				getField().getFieldID());
	}

	@Override
	public void setStartTimeCoordinate(double coord) {
		events.getFirstEvent().setStartTimeCoordinate(coord);
	}

	@Override
	public void setEndTimeCoordinate(double coord) {
		events.getLastEvent().setEndTimeCoordinate(coord);
	}

	@Override
	public ArrayList getEventsToAdd() {
		ArrayList list = new ArrayList();
		return list;
	}

	@Override
	public String toString() {
		return "MovementEntity: " + super.toString();
	}

   @Override
public Time getStartTimeInfo() {
      return getFirstEvent().getStartTimeInfo();
   }

   @Override
public Time getEndTimeInfo() {
      return getLastEvent().getEndTimeInfo();
   }

   @Override
public void setStartTimeInfo(Time startTimeInfo) {
      getFirstEvent().setStartTimeInfo(startTimeInfo); 
   }

   @Override
public void setEndTimeInfo(Time endTimeInfo) {
      getLastEvent().setEndTimeInfo(endTimeInfo);
   }



@Override
public ArrayList<Event> getEventsList() {
	return new ArrayList<Event>(events.getEvents());
}
}
