package edu.bu.signstream.grepresentation.fields.nonManualField;

import java.awt.Color;

import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.time.*;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class ChainedEvent implements Event {
	protected String id;
	protected String text;
	protected String ttText;

	protected Time startTimeInfo = TimeRange.ABSOLUTE.zero();
	protected Time endTimeInfo = TimeRange.ABSOLUTE.zero();

	public boolean isLockedStartTime() {
		return lockedStartTime;
	}

	public void setLockedStartTime(boolean lockedStartTime) {
		this.lockedStartTime = lockedStartTime;
	}

	public boolean isLockedEndTime() {
		return lockedEndTime;
	}

	public void setLockedEndTime(boolean lockedEndTime) {
		this.lockedEndTime = lockedEndTime;
	}

	private boolean lockedStartTime = true;
	private boolean lockedEndTime = true;

	protected int startTime = -1, endTime;

	private Color color = null;

	protected SignStreamSegmentPanel segmentPanel;

	// Set color schema edu.bu.signstream.common.fields.vo.Palette
	protected boolean selected = false;

	private boolean highlighted = false;

	// protected MediaToolBar1 mToolBar = SSSingleton.getMediaToolBar();

	public ChainedEvent(TrackValue value, SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;
		setTrackValue(value);
	}

	public ChainedEvent(SS3Item item, SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;
		setSS3Item(item);
	}

	public ChainedEvent(SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;
	}

    public SignStreamSegmentPanel getSignStreamSegmentPanel() {
        return segmentPanel;
    }

	@Override
	public void setColor(Color color) {
		this.color = color;
	}

	@Override
	public Color getColor() {
		return color;
	}

	@Override
	public void setID(String id) {
		this.id = id;
	}

	@Override
	public String getID() {
		return id;
	}

	public boolean isExist() {
		if (startTimeInfo.doubleValue() == -1)
			return false;
		return true;
	}

	// Event interface
	@Override
	public Time getStartTimeInfo() {
		return startTimeInfo; // value.getStartTime();
	}

	// Event interface
	@Override
	public Time getEndTimeInfo() {
		return endTimeInfo; // value.getEndTime();
	}

    @Override
    public Interval interval() {
        return new Interval(startTimeInfo, endTimeInfo);
    }

    // Event interface
	@Override
	public void setStartTimeInfo(Time startTimeInfo) {
		this.startTimeInfo = startTimeInfo;
	}

	// Event interface
	@Override
	public void setEndTimeInfo(Time endTimeInfo) {
		this.endTimeInfo = endTimeInfo;
	}

	// // Event interface
	@Override
	public double getStartTimeCoordinate() {
        return startTimeInfo.absolute().as(TimeUnit.COORDINATE, Rounding.NEAREST);
//		return segmentPanel.getZoomer().getCoordinateTimeConvertor()
//				.convertTimeInfoToCoordinate(startTimeInfo);
	}

	// Event interface
	@Override
	public double getEndTimeCoordinate() {
        return endTimeInfo.absolute().as(TimeUnit.COORDINATE, Rounding.NEAREST);
//		return segmentPanel.getZoomer().getCoordinateTimeConvertor()
//				.convertTimeInfoToCoordinate(endTimeInfo);
	}

	// Event interface
	@Override
	public void setStartTimeCoordinate(double coord) {

		double endTimeCoord = getEndTimeCoordinate();
		if (coord > endTimeCoord) {
//		    endTimeInfo = TimeRange.ABSOLUTE.time(Math.round(coord), TimeUnit.COORDINATE)
//                .in(SS3Singleton.getVideoTimeRange());
            startTimeInfo = TimeRange.ABSOLUTE.time(Math.round(endTimeCoord), TimeUnit.COORDINATE)
                .in(SS3Singleton.getVideoTimeRange());
		} else {
            startTimeInfo = TimeRange.ABSOLUTE.time(Math.round(coord), TimeUnit.COORDINATE)
                .in(SS3Singleton.getVideoTimeRange());
		}
	}

	// Event interface
	@Override
	public void setEndTimeCoordinate(double coord) {
		double startTimeCoord = getStartTimeCoordinate();

		if (startTimeCoord > coord) {
//            startTimeInfo = TimeRange.ABSOLUTE.time(Math.round(coord), TimeUnit.COORDINATE)
//                .in(SS3Singleton.getVideoTimeRange());
            endTimeInfo = TimeRange.ABSOLUTE.time(Math.round(startTimeCoord), TimeUnit.COORDINATE)
                .in(SS3Singleton.getVideoTimeRange());
        } else {
            endTimeInfo = TimeRange.ABSOLUTE.time(Math.round(coord), TimeUnit.COORDINATE)
                .in(SS3Singleton.getVideoTimeRange());
		}
	}

	public boolean contains(double x) {
		//
		// System.out.println("getStartTimeCoordinate = " +
		// getStartTimeCoordinate() + ", getEndTimeCoordinate() = " +
		// getEndTimeCoordinate());
		//
		return (getStartTimeCoordinate() <= x && x <= getEndTimeCoordinate());
	}

	public void reset() {
		unselect();
		selected = false;
	}

	@Override
	public boolean select() {
		highlight();
		selected = true;
		return selected;
	}

	@Override
	public boolean unselect() {
		this.selected = false;
		unhighlight();
		// mToolBar.setEnable(true);
		return true;
	}

	@Override
	public void highlight() {
		this.highlighted = true;
	}

	@Override
	public void unhighlight() {
		this.highlighted = false;
	}

	@Override
	public boolean isHighlighted() {
		return this.highlighted;
	}

	@Override
	public boolean isSelected() {
		return selected;
	}

	@Override
	public String getToolTipText() {
        var t = ttText != null && ttText.length() > 0 ? ttText : text;
	    var s = interval().duration().as(TimeUnit.SECOND);
	    return String.format("%s, %,.4fs", t, s);
	}

	public String getText() {
		return text == null ? "" : text;
	}

	public void setText(String text) {
		this.text = text;
	}

	public void setToolTipText(String ttText) {
		this.ttText = ttText;
	}

//	private String getValueType() {
//		if (id == Constants.TYPE_HOLD) {
//			return "HOLD";
//		} else if (id == Constants.TYPE_ONSET) {
//			return "ONSET";
//		} else if (id == Constants.TYPE_OFFSET) {
//			return "OFFSET";
//		}
//		return "";
//	}

	public void setTrackValue(TrackValue value) {
		if (value != null) {
			this.id = "" + value.getValueId();
			this.text = value.getTextValue();
			this.startTimeInfo = value.getStartTimeInfo();
			this.endTimeInfo = value.getEndTimeInfo();
		}
	}

	public void setSS3Item(SS3Item item) {
		this.id = item.getId();
		this.text = item.getText();
		this.startTimeInfo = item.getStartTimeInfo();
		this.endTimeInfo = item.getEndTimeInfo();
	}

	public SS3Item getSS3Item() {
		SS3Item item = new SS3Item();
		item.setId(id);
		item.setStartTimeInfo(startTimeInfo);
		item.setEndTimeInfo(endTimeInfo);
		item.setText(text);
		return item;
	}

	public ChainedEvent cloneChainedEvent() {
		return new ChainedEvent(getSS3Item(), segmentPanel);
	}
}
