package edu.bu.signstream.grepresentation.fields.partOfSpeechField;

import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;

import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualFieldWrapper;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class PartOfSpeechFieldWrapper extends NonManualFieldWrapper {
	//private NonManualField nmField;
	private PartOfSpeechField posField;

	public PartOfSpeechFieldWrapper(PartOfSpeechField posField, SignStreamSegmentPanel segmentPanel, TimeRange timeRange) {
		super(posField, segmentPanel, timeRange);
		this.posField = posField;
		//nmField = field;
	}
   

	@Override
	public void highlight(boolean highlight, String refID) {
		ChainedEventsEntity entity = (ChainedEventsEntity)field.getEntityWithRefID(refID);
		// ---
		System.err.println("entity = " + entity);
		
		if(entity!=null) {
			//ChainedEvent event = entity.getValue();
			entity.highlight(true);
		}
		// ---
	}

	@Override
	public void unselectAll() {
		posField.unselect();
		//highlight(false, -1);
	}

	public void mouseClicked(Point point) {
		if (!isEntitySelected()) {
			Event event = posField.getEventAt(point.x);
			if (event != null) {
				event.select();
			} else {
				posField.startSelect(point.x);
			}
		} else {
			EventsEntity entity = posField.getEntityAt(point.x);
			if (entity != null) {
				entity.select();
			} else {
				posField.startSelect(point.x);
			}
		}
	}

	@Override
	public void addEventsEntity(EventsEntity entity) {
		posField.addChainedEventEntity((PartOfSpeechEntity) entity);
	}
	// ----

	@Override
	public boolean contains(int Y) {
		return field.contains(Y);
	}

	public void mouseRightClicked(Point point, JPanel panel) {
		//System.out.println("PartOfSpeechFieldWrapper: mouseRightClicked");
	}

	@Override
	public void mouseDblLeftClicked(Point point, JPanel panel) {
		SyntacticGroup group = posField.getSyntacticGroupAt(point.x);

		if (group == null) {
			super.mouseDblLeftClicked(point, panel);
		} else {

			final ArrayList eventsToAdd = group.getAvailableSGLabels();

			if (eventsToAdd == null)
				return;

			final JPopupMenu popup = new JPopupMenu();

			for (int i = 0; i < eventsToAdd.size(); i++) {
				popup.add(new JLabel((String) eventsToAdd.get(i)));
			}

			popup.setPreferredSize(popup.getPreferredSize());
			popup.show(panel, point.x, point.y);
			popup.setVisible(true);
			
			SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);

			popup.addMouseListener(new MouseAdapter() {
				@Override
				public void mousePressed(MouseEvent ev) {
					int popupHeight = popup.getPreferredSize().height;
					int choosenHeight = ev.getPoint().y;
					int elementHeight = popupHeight / eventsToAdd.size();
					int choosenEl = choosenHeight / elementHeight;

					if (choosenHeight % elementHeight == 0)
						choosenEl--;

					//String event = (String) eventsToAdd.get(choosenEl);
					
			    	SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);

					popup.setVisible(false);
				}
			});
		}
	}

	public void mouseDblClicked(Point point) {
	}

	// do nothing!!!
	@Override
	public void mouseDblRightClicked(Point point, JPanel panel) {
		//System.out.println("PartOfSpeechFieldWrapper: mouseDblRightClicked");
	}

	// MouseMotionListener
	@Override
	public void mouseMoved(Point point, JPanel panel) {
		Event event = posField.getEventAt(point.x);
		if (event != null) {
			panel.setToolTipText(event.getToolTipText());
		} else {
			panel.setToolTipText("");
		}
		//System.out.println("PartOfSpeechFieldWrapper: mouseMoved");
	}

	@Override
	public void mouseDragged(Point point, JPanel panel) {
		// do nothing!!! 
		posField.select(point.x);
		//System.out.println("PartOfSpeechFieldWrapper: mouseDragged");
	}

	@Override
	public void mouseReleased(Point point, JPanel panel) {
		posField.createNewSyntacticGroup();
	}

	@Override
	public void delete(int eventAt) {
		EventsEntity entity = posField.getEntityAt(eventAt);
		if (entity != null)
			posField.deleteEntity(entity);
	}

	public void adjust(Event value, int eventAt) {
		Event event = posField.getEventAt(eventAt);
		if (event != null) {
			event.setStartTimeInfo(value.getStartTimeInfo());
			event.setEndTimeInfo(value.getEndTimeInfo());
		}
	}
}
