package edu.bu.signstream.grepresentation.listener;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.logging.Level;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.grepresentation.fields.customField.CustomPointFieldWrapper;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.customField.CustomIntervalFieldWrapper;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.CustomFieldsLabelView;
import edu.bu.signstream.grepresentation.view.GestureFieldsLabelView;
import edu.bu.signstream.grepresentation.view.GlossFieldsLabelView;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.NonManualFieldsLabelView;
import edu.bu.signstream.grepresentation.view.SegmentGraphicRepresentation;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class CustomFieldsListener extends SignStreamMouseListener {

	public CustomFieldsListener(SignStreamSegmentPanel segmentPanel,
			SegmentGraphicRepresentation nonManualRep,
			SegmentGraphicRepresentation glossRep,
			SegmentGraphicRepresentation gestureRep,
			SegmentGraphicRepresentation customRep,
			NonManualFieldsLabelView nmLabelView,
			GlossFieldsLabelView glsLabelView,
			GestureFieldsLabelView gesLabelView,
			CustomFieldsLabelView cstLabelView) {

		super(segmentPanel, nonManualRep, glossRep, gestureRep, customRep, nmLabelView, glsLabelView, gesLabelView, cstLabelView);
	}

    @Override
    public SegmentGraphicRepresentation view() { return customRep; }

    private boolean mouseDragged = false;
	private boolean mouseDraggedAlt = false;
	
//	// to prevent multiple call of dblClick method

	@Override
	public void mouseMoved(MouseEvent e) {
	    super.mouseMoved(e);
		
		Point point = e.getPoint();

		// GREG TODO gloss?
		glossRep.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));

		if (slider != null && point.x - 3 < slider.getSliderPosition()
				&& point.x + 3 > slider.getSliderPosition()) {
			customRep.setCursor(new Cursor(Cursor.E_RESIZE_CURSOR));
		} else {
			customRep.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}

		// change magnified events to normal
		FieldWrappersCollection wrappers = glossRep
				.getFieldWrappersCollection();

		FieldWrapper rhWrapper = wrappers
				.getFieldWrapper(Constants.TYPE_DOM_HAND_SHAPE);
		FieldWrapper lhWrapper = wrappers
				.getFieldWrapper(Constants.TYPE_NDOM_HAND_SHAPE);

		boolean minified = false;

		if (minifyAllHandShapeIcons(rhWrapper))
			minified = true;

		if (minifyAllHandShapeIcons(lhWrapper))
			minified = true;

		if (minified)
			glossRep.repaint();

		FieldWrapper wrapper = customRep.getFieldWrappersCollection().getFieldWrapperAt(point.y);
		if (wrapper != null)
			wrapper.mouseMoved(point, customRep);
	}

	@Override
	public void mouseDragged(MouseEvent e) {
        var point = e.getPoint();
        FieldWrapper wrapper = customRep.getFieldWrappersCollection().getFieldWrapperAt(point.y);

        var player = SS3Singleton.getMediaPlayer();
        if (player.playing()) player.pause();

        if (wrapper == null) {
            var time = SS3Singleton.getVideoTimeRange().time(e.getX(), TimeUnit.COORDINATE);
            player.seek(time);
            return;
        }

        if (wrapper instanceof CustomIntervalFieldWrapper) {
            CustomIntervalFieldWrapper customWrapper = (CustomIntervalFieldWrapper) wrapper;
            customRep.setCursor(new Cursor(customWrapper.getCursorType(point)));

            if (isShiftPressed(e) || isAltPressed(e) || isCtrlPressed(e))
                customWrapper.mouseDraggedAlt(point);
            else
                wrapper.mouseDragged(point, glossRep);
        }

        if (wrapper instanceof CustomPointFieldWrapper) {
            CustomPointFieldWrapper customWrapper = (CustomPointFieldWrapper) wrapper;
            customRep.setCursor(new Cursor(customWrapper.getCursorType(point)));

            if (isShiftPressed(e) || isAltPressed(e) || isCtrlPressed(e))
                customWrapper.mouseDraggedAlt(point);
            else
                wrapper.mouseDragged(point, glossRep);
        }

        repaintAll();
	}

	private int ex = -10;

	@Override
	public void mousePressed(MouseEvent e) {
		segmentPanel.getUtteranceView().getHeader().getTimelineView().getUpperLeftCorner().getZoomInOut().requestFocus();

		SS3SignStreamApplication.logger.log(Level.INFO, "Mouse Pressed");
		Util.dragging(true);

        boolean isAltPressed = isAltPressed(e);
        boolean isCtrlPressed = isCtrlPressed(e);

		if (ex == e.getPoint().x)
			return;

		ex = e.getPoint().x;

		FieldWrapper wrapper = customRep.getFieldWrappersCollection().getFieldWrapperAt(e.getPoint().y);

		if (wrapper == null) {
		    var player = SS3Singleton.getMediaPlayer();
		    var time = SS3Singleton.getVideoTimeRange().time(e.getX(), TimeUnit.COORDINATE);
		    if (player.playing()) player.pause();
		    player.seek(time);
            return;
        }
		
		Event event = wrapper.getField().getEventAt(e.getPoint().x);		
		if (event == null) {	// only check to begin dragging if nothing is occupying this space already
			if (isAltPressed || isCtrlPressed) {
				wrapper.mousePressedAlt(e.getPoint(), customRep);	// begin drawing line for new nonmanual event
				repaintAll();
				if (e.getPoint().y > SegmentGraphReprUtil.axesBottomHeight) {
					unselectAll();
					repaintAll();
				} 
			}
		}
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		
		SS3SignStreamApplication.logger.log(Level.INFO, "Mouse Released");
		Util.dragging(true);
		
		Point point = e.getPoint();
		
		if (mouseDragged) {
			mouseDragged = false;
			
//			try {
//				double time = segmentPanel.getMultipleMovieController().getCurrentTime();
//				double packetDuration = segmentPanel.getMultipleMovieController().getPacketDuration();
//				double frames = time / packetDuration;
//				double cTime = frames * packetDuration;
//				// GREG TODO ??? divide and then multiply the same value?
//				slider.setSliderPositionTime(cTime);
//			} catch(NullPointerException ex) {
//				ex.printStackTrace();
//				SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
//			}
		}	
		
		FieldWrapper wrapper = customRep.getFieldWrappersCollection().getFieldWrapperAt(point.y);

		if (wrapper != null) {
			wrapper.mouseReleased(point, customRep);
			segmentPanel.setSelectedFieldWrapper(wrapper);
			Event selectedEvent = wrapper.getSelectedEvent();
			segmentPanel.setSelectedEvent(selectedEvent);

			repaintAll();
		}

//		if (wrapper instanceof HandShapeFieldWrapper && mouseDraggedAlt) {		// this is dead code also - b/c hs wrapper
//			HandShapeFieldWrapper hsWrapper = (HandShapeFieldWrapper) wrapper;
//			hsWrapper.mouseReleasedAlt(point);
//			mouseDraggedAlt = false;
//			repaintAll();
//		}
	}

	// select chained event or chained events entity
	@Override
	public void mouseClicked(final MouseEvent e) {

		SS3SignStreamApplication.logger.log(Level.INFO, "Mouse Clicked");

		if (mouseDragged) {
			// do not select
			mouseDragged = false;
			return;
		}

		Point point = e.getPoint();
		
		unselectAll();
		
		FieldWrapper wrapper = customRep.getFieldWrappersCollection().getFieldWrapperAt(point.y);

		if (wrapper != null) {
            int clickCount = e.getClickCount();
            //
            System.out.println("clickCount = " + clickCount);
            //
            
			// right btn
			if (e.getButton() == MouseEvent.BUTTON3 || isCtrlPressed(e) || isAltPressed(e)) {
				if (wrapper instanceof CustomIntervalFieldWrapper) {
					CustomIntervalFieldWrapper nmFieldWrapper = (CustomIntervalFieldWrapper) wrapper;
					nmFieldWrapper.mouseCtrlAltClicked(point, customRep);
					highlightLabel(wrapper);
				}
			} else if (e.getClickCount() == 2 && e.getButton() == MouseEvent.BUTTON1) {	// left double click
				wrapper.mouseDblLeftClicked(point, customRep);
				highlightLabel(wrapper);
				return;
			}

			wrapper.mouseClicked(point, customRep);

		}
		repaintAll();
	}

	@Override
	protected void highlightLabel(FieldWrapper wrapper) {
		String fieldID = wrapper.getField().getFieldID();

		try {
			ArrayList labels = nmLabelView.getLabels();
			// highlight only selected field label
			for (int i = 0; i < labels.size(); i++) {
				LabelObject object = (LabelObject) labels.get(i);
				String id = object.getFieldID();
				if (fieldID.equalsIgnoreCase(id)) {
					object.setUpdatable(true);
				} else {
					object.setUpdatable(false);
				}
			}
			nmLabelView.repaint();

			ArrayList glsLabels = glsLabelView.getLabels();
			// unhighlight all labels
			for (int j = 0; j < glsLabels.size(); j++) {
				LabelObject object = (LabelObject) glsLabels.get(j);
				object.setUpdatable(false);
			}
			glsLabelView.repaint();
		} catch (NullPointerException ex) {
			return;
		}
	}
}
