package edu.bu.signstream.grepresentation.view;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.util.HashMap;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JTree;
import javax.swing.border.Border;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.handshapes.GestureHandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.orientationField.GestureOrientationFieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeFieldWrapper;

public class CustomTreeRenderer extends DefaultTreeCellRenderer {
    private final Color color;
    private final int w;
    private final int h;
    private static HashMap<String, Integer> largestWidth = new HashMap<>();

    public CustomTreeRenderer(Color color, int w, int h) {
        this.color = color;
        this.w = w;
        this.h = h;
    }

    @Override
    public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {

        super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);

        Component com = super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);

        JLabel label = (JLabel) com;

        if (value != null) {
            LabelObject lb = null;

            if (value instanceof JTree.DynamicUtilTreeNode) {
                JTree.DynamicUtilTreeNode node = (JTree.DynamicUtilTreeNode) value;
                Object userObj = node.getUserObject();

                if (userObj instanceof TreeNodeVector) {
                    TreeNodeVector vector = (TreeNodeVector) node.getUserObject();
                    lb = vector.getObject();
                } else {
                    lb = (LabelObject) node.getUserObject();
                }
            } else if (value instanceof DefaultMutableTreeNode) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
                Object userObj = node.getUserObject();

                if (userObj instanceof LabelObject) {
                    lb = (LabelObject) node.getUserObject();
                }
            } else if (value instanceof LabelObject) {
                lb = (LabelObject) value;
            } else {
                return this;
            }

            if (lb != null) {
                // should be utilized only by LabelViewTemporalPartitions to display default segment tiers
                if (lb.isDefaultSegmentTier()) {
                    Border border = BorderFactory.createLineBorder(Color.RED, 1);
                    label.setBorder(border);
                } else {
                    label.setBorder(null);
                }

                int labelHeight = SegmentGraphReprUtil.cellHeight;

                FieldWrapper wrp = lb.getWrapper();
                if (wrp instanceof GestureHandShapeFieldWrapper || wrp instanceof GestureOrientationFieldWrapper || wrp instanceof HandShapeFieldWrapper) {
                    labelHeight = SegmentGraphReprUtil.hsAndOrientationHeight;
                }

                label.revalidate();
                // Set the height but leave the width alone


                // Use Map's compute method to simplify getting and setting values

                largestWidth.compute(wrp.getField().getFieldID(), (key, val) -> {
                    if (val == null || label.getPreferredSize().width > val) {
                        return label.getPreferredSize().width;
                    }
                    return val;
                });

                label.setPreferredSize(new Dimension(largestWidth.get(wrp.getField().getFieldID()), labelHeight));


                label.repaint();


/*                Dimension d = this.getPreferredSize();
                d.setSize(d.getWidth(), labelHeight);
                label.setMinimumSize(d);
                label.setPreferredSize(d);
                label.setMaximumSize(d);*/



/*                if (label.getPreferredSize().width > largestWidth) {
                    largestWidth = label.getPreferredSize().width;

                    label.setPreferredSize(new Dimension(label.getPreferredSize().width, labelHeight));
                    label.setSize(new Dimension(label.getPreferredSize().width, labelHeight));
                    label.setMinimumSize(new Dimension(label.getPreferredSize().width, labelHeight));
                }

                label.setPreferredSize(new Dimension(largestWidth, labelHeight));*/




                // do not add case for if-selected here
                // selected is taken care of by JTree.setSelectionPath(TreePath)
            }
        }

        return this;
    }
}
