package edu.bu.signstream.grepresentation.view;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.graphField.GraphFieldWrapper;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.LabelViewIndependentItems.Listener;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

public class LabelViewNonManualFields extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private LabelViewIndependentItems smoother;
	ArrayList<LabelObject> DATA = new ArrayList<LabelObject>();
	JList list;
	boolean updating = false;

	private SegmentGraphicRepresentation rep;
	private SegmentControlPanel controlPanel;

	public LabelViewNonManualFields(final ArrayList<LabelObject> data, final SegmentGraphicRepresentation rep,
									final SegmentControlPanel controlPanel) {

		DATA.addAll(data);
		this.rep = rep;
		this.controlPanel = controlPanel;

		sortData();

		list = new JList(new AbstractListModel() {
			@Override
			public int getSize() {
				return DATA.size();
			}

			@Override
			public Object getElementAt(int index) {
				return DATA.get(index);
			}
		});

		list.setBackground(UIPalette.getBackground());

		smoother = new LabelViewIndependentItems(list) {
			@Override
			protected void move(int fromIndex, int toIndex) {
				SS3Database currentDb = controlPanel.getSignStreamSegmentPanel().getSS3Database();
				SS3CodingScheme scheme = currentDb.getSS3CodingScheme();
				
				LabelObject fromLabel = DATA.get(fromIndex);
				LabelObject toLabel = DATA.get(toIndex);
				
				LabelObject o = DATA.remove(fromIndex);
				DATA.add(toIndex, o);
				list.invalidate();
				recreateList(); // call method to recreate jlist
				rep.getFieldWrappersCollection().moveField(fromIndex, toIndex);

				SS3Field fromField = scheme.getField(fromLabel.getFieldID());
				SS3Field toField = scheme.getField(toLabel.getFieldID());
				
				scheme.moveFieldOrderTo(fromField, toField);

				for (LabelObject obj : DATA) {
					obj.getWrapper().setSelectedNHighlighted(false);
				}

	            SS3Singleton.getMediaToolBar().selectLabel(o);
				list.setSelectedValue(o, true);
			}
		};

		GraphLabelDoubleClickListener graphListener = new GraphLabelDoubleClickListener(this, rep);
		list.addMouseListener(graphListener);

		Listener listener = new Listener(smoother);
		list.addMouseListener(listener);
		list.addMouseMotionListener(listener);
		list.setSelectionBackground(UIPalette.getSelectedWrapperColor());
		list.getSelectionModel().setSelectionMode(DefaultListSelectionModel.SINGLE_SELECTION);
		list.getSelectionModel().addListSelectionListener(new ListSelectionHandler(list, controlPanel, rep, this));
		list.addKeyListener(SS3Singleton.getVideoControlKeyListener());

		recreateList();
		setLayout(new BorderLayout());
		
		// just to buffer space, to get the labels and the field lines in sync horizontally
		JPanel tmp = new JPanel();
		tmp.setBackground(UIPalette.getBackground());
		tmp.setPreferredSize(new Dimension(tmp.getWidth(), SegmentGraphReprUtil.axesBottomHeight));
		tmp.setMinimumSize(new Dimension(tmp.getWidth(), SegmentGraphReprUtil.axesBottomHeight));
		add(tmp, BorderLayout.NORTH);		
		
		add(list, BorderLayout.CENTER);

		rep.setPreferredSize(rep.getPreferredDimension());
	}

	public void recreateList() {

		list.setCellRenderer(new DefaultListCellRenderer() {

			@Override
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean sel,
					boolean focus) {
				Component c = super.getListCellRendererComponent(list, value, index, sel, focus);

				JLabel label = (JLabel) c;
				label.setPreferredSize(new Dimension(200, SegmentGraphReprUtil.cellHeight));
				//label.setBorder(new LineBorder(Palette.AXIS_COLOR));
				label.setBorder(BorderFactory.createMatteBorder(index == 0 ? 1 : 0, 0, 1, 0, Palette.AXIS_COLOR));

				UserInterfaceUtil.setLabelLNF(label);

				LabelObject object = (LabelObject) value;
//				if (object.isUpdatable()) {
//					label.setForeground(Color.RED);
//				} else {
					label.setForeground(Color.black);
//				}
				FieldWrapper wrapper = object.getWrapper();
				if (wrapper instanceof GraphFieldWrapper) {
					int height =
							SegmentGraphReprUtil.getCorrectCellHeight(((GraphFieldWrapper) wrapper).getGraphField());
					label.setPreferredSize(new Dimension(200, height));
				}

				if ((index % 2) == 0) {
					((JComponent) c).setOpaque(true);
				} else if (c.getBackground().equals(list.getBackground())) {
					((JComponent) c).setOpaque(false);
				}

				return c;
			}
		});
		list.revalidate();
		list.repaint();
		rep.repaint();
		// setLayout(new BorderLayout());
		// add(list, BorderLayout.CENTER);
	}

	public LabelViewIndependentItems getLabelViewIndependentItems() {
		return smoother;
	}

	public void update() {
		sortData();
		recreateList();
	}

	private void sortData() {
		controlPanel.getSignStreamSegmentPanel().getSS3Database().getSS3CodingScheme().sortLabelObjectList(DATA);
	}
}

class ListSelectionHandler implements ListSelectionListener {
	private JList list = null;
	private SegmentControlPanel controlPanel = null;
	private SegmentGraphicRepresentation rep = null;
	private LabelViewNonManualFields self;

	public ListSelectionHandler(JList list, SegmentControlPanel controlPanel, SegmentGraphicRepresentation rep, LabelViewNonManualFields self) {
		this.list = list;
		this.rep = rep;
		this.controlPanel = controlPanel;
		this.self = self;
	}

	@Override
	public void valueChanged(ListSelectionEvent e) {
		Object obj = list.getSelectedValue();

		self.updating = true;
		if (e.getValueIsAdjusting() && obj != null)
            SS3Singleton.getMediaToolBar().selectLabel((LabelObject)obj);
		if (e.getValueIsAdjusting() && obj == null)
            SS3Singleton.getMediaToolBar().deselectLabel();
        self.updating = false;

		controlPanel.focusUtteranceList();
	}
}

class GraphLabelDoubleClickListener extends MouseAdapter {
	private LabelViewNonManualFields lblView;
	private SegmentGraphicRepresentation rep;

	public GraphLabelDoubleClickListener(LabelViewNonManualFields lblView, SegmentGraphicRepresentation rep) {
		this.rep = rep;
		this.lblView = lblView;
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		if (e.getClickCount() == 2) {
			LabelObject obj = (LabelObject) lblView.list.getModel().getElementAt(lblView.list.locationToIndex(e.getPoint()));
			if (obj.getWrapper() instanceof GraphFieldWrapper) {
				((GraphFieldWrapper)obj.getWrapper()).getGraphField().toggleMaximized();
			}
		}
		lblView.recreateList();
		rep.setPreferredSize(rep.getPreferredDimension()); // update scrollpane viewport height
		rep.setSize(rep.getPreferredDimension()); // update scrollpane viewport height
	}
}