package edu.bu.signstream.grepresentation.view.main;

import java.awt.BorderLayout;
import java.util.*;
import java.util.logging.Level;
import java.util.stream.Collectors;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.Panel;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.ViewMode;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindow;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3StatementField;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.view.*;
import edu.bu.signstream.time.*;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.Painter;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.displayTimePeriod.DisplayTimeViewWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEnteredText;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEventsEntityCollection;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureField;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureText;
import edu.bu.signstream.grepresentation.fields.glossField.EnteredText;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntityCollection;
import edu.bu.signstream.grepresentation.fields.glossField.GlossField;
import edu.bu.signstream.grepresentation.fields.glossField.Text;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEvent;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationFieldWrapper;
import edu.bu.signstream.grepresentation.fields.presentation.SegmentFieldWrapper;
import edu.bu.signstream.grepresentation.listener.SegmentMouseListener;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.media.fileNavigation.LoadedMediaFile;
import edu.bu.signstream.media.fileNavigation.MediaDelegate;
import edu.bu.signstream.navigation.DatabaseDelegate;
import edu.bu.signstream.navigation.DbNavigationPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import edu.bu.signstream.ui.panels.search.SearchSavedState;
import edu.bu.signstream.ui.popup.TextEntryPopupGenerator;
import edu.bu.signstream.ui.video.MultipleMovieController;

public class SignStreamSegmentPanel extends JPanel {
	private static final long serialVersionUID = 1L;

	private boolean isUtteranceLoaded = false;

	private DbNavigationPanel navigationPanel;

	private SS3Database database;

	private JFrame parentFrame;

	private final Hashtable<JDialog, java.io.File> openedMediaDialogs = new Hashtable<JDialog, java.io.File>();

	private Painter painter = null;

	private ArrayList<Segment> segments = new ArrayList<>();

	private Segment selectedSegment = null;
	public Segment selectedSegment() { return selectedSegment; }

	private boolean entityLevelSelected = false;

	private ToolBarPanels tbPanel = null;

	private final JSplitPane sp = new JSplitPane();

	private Event selectedEvent = null;

	private FieldWrapper selectedWrapper = null;

	private Optional<UtteranceViewState> utteranceViewState = Optional.empty();
	private DatabaseViewState databaseViewState;

	private final Map<String, ViewMode> segmentViewModes = new HashMap<>();
	public ViewMode getSegmentViewMode(String id) { return segmentViewModes.get(id); }
	public void setSegmentViewMode(String id, ViewMode mode) { segmentViewModes.put(id, mode); }

    private boolean zoomLocked = false;
    public boolean zoomLocked() {
		return zoomLocked;
	}
    public void zoomLock(boolean locked) {
		zoomLocked = locked;
	}

    private boolean displaySelected = true;
    public boolean displaySelected() { return displaySelected; }
    public void setDisplaySelected(boolean enabled) { displaySelected = enabled; }

    private boolean displayAnnotated = true;
    public boolean displayAnnotated() { return displayAnnotated; }
    public void setDisplayAnnotated(boolean enabled) { displayAnnotated = enabled; }

	// need
	SegmentMouseListener listener = null;

	// need
	private DatabaseView databaseView = null;
	private UtteranceView utteranceView = null;

	// need
	public UtteranceView getUtteranceView() {
		return utteranceView;
	}

	// need
	public DatabaseView getDatabaseView() {
		return databaseView;
	}

	// need
	public void setSS3Database(SS3Database database) {
		this.database = database;
	}

	// need
	public SignStreamSegmentPanel(DatabaseDelegate delegate, JFrame parentFrame, ViewMode viewMode) {
		this.parentFrame = parentFrame;

		SS3GlossWindow glsWindow = new SS3GlossWindow();

		SS3GlossWindowSegment displayTimePeriod = new SS3GlossWindowSegment();
		displayTimePeriod.setLabel("Display Time Period");
		glsWindow.setDisplayTimePeriod(displayTimePeriod);

		SS3TemporalPartition temporalPartition = new SS3TemporalPartition();
		temporalPartition.setLabel("Temporal Partition 1");

		SS3GlossWindowSegment tpSegment = new SS3GlossWindowSegment();
		tpSegment.setLabel("Segment Tier 1");
		tpSegment.setID("" + Util.getLongUniqueNumber());

		temporalPartition.addTemporalPartitionSegment(tpSegment);
		temporalPartition.setDefaultSegment(tpSegment);

		ArrayList<SS3TemporalPartition> partitions = new ArrayList<>();
		partitions.add(temporalPartition);
		glsWindow.setTemporalPartitions(partitions);


		this.database = new SS3Database();
		database.setSS3GlossWindow(glsWindow);
		Excerpt excerpt = new Excerpt(Util.getLongUniqueNumber(),
				database.getFileName());
		database.setExcerptObj(excerpt);

		SS3GlossWindow window = database.getSS3GlossWindow();
		Segment segment = window.getSegment();
		if (segment == null) {
			segment = !segments.isEmpty() ? segments.get(0) : new Segment(this, null);
			window.setSegment(segment);
		}
		this.selectedSegment = segment;

		RootLabelObject rootObject = new RootLabelObject();
		LabelObject presentationObject = new LabelObject();
		rootObject.setPresentationLabelObject(presentationObject);
		
		TimeRange timeRange = SS3Singleton.getVideoTimeRange();
//		TimeRange timeRange = TimeRange.ABSOLUTE;

		SS3GlossWindowSegment dislayTimePeriod = window.getDisplayTimePeriod();

		PresentationField displayTimePeriodField = new PresentationField(null,
				dislayTimePeriod, this);
		DisplayTimeViewWrapper displayTimePeriodFieldWrapper = new DisplayTimeViewWrapper(
				displayTimePeriodField, this, timeRange);
		displayTimePeriodFieldWrapper.setVisible(true);

		LabelObject dObj = new LabelObject();
		dObj.setLabel(dislayTimePeriod.getLabel());
		dObj.setFieldID(displayTimePeriodField.getFieldID());
		dObj.setWrapper(displayTimePeriodFieldWrapper);

		presentationObject.addChild(dObj);

		segment.addFieldWrapper(displayTimePeriodFieldWrapper);

		for (SS3TemporalPartition partition : partitions) {
			ArrayList<SS3GlossWindowSegment> segments = partition
					.getTemporalPartitionSegments();

			PresentationField pf = new PresentationField(partition, this);
			PresentationFieldWrapper pfw = new PresentationFieldWrapper(pf,
					this, timeRange);
			pfw.setVisible(true);

			LabelObject temporalPartitionObject = new LabelObject();
			temporalPartitionObject.setLabel(partition.getLabel());
			temporalPartitionObject.setFieldID(pf.getFieldID());
			temporalPartitionObject.setWrapper(pfw);
			segment.addFieldWrapper(pfw);

			for (SS3GlossWindowSegment sgm : segments) {
				PresentationField spf = new PresentationField(partition, sgm, this);
				spf.setParentField(pf);

				ArrayList segmentEntities = spf.getEntities();
				for (int i = 0; i < segmentEntities.size(); i++) {
					PresentationEventsEntity pEntity = (PresentationEventsEntity) segmentEntities
							.get(i);
					PresentationEventsEntity masterEntity = pf
							.getEntityWithStartTime(pEntity.getStartTimeInfo());
					if (masterEntity != null) {
						masterEntity.addDependentEntity(pEntity);
					} else {
						SS3SignStreamApplication.logger.setLevel(Level.SEVERE);
						SS3SignStreamApplication.logger
								.info("Could not find master entity for entity "
										+ pEntity.getToolTipText());
					}
				}

				SegmentFieldWrapper sw = new SegmentFieldWrapper(spf, this,
						partition.getLabel(), sgm.getLabel(), timeRange);
				sw.setVisible(true);

				pfw.addSegmentFieldWrapper(sw);

				LabelObject temporalPartitionPresentationObject = new LabelObject();
				temporalPartitionPresentationObject
						.setFieldID(spf.getFieldID());
				temporalPartitionPresentationObject.setLabel(sgm.getParticipantLabel() + sgm.getLabel());
				temporalPartitionPresentationObject.setWrapper(sw);

				temporalPartitionObject
						.addChild(temporalPartitionPresentationObject);
				segment.addFieldWrapper(sw);
			}

			presentationObject.addChild(temporalPartitionObject);
		}

		segment.setRootLabelObject(rootObject);

		delegate = new DatabaseDelegate(this);
		this.navigationPanel = new DbNavigationPanel(this, delegate);
		databaseView = new DatabaseView(segment, this);

		TitledBorder border = new TitledBorder("");
		databaseView.setBorder(border);

		tbPanel = new ToolBarPanels(this);
		var mediaToolBar = new MediaToolBar(this);
		SS3Singleton.setMediaToolBar(mediaToolBar);
		tbPanel.addMediaToolBar(mediaToolBar);

		sp.setLeftComponent(navigationPanel);
		sp.setRightComponent(databaseView);

		sp.setDividerLocation(200);

		setLayout(new BorderLayout());
		add(tbPanel, BorderLayout.NORTH);
		add(sp, BorderLayout.CENTER);

		updateJMenuBar();

		repaint();
	}

	public static ViewDataManager initializeDefaultViewDataManager(ViewMode viewMode) {
		ViewDataManager viewDataManager = new ViewDataManager();
		Set<Panel> visiblePanels = viewMode.getPanels();
		int totalNumberOfPanels = visiblePanels.size();

		Iterator<Panel> panelIterator = visiblePanels.iterator();

		int order = 0;

		while (panelIterator.hasNext()) {
			Panel panel = panelIterator.next();
			String id = panel.id;
			double panelPercentage = 100.0 / (double) totalNumberOfPanels;
			ViewData viewData = new ViewData(id, Constants.panelOrders.get(id), panelPercentage);
			viewDataManager.addViewData(viewData);
		}

		return viewDataManager;
	}

	// need
	public void loadDatabase(SS3Database database) {
        if (database == null) {
            utteranceViewState = Optional.empty();

			SS3GlossWindow glsWindow = new SS3GlossWindow();

			SS3GlossWindowSegment displayTimePeriod = new SS3GlossWindowSegment();
			displayTimePeriod.setLabel("Display Time Period");
			glsWindow.setDisplayTimePeriod(displayTimePeriod);

			SS3TemporalPartition temporalPartition = new SS3TemporalPartition();
			temporalPartition.setLabel("Temporal Partition 1");

			SS3GlossWindowSegment tpSegment = new SS3GlossWindowSegment();
			tpSegment.setLabel("Segment Tier 1");
			tpSegment.setID("" + Util.getLongUniqueNumber());
			temporalPartition.addTemporalPartitionSegment(tpSegment);
			temporalPartition.setDefaultSegment(tpSegment);

			ArrayList<SS3TemporalPartition> partitions = new ArrayList<SS3TemporalPartition>();

			partitions.add(temporalPartition);

			glsWindow.setTemporalPartitions(partitions);

			database = new SS3Database();
			database.setSS3GlossWindow(glsWindow);
			Excerpt excerpt = new Excerpt(Util.getLongUniqueNumber(),
					database.getFileName());
			database.setExcerptObj(excerpt);
		}

		// regardless of which db is being loaded, we always want to do this check below
		// in case we are switching between two different db's, from the UtteranceView perspective
		if (sp.getRightComponent() instanceof DatabaseView) {
			sp.repaint();
		} else {
			sp.remove(utteranceView);
			//remove all View menu items
//			SS3Singleton.getSignStreamApplication().getJMenuBar().getMenu(2).removeAll();
			
			sp.setLeftComponent(navigationPanel);
			sp.setRightComponent(databaseView);

			sp.setDividerLocation(200);
		}

		if (this.database != database) {


			DefaultMutableTreeNode databaseRoot = database.getDbNavigationPanelNode();

			DefaultMutableTreeNode delegateRoot = SS3Singleton.getSignStreamApplication().
					getDatabaseDelegate().getMutableTreeNode();

			DefaultMutableTreeNode refreshRoot = databaseRoot != null ? databaseRoot : delegateRoot;

			// update dbnavigation panel
			getDbNavigationPanel().refresh(refreshRoot);

			// update panel title
			if (refreshRoot.getChildCount() > 0) {
				DefaultMutableTreeNode node1 = (DefaultMutableTreeNode) refreshRoot
						.getChildAt(0);

				Object uo = node1.getUserObject();
				if (uo instanceof String) {
					SS3Singleton.getSignStreamApplication().setTitle((String) uo);
				}
				
				getDbNavigationPanel().getJTree().expandRow(1);
				getDbNavigationPanel().getJTree().setSelectionRow(2);
			}

			SS3SignStreamApplication.logger.log(Level.INFO,
					"Loading a database [" + database.getExcerpt()
							+ "] presentation view.");

			ArrayList<SS3MediaFile> movies = new ArrayList<>();
			String loadingMediaFilesMsg = "Loading media files [";

			Excerpt excerpt = database.getExcerptObj();

			ArrayList<Object> mIDs = excerpt.getMediaFiles();
			for (int i = 0; i < mIDs.size(); i++) {
				String id = (String) mIDs.get(i);

				SS3MediaFile mediaFile = database.getMediaFile(id);
				if (mediaFile != null) {
					movies.add(mediaFile);
					loadingMediaFilesMsg = loadingMediaFilesMsg
							.concat(mediaFile.getFile().getName());

					if (i < mIDs.size() - 1)
						loadingMediaFilesMsg = loadingMediaFilesMsg
								.concat(", ");
				}
			}

			loadingMediaFilesMsg = loadingMediaFilesMsg.concat("]...");

			SS3SignStreamApplication.logger.log(Level.INFO,
					loadingMediaFilesMsg);

//			showVideoFiles(movies, database.getVideoOnScreenLocations());

//            var player = SS3Singleton.getMediaPlayer();
//            for (var a : database.getAudios()) {
//                var uri = a.uri();
//
//                var audio = player.audio(uri).orElse(null);
//                if (audio == null)
//                    audio = player.addAudio(uri);
//
//                if (!audio.displayed())
//                    audio.display();
//
//                database.addAudio(audio);
//            }

//            var videos = database.getVideos().stream()
//                .map(v -> {
//                    var file = new SS3MediaFile();
//                    file.setFile(new File(v.uri()));
//                    file.setId(v.uri().toString());
//                    return file;
//                })
//                .collect(Collectors.toCollection(ArrayList::new));
//            showVideoFiles(videos, database.getVideoOnScreenLocations());

//			SS3Singleton.getMediaPlayer().seek(database.getStartTime());

//			contr.selectSegment(currentTime, database.getEndTimeInfo()
//					.getMovieTime());

//			int movieDuration = getMultipleMovieController().getMovieDuration();
//			int packetDuration = (int) getMultipleMovieController()
//					.getPacketDuration();
//
//			if (movieDuration == 0) {
//				movieDuration = database.getMovieDuration();
//				packetDuration = database.getPacketDuration();
//			}

            int width = databaseView.getVisibleWidth2();
			getZoomer().getCoordinateTimeConvertor().resize(width);
//			getZoomer().fillEntirePage(movieDuration, packetDuration, width);

			update(database);

//			if (movieDuration == 0) {
//				SS3GlossWindow glossWindow = database.getSS3GlossWindow();
//				ArrayList<SS3TemporalPartition> temporalPartitions = glossWindow.getTemporalPartitions();
//
//
//				for (SS3TemporalPartition temporalPartition : temporalPartitions) {
//					SS3GlossWindowSegment segment = temporalPartition
//							.getGlossWindowSegment();
//					ArrayList<SS3GlossWindowItem> items = segment
//							.getSS3GlossWindowItems();
//					for (SS3GlossWindowItem item : items) {
//						int time = item.getEndTimeInfo().getMovieTime();
//						if (time > movieDuration)
//							movieDuration = time;
//					}
//				}
//			}


			databaseView.updateDrawingsAreaSize();
			getZoomer().zoom(0);
		}

		repaint();
		this.database = database;

		// segments are from UtteranceView - clean from memory after going back to DatabaseView
		segments.clear();

		repaint();
		updateUI();
		databaseView.getUpperLeft().lock(this.zoomLocked);
		isUtteranceLoaded = false;
	}

	public void updateSize() {
	    databaseView.updateDrawingsAreaSize();
    }

    public void zoom(double fr) {
	    if (isUtteranceLoaded()) {
	        utteranceView.zoom(fr);
        } else {
            databaseView.zoom(fr);
        }
    }

    public void fitZoom(Duration duration) {
        getZoomer().fit(duration);
        zoom(getZoomer().zoom());
    }

    @Deprecated
    public void fitZoom(double duration) {
	    getZoomer().fit(duration);
        zoom(getZoomer().zoom());
    }

	// value - SS3MediaFile
	private final ArrayList<SS3MediaFile> availableMediaFiles = new ArrayList<SS3MediaFile>();

	public void setVisible(Event event) {
		double startCoord = event.getStartTimeCoordinate();
		double endCoord = event.getEndTimeCoordinate();

		/*
		 * Note: if you are working within the utterance view, but pass in the "parent" as a PresentationEvent, you may
		 * unexpectedly go into the if-block below where you may have intended to go into the else-block instead.
		 * Probably better to use the other setVisible(int, int, boolean) below
		 */
		if (event instanceof PresentationEventsEntity
				|| event instanceof PresentationEvent) {
			databaseView.setVisible(startCoord, endCoord);
		} else {
			utteranceView.setVisible(startCoord, endCoord, false);
		}

		repaint();
	}

	public void scroll(Time time) {
	    var coord = time.as(TimeUnit.COORDINATE);
	    if (isUtteranceLoaded()) {
	        utteranceView.setVisible(coord, coord, true);
        } else {
            databaseView.setVisible(coord, coord);
        }
    }

	public void setVisible(double startCoord, double endCoord) {
		if (isUtteranceLoaded() && utteranceView != null) {
            utteranceView.setVisible(startCoord, endCoord, false);
		} else if (!isUtteranceLoaded() && databaseView != null) {
            databaseView.setVisible(startCoord, endCoord);
		}

		repaint();
	}

	public void setVisible(Time t) {
	    var x = t.as(TimeUnit.COORDINATE);
        setVisible(x, x);
    }

	public void setSelectedFieldWrapper(FieldWrapper selectedWrapper) {
		this.selectedWrapper = selectedWrapper;
	}

	public FieldWrapper getSelectedFieldWrapper() {
		return selectedWrapper;
	}

	public void setSelectedEvent(Event selectedEvent) {
		this.selectedEvent = selectedEvent;
	}

	public Event getSelectedEvent() {
		return selectedEvent;
	}

	public Segment getSegment(String segmentID) {
		for (Segment segment : segments) {
			if (segment.getID().equals(segmentID)) {
				return segment;
			}
		}
		return null;
	}

	public ToolBarPanels getToolBarPanels() {
		return tbPanel;
	}

	public boolean isEntityLevelSelected() {
		return entityLevelSelected;
	}

	public void resetPainter() {
	    this.painter = null;
    }

	public Painter getPainter() {
		if (painter == null)
			painter = new Painter(getSS3Database().getSS3CodingScheme());
		return painter;
	}

	public void update(SS3Database database) {
		setSS3Database(database);

		SS3GlossWindow window = database.getSS3GlossWindow();
		Segment segment = selectedSegment;

		segment.getFieldWrappersCollection().removeAll();

		// most of below code is transplanted from SignStreamSegmentPanel constructor
		// since when an update is performed on the view it must act like a new creation

		RootLabelObject rootObject = new RootLabelObject();
		LabelObject presentationObject = new LabelObject();
		rootObject.setPresentationLabelObject(presentationObject);
		
		TimeRange timeRange = SS3Singleton.getVideoTimeRange();
//		TimeRange timeRange = TimeRange.ABSOLUTE;

		SS3GlossWindowSegment dislayTimePeriod = window.getDisplayTimePeriod();

		PresentationField displayTimePeriodField = new PresentationField(null,
				dislayTimePeriod, this);
		DisplayTimeViewWrapper displayTimePeriodFieldWrapper = new DisplayTimeViewWrapper(
				displayTimePeriodField, this, timeRange);
		displayTimePeriodFieldWrapper.setVisible(true);

		LabelObject dObj = new LabelObject();
		dObj.setLabel(dislayTimePeriod.getLabel());
		dObj.setFieldID(displayTimePeriodField.getFieldID());
		dObj.setWrapper(displayTimePeriodFieldWrapper);

		presentationObject.addChild(dObj);

		segment.addFieldWrapper(displayTimePeriodFieldWrapper);

		//Hashtable<String, SS3TemporalPartition> partitions = window.getTemporalPartitions();
		ArrayList<SS3TemporalPartition> partitions = window.getTemporalPartitions();

		//for (SS3TemporalPartition partition : partitions.values()) {
		for (SS3TemporalPartition partition : partitions) {
			ArrayList<SS3GlossWindowSegment> segments = partition.getTemporalPartitionSegments();

			PresentationField pf = new PresentationField(partition, this);
			PresentationFieldWrapper pfw = new PresentationFieldWrapper(pf,	this, timeRange);
			pfw.setVisible(true);

			LabelObject temporalPartitionObject = new LabelObject();
			temporalPartitionObject.setLabel(partition.getLabel());
			temporalPartitionObject.setFieldID(pf.getFieldID());
			temporalPartitionObject.setWrapper(pfw);
			segment.addFieldWrapper(pfw);

			for (SS3GlossWindowSegment sgm : segments) {
				PresentationField spf = new PresentationField(partition, sgm, this);
				spf.setParentField(pf);

				ArrayList<PresentationEventsEntity> segmentEntities = spf.getEntities();
                for (PresentationEventsEntity segmentEntity : segmentEntities) {
                    PresentationEventsEntity masterEntity = pf
                            .getEntityWithStartTime(segmentEntity.getStartTimeInfo());
                    if (masterEntity != null) {
                        masterEntity.addDependentEntity(segmentEntity);
                    } else {
                        SS3SignStreamApplication.logger.setLevel(Level.SEVERE);
                        SS3SignStreamApplication.logger
                                .info("Could not find master entity for entity "
                                        + segmentEntity.getToolTipText());
                    }
                }

				SegmentFieldWrapper sw = new SegmentFieldWrapper(spf, this,
						partition.getLabel(), sgm.getLabel(), timeRange);
				sw.setVisible(true);

				pfw.addSegmentFieldWrapper(sw);

				LabelObject temporalPartitionPresentationObject = new LabelObject();
				temporalPartitionPresentationObject
						.setFieldID(spf.getFieldID());
				String temporalPartitionPresentationObjectLabel =
						sgm.getParticipantLabel() == null || sgm.getParticipantLabel().isEmpty()
								? sgm.getLabel()
								: sgm.getParticipantLabel() + Util.SEGMENT_TIER_SEPARATOR + sgm.getLabel();
				temporalPartitionPresentationObject.setLabel(temporalPartitionPresentationObjectLabel);
				temporalPartitionPresentationObject.setWrapper(sw);
				temporalPartitionPresentationObject.setDefaultSegmentTier(sgm.isDefault());

				temporalPartitionObject
						.addChild(temporalPartitionPresentationObject);
				segment.addFieldWrapper(sw);
			}

			presentationObject.addChild(temporalPartitionObject);
		}

		segment.setRootLabelObject(rootObject);

//		databaseView.zoom(1);
		databaseView.updatePanels();
	}

	/**
	 * Add a new LabelObject and Wrapper to represent a new segment tier
	 * Then call update() to reflect changes in the view.
	 */
	public void createSegment(SS3Participant participant) {
		DefaultMutableTreeNode node = databaseView.getLabelView().getSelectedNode();

		Object obj = node.getUserObject();

		// if child node (segment tier) is selected, then select its temporal partition "parent"
		if (node.getLevel() == 2) {
			DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) node
					.getParent();
			obj = parentNode.getUserObject();
			node = parentNode;
		}

		LabelObject partitionObj = null;

		if (obj instanceof LabelObject) {
			partitionObj = (LabelObject) obj;
		} else if (obj instanceof TreeNodeVector) {
			TreeNodeVector vector = (TreeNodeVector) obj;
			partitionObj = vector.getObject();
		}

		String partitionLabel = partitionObj.getLabel();
		ArrayList<SS3TemporalPartition> partitions = database.getSS3GlossWindow().getTemporalPartitions();

		SS3TemporalPartition partition = new SS3TemporalPartition();

		for (SS3TemporalPartition tempPartition : partitions) {
			if (tempPartition.getLabel().equals(partitionLabel)) {
				partition = tempPartition;
			}
		}

		// check that no children of node (segments) have children of their own
		if (node.getDepth() <= 1) {
			String segmentLabel = "Segment Tier 1";
			if (node.getChildCount() > 0) {
				int counter = node.getChildCount() + 1;
				segmentLabel = "Segment Tier " + counter;

				while (partition.getTemporalPartitionSegmentByLabel(segmentLabel) != null) {
					counter++;
					segmentLabel = "Segment Tier " + counter;
				}
			}

			// segment to add
			SS3GlossWindowSegment tpSegment = new SS3GlossWindowSegment();
			tpSegment.setParticipant(participant);
			tpSegment.setLabel(segmentLabel);
			partition.addTemporalPartitionSegment(tpSegment);

			// addChild() makes segmentObj visible to LabelViewTemporalPartitions.populateTree()
			LabelObject segmentObj = new LabelObject("", tpSegment.getLabel());
			partitionObj.addChild(segmentObj);

			TimeRange timeRange = SS3Singleton.getVideoTimeRange();
			
			// wrappers to make the new segment row clickable, and wrap segment-wrapper inside partition-wrapper
			PresentationField spf = new PresentationField(partition, tpSegment, this); // creates unique ID here
			SegmentFieldWrapper sw = new SegmentFieldWrapper(spf, this,
					partition.getLabel(), tpSegment.getLabel(), timeRange);
			
			PresentationFieldWrapper pfw = (PresentationFieldWrapper)partitionObj.getWrapper();

			PresentationField pf = (PresentationField) pfw.getField();
            spf.setParentField(pf);

			// clone the entities from the partition field, along with the item inside each
			// see PresentationFieldWrapper.mouseReleased() for source of most of below code
			ArrayList<PresentationEventsEntity> entities = pf.getEntities();
			for (EventsEntity entity : entities) {
				PresentationEventsEntity pEntity = (PresentationEventsEntity) entity;    // this cast should always work
				PresentationEventsEntity clonedEntity = pEntity.clone();
				
				pEntity.getStartTimeInfo().addDependentTimeInfo(
						clonedEntity.getStartTimeInfo());
				pEntity.getEndTimeInfo().addDependentTimeInfo(
						clonedEntity.getEndTimeInfo());
				pEntity.addDependentEntity(clonedEntity);
				spf.addEventsEntity(clonedEntity);    // spf wrapped by sw

				SS3GlossWindowItem pItem = pEntity.getEvent().getSS3GlossWindowItem();
				SS3GlossWindowItem clonedItem
						= new SS3GlossWindowItem(partition.getLabel(), tpSegment.getLabel());
				clonedItem.setId(Util.getLongUniqueNumber());
				clonedItem.setStartTimeInfo(pItem.getStartTimeInfo());
				clonedItem.setEndTimeInfo(pItem.getEndTimeInfo());
				clonedItem.setLabel(pEntity.getEvent().getText());

				PresentationEvent pEvent = clonedEntity.getEvent();
				pEvent.setSS3GlossWindowItem(clonedItem);
				pItem.addSS3GlossWindowItem(clonedItem);

				spf.getSS3GlossWindowSegment().addSS3GlossWindowItem(clonedItem);    // spf wrapped by sw
			}

			pfw.addSegmentFieldWrapper(sw);
			segmentObj.setWrapper(sw);    // sw wrapping spf

			update(database);

            databaseView.getLabelView().selectSegment(tpSegment.getID());
        }
	}

	/**
	 * Add a new LabelObject and Wrapper to represent a new temporal partition
	 * Then call update() to reflect changes in the view.
	 */
	public void createTemporalPartition(SS3Participant participant) {
		DefaultMutableTreeNode node = databaseView.getLabelView().getSelectedNode();

		// if child node (segment tier) is selected, then select its temporal partition "parent"
		if (node.getLevel() == 2) {
			DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) node
					.getParent();
			node = parentNode;
		}

		LabelObject partitionObj = new LabelObject();

		String partitionLabel = partitionObj.getLabel();


		// check that no children of node (segments) have children of their own
		if (node.getDepth() <= 1) {
			String tpLabel = "Temporal Partition 1";
			String segmentLabel = "Segment Tier 1";

			DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) node.getParent();

			int index = parentNode.getChildCount();

			if (parentNode.getChildCount() > 0) {

				do {
					tpLabel = "Temporal Partition " + index++;
				} while (database.getSS3GlossWindow().getTemporalPartitionByLabel(tpLabel) != null);
			}

			SS3TemporalPartition newPartition = new SS3TemporalPartition();
			newPartition.setLabel(tpLabel);

			RootLabelObject rootObject = new RootLabelObject();
			rootObject.setPresentationLabelObject(partitionObj);
			
			TimeRange timeRange = SS3Singleton.getVideoTimeRange();
			 
			// segment to add
			SS3GlossWindowSegment tpSegment = new SS3GlossWindowSegment();
			tpSegment.setParticipant(participant);
			tpSegment.setLabel(segmentLabel);
			tpSegment.setID("" + Util.getLongUniqueNumber());
			newPartition.addTemporalPartitionSegment(tpSegment);
			newPartition.setDefaultSegment(tpSegment);    // tpSegment is the only ST in this new TP, thus it must be the default

			// addChild() makes segmentObj visible to LabelViewTemporalPartitions.populateTree()
			LabelObject segmentObj = new LabelObject("", tpSegment.getLabel());
			partitionObj.addChild(segmentObj);

			// wrappers to make the new segment row clickable, and wrap segment-wrapper inside partition-wrapper
			PresentationField spf = new PresentationField(newPartition, tpSegment, this); // creates unique ID here
			SegmentFieldWrapper sw = new SegmentFieldWrapper(spf, this, 
					newPartition.getLabel(), tpSegment.getLabel(), timeRange);
			
			PresentationField pf = new PresentationField(newPartition, this);
			PresentationFieldWrapper pfw = new PresentationFieldWrapper(pf, this, timeRange);
			pfw.setVisible(true);
			spf.setParentField(pf);

			// clone the entities from the partition field, along with the item inside each
			// see PresentationFieldWrapper.mouseReleased() for source of most of below code
			ArrayList<PresentationEventsEntity> entities = pf.getEntities();
			for (EventsEntity entity : entities) {
				PresentationEventsEntity pEntity = (PresentationEventsEntity) entity;    // this cast should always work
				PresentationEventsEntity clonedEntity = pEntity.clone();
				
				pEntity.getStartTimeInfo().addDependentTimeInfo(
						clonedEntity.getStartTimeInfo());
				pEntity.getEndTimeInfo().addDependentTimeInfo(
						clonedEntity.getEndTimeInfo());
				pEntity.addDependentEntity(clonedEntity);
				spf.addEventsEntity(clonedEntity);    // spf wrapped by sw

				SS3GlossWindowItem pItem = pEntity.getEvent().getSS3GlossWindowItem();
				SS3GlossWindowItem clonedItem
						= new SS3GlossWindowItem(newPartition.getLabel(), tpSegment.getLabel());
				clonedItem.setId(Util.getLongUniqueNumber());
				clonedItem.setStartTimeInfo(pItem.getStartTimeInfo());
				clonedItem.setEndTimeInfo(pItem.getEndTimeInfo());
				clonedItem.setLabel(pEntity.getEvent().getText());

				PresentationEvent pEvent = clonedEntity.getEvent();
				pEvent.setSS3GlossWindowItem(clonedItem);
				pItem.addSS3GlossWindowItem(clonedItem);

				spf.getSS3GlossWindowSegment().addSS3GlossWindowItem(clonedItem);    // spf wrapped by sw
			}

			pfw.addSegmentFieldWrapper(sw);
			segmentObj.setWrapper(sw);    // sw wrapping spf

			database.getSS3GlossWindow().getTemporalPartitions().add(newPartition);


			update(database);

            databaseView.getLabelView().selectPartition(tpLabel);
		}
	}

	/**
	 * Delete the LabelObject and Wrapper representing the selected segment tier
	 * Then call update() to reflect changes in the view.
	 */
	public void deleteSegment() {
		LabelViewTemporalPartitions labelView = databaseView.getLabelView();
		DefaultMutableTreeNode node = labelView.getSelectedNode();

		if (node == null)
			return;

		DefaultMutableTreeNode prevSiblingNode = node.getPreviousSibling();
		DefaultMutableTreeNode nextSiblingNode = node.getNextSibling();
		DefaultMutableTreeNode prevNode = node.getPreviousNode();
		DefaultMutableTreeNode nextNode = node.getNextNode();

		// if selected DisplayTimePeriod layer
		System.out.println(node.getLevel());
		if (prevSiblingNode == null && node.getLevel() == 1) {
			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_SEGMENT_TIER);
			return;
		}

		if (prevSiblingNode != null) {
			labelView.setSelectedNode(new TreePath(prevSiblingNode.getPath()));
		} else if (nextSiblingNode != null) {
			labelView.setSelectedNode(new TreePath(nextSiblingNode.getPath()));
		} else if (prevNode != null) {
			labelView.setSelectedNode(new TreePath(prevNode.getPath()));
		} else if (nextNode != null) {
			labelView.setSelectedNode(new TreePath(nextNode.getPath()));
		}

		Object obj = node.getUserObject();

		LabelObject lObj = null;

		if (obj instanceof LabelObject) {
			lObj = (LabelObject) obj;
		} else if (obj instanceof TreeNodeVector) {
			TreeNodeVector vector = (TreeNodeVector) obj;
			lObj = vector.getObject();
		}

		FieldWrapper wrp = lObj.getWrapper();

		if (wrp instanceof PresentationFieldWrapper) {
			System.out.println("Call to delete Temporal Partition layer - no action taken");
			return;
		} else if (wrp instanceof SegmentFieldWrapper) {

			ArrayList<SS3TemporalPartition> partitions = database.getSS3GlossWindow().getTemporalPartitions();

			DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) node.getParent();
			Object parentObj = parentNode.getUserObject();

			LabelObject partitionObj = null;

			if (parentObj instanceof LabelObject) {
				partitionObj = (LabelObject) parentObj;
			} else if (obj instanceof TreeNodeVector) {
				TreeNodeVector vector = (TreeNodeVector) parentObj;
				partitionObj = vector.getObject();
			}

			// removeChild() makes lObj invisible to LabelViewTemporalPartitions.populateTree()
			// partitionObj is inside selectedSegment object - so still being used here
			partitionObj.removeChild(lObj);

			String partitionLabel = partitionObj.getLabel();
			SS3TemporalPartition partition = new SS3TemporalPartition();

			for (SS3TemporalPartition tempPartition : partitions) {
				if (tempPartition.getLabel().equals(partitionLabel)) {
					partition = tempPartition;
				}
			}

			String participantAndSegmentLabel = lObj.getLabel();
			String segmentLabelOnly = participantAndSegmentLabel.substring(
					participantAndSegmentLabel.indexOf(Util.SEGMENT_TIER_SEPARATOR) + 1);

			SS3GlossWindowSegment segmentToRemove = partition.getTemporalPartitionSegmentByLabel(segmentLabelOnly);
			partition.removeTemporalPartitionSegment(segmentToRemove);

			String segmentTierID = segmentToRemove.getID();
			if (utteranceView != null) {
				for (Segment s: utteranceView.getSegments()) {
					if (s.getID().equals(segmentTierID))
						utteranceView.removeSegment(s);
				}
			}
			
			for (SS3StatementField statementField: segmentToRemove.getStatementFields()) {
				String fieldID = statementField.getId();
				SS3Track track = database.getTrack(fieldID);
				for (SS3Entity ss3Entity: statementField.getSS3Entities()) {
					track.removeSS3Entity(ss3Entity.getID());
				}
				
				statementField.getSS3Entities().clear();				
			}
		}

		update(database);
	}

	/**
	 * Delete the LabelObject and Wrapper representing the selected segment tier
	 * Then call update() to reflect changes in the view.
	 */
	public void deleteTemporalPartition() {
		LabelViewTemporalPartitions labelView = databaseView.getLabelView();
		DefaultMutableTreeNode node = labelView.getSelectedNode();

		if (node == null)
			return;

		DefaultMutableTreeNode prevSiblingNode = node.getPreviousSibling();
		DefaultMutableTreeNode nextSiblingNode = node.getNextSibling();
		DefaultMutableTreeNode prevNode = node.getPreviousNode();
		DefaultMutableTreeNode nextNode = node.getNextNode();

		// if selected DisplayTimePeriod layer
		System.out.println(node.getLevel());
		if (prevSiblingNode == null && node.getLevel() < 1) {
			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_TEMPORAL_PARTITION);
			return;
		}

		if (prevSiblingNode != null) {
			labelView.setSelectedNode(new TreePath(prevSiblingNode.getPath()));
		} else if (nextSiblingNode != null) {
			labelView.setSelectedNode(new TreePath(nextSiblingNode.getPath()));
		} else if (prevNode != null) {
			labelView.setSelectedNode(new TreePath(prevNode.getPath()));
		} else if (nextNode != null) {
			labelView.setSelectedNode(new TreePath(nextNode.getPath()));
		}

		Object obj = node.getUserObject();
		Object parentObj = null;

		// if child node (segment tier) is selected, then select its temporal partition "parent" also
		if (node.getLevel() == 2) {
			DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) node.getParent();
			parentObj = parentNode.getUserObject();
		} else
			parentObj = node.getUserObject();


		LabelObject lObj = null;

		if (obj instanceof LabelObject) {
			lObj = (LabelObject) obj;
		} else if (obj instanceof TreeNodeVector) {
			TreeNodeVector vector = (TreeNodeVector) obj;
			lObj = vector.getObject();
		}

		LabelObject partitionObj = null;

		if (parentObj instanceof LabelObject) {
			partitionObj = (LabelObject) parentObj;
		} else if (obj instanceof TreeNodeVector) {
			TreeNodeVector vector = (TreeNodeVector) parentObj;
			partitionObj = vector.getObject();
		}

		// removeChild() makes lObj invisible to LabelViewTemporalPartitions.populateTree()
		// partitionObj is inside selectedSegment object - so still being used here
		partitionObj.removeChild(lObj);

		String partitionLabel = partitionObj.getLabel();

		SS3TemporalPartition partitionToRemove = database.getSS3GlossWindow().getTemporalPartitionByLabel(partitionLabel);
		database.getSS3GlossWindow().removeTemporalPartition(partitionToRemove);
		
		for (SS3GlossWindowSegment segmentToRemove: partitionToRemove.getTemporalPartitionSegments()) {
			String segmentTierID = segmentToRemove.getID();
			if (utteranceView != null) {
				for (Segment s: utteranceView.getSegments()) {
					if (s.getID().equals(segmentTierID))
						utteranceView.removeSegment(s);
				}
			}
			
			for (SS3StatementField statementField: segmentToRemove.getStatementFields()) {
				String fieldID = statementField.getId();
				SS3Track track = database.getTrack(fieldID);
				for (SS3Entity ss3Entity: statementField.getSS3Entities()) {
					track.removeSS3Entity(ss3Entity.getID());
				}
				
				statementField.getSS3Entities().clear();
			}
		}

		update(database);
	}

	/**
	 * Rename the selected segment tier's label
	 * Then call update() to reflect changes in the view.
	 *
	 * @param newLabel - The label which the selected segment will now acquire.
	 */
	public void renameSegment(String newLabel) {
		DefaultMutableTreeNode node = databaseView.getLabelView().getSelectedNode();

		Object obj = node.getUserObject();
		LabelObject segmentObj = null;

		if (obj instanceof LabelObject) {
			segmentObj = (LabelObject) obj;
		} else if (obj instanceof TreeNodeVector) {
			TreeNodeVector vector = (TreeNodeVector) obj;
			segmentObj = vector.getObject();
		}

		DefaultMutableTreeNode parentNode = null;
		LabelObject partitionObj = null;

		// if child node (segment tier) is selected, then select its temporal partition "parent" also
		if (node.getLevel() == 2) {
			parentNode = (DefaultMutableTreeNode) node.getParent();
			partitionObj = (LabelObject) parentNode.getUserObject();
		} else {
			// this case is handled in 
			// SS3SignStreamApplication.actionPerformed() under RENAME_SEGMENT
			return;
		}

		SS3TemporalPartition partition = database.getSS3GlossWindow().getTemporalPartitionByLabel(partitionObj.getLabel());

		if (partition.getTemporalPartitionSegmentByLabel(newLabel) != null) {
			SS3Singleton.getSignStreamApplication().confirmationDialog("Rename Segment Tier",
					"The name '" + newLabel + "' has already been used.  Please choose a different name");
			return;
		}

		String participantAndSegmentLabel = segmentObj.getLabel();
		String segmentLabelOnly = participantAndSegmentLabel.substring(
				participantAndSegmentLabel.indexOf(Util.SEGMENT_TIER_SEPARATOR) + 1);

		SS3GlossWindowSegment segmentToRename = partition.getTemporalPartitionSegmentByLabel(segmentLabelOnly);
		segmentToRename.setLabel(newLabel);

		update(database);
	}


	/**
	 * Rename (and reset) the selected segment tier's participant
	 * Then call update() to reflect changes in the view.
	 *
	 * @param participant - The new participant which the selected segment will now acquire.
	 */
	public void renameParticipant(SS3Participant participant) {
		DefaultMutableTreeNode node = databaseView.getLabelView().getSelectedNode();

		Object obj = node.getUserObject();
		LabelObject segmentObj = null;

		if (obj instanceof LabelObject) {
			segmentObj = (LabelObject) obj;
		} else if (obj instanceof TreeNodeVector) {
			TreeNodeVector vector = (TreeNodeVector) obj;
			segmentObj = vector.getObject();
		}

		DefaultMutableTreeNode parentNode = null;
		LabelObject partitionObj = null;

		// if child node (segment tier) is selected, then select its temporal partition "parent" also
		if (node.getLevel() == 2) {
			parentNode = (DefaultMutableTreeNode) node.getParent();
			partitionObj = (LabelObject) parentNode.getUserObject();
		} else {
			// this case is handled in 
			// SS3SignStreamApplication.actionPerformed() under RENAME_PARTICIPANT
			return;
		}

		SS3TemporalPartition partition = database.getSS3GlossWindow().getTemporalPartitionByLabel(partitionObj.getLabel());

		String participantAndSegmentLabel = segmentObj.getLabel();
		String segmentLabelOnly = participantAndSegmentLabel.substring(
				participantAndSegmentLabel.indexOf(Util.SEGMENT_TIER_SEPARATOR) + 1);

		SS3GlossWindowSegment segmentToRename = partition.getTemporalPartitionSegmentByLabel(segmentLabelOnly);
		segmentToRename.setParticipant(participant);

		update(database);
	}

	/**
	 * Rename the selected Temporal Partition label
	 * Then call update() to reflect changes in the view.
	 *
	 * @param newLabel - The label which the selected Temporal Partition will now acquire.
	 */
	public void renameTemporalPartition(String newLabel) {
		DefaultMutableTreeNode node = databaseView.getLabelView().getSelectedNode();
		LabelObject partitionObj = null;

		// if child node (segment tier) is selected, then select its temporal partition "parent" also
		if (node.getLevel() == 2) {
			DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) node.getParent();
			partitionObj = (LabelObject) parentNode.getUserObject();
		} else
			partitionObj = (LabelObject) node.getUserObject();

		SS3TemporalPartition partition = new SS3TemporalPartition();

		partition = database.getSS3GlossWindow().getTemporalPartitionByLabel(partitionObj.getLabel());

		partition.setLabel(newLabel);

		update(database);
	}

	/**
	 * Sets the selected segment tier to be the default tier for its encapsulating temporal partition
	 * Then call update() to reflect changes in the view.
	 */
	public void setDefaultSegment() {
		DefaultMutableTreeNode node = databaseView.getLabelView().getSelectedNode();

		Object obj = node.getUserObject();
		LabelObject segmentObj = null;

		if (obj instanceof LabelObject) {
			segmentObj = (LabelObject) obj;
		} else if (obj instanceof TreeNodeVector) {
			TreeNodeVector vector = (TreeNodeVector) obj;
			segmentObj = vector.getObject();
		}

		DefaultMutableTreeNode parentNode = null;
		LabelObject partitionObj = null;

		// if child node (segment tier) is selected, then select its temporal partition "parent" also
		if (node.getLevel() == 2) {
			parentNode = (DefaultMutableTreeNode) node.getParent();
			partitionObj = (LabelObject) parentNode.getUserObject();
		} else {
			// this case is handled in 
			// SS3SignStreamApplication.actionPerformed() under SET_DEFAULT_SEGMENT
			return;
		}

		ArrayList<SS3TemporalPartition> partitions = database.getSS3GlossWindow().getTemporalPartitions();

		SS3TemporalPartition partition = new SS3TemporalPartition();

		for (SS3TemporalPartition tempPartition : partitions) {
			if (tempPartition.getLabel().equals(partitionObj.getLabel())) {
				partition = tempPartition;
			}
		}

		String participantAndSegmentLabel = segmentObj.getLabel();
		String segmentLabelOnly = participantAndSegmentLabel.substring(
				participantAndSegmentLabel.indexOf(Util.SEGMENT_TIER_SEPARATOR) + 1);

		SS3GlossWindowSegment defaultSegment = partition.getTemporalPartitionSegmentByLabel(segmentLabelOnly);

		partition.setDefaultSegment(defaultSegment);

		update(database);
	}

	// this updates the in-memory objects representing data: fields/tracks, holding ss3Entities
	public void updateLoadedDatabase() {
		SS3SignStreamApplication.logger.log(Level.INFO,
				"Updating loaded database...");
		
        ArrayList<String> entityIDsAlreadyUpdatedParent = new ArrayList<String>();

		if (utteranceView != null) {
			ArrayList<Segment> segments = utteranceView.getSegments();

			for (Segment segment : segments) {
				RootLabelObject rootLabelObject = segment.getRootLabelObject();
				LabelObject glossLabelObject = rootLabelObject.getGlossLabelObject();
				LabelObject nonManualLabelObject = rootLabelObject.getNonManualLabelObject();
				LabelObject gestureLabelObject = rootLabelObject.getGestureLabelObject();
				LabelObject customLabelObject = rootLabelObject.getCustomLabelObject();
				
				SS3GlossWindowSegment segmentTier = segment.getParentField().getSS3GlossWindowSegment();
				ArrayList<SS3GlossWindowItem> utterances = segmentTier.getSS3GlossWindowItems();

				SegmentTemplate template = this.utteranceView.getSegmentTemplate(segment.getID());
				Set<String> panelIDs = template.getViewDataManager().getPanelIDAsSet();
				segmentTier.setVisiblePanelIDs(panelIDs);
				
				// assume no fields are configured to permanently be visible - then below only add as required
				segmentTier.removeAllPermanentFields();

				SS3GlossWindow window = this.database.getSS3GlossWindow();
				window.removeAllPermanentFields();
				
				for (SS3GlossWindowItem utterance: utterances) {
					utterance.removeAllPermanentFields();
				}
					
					ArrayList<LabelObject> allObjects = nonManualLabelObject.getAllChildren();
					allObjects.addAll(glossLabelObject.getAllChildren());
					allObjects.addAll(gestureLabelObject.getAllChildren());
					allObjects.addAll(customLabelObject.getAllChildren());

					for (LabelObject obj : allObjects) {
						var wrapper = obj.getWrapper();
						var field = wrapper.getField();
						String fieldID = field.getFieldID();                // GraphField overrides getFieldID()

						boolean permVisibleGlobally = obj.isPermanentlyVisibleGlobally();
						boolean permVisibleAcrossSegmentTier = obj.isPermanentlyVisibleAcrossSegmentTier();

						if (permVisibleAcrossSegmentTier)
							segmentTier.addPermanentFieldAsID(fieldID);

						if (permVisibleGlobally)
							window.addPermanentFieldAsID(fieldID);

						for (SS3GlossWindowItem utterance : utterances) {
							if (obj.isUtterancePermanentlyVisible(utterance))
								utterance.addPermanentlyVisibleFieldAsID(fieldID);
						}

						// first: we delete everything from the statement field
						// and for those entities within the statement field, remove them from the db track as well
						SS3StatementField sField = segmentTier.getSS3StatementField(fieldID);

						if (sField == null) {
							sField = new SS3StatementField();
							sField.setId(fieldID);
							sField.setTimeRange(field.timeRange());
							segmentTier.addField(sField);
						} else if (wrapper.timeRange() != wrapper.defaultTimeRange()) {
							sField.setTimeRange(field.timeRange());
						}

						SS3Track track = database.getTrack(fieldID);
						if (track == null) {
							track = new SS3Track();
							track.setId(fieldID);
							database.addTrack(track);
						}

						for (SS3Entity ss3Entity : sField.getSS3Entities()) {
							track.removeSS3Entity(ss3Entity.getID());    // removing only those entities from the db track which were in the stmt field
						}

						sField.getSS3Entities().clear();        // remove all entities from the stmt field

						if (field.getEntities().isEmpty()) {
							// texts =/= entities, so gloss fields will be marked as empty of entities even though they might hold texts
							if (fieldID.equalsIgnoreCase(Util.DOM_GLOSS_FIELD_ID)
									|| fieldID.equalsIgnoreCase(Util.NDOM_GLOSS_FIELD_ID)) {
								GlossField glsField = (GlossField) field;
								GlossChainedEventsEntityCollection col = glsField.getGlossChainedEventsEntityCollection();
								EnteredText enteredText = col.getEnteredText();

								if (enteredText != null) {
									ArrayList<Text> texts = enteredText
											.getEnteredText();

									ArrayList<SS3Entity> statementFieldEntitiesList = new ArrayList<SS3Entity>();
									for (Text text : texts) {
										SS3Entity ss3Entity = text.getSS3Entity();
										track.addEntity(ss3Entity);
										statementFieldEntitiesList.add(ss3Entity);
									}
									sField.setSS3Entities(statementFieldEntitiesList);
								}
							}
						} else {
							ArrayList<EventsEntity> entities = field.getEntities();
							ArrayList<SS3Entity> statementFieldEntitiesList = new ArrayList<SS3Entity>();

							// next: add all entities...
							for (EventsEntity entity : entities) {
								String entityID = entity.getID();

								// no need to update parent multiple times in one go
								if (!entityIDsAlreadyUpdatedParent.contains(entityID)) {
									var ret = entity.updateParentEntity();
									if (ret != null && !ret.isEmpty())
										entityIDsAlreadyUpdatedParent.addAll(ret);
								}

								SS3Entity ss3Entity = entity.getSS3Entity();
								track.addEntity(ss3Entity);
								statementFieldEntitiesList.add(ss3Entity);
							}

							// ...and add all texts...
							if (fieldID.equalsIgnoreCase(Util.DOM_GLOSS_FIELD_ID)
									|| fieldID.equalsIgnoreCase(Util.NDOM_GLOSS_FIELD_ID)) {
								GlossField glsField = (GlossField) field;
								GlossChainedEventsEntityCollection col = glsField
										.getGlossChainedEventsEntityCollection();
								EnteredText enteredText = col.getEnteredText();

								ArrayList<Text> texts = enteredText
										.getEnteredText();


								for (Text text : texts) {
									SS3Entity ss3Entity = text.getSS3Entity();
									track.addEntity(ss3Entity);
									statementFieldEntitiesList.add(ss3Entity);
								}
							}

							// ...or add all gesture texts...
							if (fieldID.equalsIgnoreCase(Util.DOM_GESTURE_FIELD_ID)
									|| fieldID.equalsIgnoreCase(Util.NDOM_GESTURE_FIELD_ID)) {
								GestureField gestField = (GestureField) field;
								GestureEventsEntityCollection col = gestField
										.getGestureEventsEntityCollection();
								GestureEnteredText enteredText = col.getEnteredText();

								// GD: not fixing/cleaning up the below as I did with gloss texts because it is all dead code;
								// gestures do not use Text classes - a TODO for later is to go through the code and clean
								// up all the gesture text dead code

								ArrayList<GestureText> texts = enteredText
										.getEnteredText();
								for (GestureText text : texts) {
									SS3Item item = text.getSS3Item();
									SS3Entity ss3Entity = new SS3Entity();
									ss3Entity.addItem(item);

									String id = edu.bu.signstream.common.util.Util
											.getLongUniqueNumber();
									ss3Entity.setID(id);
									track.addEntity(ss3Entity);
									statementFieldEntitiesList.add(ss3Entity);
								}
							}

							// set the joined list of entities + texts into the stmt field
							sField.setSS3Entities(statementFieldEntitiesList);
						}
					}

				template.revalidate();
				template.repaint();
			}
		}
		
		if (databaseView != null) {

			FieldWrappersCollection collection = databaseView
					.getSegmentGraphicRepresentation()
					.getFieldWrappersCollection();

			ArrayList<FieldWrapper> wrappers = collection.getWrappers();

			SS3GlossWindow window = database.getSS3GlossWindow();

			ArrayList<SS3TemporalPartition> temporalPartitions = window.getTemporalPartitions();

			SS3TemporalPartition tp = null;
			SS3GlossWindowSegment s = null;

			for (SS3TemporalPartition temporalPartition : temporalPartitions) {
				tp = temporalPartition;

				ArrayList<SS3GlossWindowSegment> segments1 = temporalPartition
						.getTemporalPartitionSegments();
				for (SS3GlossWindowSegment segment : segments1) {

					s = segment;

				}
			}

			for (FieldWrapper wrapper : wrappers) {
				ArrayList<SS3GlossWindowItem> windowItems = new ArrayList<>();

				ArrayList<EventsEntity> entities = wrapper.getField()
						.getEntities();

				if (wrapper instanceof PresentationFieldWrapper) {
					for (EventsEntity ent : entities) {
						PresentationEventsEntity pentity = (PresentationEventsEntity) ent;
						PresentationEvent pevent = pentity.getEvent();
						SS3GlossWindowItem wItem = pevent
								.getSS3GlossWindowItem();
						windowItems.add(wItem);
					}
					tp.setItems(windowItems);
				} else if (wrapper instanceof SegmentFieldWrapper) {

					for (EventsEntity ent : entities) {
						PresentationEventsEntity sentity = (PresentationEventsEntity) ent;
						PresentationEvent sevent = sentity.getEvent();
						SS3GlossWindowItem sItem = sevent
								.getSS3GlossWindowItem();
						windowItems.add(sItem);
					}
					s.setItems(windowItems);
				}
			}

		}

		SS3SignStreamApplication.logger.log(Level.INFO,
				"Loaded database has been updated.");

	}


	public void closeAllLoadedVideoFiles() {
		Set<Map.Entry<JDialog, java.io.File>> omSet = openedMediaDialogs
				.entrySet();
		Iterator<Map.Entry<JDialog, java.io.File>> omi = omSet.iterator();

		ArrayList<JDialog> dialogsToRemove = new ArrayList<JDialog>();

		while (omi.hasNext()) {
			Map.Entry<JDialog, java.io.File> me = omi.next();

			JDialog dialog = me.getKey();
			dialog.removeAll();
			dialog.removeNotify();
			dialog.setVisible(false);
			dialogsToRemove.add(dialog);
			dialog = null;
		}

		for (JDialog d : dialogsToRemove) {
			openedMediaDialogs.remove(d);
		}

		SS3Singleton.getMediaPlayer().close();
    }

    public void loadDatabaseView() {
        utteranceViewState = Optional.of(new UtteranceViewState(utteranceView));
        utteranceViewState.get().segments.stream()
            .forEach(s -> segmentViewModes.put(s.first, s.second.viewMode));
        var state = databaseViewState;

        updateLoadedDatabase();
        loadDatabase(database);



        // Exit utterance
        SS3Singleton.getMediaToolBar().exitUtterance();

		// Get the green scrubber to the current time
		// SS3Singleton.getMediaPlayer().seek(TimeRange.ABSOLUTE.time(0, TimeUnit.COORDINATE));
		// Based on the time interval that is visible, scroll to the current time such that the current time is in the middle of the visible interval

		Time visibleStartTime = SegmentGraphReprUtil.visibleStart;
		Time visibleEndTime = SegmentGraphReprUtil.visibleEnd;
		Time centerVisibleTime = visibleStartTime.add(visibleEndTime.duration()).divide(2.0);

		this.scroll(centerVisibleTime);
		this.zoom(state.zoom);


        // Update window size
        double width = this.getDatabaseView().getVisibleWidth2();
        this.getZoomer().getCoordinateTimeConvertor().resize(Math.round(width));

/*        if (zoomLocked()) { As far as I can tell, the Zoom and optional state object are very buggy and do not function correctly. I need to investigate it further another time. I just can't wrap my head around it right now.
            zoom(state.zoom);
            scroll(state.scroll);
        } else {
            zoom(0);
        }*/

        // Update view zoom lock
        databaseView.getUpperLeft()
            .lock(zoomLocked());

        // adding here as just a failsafe
    	TextEntryPopupGenerator.deleteInstance();
    }

    public void loadUtteranceView(PresentationField field, PresentationEventsEntity utterance) {
		databaseViewState = new DatabaseViewState(databaseView);
	    var defState = UtteranceViewState.from(field, utterance, utteranceViewState);

        var state = utteranceViewState
            .filter(s -> utterance == null)
            .orElse(defState);

        updateLoadedDatabase();
        loadDatabase(database);

        var handler = SS3Singleton.getManager().getRequestHandler();
        handler.init(this);

        var allFields = databaseView.getSegment()
            .getRootLabelObject()
            .getPresentationLabelObject()
            .getAllChildren().stream()
            .map(c -> (PresentationField)c.getField())
            .collect(Collectors.toMap(
                c -> c.getSS3GlossWindowSegment(),
                c -> c));

        Map<String, SS3GlossWindowSegment> allSegments = database.getSS3GlossWindow()
            .getTemporalPartitions().stream()
            .flatMap(p -> p.getTemporalPartitionSegments().stream())
            .collect(Collectors.toMap(s -> s.getID(), s -> s));

        var stateSegments = (utterance == null ? state.segments : defState.segments).stream()
            .filter(p -> allSegments.containsKey(p.first))
            .collect(Collectors.toList());

        var stateSelectedSegment = utterance == null ? state.selectedSegment : defState.selectedSegment;

        var newSegments = stateSegments.stream()
            .map(p -> {
                String id = p.first;
                SegmentTemplateState s = p.second;

                SS3GlossWindowSegment segment = allSegments.get(id);
                PresentationField f = allFields.get(segment);

                Segment newSegment = handler.convertItemsToSegment(segment, f);
                newSegment.setViewMode(s.viewMode);

				if (segment.getViewDataManager() == null) {
					newSegment.setViewDataManager(initializeDefaultViewDataManager(s.viewMode));
				} else {
					newSegment.setViewDataManager(segment.getViewDataManager());
				}

                return newSegment;
            })
            .collect(Collectors.toCollection(ArrayList::new));

        segments.clear();
        segments.addAll(newSegments);
        loadUtterance(newSegments);

        // Select the right utterances
        stateSegments.forEach(e -> {
            var template = utteranceView.getSegmentTemplateBy(e.first);
            var s = e.second;

            template.getSegmentControlPanel().setSelectedUtterance(s.selectedUtterance);
            template.getNonmanView()
                .getCSFieldDialog()
                .getCSFieldPanel()
                .updateCurrentUtterance();
        });

        // Select the right template
        var selectedSegment = utteranceView.getSegmentTemplateBy(stateSelectedSegment);
        if (selectedSegment != null)
            selectedSegment.selectTemplate();

		// Update the current utterance if needed
        if (utterance != null)
            SS3Singleton.getMediaToolBar().enterUtterance(utterance);
        else if (selectedSegment != null && selectedSegment.getSelectedUtterance() != null)
            SS3Singleton.getMediaToolBar().enterUtterance((PresentationEventsEntity) selectedSegment.getSelectedUtterance());

        // Update window size

		// this.zoom(state.zoom);

        double width = getDatabaseView().getVisibleWidth2();
        getZoomer().getCoordinateTimeConvertor().resize(Math.round(width));

        // Update scroll
        if (utterance != null) {
            if (!zoomLocked())
                fitZoom(utterance.interval().duration());
            else
                zoom(state.zoom);

            var player = SS3Singleton.getMediaPlayer();
            var time = utterance.getStartTimeInfo();
            player.seek(time);
            scroll(time);
        } else {
            zoom(state.zoom);
            scroll(TimeRange.ABSOLUTE.time(Math.round(state.scroll), TimeUnit.COORDINATE));
        }

		if (utterance == null) { // if we are not loading a specific utterance, then we want to center the view on the green scrubber
			// Center the view on the green scrubber
			Time visibleStartTime = SegmentGraphReprUtil.visibleStart;
			Time visibleEndTime = SegmentGraphReprUtil.visibleEnd;
			
			Time centerVisibleTime = visibleStartTime.add(visibleEndTime.duration()).divide(2.0);
			this.scroll(centerVisibleTime);
			this.zoom(state.zoom);
		}




        // Update view zoom lock
        utteranceView.getHeader()
            .getTimelineView()
            .getUpperLeftCorner()
            .lock(zoomLocked());

        // Show fields
       for (var template : utteranceView.getSegmentTemplates()) {
            template.getNonmanView()
                .getCSFieldDialog()
                .getCSFieldPanel()
                .showAllFieldsCurrentUtterance();

/*			template.calcViews(template.getViewDataManager().getViewData(Constants.NONMANUAL_PANEL_ID).getPanelPercentage(),
					template.getViewDataManager().getViewData(Constants.SIGN_PANEL_ID).getPanelPercentage(),
					template.getViewDataManager().getViewData(Constants.GESTURE_PANEL_ID).getPanelPercentage());*/

			template.getSegment().setSegmentPanelSize(template.getCustomPanelizedSplitPane().getSize());
			template.resizePanel();
        }

        // adding here as just a failsafe
    	TextEntryPopupGenerator.deleteInstance();
    }

    public void loadUtterance(ArrayList<Segment> segments) {
		if (sp.getRightComponent() instanceof UtteranceView) {
			sp.remove(utteranceView);
		} else {
			sp.remove(databaseView);
		}

		sp.remove(navigationPanel);
		utteranceView = new UtteranceView(segments, this);
		for (var template : utteranceView.getSegmentTemplates()) {
            template.getNonmanView()
                .getCSFieldDialog()
                .getCSFieldPanel()
                .apply();

			template.resizePanel();

        }
		sp.setRightComponent(utteranceView);

		isUtteranceLoaded = true;
	}

	public boolean isUtteranceLoaded() {
		return isUtteranceLoaded;
	}

	@Override
	public void repaint() {
		super.repaint();

		if (parentFrame != null) {
			parentFrame.invalidate();
			parentFrame.validate();
		}

	}

	public void updateJMenuBar() {
		availableMediaFiles.clear();

		MediaDelegate delegate = new MediaDelegate();
		ArrayList<LoadedMediaFile> mFiles = delegate.getLoadedMediaFiles(this);

		for (LoadedMediaFile lFile : mFiles) {
			if (lFile.isLocated()) {
				String fileName = lFile.getMediaFile().getFile().getName();
				JCheckBoxMenuItem item = new JCheckBoxMenuItem(fileName);
				item.setSelected(lFile.isLoaded());
//				item.addActionListener(this);
				availableMediaFiles.add(lFile.getMediaFile());
				item.setActionCommand("" + (availableMediaFiles.size() - 1));
			}
		}
	}

	public SS3Database getSS3Database() {
		return database;
	}

	public Slider getSlider() {
		return SS3Singleton.getVideoControlManager().getSlider();
	}

	public ArrayList<Segment> getSegments() {
		return segments;
	}

	public DbNavigationPanel getDbNavigationPanel() {
		return navigationPanel;
	}

	public JFrame getParentFrame() {
		return parentFrame;
	}

	public Zoomer getZoomer() {
		return SS3Singleton.getVideoControlManager().getZoomer();
	}

	public MultipleMovieController getMultipleMovieController() {
		return SS3Singleton.getVideoControlManager()
				.getMultipleMovieController();
	}

	public Excerpt getExcerpt() {
		return database.getExcerptObj();
	}

	/*
	 * filesToLoad - array of SS3MediaFile objects
	 */
//	public void closeOpenedMediaDialog(JDialog dialog) {
//		openedMediaDialogs.remove(dialog);
//	}

	public Hashtable<JDialog, java.io.File> getOpenedMediaDialogs() {
		return openedMediaDialogs;
	}

}	// end class SSSP


