package edu.bu.signstream.media.fileNavigation;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.File;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.ButtonPanel;
import edu.bu.signstream.ui.panels.table.sortable.CSFieldTableSorter;

public class MediaPropertiesPanel extends JPanel {

   private MediaTable table = null;

   private MediaTableModel tableModel = null;

   private SignStreamSegmentPanel segmentPanel = null;

   private JButton addBtn = new JButton("Add");

   private JButton deleteBtn = new JButton("Delete");

   private JButton cancelBtn = new JButton("Cancel");

   private JButton okBtn = new JButton("OK");

   private JDialog parent;

   private final int ADD = 0;

   private final int DELETE = 1;

   private final int CANCEL = 2;

   private final int OK = 3;

   private Object[] columnNames = { "Display ", "File Name ", "Delete ", "" };
   //         "Frame Rate ",
   //         "Color ",
   //         "View ",
   //         "Format ",
   //         "" };

   public MediaPropertiesPanel(
      SignStreamSegmentPanel segmentPanel,
      JDialog parent) {
      this.segmentPanel = segmentPanel;
      this.parent = parent;
      initUI();
   }

   private void initUI() {
      addBtn.setActionCommand("" + ADD);
      deleteBtn.setActionCommand("" + DELETE);
      cancelBtn.setActionCommand("" + CANCEL);
      okBtn.setActionCommand("" + OK);

//      addBtn.addActionListener(this);
//      deleteBtn.addActionListener(this);
//      cancelBtn.addActionListener(this);
//      okBtn.addActionListener(this);

      JScrollPane pane = createValuesTable();
      JPanel centerPanel = createPanel(pane);
      
      table.setDefaultRenderer(File.class, new FileRenderer(true));

      ButtonPanel bPanel = new ButtonPanel();
      bPanel.add(addBtn);
      bPanel.add(deleteBtn);
      bPanel.add(cancelBtn);
      bPanel.add(okBtn);

      setLayout(new BorderLayout());
      add(centerPanel, BorderLayout.CENTER);
      add(bPanel, BorderLayout.SOUTH);

      table.getColumnModel().removeColumn(table.getColumnModel().getColumn(3));
      table.setAutoCreateColumnsFromModel(false);
   }

   private JPanel createPanel(Component fPane) {

      JPanel pane = new JPanel();
      GridBagLayout gbl = new GridBagLayout();
      GridBagConstraints gbc = new GridBagConstraints();

      pane.setLayout(gbl);

      gbc.anchor = GridBagConstraints.NORTHWEST;
      gbc.gridwidth = GridBagConstraints.REMAINDER;

      gbc.anchor = GridBagConstraints.NORTH;
      gbc.fill = GridBagConstraints.HORIZONTAL;    

      gbc.anchor = GridBagConstraints.WEST;
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      gbc.insets = new Insets(15, 3, 3, 3);
      pane.add(fPane, gbc);

      return pane;
   }

   private JScrollPane createValuesTable() {
      tableModel =
         new MediaTableModel(columnNames, createTableModelData(), parent);
      CSFieldTableSorter sorter = new CSFieldTableSorter(tableModel);

      table = new MediaTable(sorter);
      sorter.setTableHeader(table.getTableHeader());
      table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

      table
         .getSelectionModel()
         .addListSelectionListener(new ListSelectionListener() {
         @Override
		public void valueChanged(ListSelectionEvent e) {
            if (e.getValueIsAdjusting()) {
               CSFieldTableSorter sorter1 =
                  (CSFieldTableSorter) table.getModel();
               sorter1.update(
                  table.getSelectedRow(),
                  table.getSelectedColumn());

               setColumnsSize();
            }
         }
      });

      JScrollPane scrollPane = new JScrollPane(table);
      scrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

      Dimension d = new Dimension(400, 200);
      table.setMaximumSize(d);
      scrollPane.setMaximumSize(d);
      scrollPane.setPreferredSize(d);

      setColumnsSize();

      tableModel.fireTableDataChanged();
      parent.setResizable(false);
      return scrollPane;
   }

   private Object[][] createTableModelData() {
      Object[][] data = null;
      ArrayList valueList = new ArrayList();
      MediaDelegate delegate = new MediaDelegate();
      ArrayList mFiles = delegate.getLoadedMediaFiles(segmentPanel);

      for (int i = 0; i < mFiles.size(); i++) {
         LoadedMediaFile lFile = (LoadedMediaFile) mFiles.get(i);

         Object[] values =
            {
               new Boolean(lFile.isLoaded()),
               lFile.getMediaFile().getFile(),
               Boolean.FALSE,          
               lFile.getMediaFile().getId()};

         valueList.add(values);
      }

      data = new Object[valueList.size()][];

      for (int j = 0; j < valueList.size(); j++) {
         data[j] = (Object[]) valueList.get(j);
      }
      return data;
   }

   private void setColumnsSize() {

      int displayWidth = 75;
      int fileWidth = 250;
      int deleteWidth = 75;

      TableColumn displayColumn = table.getColumn(columnNames[0]);
      TableColumn fileColumn = table.getColumn(columnNames[1]);
      TableColumn deleteColumn = table.getColumn(columnNames[2]);
      //      TableColumn frameColumn = table.getColumn(columnNames[3]);
      //      TableColumn colorColumn = table.getColumn(columnNames[4]);
      //      TableColumn viewColumn = table.getColumn(columnNames[5]);
      //      TableColumn formatColumn = table.getColumn(columnNames[6]);

      displayColumn.setMinWidth(displayWidth);
      displayColumn.setMaxWidth(displayWidth);
      displayColumn.setPreferredWidth(displayWidth);

      fileColumn.setMinWidth(fileWidth);
      fileColumn.setMaxWidth(fileWidth);
      displayColumn.setPreferredWidth(fileWidth);

      displayColumn.setMinWidth(deleteWidth);
      displayColumn.setMaxWidth(deleteWidth);
      displayColumn.setPreferredWidth(deleteWidth);

      //      frameColumn.setMinWidth(deleteWidth);
      //      frameColumn.setMaxWidth(deleteWidth);
      //      frameColumn.setPreferredWidth(deleteWidth);
      //
      //      colorColumn.setMinWidth(deleteWidth);
      //      colorColumn.setMaxWidth(deleteWidth);
      //      colorColumn.setPreferredWidth(deleteWidth);
      //
      //      viewColumn.setMinWidth(deleteWidth);
      //      viewColumn.setMaxWidth(deleteWidth);
      //      viewColumn.setPreferredWidth(deleteWidth);
      //
      //      formatColumn.setMinWidth(deleteWidth);
      //      formatColumn.setMaxWidth(deleteWidth);
      //      formatColumn.setPreferredWidth(deleteWidth);

      table.revalidate();
   }

   //   private int setPreferredSize() {
   //      int displaySize = setSize(table.getColumn(columnNames[0]));
   //      int nameSize = setSize(table.getColumn(columnNames[1]));
   //      int deleteSize = setSize(table.getColumn(columnNames[2]));
   ////      int frameRateSize = setSize(table.getColumn(columnNames[3]));
   ////      int colorSize = setSize(table.getColumn(columnNames[4]));
   ////      int viewSize = setSize(table.getColumn(columnNames[5]));
   ////      int formatSize = setSize(table.getColumn(columnNames[6]));
   //
   //      int size =
   //         displaySize
   //            + nameSize
   //            + deleteSize;
   ////            + frameRateSize
   ////            + colorSize
   ////            + viewSize
   ////            + formatSize;
   //
   //      return size;
   //   }

   private int setSize(TableColumn column) {
      int columnSize = getPreferredWidthForColumn(column);
      column.setMinWidth(columnSize);
      column.setMaxWidth(columnSize);
      column.setPreferredWidth(columnSize);
      table.revalidate();
      return columnSize;
   }

   public int getPreferredWidthForColumn(TableColumn col) {
         int hw = columnHeaderWidth(col), // hw = header width
   cw = widestCellInColumn(col); // cw = column width

      return hw > cw ? hw : cw;
   }

   private int columnHeaderWidth(TableColumn col) {
      // TableCellRenderer renderer = col.getHeaderRenderer();
      return 75;
   }

   private int widestCellInColumn(TableColumn col) {
      int c = col.getModelIndex(), width = 0, maxw = 0;

      for (int r = 0; r < table.getRowCount(); ++r) {
         TableCellRenderer renderer = table.getCellRenderer(r, c);

         Component comp =
            renderer.getTableCellRendererComponent(
               table,
               table.getValueAt(r, c),
               false,
               false,
               r,
               c);

         width = comp.getPreferredSize().width;

         //         if(c == 1) {
         //        	 File file = (File)table.getValueAt(r, c);
         //        	 String name = file.getName();
         //        	 Font font = comp.getFont();
         //        	 FontMetrics fontMetrics = comp.getFontMetrics(font);
         //        	 width = fontMetrics.stringWidth(name)+15;
         //         }

         maxw = width > maxw ? width : maxw;
      }
      return maxw;
   }

//   @Override
//public void actionPerformed(ActionEvent ae) {
//      String str = ae.getActionCommand();
//
//      switch (Integer.parseInt(str)) {
//         case ADD :
//            String path =
//               SS3Singleton.getMediaFilesLocator().getLastLoadedDirectory();
//
//            JFileChooser chooser = new JFileChooser(new File(path));
//            chooser.addChoosableFileFilter(new MovieFilter());
//            int state = chooser.showOpenDialog(table);
//
//            if (state == JFileChooser.APPROVE_OPTION) {
//               File selectedFile = chooser.getSelectedFile();
//               tableModel.addNewValue(selectedFile);
//               SS3Singleton.getMediaFilesLocator().setLastLoadedDirectory(
//                  selectedFile.getParentFile().getPath());
//            }
//            break;
//         case DELETE :
//            tableModel.deleteTableRows();
//            break;
//         case CANCEL :
//            parent.dispose();
//            break;
//         case OK :
//            ArrayList files = tableModel.getMediaFiles();
//            ArrayList<SS3MediaFile> lFiles = new ArrayList<SS3MediaFile>();
//
//            for (int i = 0; i < files.size(); i++) {
//               LoadedMediaFile file = (LoadedMediaFile) files.get(i);
//               SS3MediaFile mFile = file.getMediaFile();
//               String id = mFile.getId();
//
//               if (id == null || id.length() == 0)
//                  mFile.setId(Util.getLongUniqueNumber());
//
//               segmentPanel.getSS3Database().addMediaFile(mFile);
//               segmentPanel.getExcerpt().addOrOverrideMediaFileID(
//                  mFile.getId());
//
//               if (file.isLoaded())
//                  lFiles.add(mFile);
//            }
//           // int size = segmentPanel.getSS3Database().getMediaFilesCollection().size();
//
//            boolean showVideoFiles = segmentPanel.showVideoFiles(lFiles, new HashMap<String, Point>());
//
//    		if(!showVideoFiles)
//    			return;
//
//            parent.dispose();
//            break;
//      }
//   }
}
