package edu.bu.signstream.media.gstreamer;

import edu.bu.signstream.common.files.SSResources;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.media.Video;

import javax.swing.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.atomic.AtomicBoolean;

public class UrlMediaPlayer extends GstMediaPlayer {
    private static final int BUFFER_SIZE = 1024;
    private static final int PROGRESS_BAR_MAX = 1000;

    private final File file;

    public UrlMediaPlayer(URL url) {
        file = new File(SSResources.rootPath + "/.temp." + Util.uuid() + ".mp4");

        // Create a loading progress bar
        var bar = new JProgressBar(0, PROGRESS_BAR_MAX);
        var loading = new JFrame();
        loading.setTitle("Loading \"" + url.toString() + "\" ...");
        loading.setContentPane(bar);
        loading.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        loading.setSize(400, 50);
        loading.setLocationRelativeTo(null);
        loading.setAlwaysOnTop(true);
        loading.setVisible(true);

        var error = new AtomicBoolean(false);

        // Spawn the downloader thread, update the progress bar, and signal the waiting thread
        var latch = new CountDownLatch(1);
        new Thread(() -> {
            try {
                var con = (HttpURLConnection)url.openConnection();
                var len = con.getContentLength();

                var in  = new BufferedInputStream(con.getInputStream());
                var out = new BufferedOutputStream(new FileOutputStream(file));

                byte[] buf = new byte[BUFFER_SIZE];
                int n;
                int count = 0;
                while ((n = in.read(buf, 0, BUFFER_SIZE)) >= 0) {
                    count += n;
                    var progress = (int)(((double)count / len) * PROGRESS_BAR_MAX);
                    SwingUtilities.invokeLater(() -> bar.setValue(progress));
                    out.write(buf, 0, n);
                }

                out.close();
                in.close();
            } catch (Exception e) {
                e.printStackTrace();
                error.set(true);
            }

            loading.dispose();
            if (error.get()) {
                JOptionPane.showMessageDialog(null,
                    "Unable to download video \"" + url.toString() + "\"", "Download Error",
                    JOptionPane.ERROR_MESSAGE);
            }

            latch.countDown();
        }).start();

        // Spawn the waiting thead, wait for the download, and show the video
        new Thread(() -> {
            try {
                latch.await();
            } catch (InterruptedException e) {
                e.printStackTrace();
                close();
                return;
            }

            if (error.get()) {
                close();
                return;
            }

            Video video;
            try {
                video = addVideo(file.toURI(), 1);
            } catch (Exception e) {
                JOptionPane.showMessageDialog(null,
                    "Unable to play video \"" + url.toString() + "\"", "Playback Error",
                    JOptionPane.ERROR_MESSAGE);
                close();
                return;
            }

            video.view().setAlwaysOnTop(true);
            video.view().requestFocus();
            video.view().addWindowListener(new WindowAdapter() {
                @Override
                public void windowClosing(WindowEvent e) {
                    close();
                }
            });

            preroll();
        }).start();
    }

    @Override
    public void close() {
        super.close();
        file.delete();
    }
}
