package edu.bu.signstream.time;

public class Interval {
    public final Time t0;
    public final Time t1;

    public Interval(Time t0, Time t1) {
        // Ensure t0 is always before t1
        var min = t0.lessThanOrEq(t1) ? t0 : t1;
        var max = t1.greaterThanOrEq(t0) ? t1 : t0;

        this.t0 = min;
        this.t1 = max;
    }

    public Duration duration() {
        return t1.subtract(t0);
    }

    public boolean empty() { return t0.equals(t1); }

    public boolean overlaps(Interval o, Bound b) {
        return o.t0.less(t1, b) && o.t1.greater(t0, b);
    }

    public boolean contains(Interval o, Bound b) {
        return o.t0.greater(t0, b) && o.t1.less(t1, b);
    }
    public boolean contains(Time t, Bound b) {
        return t.greater(t0, b) && t.less(t1, b);
    }

    public Interval convert(TimeUnit u, Rounding r) {
        return t0.convert(u, r).to(t1.convert(u, r));
    }

    @Override
    public boolean equals(Object o) {
        return o instanceof Interval && ((Interval)o).t0.equals(t0) && ((Interval)o).t1.equals(t1);
    }

    @Override
    public String toString() {
        return "Interval{" + t0 + " -> " + t1 + "}";
    }
}
