package edu.bu.signstream.ui.panels.newDatabase;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.media.Audio;
import edu.bu.signstream.media.fileNavigation.LoadedMediaFile;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.stream.Collectors;

public class AudioPanelScrolledView extends JPanel {

	private static final long serialVersionUID = 1L;

	private ArrayList<AudioPanel> videoPanels = new ArrayList<AudioPanel>();

	private AudioPanel videoPanel1 = null;
	private AudioPanel videoPanel2 = null;
	private AudioPanel videoPanel3 = null;
	private AudioPanel videoPanel4 = null;

	private JScrollPane scroller = null;
	private VideoFilesPane videoFilesPane;

	private JLabel videoLabel = new JLabel(" Audio ");
	private JLabel fileNameLabel = new JLabel(" File Name ");

	public AudioPanelScrolledView(ArrayList<LoadedMediaFile> mFiles, Dimension d, JFrame parentFrame) {
		super(new BorderLayout());

        mFiles = mFiles.stream()
            .filter(LoadedMediaFile::isAudio)
            .collect(Collectors.toCollection(ArrayList::new));

		UserInterfaceUtil.setLabelLNF(videoLabel);
		UserInterfaceUtil.setLabelLNF(fileNameLabel);

		if(mFiles.size() >= 1) {
			videoPanel1 = new AudioPanel(mFiles.get(0), parentFrame);
		} else {
			videoPanel1 = new AudioPanel(null, parentFrame);
		}
		videoPanels.add(videoPanel1);
		
		if(mFiles.size() >= 2) {
			videoPanel2 = new AudioPanel(mFiles.get(1), parentFrame);
		} else {
			videoPanel2 = new AudioPanel(null, parentFrame);
		}
		videoPanels.add(videoPanel2);
		
		if(mFiles.size() >= 3) {
			videoPanel3 = new AudioPanel(mFiles.get(2), parentFrame);
		} else {
			videoPanel3 = new AudioPanel(null, parentFrame);
		}
		videoPanels.add(videoPanel3);
		
		if(mFiles.size() >= 4) {
			videoPanel4 = new AudioPanel(mFiles.get(3), parentFrame);
		} else {
			videoPanel4 = new AudioPanel(null, parentFrame);
		}
		videoPanels.add(videoPanel4);

		videoFilesPane = new VideoFilesPane();
		videoFilesPane.setBackground(Color.white);

		scroller = new JScrollPane(videoFilesPane);
		scroller.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		scroller.setPreferredSize(d);

		add(scroller, BorderLayout.CENTER);
	}
	
	public void update(ArrayList<LoadedMediaFile> mFiles) {
		if(mFiles.size() >= 1) {
			videoPanel1.setMediaFile(mFiles.get(0));
		} else {
			videoPanel1.setMediaFile(null);
		}
		videoPanel1.repaint();
		
		if(mFiles.size() >= 2) {
			videoPanel2.setMediaFile(mFiles.get(1));
		} else {
			videoPanel2.setMediaFile(null);
		}
		videoPanel2.repaint();
		
		if(mFiles.size() >= 3) {
			videoPanel3.setMediaFile(mFiles.get(2));
		} else {
			videoPanel3.setMediaFile(null);
		}
		videoPanel3.repaint();
		
		if(mFiles.size() >= 4) {
			videoPanel4.setMediaFile(mFiles.get(3));
		} else {
			videoPanel4.setMediaFile(null);
		}
		videoPanel4.repaint();
	}
	
	public boolean isAtLeastOneValidVideoFileAvailable() {
		for(AudioPanel vp : videoPanels) {
			LoadedMediaFile lFile = vp.getMediaFile();
			if(lFile != null && lFile.getMediaFile() != null && lFile.getMediaFile().getFile() != null && lFile.getMediaFile().getFile().exists()) {
				return true;
			}
		}
		return false;
	}
	
	public ArrayList<AudioPanel> getAudioPanels() {
		return videoPanels;
	}

	/** The component inside the scroll pane. */
	public class VideoFilesPane extends JPanel {

		private static final long serialVersionUID = 1L;

		public VideoFilesPane() {
			JPanel videoPanel = new JPanel();
			GridBagConstraints c = new GridBagConstraints();
			videoPanel.setLayout(new GridBagLayout());
			videoPanel.setBackground(Color.WHITE);
			videoPanel.setForeground(Color.white);

			c.anchor = GridBagConstraints.WEST;
			c.insets = new Insets(0, 1, 0, 1);
			
			c.gridx = 1;
			c.gridwidth = 3;
			videoPanel.add(videoLabel, c);
			c.gridx = 4;
			c.gridwidth = 1;
			videoPanel.add(fileNameLabel, c);

			addComponent( videoPanel1, c, 1, videoPanel);
			addComponent( videoPanel2, c, 2, videoPanel);
			addComponent( videoPanel3, c, 3, videoPanel);
			addComponent( videoPanel4, c, 4, videoPanel);

			setLayout(new FlowLayout(FlowLayout.LEFT));
			add(videoPanel);
		}
	}
	
	private void addComponent(AudioPanel vp, GridBagConstraints c, int gridy, JPanel videoPanel) {
		c.anchor = GridBagConstraints.WEST;
		c.gridx = 1;
		c.gridy = gridy;
		videoPanel.add(vp.getInfoBtn(), c);
		c.gridx = 2;
		videoPanel.add(vp.getSelectBtn(), c);
		c.gridx = 3;
		videoPanel.add(vp.getRemoveBtn(), c);
		c.gridx = 4;
		c.anchor = GridBagConstraints.CENTER;
		videoPanel.add(vp.getDisplayCb(), c);
//		c.gridx = 5;
		// TODO: Re-add lookup checkbox when it is fixed
//		videoPanel.add(vp.getLookupCb(), c);
		c.anchor = GridBagConstraints.WEST;
		c.gridx = 6;
		videoPanel.add(vp.getFileNameLbl(), c);
	}
}
