package edu.bu.signstream.ui.panels.search;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.NoSuchElementException;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;

public class BinarySearchGroup extends SearchGroup implements ActionListener{

	private static final long serialVersionUID = 1L;
	
	// LIST OF EXPRESSIONS (LIMIT IS 2 AS THIS IS BINARY)
	private SearchRule firstRule;
	private SearchRule secondRule;
	
	// GUI COMPONENTS
	
	private JPanel negatePanel;
	private JPanel leftPanel;
	private JPanel rightPanel;
	
	private JPanel operatorPanel;	
	private JRadioButton withButton;
	private JRadioButton beforeButton;
	private JRadioButton afterButton;
	
	// ACTION COMMANDS
	
	private final int REMOVE = 0;
	private final int NEGATE_GROUP = 10;
	private final int WITH = 20;
	private final int BEFORE = 30;
	private final int AFTER = 40;
	
	// LABELS
	
	private final String origGroupButtonLabel = "";
	private final String negateGroupButtonLabel = "NOT";
	
	private final String withRadioLabel = SearchOperator.WITH.toString();
	private final String beforeRadioLabel = SearchOperator.BEFORE.toString();
	private final String afterRadioLabel = SearchOperator.AFTER.toString();

	public BinarySearchGroup(SS3Database database, SearchGroup parentGroup) {
		super(database, parentGroup);
		
		searchTreeNode = new SearchTreeNode(null, SearchOperator.WITH);	// all binary search groups default to WITH
		
		createUI();
	}

	@Override
	public void createUI() {
		
		// color background (no need to alternate since there can be no sub-group to a binary search group)

		int panelBorderWidth = 2;

		java.awt.Color backgroundColor = java.awt.Color.RED.darker();

		setBackground(backgroundColor);
		setOpaque(true);
		setBorder(BorderFactory.createLineBorder(backgroundColor, panelBorderWidth));
		
		// action listener for removeButton
		
		removeButton.addActionListener(this);
		removeButton.setActionCommand("" + REMOVE);
		
		// instantiate and add components for negateButtonPanel
		
		negatePanel = new JPanel();
		negatePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		
		negateGroupButton = new JButton("");
		negateGroupButton.addActionListener(this);
		negateGroupButton.setActionCommand("" + NEGATE_GROUP);

		negatePanel.add(negateGroupButton);
		negatePanel.setBackground(backgroundColor);
		negatePanel.setOpaque(true);
		
		// instantiate and add components for firstRulePanel
		
		firstRule = new SearchRule(database, this);
		searchTreeNode.add(firstRule.getSearchTreeNode());		// must edit this object's node in place after expression creation
		firstRule.enableRemoveButton(false);
		firstRule.enableNegateRuleButton(false);
		firstRule.enableNegateFieldAndValueButton(false);
		firstRule.enableFrameComponents(true);
		
		// instantiate and add components for operatorPanel
		
		operatorPanel = new JPanel();
		operatorPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		
		withButton = new JRadioButton(withRadioLabel);
		withButton.setSelected(true);
		withButton.addActionListener(this);
		withButton.setActionCommand("" + WITH);
		beforeButton = new JRadioButton(beforeRadioLabel);
		beforeButton.addActionListener(this);
		beforeButton.setActionCommand("" + BEFORE);
		afterButton = new JRadioButton(afterRadioLabel);
		afterButton.addActionListener(this);
		afterButton.setActionCommand("" + AFTER);

		ButtonGroup buttonGroup = new ButtonGroup();
		buttonGroup.add(withButton);
		buttonGroup.add(beforeButton);
		buttonGroup.add(afterButton);
		
		operatorPanel.add(withButton);
		operatorPanel.add(beforeButton);
		operatorPanel.add(afterButton);
		operatorPanel.setBackground(backgroundColor);
		operatorPanel.setOpaque(true);
		
		// instantiate and add components for leftPanel

		secondRule = new SearchRule(database, this);
		searchTreeNode.add(secondRule.getSearchTreeNode());		// must edit this object's node in place after expression creation
		secondRule.enableRemoveButton(false);
		secondRule.enableNegateFieldAndValueButton(false);
		secondRule.enableNegateRuleButton(false);
		secondRule.enableFrameComponents(true);
		
		// add sub-panels to this binary search group panel
		
		leftPanel = new JPanel();
		leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.PAGE_AXIS));
		leftPanel.add(removeButton);
		leftPanel.setOpaque(false);
		
		rightPanel = new JPanel();
		rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.PAGE_AXIS));
		rightPanel.add(negatePanel);
		rightPanel.add(firstRule);
		rightPanel.add(operatorPanel);
		rightPanel.add(secondRule);
		rightPanel.setOpaque(false);
		rightPanel.setBorder(BorderFactory.createLineBorder(backgroundColor, 10));
		
		setLayout(new BorderLayout());
		add(leftPanel, BorderLayout.WEST);
		add(rightPanel, BorderLayout.CENTER);		

	}

	@Override
	public void loadSavedSearchTreeNode(SearchTreeNode savedSearchTreeNode) {
		
		if ( savedSearchTreeNode == null || !savedSearchTreeNode.isGroupNode()) 
			return;
		
		searchTreeNode = savedSearchTreeNode;
		
		String savedText = searchTreeNode.isNegateGroup() ? negateGroupButtonLabel : origGroupButtonLabel;
		negateGroupButton.setText(savedText);
		
		SearchOperator savedOperator = searchTreeNode.getSearchOperator();
		if (savedOperator.equals(SearchOperator.WITH))
			withButton.setSelected(true);
		else if (savedOperator.equals(SearchOperator.BEFORE))
			beforeButton.setSelected(true);
		else if (savedOperator.equals(SearchOperator.AFTER))
			afterButton.setSelected(true);

		SearchTreeNode firstChildNode = null, secondChildNode = null;
		try {
			firstChildNode = (SearchTreeNode) savedSearchTreeNode.getFirstChild();
			secondChildNode = (SearchTreeNode) savedSearchTreeNode.getLastChild();
		} catch (NoSuchElementException e) {
			System.out.println("Something bad happened with save/load");
		}

		firstRule = new SearchRule(database, this);
		firstRule.loadSavedSearchTreeNode(firstChildNode);
		firstRule.enableRemoveButton(false);
		firstRule.enableNegateRuleButton(false);
		firstRule.enableNegateFieldAndValueButton(false);
		firstRule.enableFrameComponents(true);

		secondRule = new SearchRule(database, this);
		secondRule.loadSavedSearchTreeNode(secondChildNode);
		secondRule.enableRemoveButton(false);
		secondRule.enableNegateFieldAndValueButton(false);
		secondRule.enableNegateRuleButton(false);
		secondRule.enableFrameComponents(true);
	}

	@Override
	public ArrayList<SearchExpression> getAllExpressions() {
		ArrayList<SearchExpression> expressionsList = new ArrayList<SearchExpression>();
		expressionsList.add(firstRule);
		expressionsList.add(secondRule);
		
		return expressionsList;
	}
	
	@Override
	protected void clear() {
		searchTreeNode.removeAllChildren();
		searchTreeNode.setData(null);
		searchTreeNode.setSearchOperator(SearchOperator.WITH);
		
		firstRule.getSearchTreeNode().removeAllChildren();
		secondRule.getSearchTreeNode().removeAllChildren();
		
		searchTreeNode.add(firstRule.getSearchTreeNode());	
		searchTreeNode.add(secondRule.getSearchTreeNode());
		
		revalidate();
		repaint();
	}

	@Override
	public void removeExpression(SearchExpression expression) {
		// do nothing - can only do nothing
		// should never be called to begin with
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		
		switch(Integer.parseInt(command)) {
		case REMOVE:
			parentGroup.getSearchTreeNode().remove(searchTreeNode);
			parentGroup.removeExpression(this);
			break;
		case NEGATE_GROUP:
			String text = negateGroupButton.getText();
			if (text.equals("")) {
				negateGroupButton.setText(negateGroupButtonLabel);
				searchTreeNode.setNegateGroupFlag(true);
				
			} else {
				negateGroupButton.setText(origGroupButtonLabel);
				searchTreeNode.setNegateGroupFlag(false);
			}
			break;
		case WITH:
			searchTreeNode.setSearchOperator(SearchOperator.WITH);
			break;
		case BEFORE:
			searchTreeNode.setSearchOperator(SearchOperator.BEFORE);
			break;
		case AFTER:
			searchTreeNode.setSearchOperator(SearchOperator.AFTER);
			break;
		}
	}

}
